@@ -21,6 +21,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "client.h"
 
+clentity_t *active_clentities, *free_clentities;
+clentity_t clentities[MAX_CLENTITIES];
+
 /*
 =========================================================================
 
@@ -606,58 +609,6 @@ INTERPOLATE BETWEEN FRAMES TO GET RENDERING PARMS
 ==========================================================================
 */
 
-struct model_s *S_RegisterSexedModel (entity_state_t *ent, char *base)
-{
-	int				n;
-	char			*p;
-	struct model_s	*mdl;
-	char			model[MAX_QPATH];
-	char			buffer[MAX_QPATH];
-
-	// determine what model the client is using
-	model[0] = 0;
-	n = CS_PLAYERSKINS + ent->number - 1;
-	if (cl.configstrings[n][0])
-	{
-		p = strchr(cl.configstrings[n], '\\');
-		if (p)
-		{
-			p += 1;
-			strcpy(model, p);
-			p = strchr(model, '/');
-			if (p)
-				*p = 0;
-		}
-	}
-
-	// if we can't figure it out, they're male
-	if (!model[0])
-		strcpy(model, "male");
-
-	Com_sprintf (buffer, sizeof(buffer), "players/%s/%s", model, base+1);
-	mdl = R_RegisterModel(buffer);
-	if (!mdl) {
-		// not found, try default weapon model
-		Com_sprintf (buffer, sizeof(buffer), "players/%s/weapon.md2", model);
-		mdl = R_RegisterModel(buffer);
-		if (!mdl) {
-			// no, revert to the male model
-			Com_sprintf (buffer, sizeof(buffer), "players/%s/%s", "male", base+1);
-			mdl = R_RegisterModel(buffer);
-			if (!mdl) {
-				// last try, default male weapon.md2
-				Com_sprintf (buffer, sizeof(buffer), "players/male/weapon.md2");
-				mdl = R_RegisterModel(buffer);
-			}
-		}
-	}
-
-	return mdl;
-}
-
-// PMM - used in shell code
-extern int Developer_searchpath (int who);
-// pmm
 /*
 ===============
 CL_AddPacketEntities
@@ -668,21 +619,19 @@ struct model_s
 {
 	char		name[MAX_QPATH];
 };
-extern void CL_MuzzleFlashParticle (vec3_t org, vec3_t angles, qboolean from_client);
+
 void CL_AddPacketEntities (frame_t *frame)
 {
 	entity_t			ent;
 	entity_state_t		*s1;
 	float				autorotate;
 	float				bob, bob_scale;
-	int					i, done, len;
+	int					i;
 	int					pnum;
 	centity_t			*cent;
 	int					autoanim;
 	clientinfo_t		*ci;
 	unsigned int		effects, renderfx;
-	char				modelpath[MAX_OSPATH];
-	char				modelname[MAX_OSPATH];
 	qboolean			playermodel;
 
 	// bonus items rotate at a fixed rate
@@ -700,7 +649,7 @@ void CL_AddPacketEntities (frame_t *frame)
 		cent = &cl_entities[s1->number];
 
 		playermodel = false;
-
+		
 		effects = s1->effects;
 		renderfx = s1->renderfx;
 
@@ -736,39 +685,38 @@ void CL_AddPacketEntities (frame_t *frame)
 
 		// create a new entity
 
-		// tweak the color of beams
-		if ( renderfx & RF_BEAM )
-		{	// the four beam colors are encoded in 32 bits of skinnum (hack)
-			ent.alpha = 0.30;
-			ent.skinnum = (s1->skinnum >> ((rand() % 4)*8)) & 0xff;
-			ent.model = NULL;
+		ent.lod1 = NULL;
+		ent.lod2 = NULL;
+		ent.team = 0;
+
+		// set skin
+		if (s1->modelindex == 255)
+		{	// use custom player skin
+			ent.skinnum = 0;
+			ci = &cl.clientinfo[s1->skinnum & 0xff];
+			ent.skin = ci->skin;
+			ent.model = ci->model;
+
+			ent.lod1 = ci->lod1;
+			
+			ent.lod2 = ci->lod2;
+			
+			if (!ent.skin || !ent.model)
+			{
+				ent.skin = cl.baseclientinfo.skin;
+				ent.model = cl.baseclientinfo.model;
+			}
+			playermodel = true;
+
 		}
 		else
 		{
-			// set skin
-			if (s1->modelindex == 255)
-			{	// use custom player skin
-				ent.skinnum = 0;
-				ci = &cl.clientinfo[s1->skinnum & 0xff];
-				ent.skin = ci->skin;
-				ent.model = ci->model;
-				if (!ent.skin || !ent.model)
-				{
-					ent.skin = cl.baseclientinfo.skin;
-					ent.model = cl.baseclientinfo.model;
-				}
-				playermodel = true;
-
-			}
-			else
-			{
-				ent.skinnum = s1->skinnum;
-				ent.skin = NULL;
-				ent.model = cl.model_draw[s1->modelindex];
-			}
+			ent.skinnum = s1->skinnum;
+			ent.skin = NULL;
+			ent.model = cl.model_draw[s1->modelindex];
 		}
-
-		if (renderfx & (RF_FRAMELERP|RF_BEAM))
+		
+		if (renderfx & (RF_FRAMELERP))
 		{	// step origin discretely, because the frames
 			// do the animation properly
 			VectorCopy (cent->current.origin, ent.origin);
@@ -790,10 +738,6 @@ void CL_AddPacketEntities (frame_t *frame)
 			}
 		}
 
-		// only used for black hole model right now, FIXME: do better
-		if (renderfx == RF_TRANSLUCENT)
-			ent.alpha = 0.70;
-
 		// render effects (fullbright, translucent, etc)
 		if ((effects & EF_COLOR_SHELL))
 			ent.flags = 0;	// renderfx go on color shell entity
@@ -841,77 +785,46 @@ void CL_AddPacketEntities (frame_t *frame)
 		if (!s1->modelindex)
 			continue;
 
-		if (effects & EF_BFG)
-		{
-			ent.flags |= RF_TRANSLUCENT;
-			ent.alpha = 0.30;
-		}
-
-		// RAFAEL
 		if (effects & EF_PLASMA)
 		{
 			ent.flags |= RF_TRANSLUCENT;
 			ent.alpha = 0.6;
 		}
 
-
-		if (renderfx & RF_CUSTOMSKIN) { //wasn't used for anything, so now used for shiny surfaces
-			ent.flags |= RF_CUSTOMSKIN;
+		if (effects & EF_FLIES)
+		{
+			CL_PoweredEffects (ent.origin);
 		}
 
+		//cool new ctf flag effects
+		if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex)], "models/items/flags/flag1.md2")) {
+			CL_FlagEffects(ent.origin, 0);
+			ent.team = 1;
+		}
+		else if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex)], "models/items/flags/flag2.md2")) {
+			CL_FlagEffects(ent.origin, 1);
+			ent.team = 2;
+		}
+		
 		// add to refresh list
 		V_AddEntity (&ent);
 
 		// color shells generate a seperate entity for the main model
 		if ((effects & EF_COLOR_SHELL) && !(s1->number == cl.playernum+1))
 		{
-			// PMM - at this point, all of the shells have been handled
-			// if we're in the rogue pack, set up the custom mixing, otherwise just
-			// keep going
-//			if(Developer_searchpath(2) == 2)
-//			{
-				// all of the solo colors are fine.  we need to catch any of the combinations that look bad
-				// (double & half) and turn them into the appropriate color, and make double/quad something special
-				if (renderfx & RF_SHELL_HALF_DAM)
-				{
-					if(Developer_searchpath(2) == 2)
-					{
-						// ditch the half damage shell if any of red, blue, or double are on
-						if (renderfx & (RF_SHELL_RED|RF_SHELL_BLUE|RF_SHELL_DOUBLE))
-							renderfx &= ~RF_SHELL_HALF_DAM;
-					}
-				}
-
-				if (renderfx & RF_SHELL_DOUBLE)
-				{
-					if(Developer_searchpath(2) == 2)
-					{
-						// lose the yellow shell if we have a red, blue, or green shell
-						if (renderfx & (RF_SHELL_RED|RF_SHELL_BLUE|RF_SHELL_GREEN))
-							renderfx &= ~RF_SHELL_DOUBLE;
-						// if we have a red shell, turn it to purple by adding blue
-						if (renderfx & RF_SHELL_RED)
-							renderfx |= RF_SHELL_BLUE;
-						// if we have a blue shell (and not a red shell), turn it to cyan by adding green
-						else if (renderfx & RF_SHELL_BLUE)
-							// go to green if it's on already, otherwise do cyan (flash green)
-							if (renderfx & RF_SHELL_GREEN)
-								renderfx &= ~RF_SHELL_BLUE;
-							else
-								renderfx |= RF_SHELL_GREEN;
-					}
-				}
-//			}
-			// pmm
+		
 			ent.flags = renderfx | RF_TRANSLUCENT;
 			ent.alpha = 0.30;
-			V_AddEntity (&ent);
+			V_AddViewEntity (&ent);			
 		}
 
 		ent.skin = NULL;		// never use a custom skin on others
 		ent.skinnum = 0;
 		ent.flags = 0;
 		ent.alpha = 0;
+		ent.lod1 = NULL;		// only player models get lods
+		ent.lod2 = NULL;
+		ent.team = 0;
 
 		ci = &cl.clientinfo[s1->skinnum & 0xff];
 
@@ -974,41 +887,6 @@ void CL_AddPacketEntities (frame_t *frame)
 				ent.alpha = 0.4;
 				ent.flags = RF_TRANSLUCENT;
 			}
-			else if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex2)], "models/misc/tube/cover.md2"))
-			{
-				ent.alpha = 0.4;
-				ent.flags = RF_TRANSLUCENT;
-			}
-			else if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex2)], "models/misc/receptor/bulb.md2"))
-			{
-				ent.alpha = 0.4;
-				ent.flags = RF_TRANSLUCENT;
-			}
-			else if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex2)], "models/misc/electrode/cover.md2"))
-			{
-				ent.alpha = 0.4;
-				ent.flags = RF_TRANSLUCENT;
-			}
-			else if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex2)], "models/misc/scope/cover.md2"))
-			{
-				ent.alpha = 0.4;
-				ent.flags = RF_TRANSLUCENT;
-			}
-			else if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex2)], "models/misc/console/cover.md2"))
-			{
-				ent.alpha = 0.4;
-				ent.flags = RF_TRANSLUCENT;
-			}
-			else if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex2)], "players/war/helmet.md2"))
-			{
-				ent.alpha = 0.4;
-				ent.flags = RF_TRANSLUCENT;
-			}
-			else if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex)], "models/objects/blank/tris.md2"))
-			{
-				ent.alpha = 0.1;
-				ent.flags = RF_TRANSLUCENT;
-			}
 
 			if (s1->number == cl.playernum+1) {
 				ent.flags |= RF_VIEWERMODEL;
@@ -1024,42 +902,16 @@ void CL_AddPacketEntities (frame_t *frame)
 
 		}
 
-		if (s1->modelindex3 || (ci->helmet && playermodel))
+		if (s1->modelindex3 || (ci->helmet && playermodel)) //index 3 is only used for clear entities now, no need for all that checking
 		{
 			if(ci->helmet && playermodel)
 				ent.model = ci->helmet;
 			else
 				ent.model = cl.model_draw[s1->modelindex3];
 
-			//look for clear player helmets and flag accordingly
-			strcpy(modelpath, cl.configstrings[CS_MODELS+(s1->modelindex3)]);
-			len = strlen (modelpath);
-
-			i = -1;
-			strcpy(modelname, " ");
-
-			done = false;
-
-
-			while(!done) {
-				if(modelpath[len] == 'h')
-					done = true;
-				else {
-					modelname[i] = modelpath[len];
-
-				}
-				if(len < 1)
-					done = true;
-				len--;
-				i++;
-			}
-			modelname[i-1] = 0;
-			if (!Q_strcasecmp (modelname, "2dm.temle") || ci->helmet)
-			{
-				ent.alpha = 0.4;
-				ent.flags = RF_TRANSLUCENT;
-			}
-
+			ent.alpha = 0.4;
+			ent.flags = RF_TRANSLUCENT;
+			
 			if (s1->number == cl.playernum+1) {
 				ent.flags |= RF_VIEWERMODEL;
 			}
@@ -1072,16 +924,22 @@ void CL_AddPacketEntities (frame_t *frame)
 		if (s1->modelindex4)
 		{
 			ent.flags = 0;
-			ent.alpha = 1.0;//fix the warmach player bug?
-			ent.model = cl.model_draw[s1->modelindex4];
+			ent.alpha = 1.0;
+
+			//cool new ctf flag effects
+			if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex4)], "models/items/flags/flag1.md2")) {
+				CL_FlagEffects(ent.origin, 0);
+				ent.model = 0;
+			}
+			else if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex4)], "models/items/flags/flag2.md2")) {
+				CL_FlagEffects(ent.origin, 1);
+				ent.model = 0;
+			}
+			else {
+				ent.model = cl.model_draw[s1->modelindex4];
 
-			if (!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex4)], "models/items/flags/flag1.md2")
-				||!Q_strcasecmp (cl.configstrings[CS_MODELS+(s1->modelindex4)], "models/items/flags/flag2.md2"))
-				V_AddEntity (&ent);
-			else if(!ci->helmet) //because we don't want any extraneous models like brainlet
-				//gunracks or war machine riders being drawn if helmt exists, meaning that
-				//the entity had defaulted
 				V_AddEntity (&ent);
+			}
 		}
 
 		// add automatic particle trails
@@ -1103,7 +961,7 @@ void CL_AddPacketEntities (frame_t *frame)
 			}
 			else if (effects & EF_HYPERBLASTER)
 			{
-				V_AddLight (ent.origin, 100, 1, 1, 0);
+				V_AddLight (ent.origin, 400*crand(), 1, 0, 1);
 			}
 			else if (effects & EF_GIB)
 			{
@@ -1113,7 +971,7 @@ void CL_AddPacketEntities (frame_t *frame)
 			{
 				CL_DiminishingTrail (cent->lerp_origin, ent.origin, cent, effects);
 			}
-			else if (effects & EF_FLAG1)
+			else if (effects & EF_TEAM1)
 			{
 				vec3_t right;
 				vec3_t start;
@@ -1128,7 +986,7 @@ void CL_AddPacketEntities (frame_t *frame)
 				VectorMA (start, 32, up, start);
 				CL_RedTeamLight(start);
 			}
-			else if (effects & EF_FLAG2)
+			else if (effects & EF_TEAM2)
 			{
 
 				vec3_t right;
@@ -1155,12 +1013,62 @@ void CL_AddPacketEntities (frame_t *frame)
 				CL_BlasterBall (cent->lerp_origin, ent.origin);
 				V_AddLight (ent.origin, 200, 0, 1, 1);
 			}
+			else if (effects & EF_BLASTER)
+			{
+				CL_BlasterTrail(cent->lerp_origin, ent.origin, cent);
+				V_AddLight (ent.origin, 50, 0, .5, 1);
+			}
 		}
 end:
 		VectorCopy (ent.origin, cent->lerp_origin);
 	}
 }
 
+/*
+=======================
+Berserker@quake2  shell
+brass effect. No change
+=======================
+*/
+
+void CL_BrassShells(vec3_t org, vec3_t dir, int count)
+{
+	int i, j;
+	clentity_t *le;
+	float d;
+
+	if (!cl_brass->value || !count)
+		return;
+
+	for (i = 0; i < count; i++) {
+
+		if (!free_clentities)
+			return;
+
+		le = free_clentities;
+		free_clentities = le->next;
+		le->next = active_clentities;
+		active_clentities = le;
+		le->time = cl.time;
+		d = (192 + rand()) & 63;
+		VectorClear(le->accel);
+		VectorClear(le->vel);
+		le->accel[0] = le->accel[1] = 0;
+		le->accel[2] = -6 * PARTICLE_GRAVITY;
+		le->alpha = 1.0;
+		le->alphavel = -0.1;
+		le->flags = CLM_BOUNCE | CLM_FRICTION | CLM_ROTATE | CLM_NOSHADOW | CLM_BRASS;
+		le->model = R_RegisterModel("models/objects/brass/tris.md2");
+		le->ang = crand() * 360;
+		le->avel = crand() * 500;
+
+		for (j = 0; j < 3; j++) {
+			le->lastOrg[j] = le->org[j] = org[j];
+			le->vel[j] = crand() * 24 + d * dir[j];
+		}
+	}
+}
+
 /*
 ==============
 CL_AddViewWeapon
@@ -1171,14 +1079,14 @@ void CL_AddViewWeapon (player_state_t *ps, player_state_t *ops)
 {
 	entity_t	gun;		// view model
 	int			i;
-
+	qboolean	useFX = false;
 
 	memset (&gun, 0, sizeof(gun));
 
-	if (gun_model)
-		gun.model = gun_model;	// development tool
-	else
-		gun.model = cl.model_draw[ps->gunindex];
+	gun.model = cl.model_draw[ps->gunindex];
+
+	gun.lod1 = NULL;
+	gun.lod2 = NULL;
 
 	if (!gun.model)
 		return;
@@ -1192,32 +1100,29 @@ void CL_AddViewWeapon (player_state_t *ps, player_state_t *ops)
 			ps->gunangles[i], cl.lerpfrac);
 	}
 
-	if (gun_frame)
-	{
-		gun.frame = gun_frame;	// development tool
-		gun.oldframe = gun_frame;	// development tool
-	}
+	gun.frame = ps->gunframe;
+	if (gun.frame == 0)
+		gun.oldframe = 0;	// just changed weapons, don't lerp from old
 	else
-	{
-		gun.frame = ps->gunframe;
-		if (gun.frame == 0)
-			gun.oldframe = 0;	// just changed weapons, don't lerp from old
-		else
-			gun.oldframe = ops->gunframe;
-	}
-
-
+		gun.oldframe = ops->gunframe;
+	
 	gun.flags = RF_MINLIGHT | RF_DEPTHHACK | RF_WEAPONMODEL;
 	gun.backlerp = 1.0 - cl.lerpfrac;
 	VectorCopy (gun.origin, gun.oldorigin);	// don't lerp at all
 
-	//add a muzzleflash for chaingun
+	//add an attached muzzleflash for chaingun
 	if(!(strcmp("models/weapons/v_shotg2/tris.md2", gun.model->name))) {
 		if(gun.frame > 4 && gun.frame < 14)
 			CL_MuzzleFlashParticle(gun.origin, gun.angles, true);
+	}	
+	else if(!(strcmp("models/weapons/v_hyperb/tris.md2", gun.model->name))) {
+		if(gun.frame > 5 && gun.frame < 7) {
+			CL_PlasmaFlashParticle(gun.origin, gun.angles, true);
+			useFX = true;
+		}
 	}
 
-	V_AddEntity (&gun);
+	V_AddViewEntity (&gun);
 	//add shells for viewweaps (all of em!)
 	{
 		int oldeffects = gun.flags, pnum;
@@ -1228,7 +1133,7 @@ void CL_AddViewWeapon (player_state_t *ps, player_state_t *ops)
 			{
 				int effects = s1->renderfx;
 
-				if (effects & (RF_SHELL_RED|RF_SHELL_BLUE|RF_SHELL_GREEN) || s1->effects&(EF_PENT|EF_QUAD))
+				if (effects & (RF_SHELL_RED|RF_SHELL_BLUE|RF_SHELL_GREEN) || s1->effects&(EF_PENT|EF_QUAD) || useFX)
 				{
 					if (effects & RF_SHELL_RED)
 						gun.flags |= RF_SHELL_RED;
@@ -1237,28 +1142,27 @@ void CL_AddViewWeapon (player_state_t *ps, player_state_t *ops)
 					if (effects & RF_SHELL_GREEN)
 						gun.flags |= RF_SHELL_GREEN;
 
+					if (useFX) 
+						gun.flags |= RF_SHELL_GREEN;
+
 					gun.flags |= RF_TRANSLUCENT;
 					gun.alpha = 0.30;
 
-					if (s1->effects & EF_COLOR_SHELL && gun.flags & (RF_SHELL_RED|RF_SHELL_BLUE|RF_SHELL_GREEN))
+					if (s1->effects & EF_COLOR_SHELL && gun.flags & (RF_SHELL_RED|RF_SHELL_BLUE|RF_SHELL_GREEN) || useFX)
 					{
-						gun.skin = R_RegisterSkin ("gfx/shell.pcx");
-
-						V_AddEntity (&gun);
+						V_AddViewEntity (&gun);
 					}
 					if (s1->effects & EF_PENT)
 					{
-						gun.skin = R_RegisterSkin ("gfx/shell_pent.pcx");
 						gun.flags = oldeffects | RF_TRANSLUCENT | RF_SHELL_RED;
 
-						V_AddEntity (&gun);
+						V_AddViewEntity (&gun);
 					}
 					if (s1->effects & EF_QUAD && cl_gun->value)
 					{
-						gun.skin = R_RegisterSkin ("gfx/shell_quad.pcx");
 						gun.flags = oldeffects | RF_TRANSLUCENT | RF_SHELL_BLUE;
 
-						V_AddEntity (&gun);
+						V_AddViewEntity (&gun);
 					}
 				}
 			}
@@ -1266,36 +1170,19 @@ void CL_AddViewWeapon (player_state_t *ps, player_state_t *ops)
 		gun.flags = oldeffects;
 	}
 
-	//add windows for vehicles, cover for RL
-	if(!(strcmp("vehicles/bomber/v_wep.md2", gun.model->name))) {
-		gun.model = R_RegisterModel("vehicles/bomber/window.md2");
-		gun.flags |= RF_TRANSLUCENT;
-		gun.alpha = 0.30;
-		V_AddEntity (&gun);
-	}
-	if(!(strcmp("vehicles/strafer/v_wep.md2", gun.model->name))) {
-		gun.model = R_RegisterModel("vehicles/bomber/window.md2");
-		gun.flags |= RF_TRANSLUCENT;
-		gun.alpha = 0.30;
-		V_AddEntity (&gun);
-	}
-	if(!(strcmp("vehicles/hover/v_wep.md2", gun.model->name))) {
-		gun.model = R_RegisterModel("vehicles/hover/window.md2");
-		gun.flags |= RF_TRANSLUCENT;
-		gun.alpha = 0.30;
-		V_AddEntity (&gun);
-	}
+	//add glass pieces
+	
 	if(!(strcmp("models/weapons/v_rocket/tris.md2", gun.model->name))) {
 		gun.model = R_RegisterModel("models/weapons/v_rocket/cover.md2");
 		gun.flags |= RF_TRANSLUCENT;
 		gun.alpha = 0.30;
-		V_AddEntity (&gun);
+		V_AddViewEntity (&gun);
 	}
 	if(!(strcmp("models/weapons/v_hyperb/tris.md2", gun.model->name))) {
 		gun.model = R_RegisterModel("models/weapons/v_hyperb/cover.md2");
 		gun.flags |= RF_TRANSLUCENT;
 		gun.alpha = 0.30;
-		V_AddEntity (&gun);
+		V_AddViewEntity (&gun);
 	}
 }
 
@@ -1425,6 +1312,7 @@ void CL_AddEntities (void)
 	CL_AddTEnts ();
 	CL_AddParticles ();
 	CL_AddDLights ();
+	CL_AddClEntities();
 	CL_AddLightStyles ();
 }
 
@@ -1448,3 +1336,285 @@ void CL_GetEntitySoundOrigin (int ent, vec3_t org)
 
 	// FIXME: bmodel issues...
 }
+
+/*
+================
+Client side entities(ejecting brass, etc
+================
+*/
+
+/*
+-----------------------------
+ClipMoveEntitiesWorld
+-----------------------------
+*/
+void CL_ClipMoveToEntitiesWorld(vec3_t start, vec3_t mins, vec3_t maxs,
+								vec3_t end, trace_t * tr, int mask)
+{
+	int i;
+	trace_t trace;
+	int headnode;
+	float *angles;
+	entity_state_t *ent;
+	int num;
+	cmodel_t *cmodel;
+
+	for (i = 0; i < cl.frame.num_entities; i++) {
+		num = (cl.frame.parse_entities + i) & (MAX_PARSE_ENTITIES - 1);
+		ent = &cl_parse_entities[num];
+
+		if (!ent->solid)
+			continue;
+
+		if (ent->solid != 31)	// special value for bmodel
+			continue;
+
+		cmodel = cl.model_clip[ent->modelindex];
+		if (!cmodel)
+			continue;
+		headnode = cmodel->headnode;
+		angles = ent->angles;
+
+		if (tr->allsolid)
+			return;
+
+		trace =
+			CM_TransformedBoxTrace(start, end, mins, maxs, headnode, mask,
+								   ent->origin, angles);
+
+		if (trace.allsolid || trace.startsolid
+			|| trace.fraction < tr->fraction) {
+			trace.ent = (struct edict_s *) ent;
+			if (tr->startsolid) {
+				*tr = trace;
+				tr->startsolid = true;
+			} else
+				*tr = trace;
+		} else if (trace.startsolid)
+			tr->startsolid = true;
+	}
+}
+
+
+trace_t CL_PMTraceWorld(vec3_t start, vec3_t mins, vec3_t maxs, vec3_t end,
+						int mask)
+{
+	trace_t t;
+
+	// check against world
+	t = CM_BoxTrace(start, end, mins, maxs, 0, mask);
+	if (t.fraction < 1.0)
+		t.ent = (struct edict_s *) 1;
+
+	// check all other solid models
+	CL_ClipMoveToEntitiesWorld(start, mins, maxs, end, &t, mask);
+
+	return t;
+}
+
+void VectorReflect(const vec3_t v, const vec3_t normal, vec3_t out)
+{
+	float d;
+
+	d = 2.0 * (v[0] * normal[0] + v[1] * normal[1] + v[2] * normal[2]);
+
+	out[0] = v[0] - normal[0] * d;
+	out[1] = v[1] - normal[1] * d;
+	out[2] = v[2] - normal[2] * d;
+}
+
+extern int CL_PMpointcontents (vec3_t point);
+void CL_AddClEntities()
+{
+	entity_t ent;
+	clentity_t *le, *next;
+	clentity_t *active, *tail;
+	vec3_t org, dir;
+	float alpha, bak;
+	int contents;
+	qboolean onground;
+	char soundname[64];
+	float time, time2, grav = Cvar_VariableValue("sv_gravity");
+	static vec3_t mins = { -2, -2, -2 }; 
+    static vec3_t maxs = { 2, 2, 2 }; 
+
+	if (!grav)
+		grav = 1;
+	else
+		grav /= 800;
+
+	active = NULL;
+	tail = NULL;
+
+	memset(&ent, 0, sizeof(ent));
+
+
+	for (le = active_clentities; le; le = next) {
+		next = le->next;
+
+		time = (cl.time - le->time) * 0.001;
+		alpha = le->alpha + time * le->alphavel;
+
+		if (alpha <= 0) {		// faded out
+			le->next = free_clentities;
+			free_clentities = le;
+			continue;
+		}
+
+		time2 = time * time;
+		org[0] = le->org[0] + le->vel[0] * time + le->accel[0] * time2;
+		org[1] = le->org[1] + le->vel[1] * time + le->accel[1] * time2;
+		bak = le->org[2] + le->vel[2] * time + le->accel[2] * time2 * grav;
+
+		org[2] = bak - 1;
+		contents = CL_PMpointcontents(org);
+		org[2] = bak;
+		onground = contents & MASK_SOLID;
+		if (onground) {
+			le->flags &= ~CLM_ROTATE;
+			le->avel = 0;
+		} else if (le->flags & CLM_STOPPED) {
+			le->flags &= ~CLM_STOPPED;
+			le->flags |= CLM_BOUNCE;
+			le->accel[2] = -15 * PARTICLE_GRAVITY;
+			// Reset
+			le->alpha = 1;
+			le->time = cl.time;
+			VectorCopy(org, le->org);
+		}
+
+		le->next = NULL;
+		if (!tail)
+			active = tail = le;
+		else {
+			tail->next = le;
+			tail = le;
+		}
+
+		if (alpha > 1.0)
+			alpha = 1;
+		
+		if(le->flags & CLM_NOSHADOW)
+			ent.flags |= RF_NOSHADOWS;
+
+		if (le->flags & CLM_FRICTION) {
+			// Water friction affected cl model
+			if (contents & MASK_WATER) {
+				if (contents & CONTENTS_LAVA) {	// kill entity in lava
+					VectorSet(dir, 0, 0, 1);
+					le->alpha = 0;
+					continue;
+				} else {
+					// Add friction
+					VectorScale(le->vel, 0.25, le->vel);
+					VectorScale(le->accel, 0.25, le->accel);
+
+					// Don't add friction again
+					le->flags &= ~(CLM_FRICTION | CLM_ROTATE);
+
+					// Reset
+					le->time = cl.time;
+					VectorCopy(org, le->org);
+				}
+			}
+		}
+		
+		if (le->flags & CLM_BOUNCE)
+		{
+			trace_t trace = CL_PMTraceWorld (le->lastOrg, mins, maxs, org, MASK_SOLID);
+
+			if (trace.fraction > 0.0001f && trace.fraction < 0.9999f )
+			{
+				vec3_t	vel;
+				// Reflect velocity
+				float time =
+					cl.time - (cls.frametime +
+							   cls.frametime * trace.fraction) * 1000;
+				time = (time - le->time) * 0.001;
+
+				VectorSet(vel, le->vel[0], le->vel[1], le->vel[2] + le->accel[2] * time * grav);
+				VectorReflect(vel, trace.plane.normal, le->vel);
+				
+				// Check for stop or slide along the plane
+				if (trace.plane.normal[2] > 0 && le->vel[2] < 1)
+				{
+					if (trace.plane.normal[2] > 0.9)
+					{
+						VectorClear(le->vel);
+						VectorClear(le->accel);
+						le->avel = 0;
+						le->flags &= ~CLM_BOUNCE;
+						le->flags |= CLM_STOPPED;
+					}
+					else
+					{
+						// FIXME: check for new plane or free fall
+						float dot = DotProduct(le->vel, trace.plane.normal);
+						VectorMA(le->vel, -dot, trace.plane.normal, le->vel);
+
+						dot = DotProduct(le->accel, trace.plane.normal);
+						VectorMA(le->accel, -dot, trace.plane.normal, le->accel);
+					}
+				}
+
+				VectorCopy(trace.endpos, org);
+
+				// Reset
+				le->time = cl.time;
+				VectorCopy(org, le->org);
+
+				//play a sound if brass
+				if ( (le->flags & (CLM_BRASS))  &&  (le->flags & (CLM_BOUNCE)) ) {
+					Com_sprintf(soundname, sizeof(soundname), "weapons/clink0%i.wav", (rand() % 2) + 1);
+					S_StartSound (le->org, 0, CHAN_WEAPON, S_RegisterSound(soundname), 1.0, ATTN_NORM, 0);
+				}
+			}
+		}
+		// Save current origin if needed
+		if (le->flags & (CLM_BOUNCE)) {
+			VectorCopy(le->lastOrg, ent.origin);
+			VectorCopy(org, le->lastOrg);	// FIXME: pause
+		} else
+			VectorCopy(org, ent.origin);
+
+		if (CL_PMpointcontents(ent.origin) & MASK_SOLID) {	// kill entity
+															// in solid
+			le->alpha = 0;
+			continue;
+		}
+		
+		ent.model = le->model;
+		if (!ent.model)
+			continue;
+
+		ent.lod1 = NULL;
+		ent.lod2 = NULL;
+
+		ent.angles[0] = le->ang +
+			((le->flags & CLM_ROTATE) ? (time * le->avel) : 0);;
+		ent.angles[2] = le->ang +
+			((le->flags & CLM_ROTATE) ? (time * le->avel) : 0);;
+		ent.angles[1] =
+			le->ang +
+			((le->flags & CLM_ROTATE) ? (time * le->avel) : 0);
+
+		ent.frame = ent.oldframe = 0;
+			
+		V_AddEntity(&ent);
+		
+	}
+
+	active_clentities = active;
+}
+
+void CL_ClearClEntities()
+{
+	int i;
+
+	free_clentities = &clentities[0];
+	active_clentities = NULL;
+
+	for (i = 0; i < MAX_CLENTITIES; i++)
+		clentities[i].next = &clentities[i + 1];
+	clentities[MAX_CLENTITIES - 1].next = NULL;
+}
