@@ -61,7 +61,12 @@ cvar_t	*cl_timeout;
 cvar_t	*cl_predict;
 cvar_t	*cl_maxfps;
 cvar_t	*cl_gun;
+cvar_t  *cl_brass;
 cvar_t  *cl_showPlayerNames;
+cvar_t	*cl_playtaunts;
+
+cvar_t	*cl_paindist;
+cvar_t	*cl_explosiondist;
 
 cvar_t	*cl_add_particles;
 cvar_t	*cl_add_lights;
@@ -79,13 +84,12 @@ cvar_t	*lookspring;
 cvar_t	*lookstrafe;
 cvar_t	*sensitivity;
 
+cvar_t	*m_smoothing;
 cvar_t	*m_pitch;
 cvar_t	*m_yaw;
 cvar_t	*m_forward;
 cvar_t	*m_side;
 
-cvar_t	*cl_lightlevel;
-
 //
 // userinfo
 //
@@ -121,6 +125,13 @@ cvar_t	*cl_healthaura;
 //blood
 cvar_t  *cl_noblood;
 
+//beam color for disruptor
+cvar_t  *cl_disbeamclr;
+
+//IRC
+cvar_t	*cl_IRC_connect_at_startup;
+cvar_t	*cl_IRC_server;
+
 client_static_t	cls;
 client_state_t	cl;
 
@@ -270,6 +281,7 @@ void CL_Record_f (void)
 	// write out messages to hold the startup information
 	//
 	SZ_Init (&buf, buf_data, sizeof(buf_data));
+	SZ_SetName ( &buf, "CL_Record_f", false );
 
 	// send the serverdata
 	MSG_WriteByte (&buf, svc_serverdata);
@@ -596,34 +608,42 @@ CL_Rcon_f
 */
 void CL_Rcon_f (void)
 {
-	char	message[1024];
+	char		message[1024];
+	sizebuf_t	buffer;
 	int		i;
 	netadr_t	to;
 
-	if (!rcon_client_password->string)
+	if ( !(rcon_client_password->string && rcon_client_password->string[0]) && Cmd_Argc() < 3)
 	{
-		Com_Printf ("You must set 'rcon_password' before\n"
-					"issuing an rcon command.\n");
+		Com_Printf ("You must set 'rcon_password' before issuing an rcon command.\n");
 		return;
 	}
 
-	message[0] = (char)255;
-	message[1] = (char)255;
-	message[2] = (char)255;
-	message[3] = (char)255;
-	message[4] = 0;
-
 	NET_Config (true);		// allow remote
 
-	strcat (message, "rcon ");
+	SZ_Init( &buffer, (byte *) message, 1024 );
+	buffer.allowoverflow = true;
+	SZ_SetName( &buffer, "RCon buffer", false );
 
-	strcat (message, rcon_client_password->string);
-	strcat (message, " ");
+	SZ_Print (&buffer, "\xff\xff\xff\xffrcon ");
+	if ( rcon_client_password->string && rcon_client_password->string[0] )
+	{
+		SZ_Print (&buffer, "\"");
+		SZ_Print (&buffer, rcon_client_password->string);
+		SZ_Print (&buffer, "\" ");
+	}
 
 	for (i=1 ; i<Cmd_Argc() ; i++)
 	{
-		strcat (message, Cmd_Argv(i));
-		strcat (message, " ");
+		SZ_Print (&buffer, "\"");
+		SZ_Print (&buffer, Cmd_Argv(i));
+		SZ_Print (&buffer, "\" ");
+	}
+
+	if ( buffer.overflowed )
+	{
+		Com_Printf ("Rcon command too long\n");
+		return;
 	}
 
 	if (cls.state >= ca_connected)
@@ -632,9 +652,7 @@ void CL_Rcon_f (void)
 	{
 		if (!strlen(rcon_address->string))
 		{
-			Com_Printf ("You must either be connected,\n"
-						"or set the 'rcon_address' cvar\n"
-						"to issue rcon commands\n");
+			Com_Printf ("You must either be connected, or set the 'rcon_address' cvar to issue rcon commands\n");
 
 			return;
 		}
@@ -700,8 +718,6 @@ void CL_Disconnect (void)
 
 	cls.connect_time = 0;
 
-	SCR_StopCinematic ();
-
 	if (cls.demorecording)
 		CL_Stop_f ();
 
@@ -1010,7 +1026,7 @@ void CL_PingServers_f (void)
 	}
 
 	// -JD restart the menu music that was stopped during this procedure
-	S_StartMusic("misc/menumusic.wav");
+	S_StartMenuMusic();
 }
 
 
@@ -1147,7 +1163,7 @@ void CL_DumpPackets (void)
 {
 	while (NET_GetPacket (NS_CLIENT, &net_from, &net_message))
 	{
-		Com_Printf ("dumnping a packet\n");
+		Com_Printf ("dumping a packet\n");
 	}
 }
 
@@ -1599,10 +1615,16 @@ void CL_InitLocal (void)
 	cls.state = ca_disconnected;
 	cls.realtime = Sys_Milliseconds ();
 
-	CL_InitInput ();
+	CL_InitInput ();	
 
 	CL_InitHttpDownload();
 
+	cl_IRC_connect_at_startup = Cvar_Get("cl_IRC_connect_at_startup", "1", CVAR_ARCHIVE);
+	cl_IRC_server = Cvar_Get("cl_IRC_server", "irc.prison.net", CVAR_ARCHIVE);
+
+	if(cl_IRC_connect_at_startup->value)
+		CL_InitIRC();
+
 	adr0 = Cvar_Get( "adr0", "", CVAR_ARCHIVE );
 	adr1 = Cvar_Get( "adr1", "", CVAR_ARCHIVE );
 	adr2 = Cvar_Get( "adr2", "", CVAR_ARCHIVE );
@@ -1619,7 +1641,7 @@ void CL_InitLocal (void)
 	cl_stereo_separation = Cvar_Get( "cl_stereo_separation", "0.4", CVAR_ARCHIVE );
 	cl_stereo = Cvar_Get( "cl_stereo", "0", 0 );
 	background_music = Cvar_Get("background_music", "1", CVAR_ARCHIVE);
-	background_music_vol = Cvar_Get("background_music_vol", "1", CVAR_ARCHIVE);
+	background_music_vol = Cvar_Get("background_music_vol", "0.5", CVAR_ARCHIVE);
 
 	cl_add_blend = Cvar_Get ("cl_blend", "1", 0);
 	cl_add_lights = Cvar_Get ("cl_lights", "1", 0);
@@ -1634,6 +1656,12 @@ void CL_InitLocal (void)
 	cl_showPlayerNames = Cvar_Get ("cl_showplayernames", "0", CVAR_ARCHIVE);
 	cl_healthaura = Cvar_Get ("cl_healthaura", "1", CVAR_ARCHIVE);
 	cl_noblood = Cvar_Get ("cl_noblood", "0", CVAR_ARCHIVE);
+	cl_disbeamclr = Cvar_Get("cl_disbeamclr", "0", CVAR_ARCHIVE);
+	cl_brass = Cvar_Get ("cl_brass", "1", CVAR_ARCHIVE);
+	cl_playtaunts = Cvar_Get ("cl_playtaunts", "1", CVAR_ARCHIVE);
+
+	cl_paindist = Cvar_Get ("cl_paindist", "1", CVAR_ARCHIVE);
+	cl_explosiondist = Cvar_Get ("cl_explosiondist", "1", CVAR_ARCHIVE);
 
 	cl_upspeed = Cvar_Get ("cl_upspeed", "200", 0);
 	cl_forwardspeed = Cvar_Get ("cl_forwardspeed", "200", 0);
@@ -1648,6 +1676,7 @@ void CL_InitLocal (void)
 	lookstrafe = Cvar_Get ("lookstrafe", "0", CVAR_ARCHIVE);
 	sensitivity = Cvar_Get ("sensitivity", "3", CVAR_ARCHIVE);
 
+	m_smoothing = Cvar_Get("m_smoothing", "0", CVAR_ARCHIVE);
 	m_pitch = Cvar_Get ("m_pitch", "0.022", CVAR_ARCHIVE);
 	m_yaw = Cvar_Get ("m_yaw", "0.022", 0);
 	m_forward = Cvar_Get ("m_forward", "1", 0);
@@ -1663,8 +1692,6 @@ void CL_InitLocal (void)
 	rcon_client_password = Cvar_Get ("rcon_password", "", 0);
 	rcon_address = Cvar_Get ("rcon_address", "", 0);
 
-	cl_lightlevel = Cvar_Get ("r_lightlevel", "0", 0);
-
 	//
 	// userinfo
 	//
@@ -1724,12 +1751,17 @@ void CL_InitLocal (void)
 	Cmd_AddCommand ("shell", CL_Shell_f);
 	#endif
 
+	Cmd_AddCommand ("irc_connect", CL_InitIRC);
+	Cmd_AddCommand ("irc_quit", CL_IRCShutdown);
+	Cmd_AddCommand ("irc_say", CL_IRCSay);
+
 	//
 	// forward to server commands
 	//
 	// the only thing this does is allow command completion
 	// to work -- all unknown commands are automatically
-	// forwarded to the server
+	// forwarded to the server. It also prevents the commands
+	// from being ignored if they are issued in 'forced' mode.
 	Cmd_AddCommand ("wave", NULL);
 	Cmd_AddCommand ("inven", NULL);
 	Cmd_AddCommand ("kill", NULL);
@@ -1744,11 +1776,23 @@ void CL_InitLocal (void)
 	Cmd_AddCommand ("notarget", NULL);
 	Cmd_AddCommand ("noclip", NULL);
 	Cmd_AddCommand ("invuse", NULL);
+	Cmd_AddCommand ("invprevw", NULL);
+	Cmd_AddCommand ("invnextw", NULL);
+	Cmd_AddCommand ("invprevp", NULL);
+	Cmd_AddCommand ("invnextp", NULL);
 	Cmd_AddCommand ("invprev", NULL);
 	Cmd_AddCommand ("invnext", NULL);
 	Cmd_AddCommand ("invdrop", NULL);
 	Cmd_AddCommand ("weapnext", NULL);
 	Cmd_AddCommand ("weapprev", NULL);
+	Cmd_AddCommand ("weaplast", NULL);
+	Cmd_AddCommand ("chatbubble", NULL);
+	Cmd_AddCommand ("players", NULL);
+	Cmd_AddCommand ("score", NULL);
+	Cmd_AddCommand ("help", NULL);
+	Cmd_AddCommand ("vote", NULL);
+	Cmd_AddCommand ("putaway", NULL);
+	Cmd_AddCommand ("playerlist", NULL);
 
 	Cvar_SetValue("scriptsloaded", 0);
 
@@ -1767,6 +1811,12 @@ void CL_InitLocal (void)
 	R_RegisterPic("m_mutators");
 	R_RegisterPic("m_single");
 	R_RegisterPic("m_quit");
+	R_RegisterPic("m_main_mont0");
+	R_RegisterPic("m_main_mont1");
+	R_RegisterPic("m_main_mont2");
+	R_RegisterPic("m_main_mont3");
+	R_RegisterPic("m_main_mont4");
+	R_RegisterPic("m_main_mont5");
 }
 
 
@@ -1826,6 +1876,7 @@ cheatvar_t	cheatvars[] = {
 	{"paused", "0"},
 	{"fixedtime", "0"},
 	{"gl_lightmap", "0"},
+	{"gl_showpolys", "0"},
 	{NULL, NULL}
 };
 
@@ -1897,7 +1948,8 @@ CL_Frame
 
 ==================
 */
-qboolean send_packet_now = false;  // instant packets, like firing weapons
+qboolean send_packet_now = false;  // instant packets
+extern float	r_frametime;
 
 void CL_Frame (int msec)
 {
@@ -1927,6 +1979,8 @@ void CL_Frame (int msec)
 	cl.time += extratime;
 	cls.realtime = curtime;
 
+	r_frametime = cls.frametime;
+
 	extratime = 0;
 
 	if (cls.frametime > (1.0 / 5))
@@ -1966,7 +2020,6 @@ void CL_Frame (int msec)
 	// advance local effects for next frame
 	CL_RunDLights ();
 	CL_RunLightStyles ();
-	SCR_RunCinematic ();
 	SCR_RunConsole ();
 
 	cls.framecount++;
@@ -2056,11 +2109,12 @@ void CL_Shutdown(void)
 	isdown = true;
 
 	CL_ShutdownHttpDownload();
+	CL_IRCShutdown();
 	CL_WriteConfiguration ();
 
 	S_Shutdown();
 	IN_Shutdown ();
-	VID_Shutdown();
+	VID_Shutdown();	
 
 	RS_FreeAllScripts();
 }
