@@ -391,19 +391,12 @@ void CL_ParseServerData (void)
 	// get the full level name
 	str = MSG_ReadString (&net_message);
 
-	if (cl.playernum == -1)
-	{	// playing a cinematic or showing a pic, not a level
-		SCR_PlayCinematic (str);
-	}
-	else
-	{
-		// seperate the printfs so the server message can have a color
-		Com_Printf("\n\n\35\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\37\n\n");
-		Com_Printf ("%c%s\n", 2, str);
+	// seperate the printfs so the server message can have a color
+	Com_Printf("\n\n\35\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\36\37\n\n");
+	Com_Printf ("%c%s\n", 2, str);
 
-		// need to prep refresh at next oportunity
-		cl.refresh_prepped = false;
-	}
+	// need to prep refresh at next oportunity
+	cl.refresh_prepped = false;
 }
 
 /*
@@ -441,6 +434,7 @@ void CL_LoadClientinfo (clientinfo_t *ci, char *s)
 	char		model_filename[MAX_QPATH];
 	char		skin_filename[MAX_QPATH];
 	char		weapon_filename[MAX_QPATH];
+	FILE		*file;
 
 	strncpy(ci->cinfo, s, sizeof(ci->cinfo));
 	ci->cinfo[sizeof(ci->cinfo)-1] = 0;
@@ -455,7 +449,18 @@ void CL_LoadClientinfo (clientinfo_t *ci, char *s)
 		s = t+1;
 	}
 
+	// isolate the model name
+	strcpy (model_name, s);
+	t = strstr(model_name, "/");
+	if (!t)
+		t = strstr(model_name, "\\");
+	if (!t)
+		t = model_name;
+	*t = 0;
+
 	ci->helmet = NULL; //we only worry about this in these cases of missing textures or models
+	ci->lod1 = NULL;
+	ci->lod2 = NULL;
 
 	if (cl_noskins->value || *s == 0)
 	{
@@ -477,15 +482,6 @@ void CL_LoadClientinfo (clientinfo_t *ci, char *s)
 	}
 	else
 	{
-		// isolate the model name
-		strcpy (model_name, s);
-		t = strstr(model_name, "/");
-		if (!t)
-			t = strstr(model_name, "\\");
-		if (!t)
-			t = model_name;
-		*t = 0;
-
 		// isolate the skin name
 		strcpy (skin_name, s + strlen(model_name) + 1);
 
@@ -529,6 +525,40 @@ void CL_LoadClientinfo (clientinfo_t *ci, char *s)
 		ci->icon = R_RegisterPic (ci->iconname);
 	}
 
+	//check for level of detail models
+	if (cl_noskins->value || *s == 0)
+		strcpy(model_name, "martianenforcer");
+
+	Com_sprintf(model_filename, sizeof(model_filename), "players/%s/lod1.md2", model_name);
+	i = 0;
+	do
+		model_filename[i] = tolower(model_filename[i]);
+	while (model_filename[i++]);
+
+	FS_FOpenFile (model_filename, &file); 
+	if(file) {
+		//exists
+		fclose(file);
+		ci->lod1 = R_RegisterModel(model_filename);	
+	}
+	else 
+		ci->lod1 = NULL;
+
+	Com_sprintf(model_filename, sizeof(model_filename), "players/%s/lod2.md2", model_name);
+	i = 0;
+	do
+		model_filename[i] = tolower(model_filename[i]);
+	while (model_filename[i++]);
+
+	FS_FOpenFile (model_filename, &file); 
+	if(file) {
+		//exists
+		fclose(file);
+		ci->lod2 = R_RegisterModel(model_filename);		
+	}
+	else
+		ci->lod2 = NULL;
+	
 	// must have loaded all data types to be valid
 	if (!ci->skin || !ci->icon || !ci->model || !ci->weaponmodel[0])
 	{
@@ -560,6 +590,30 @@ void CL_ParseClientinfo (int player)
 	CL_LoadClientinfo (ci, s);
 }
 
+void CL_ParseTaunt( char *s) 
+{
+	int l, j;
+	char tauntsound[MAX_OSPATH];
+
+	//parse
+	strcpy( scr_playericon, COM_Parse( &s ) );
+	l = strlen(scr_playericon);
+
+	for (j=0 ; j<l ; j++)
+		scr_playericon[j] = tolower(scr_playericon[j]);
+
+	Com_sprintf(scr_playericon, sizeof(scr_playericon), "%s_i", scr_playericon);
+	
+	strcpy( tauntsound, COM_Parse( &s ) );
+	
+	strcpy( scr_playername, COM_Parse( &s ) ); //fix	
+
+	if(cl_playtaunts->value) {
+		S_StartSound (NULL, 0, 0, S_RegisterSound (tauntsound), 1, ATTN_NONE, 0);		
+		scr_playericonalpha = 2.0;
+	}
+	
+}
 
 /*
 ================
@@ -636,6 +690,8 @@ void CL_ParseConfigString (void)
 		if (cl.refresh_prepped && strcmp(olds, s))
 			CL_ParseClientinfo (i-CS_PLAYERSKINS);
 	}
+	else if ( i == CS_GENERAL)
+		CL_ParseTaunt(s);
 }
 
 
