@@ -25,7 +25,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   put up loading plaque
   blanked background with loading plaque
   blanked background with menu
-  cinematics
   full screen image for quit and victory
 
   end of unit intermissions
@@ -44,8 +43,6 @@ int			scr_draw_loading;
 
 vrect_t		scr_vrect;		// position of render window on screen
 
-
-cvar_t		*scr_viewsize;
 cvar_t		*scr_conspeed;
 cvar_t		*scr_centertime;
 cvar_t		*scr_showturtle;
@@ -58,7 +55,6 @@ cvar_t		*scr_debuggraph;
 cvar_t		*scr_graphheight;
 cvar_t		*scr_graphscale;
 cvar_t		*scr_graphshift;
-cvar_t		*scr_drawall;
 
 cvar_t		*scr_consize;
 
@@ -80,6 +76,7 @@ int			crosshair_width, crosshair_height;
 void SCR_TimeRefresh_f (void);
 void SCR_Loading_f (void);
 
+extern void R_VCFreeFrame(void);
 
 /*
 ===============================================================================
@@ -282,6 +279,129 @@ void SCR_CheckDrawCenterString (void)
 	SCR_DrawCenterString ();
 }
 
+/*
+===============================================================================
+
+IRC PRINTING
+
+  we'll have 5 lines stored.
+  once full, start bumping lines, as in copying line 1 into line 0, and so forth
+
+===============================================================================
+*/
+
+char		scr_IRCstring[5][1024];
+float		scr_IRCtime_start;	// for slow victory printing
+float		scr_IRCtime_off;
+int			scr_IRC_lines = -1;
+
+/*
+==============
+SCR_IRCPrint
+
+Called for IRC messages
+==============
+*/
+
+void IRC_BumpMsg(void)
+{
+	int i;
+
+	for(i = 0; i < scr_IRC_lines; i++)		
+		strcpy(scr_IRCstring[i], scr_IRCstring[i+1]);
+}
+
+void SCR_IRCPrintf (char *fmt, ...)	
+{
+	va_list		argptr;
+	char		msg[1024];
+
+	va_start (argptr,fmt);
+	vsnprintf(msg, sizeof(msg), fmt, argptr);
+	va_end (argptr);
+
+	scr_IRCtime_off = 15;
+	scr_IRCtime_start = cl.time;
+
+	// count the number of lines
+	if(scr_IRC_lines < 4)
+		scr_IRC_lines += 1;
+	else
+		IRC_BumpMsg();
+
+	strncpy (scr_IRCstring[scr_IRC_lines], msg, sizeof(scr_IRCstring[scr_IRC_lines])-1);
+}
+
+extern vec4_t	Color_Table[8];
+
+void SCR_DrawColorString ( int x, int y, const char *str )
+{
+	int		num;
+	vec4_t	scolor;
+	int	charscale;
+
+	charscale = (float)(viddef.height)*8/600;
+	if(charscale < 8)
+		charscale = 8;
+
+	scolor[0] = 0; 
+	scolor[1] = 1;
+	scolor[2] = 0;
+	scolor[3] = 1;
+
+	while (*str) {
+		if ( Q_IsColorString( str ) ) {
+			VectorCopy ( Color_Table[ColorIndex(str[1])], scolor );
+			str += 2;
+			continue;
+		}
+		
+		Draw_ScaledColorChar (x, y, *str, scolor, charscale); //this is only ever used for names.
+		
+		num = *str++;
+		num &= 255;
+			
+		if ( (num&127) == 32 ) { //spaces reset colors
+			scolor[0] = 0;
+			scolor[1] = 1;
+			scolor[2] = 0;
+			scolor[3] = 1;
+		}
+
+		x += charscale;
+	}
+}
+void SCR_DrawIRCString (int lineNum)
+{
+	int		x, y;
+	int		charscale;
+
+	charscale = (float)(viddef.height)*8/600;
+	if(charscale < 8)
+		charscale = 8;
+
+	y = charscale*5 + lineNum*charscale;
+	x = 0;
+
+	SCR_AddDirtyPoint (x, y);
+	SCR_DrawColorString (x, y, scr_IRCstring[lineNum]);
+	SCR_AddDirtyPoint (x, y+charscale);
+
+}
+
+void SCR_CheckDrawIRCString (void)
+{
+	int i;
+
+	scr_IRCtime_off -= cls.frametime;
+
+	if (scr_IRCtime_off <= 0)
+		return;
+
+	for(i = 0; i <= scr_IRC_lines; i++)
+		SCR_DrawIRCString (i);
+}
+
 //=============================================================================
 
 /*
@@ -293,52 +413,17 @@ Sets scr_vrect, the coordinates of the rendered window
 */
 static void SCR_CalcVrect (void)
 {
-	int		size;
 
-	// bound viewsize
-	if (scr_viewsize->value < 40)
-		Cvar_Set ("viewsize","40");
-	if (scr_viewsize->value > 100)
-		Cvar_Set ("viewsize","100");
-
-	size = scr_viewsize->value;
-
-	scr_vrect.width = viddef.width*size/100;
+	scr_vrect.width = viddef.width;
 	scr_vrect.width &= ~7;
 
-	scr_vrect.height = viddef.height*size/100;
+	scr_vrect.height = viddef.height;
 	scr_vrect.height &= ~1;
 
 	scr_vrect.x = (viddef.width - scr_vrect.width)/2;
 	scr_vrect.y = (viddef.height - scr_vrect.height)/2;
 }
 
-
-/*
-=================
-SCR_SizeUp_f
-
-Keybinding command
-=================
-*/
-void SCR_SizeUp_f (void)
-{
-	Cvar_SetValue ("viewsize",scr_viewsize->value+10);
-}
-
-
-/*
-=================
-SCR_SizeDown_f
-
-Keybinding command
-=================
-*/
-void SCR_SizeDown_f (void)
-{
-	Cvar_SetValue ("viewsize",scr_viewsize->value-10);
-}
-
 /*
 =================
 SCR_Sky_f
@@ -385,20 +470,18 @@ SCR_Init
 */
 void SCR_Init (void)
 {
-	scr_viewsize = Cvar_Get ("viewsize", "100", CVAR_ARCHIVE);
-	scr_conspeed = Cvar_Get ("scr_conspeed", "3", 0);
-	scr_consize = Cvar_Get ("scr_consize", "0.5", 0);
+	scr_conspeed = Cvar_Get ("scr_conspeed", "3", CVAR_ARCHIVE);
+	scr_consize = Cvar_Get ("scr_consize", "0.5", CVAR_ARCHIVE);
 	scr_showturtle = Cvar_Get ("scr_showturtle", "0", 0);
 	scr_showpause = Cvar_Get ("scr_showpause", "1", 0);
-	scr_centertime = Cvar_Get ("scr_centertime", "2.5", 0);
-	scr_printspeed = Cvar_Get ("scr_printspeed", "8", 0);
+	scr_centertime = Cvar_Get ("scr_centertime", "2.5", CVAR_ARCHIVE);
+	scr_printspeed = Cvar_Get ("scr_printspeed", "8", CVAR_ARCHIVE);
 	scr_netgraph = Cvar_Get ("netgraph", "0", 0);
 	scr_timegraph = Cvar_Get ("timegraph", "0", 0);
 	scr_debuggraph = Cvar_Get ("debuggraph", "0", 0);
 	scr_graphheight = Cvar_Get ("graphheight", "32", 0);
 	scr_graphscale = Cvar_Get ("graphscale", "1", 0);
 	scr_graphshift = Cvar_Get ("graphshift", "0", 0);
-	scr_drawall = Cvar_Get ("scr_drawall", "0", 0);
 
 	cl_drawfps = Cvar_Get ("cl_drawfps", "0", CVAR_ARCHIVE);
 	cl_drawtimer = Cvar_Get("cl_drawtimer", "0", CVAR_ARCHIVE);
@@ -408,8 +491,6 @@ void SCR_Init (void)
 //
 	Cmd_AddCommand ("timerefresh",SCR_TimeRefresh_f);
 	Cmd_AddCommand ("loading",SCR_Loading_f);
-	Cmd_AddCommand ("sizeup",SCR_SizeUp_f);
-	Cmd_AddCommand ("sizedown",SCR_SizeDown_f);
 	Cmd_AddCommand ("sky",SCR_Sky_f);
 
 	scr_initialized = true;
@@ -480,6 +561,60 @@ void SCR_DrawPause (void)
 SCR_DrawLoading
 ==============
 */
+
+
+void SCR_DrawRotatingIcon( void )
+{
+	extern float CalcFov( float fov_x, float w, float h );
+	refdef_t refdef;
+	static float yaw;
+	entity_t entity;
+
+	memset( &refdef, 0, sizeof( refdef ) );
+
+	refdef.width = viddef.width;
+	refdef.height = viddef.height;
+	refdef.x = 0;
+	refdef.y = 0;
+	refdef.fov_x = 90;
+	refdef.fov_y = CalcFov( refdef.fov_x, refdef.width, refdef.height );
+	refdef.time = cls.realtime*0.001;
+
+	memset( &entity, 0, sizeof( entity ) );
+
+	yaw += cls.frametime*50;
+	if (yaw > 360)
+		yaw = 0;
+
+	//i think this is where it's getting blown out, the model keeps getting overwritten or lost at map load
+	entity.model = R_RegisterModel( "models/objects/icon/tris.md2" );
+		
+	entity.flags = RF_FULLBRIGHT | RF_MENUMODEL;
+	entity.origin[0] = 80;
+	entity.origin[1] = 30;
+	entity.origin[2] = -5;
+	
+	VectorCopy( entity.origin, entity.oldorigin );
+	
+	entity.frame = 0;
+	entity.oldframe = 0;
+	entity.backlerp = 0.0;
+	entity.angles[1] = (int)yaw;
+
+	refdef.areabits = 0;
+	refdef.num_entities = 1;
+
+	refdef.entities = &entity;
+	refdef.lightstyles = 0;
+	refdef.rdflags = RDF_NOWORLDMODEL;
+
+	refdef.height += 4;
+
+	R_RenderFrame( &refdef );
+
+	free(entity.model);
+}
+
 void SCR_DrawLoadingBar (int percent, int scale)
 {
 	float hudscale;
@@ -689,6 +824,8 @@ void SCR_DrawLoading (void)
 	}
 	else if (isMap) //loading bar...
 	{
+		//to do - fix
+		//SCR_DrawRotatingIcon();
 
 		SCR_DrawLoadingBar(loadingPercent, font_size);
 
@@ -737,6 +874,7 @@ void SCR_RunConsole (void)
 SCR_DrawConsole
 ==================
 */
+float sendBubbleNow;
 void SCR_DrawConsole (void)
 {
 	Con_CheckResize ();
@@ -760,9 +898,20 @@ void SCR_DrawConsole (void)
 	}
 	else
 	{
-		if (cls.key_dest == key_game || cls.key_dest == key_message)
+		if (cls.key_dest == key_game || cls.key_dest == key_message) 
 			Con_DrawNotify ();	// only draw notify in game
 	}
+
+	//draw chat bubble 
+	if(cls.key_dest == key_message || scr_con_current) { 
+		//only send this every couple of seconds no need to flood
+		sendBubbleNow += cls.frametime;
+		if(sendBubbleNow >= 1) {
+			Cbuf_AddText("chatbubble\n"); 
+			Cbuf_Execute ();
+			sendBubbleNow = 0; 
+		}
+	}
 }
 
 //=============================================================================
@@ -786,10 +935,7 @@ void SCR_BeginLoadingPlaque (void)
 	if (developer->value)
 		return;
 
-	if (cl.cinematictime > 0)
-		scr_draw_loading = 2;	// clear to balack first
-	else
-		scr_draw_loading = 1;
+	scr_draw_loading = 1;
 
 	SCR_UpdateScreen ();
 	cls.disable_screen = Sys_Milliseconds ();
@@ -903,97 +1049,6 @@ void SCR_DirtyScreen (void)
 	SCR_AddDirtyPoint (viddef.width-1, viddef.height-1);
 }
 
-/*
-==============
-SCR_TileClear
-
-Clear any parts of the tiled background that were drawn on last frame
-==============
-*/
-void SCR_TileClear (void)
-{
-	int		i;
-	int		top, bottom, left, right;
-	dirty_t	clear;
-
-	if (scr_drawall->value)
-		SCR_DirtyScreen ();	// for power vr or broken page flippers...
-
-	if (scr_con_current == 1.0)
-		return;		// full screen console
-	if (scr_viewsize->value == 100)
-		return;		// full screen rendering
-	if (cl.cinematictime > 0)
-		return;		// full screen cinematic
-
-	// erase rect will be the union of the past three frames
-	// so tripple buffering works properly
-	clear = scr_dirty;
-	for (i=0 ; i<2 ; i++)
-	{
-		if (scr_old_dirty[i].x1 < clear.x1)
-			clear.x1 = scr_old_dirty[i].x1;
-		if (scr_old_dirty[i].x2 > clear.x2)
-			clear.x2 = scr_old_dirty[i].x2;
-		if (scr_old_dirty[i].y1 < clear.y1)
-			clear.y1 = scr_old_dirty[i].y1;
-		if (scr_old_dirty[i].y2 > clear.y2)
-			clear.y2 = scr_old_dirty[i].y2;
-	}
-
-	scr_old_dirty[1] = scr_old_dirty[0];
-	scr_old_dirty[0] = scr_dirty;
-
-	scr_dirty.x1 = 9999;
-	scr_dirty.x2 = -9999;
-	scr_dirty.y1 = 9999;
-	scr_dirty.y2 = -9999;
-
-	// don't bother with anything convered by the console)
-	top = scr_con_current*viddef.height;
-	if (top >= clear.y1)
-		clear.y1 = top;
-
-	if (clear.y2 <= clear.y1)
-		return;		// nothing disturbed
-
-	top = scr_vrect.y;
-	bottom = top + scr_vrect.height-1;
-	left = scr_vrect.x;
-	right = left + scr_vrect.width-1;
-
-	if (clear.y1 < top)
-	{	// clear above view screen
-		i = clear.y2 < top-1 ? clear.y2 : top-1;
-		Draw_TileClear (clear.x1 , clear.y1,
-			clear.x2 - clear.x1 + 1, i - clear.y1+1, "backtile");
-		clear.y1 = top;
-	}
-	if (clear.y2 > bottom)
-	{	// clear below view screen
-		i = clear.y1 > bottom+1 ? clear.y1 : bottom+1;
-		Draw_TileClear (clear.x1, i,
-			clear.x2-clear.x1+1, clear.y2-i+1, "backtile");
-		clear.y2 = bottom;
-	}
-	if (clear.x1 < left)
-	{	// clear left of view screen
-		i = clear.x2 < left-1 ? clear.x2 : left-1;
-		Draw_TileClear (clear.x1, clear.y1,
-			i-clear.x1+1, clear.y2 - clear.y1 + 1, "backtile");
-		clear.x1 = left;
-	}
-	if (clear.x2 > right)
-	{	// clear left of view screen
-		i = clear.x1 > right+1 ? clear.x1 : right+1;
-		Draw_TileClear (i, clear.y1,
-			clear.x2-i+1, clear.y2 - clear.y1 + 1, "backtile");
-		clear.x2 = right;
-	}
-
-}
-
-
 //===============================================================
 
 
@@ -1129,7 +1184,7 @@ void SCR_DrawField (int x, int y, int color, int width, int value, float scale)
 		else
 			frame = *ptr -'0';
 
-		Draw_ScaledPic (x,y,scale,sb_nums[color][frame]);
+		Draw_ScaledPic (x,y,scale/1.8,sb_nums[color][frame]);
 		x += CHAR_WIDTH*scale;
 		ptr++;
 		l--;
@@ -1170,7 +1225,7 @@ SCR_ExecuteLayoutString
 */
 void SCR_ExecuteLayoutString (char *s)
 {
-	int		x, y;
+	int		x, y, ny; //ny is coordinates used for new sb layout client tags only
 	int		value;
 	char	*token;
 	int		width;
@@ -1178,6 +1233,7 @@ void SCR_ExecuteLayoutString (char *s)
 	clientinfo_t	*ci;
 	int		charscale;
 	float	scale;
+	qboolean newSBlayout = false;
 
 	if (cls.state != ca_active || !cl.refresh_prepped)
 		return;
@@ -1236,6 +1292,7 @@ void SCR_ExecuteLayoutString (char *s)
 		{
 			token = COM_Parse (&s);
 			y = viddef.height/2 - 100*scale + atoi(token)*scale;
+			ny = viddef.height/2 - 100*scale + atoi(token)*scale/2;
 			continue;
 		}
 
@@ -1257,6 +1314,43 @@ void SCR_ExecuteLayoutString (char *s)
 			continue;
 		}
 
+		if (!strcmp(token, "newsb"))
+		{		
+			//print header here
+			x = viddef.width/2 - 160*scale;
+			y = viddef.height/2 - 64*scale;
+
+			DrawString (x+2*scale, y, va("Player"));
+			DrawString (x+176*scale, y, va("Score"));
+			DrawString (x+232*scale, y, va("Ping"));
+			DrawString (x+280*scale, y, va("time"));
+
+			newSBlayout = true;
+			continue;
+		}
+
+		if (!strcmp(token, "newctfsb"))
+		{		
+			//print header here
+			x = viddef.width/2 - 160*scale;
+			y = viddef.height/2 - 72*scale;
+
+			//team 1
+			DrawString (x-82*scale, y, va("Player"));
+			DrawString (x+76*scale, y, va("Score"));
+			DrawString (x+124*scale, y, va("Ping"));
+
+			x += 190*scale;
+
+			//team 2
+			DrawString (x-16*scale, y, va("Player"));
+			DrawString (x+142*scale, y, va("Score"));
+			DrawString (x+190*scale, y, va("Ping"));
+
+			newSBlayout = true;
+			continue;
+		}
+
 		if (!strcmp(token, "client"))
 		{	// draw a deathmatch client block
 			int		score, ping, time;
@@ -1264,7 +1358,10 @@ void SCR_ExecuteLayoutString (char *s)
 			token = COM_Parse (&s);
 			x = viddef.width/2 - 160*scale + atoi(token)*scale;
 			token = COM_Parse (&s);
-			y = viddef.height/2 - 100*scale + atoi(token)*scale;
+			if(newSBlayout)
+				y = viddef.height/2 - 100*scale + atoi(token)*scale/2;
+			else
+				y = viddef.height/2 - 100*scale + atoi(token)*scale;
 			SCR_AddDirtyPoint (x, y);
 			SCR_AddDirtyPoint (x+159*scale, y+31*scale);
 
@@ -1279,19 +1376,30 @@ void SCR_ExecuteLayoutString (char *s)
 
 			token = COM_Parse (&s);
 			ping = atoi(token);
-
+ 
 			token = COM_Parse (&s);
 			time = atoi(token);
 
-			Draw_ColorString (x+32*scale, y, ci->name);
-			DrawString (x+32*scale, y+8*scale,  "Score: ");
-			DrawAltString (x+32*scale+7*charscale, y+8*scale,  va("%i", score));
-			DrawString (x+32*scale, y+16*scale, va("Ping:  %i", ping));
-			DrawString (x+32*scale, y+24*scale, va("Time:  %i", time));
+			if(newSBlayout) { //new scoreboard layout
+
+				Draw_ColorString (x+2*scale, y+34*scale, ci->name, 1.2);
+				DrawAltString (x+176*scale, y+34*scale, va("%i", score));
+				DrawString (x+232*scale, y+34*scale, va("%i", ping));
+				DrawString (x+280*scale, y+34*scale, va("%i", time));
+			}
+			else
+			{
+				Draw_ColorString (x+32*scale, y, ci->name, 1);
+				DrawString (x+32*scale, y+8*scale,  "Score: ");
+				DrawAltString (x+32*scale+7*charscale, y+8*scale,  va("%i", score));
+				DrawString (x+32*scale, y+16*scale, va("Ping:  %i", ping));
+				DrawString (x+32*scale, y+24*scale, va("Time:  %i", time));
+
+				if (!ci->icon)
+					ci = &cl.baseclientinfo;
+				Draw_ScaledPic (x, y, scale/2, ci->iconname);
+			}
 
-			if (!ci->icon)
-				ci = &cl.baseclientinfo;
-			Draw_ScaledPic (x, y, scale, ci->iconname);
 			continue;
 		}
 		if (!strcmp(token, "queued"))
@@ -1301,7 +1409,10 @@ void SCR_ExecuteLayoutString (char *s)
 			token = COM_Parse (&s);
 			x = viddef.width/2 - 160*scale + atoi(token)*scale;
 			token = COM_Parse (&s);
-			y = viddef.height/2 - 100*scale + atoi(token)*scale;
+			if(newSBlayout)
+				y = viddef.height/2 - 100*scale + atoi(token)*scale/2;
+			else
+				y = viddef.height/2 - 100*scale + atoi(token)*scale;
 			SCR_AddDirtyPoint (x, y);
 			SCR_AddDirtyPoint (x+159*scale, y+31*scale);
 
@@ -1320,24 +1431,43 @@ void SCR_ExecuteLayoutString (char *s)
 			token = COM_Parse (&s);
 			time = atoi(token);
 
-			Draw_ColorString (x+32*scale, y, ci->name);
-			DrawString (x+32*scale, y+8*scale,  "Score: ");
-			DrawAltString (x+32*scale+7*charscale, y+8*scale,  va("%i", score));
-			DrawString (x+32*scale, y+16*scale, va("Ping:  %i", ping));
-			Draw_ColorString (x+32*scale, y+24*scale, va("^1Queue:  ^1%i", time));
+			token = COM_Parse (&s);
+
+			if(newSBlayout) { //new scoreboard layout
+
+				Draw_ColorString (x+2*scale, y+34*scale, ci->name, 1.2);
+				DrawAltString (x+176*scale, y+34*scale, va("%i", score));
+				DrawString (x+232*scale, y+34*scale, va("%i", ping));
+				Draw_ColorString (x+280*scale, y+34*scale, va("^1%i", time), 1.2);
+
+			}
+			else {
+
+				Draw_ColorString (x+32*scale, y, ci->name, 1);
+				DrawString (x+32*scale, y+8*scale,  "Score: ");
+				DrawAltString (x+32*scale+7*charscale, y+8*scale,  va("%i", score));
+				DrawString (x+32*scale, y+16*scale, va("Ping:  %i", ping));
+				Draw_ColorString (x+32*scale, y+24*scale, va("^1Queue:  ^1%i", time), 1);
+
+				if (!ci->icon)
+					ci = &cl.baseclientinfo;
+				Draw_ScaledPic (x, y, scale/2, ci->iconname);
+			}
 
-			if (!ci->icon)
-				ci = &cl.baseclientinfo;
-			Draw_ScaledPic (x, y, scale, ci->iconname);
 			continue;
 		}
 		if (!strcmp(token, "ctf"))
 		{	// draw a ctf client block
-			int		score, ping;
+			int	score, ping;
 			char	block[80];
+			int		team;
 
 			token = COM_Parse (&s);
 			x = viddef.width/2 - 160*scale + atoi(token)*scale;
+			if(atoi(token) < 0)
+				team = 0;
+			else
+				team = 1;
 			token = COM_Parse (&s);
 			y = viddef.height/2 - 100*scale + atoi(token)*scale;
 			SCR_AddDirtyPoint (x, y);
@@ -1357,9 +1487,21 @@ void SCR_ExecuteLayoutString (char *s)
 			if (ping > 999)
 				ping = 999;
 
-			sprintf(block, "%3d %3d %-12.12s", score, ping, ci->name);
+			if(newSBlayout) {
+				Draw_ColorString (x+16, y, ci->name, 1.2);
+				DrawAltString (x+172*scale, y+1*scale, va("%i", score));
+				DrawString (x+220*scale, y+1*scale, va("%i", ping));
 
-			Draw_ColorString (x, y, block);
+				if(team) { //draw pic on left side(done client side to save packetsize
+					Draw_ScaledPic (x, y-2*scale, scale, "/pics/blueplayerbox");
+				}
+				else 
+					Draw_ScaledPic(x, y-2*scale, scale, "/pics/redplayerbox");
+			}
+			else {
+				sprintf(block, "%3d %3d %s", score, ping, ci->name);
+				Draw_ColorString (x, y, block, 1);
+			}
 
 			continue;
 		}
@@ -1367,9 +1509,16 @@ void SCR_ExecuteLayoutString (char *s)
 		if (!strcmp(token, "picn"))
 		{	// draw a pic from a name
 			token = COM_Parse (&s);
-			SCR_AddDirtyPoint (x, y);
-			SCR_AddDirtyPoint (x+23*scale, y+23*scale);
-			Draw_ScaledPic (x, y, scale, token);
+			if(newSBlayout && !strcmp(token, "playerbox")) { //cannot simply fill y = ny here
+				SCR_AddDirtyPoint (x, ny+32*scale);
+				SCR_AddDirtyPoint (x+23*scale, ny+55*scale);
+				Draw_ScaledPic (x, ny+32*scale, scale, token);
+			}
+			else {
+				SCR_AddDirtyPoint (x, y);
+				SCR_AddDirtyPoint (x+23*scale, y+23*scale);
+				Draw_ScaledPic (x, y, scale, token);
+			}
 			continue;
 		}
 
@@ -1406,11 +1555,9 @@ void SCR_ExecuteLayoutString (char *s)
 			//draw the zoom scope pic if we are using the zoom alt-fire of disruptor
 			if (cl.frame.playerstate.stats[STAT_ZOOMED])
 			{
-				int		w, h;
 				char zoompic[32];
 				sprintf(zoompic, "zoomscope%i", cl.frame.playerstate.stats[STAT_ZOOMED]);
-				Draw_GetPicSize (&w, &h, zoompic);
-				Draw_ScaledPic ((viddef.width-w*scale)/2, (viddef.height-h*scale)/2, scale, zoompic);
+				Draw_StretchPic (0, 0, viddef.width, viddef.height, zoompic);
 			}
 
 			continue;
@@ -1543,6 +1690,46 @@ void SCR_DrawLayout (void)
 	SCR_ExecuteLayoutString (cl.layout);
 }
 
+/*
+=================
+SCR_DrawPlayerIcon
+=================
+*/
+char		scr_playericon[MAX_OSPATH];
+char		scr_playername[32];
+float		scr_playericonalpha;
+void SCR_DrawPlayerIcon(void) {
+	
+	int w, h;
+	float scale, iconPos;
+
+	if (cls.key_dest == key_menu || cls.key_dest == key_console)
+		return;
+
+	if(scr_playericonalpha <= 0.0)
+		return;
+
+	scr_playericonalpha -= cls.frametime; //fade map pic in
+
+	if(scr_playericonalpha < 1.0)
+		iconPos = scr_playericonalpha;
+	else
+		iconPos = 1.0;
+
+	scale = (float)(viddef.height)/600;
+	if(scale < 1)
+		scale = 1;
+
+	w = h = 64; //icon size, will change to be larger
+
+	w*=scale;
+	h*=scale;
+
+	Draw_AlphaStretchPlayerIcon( -w+(w*iconPos), viddef.height/2 + h/2, w, h, scr_playericon, scr_playericonalpha);
+	Menu_DrawColorString(-w+(w*iconPos), viddef.height/2 + h + 32*scale, scr_playername);
+
+}
+
 /*
 ================
 
@@ -1550,26 +1737,36 @@ SCR_showTimer
 
 ================
 */
-clock_t 	t1;
 char		temptime[32];
 int		timecounter;
+int		seconds, minutes;
 void SCR_showTimer(void)
 {
 	float scale;
+
+	if (cls.key_dest == key_menu || cls.key_dest == key_console)
+		return;
+
 	scale = (float)(viddef.height)/600;
 	if(scale < 1)
 		scale = 1;
 
-	if ((cl.time + 2000) < timecounter)
+	if ((cls.realtime + 2000) < timecounter)
 		timecounter = cl.time + 1000;
 
-	if (cl.time >timecounter)
+	if (cls.realtime > timecounter)
 	{
-		t1 = clock();
-		Com_sprintf(temptime, sizeof(temptime),"%i", (int)t1 / CLOCKS_PER_SEC);
-		timecounter = cl.time + 1000;
+		seconds += 1;
+		if(seconds >= 60) {
+			minutes++;
+			seconds = 0;
+		}
+		Com_sprintf(temptime, sizeof(temptime),"%i:%2i", minutes, seconds);
+		timecounter = cls.realtime + 1000;
 	}
 
+	Draw_StretchPic (viddef.width - 104*scale, viddef.height -42*scale, 32*scale, 32*scale, "timer");
+
 	DrawString(viddef.width - 64*scale, viddef.height - 32*scale, temptime);
 }
 
@@ -1587,20 +1784,77 @@ char		temp[32];
 void SCR_showFPS(void)
 {
 	float scale;
+	static qboolean restart = true;
+	static float update_trigger = 0.0f;
+	static float framecounter = 0.0f;	
+	static int start_msec;
+	int end_msec;
+	float time_sec;
+	float framerate;
+
+	if (cls.key_dest == key_menu || cls.key_dest == key_console)
+	{
+		restart = true;
+		return;
+	}		
+
+	if( restart )
+	{
+		start_msec = cls.realtime; 
+		framecounter = 0.0f;
+		update_trigger = 10.0f; // initial delay to update
+		temp[0] = 0; // blank the text buffer
+		restart = false;
+	}
+	else
+	{
+		framecounter += 1.0f;
+		if( framecounter >= update_trigger )
+		{
+			// calculate frames-per-second, using client frame current time
+			end_msec = cls.realtime;
+			time_sec = ((float)(end_msec - start_msec)) / 1000.0f;
+			framerate = framecounter / time_sec ;
+			
+			// update text buffer for display
+			Com_sprintf( temp, sizeof(temp), "%3.0ffps", framerate );
+			
+			// setup for next update
+			framecounter = 0.0f;
+			start_msec = end_msec;
+			update_trigger = framerate / 2.0 ; // for .5 sec update interval
+		}
+	}
 
 	scale = (float)(viddef.height)/600;
 	if(scale < 1)
 		scale = 1;
-
-	if ((cl.time + 1000) < fpscounter)
-		fpscounter = cl.time + 100;
-
-	if (cl.time > fpscounter)
-	{
-		Com_sprintf(temp, sizeof(temp),"%3.0ffps", 1/cls.frametime);
-		fpscounter = cl.time + 100;
-	}
+	
 	DrawString(viddef.width - 64*scale, viddef.height - 24*scale, temp);
+	
+}
+
+/*
+ * SCR_ShowRSpeeds()
+ *   display the cvar r_speeds activated performance counters
+ */
+void SCR_showRSpeeds( void )
+{
+	char prtstring[32];
+ 	float scale;
+ 	extern int c_brush_polys;
+ 	extern int c_alias_polys;
+ 
+ 	if (cls.key_dest == key_menu || cls.key_dest == key_console)
+ 		return;
+ 
+ 	scale = (float)(viddef.height)/600;
+ 	if(scale < 1)
+ 		scale = 1;
+ 
+ 	Com_sprintf( prtstring, sizeof( prtstring ),
+ 		"%5.5i wpoly %5.5i epoly",  c_brush_polys, c_alias_polys );
+ 	DrawString(	viddef.width/2, viddef.height - 48*scale,	prtstring );
 }
 
 /*
@@ -1664,67 +1918,20 @@ void SCR_UpdateScreen (void)
 		if (scr_draw_loading == 2)
 		{	//  loading plaque over black screen
 
-			//re.CinematicSetPalette(NULL);
 			SCR_DrawLoading ();
 
 			if (cls.disable_screen)
 				scr_draw_loading = 2;
-
-			//if (cls.consoleActive)
-			//	Con_DrawConsole (0.5, false);
-
+	
 			//NO CONSOLE!!!
 			continue;
-		}
-		// if a cinematic is supposed to be running, handle menus
-		// and console specially
-		else if (cl.cinematictime > 0)
-		{
-			if (cls.key_dest == key_menu)
-			{
-				if (cl.cinematicpalette_active)
-				{
-					R_SetPalette(NULL);
-					cl.cinematicpalette_active = false;
-				}
-				M_Draw ();
-//				R_EndFrame();
-//				return;
-			}
-			else if (cls.key_dest == key_console)
-			{
-				if (cl.cinematicpalette_active)
-				{
-					R_SetPalette(NULL);
-					cl.cinematicpalette_active = false;
-				}
-				SCR_DrawConsole ();
-//				R_EndFrame();
-//				return;
-			}
-			else
-			{
-				SCR_DrawCinematic();
-//				R_EndFrame();
-//				return;
-			}
-		}
+		}		
 		else
 		{
-
-			// make sure the game palette is active
-			if (cl.cinematicpalette_active)
-			{
-				R_SetPalette(NULL);
-				cl.cinematicpalette_active = false;
-			}
-
 			// do 3D refresh drawing, and then update the screen
 			SCR_CalcVrect ();
 
-			// clear any dirty part of the background
-			SCR_TileClear ();
-
+			need_free_vbo = true;
 			V_RenderView ( separation[i] );
 
 			SCR_DrawStats ();
@@ -1735,6 +1942,7 @@ void SCR_UpdateScreen (void)
 
 			SCR_DrawNet ();
 			SCR_CheckDrawCenterString ();
+			SCR_CheckDrawIRCString();
 
 			if (scr_timegraph->value)
 				SCR_DebugGraph (cls.frametime*300, 0);
@@ -1746,6 +1954,9 @@ void SCR_UpdateScreen (void)
 
 			SCR_DrawConsole ();
 
+			if (need_free_vbo)		
+				R_VCFreeFrame();
+
 			M_Draw ();
 
 			SCR_DrawLoading ();
@@ -1759,7 +1970,12 @@ void SCR_UpdateScreen (void)
 			{
 				SCR_showTimer();
 			}
-
+			{
+				extern cvar_t* r_speeds;
+				if( r_speeds->value == 2 )
+					SCR_showRSpeeds();
+			}
+			SCR_DrawPlayerIcon();
 		}
 	}
 	R_EndFrame();
