@@ -76,11 +76,17 @@ typedef struct
 	struct model_s	*model;
 	struct model_s	*weaponmodel[MAX_CLIENTWEAPONMODELS];
 	struct model_s  *helmet;
+	struct model_s  *lod1;
+	struct model_s  *lod2;
 } clientinfo_t;
 
 extern char cl_weaponmodels[MAX_CLIENTWEAPONMODELS][MAX_QPATH];
 extern int num_cl_weaponmodels;
 
+extern char		scr_playericon[MAX_OSPATH];
+extern char		scr_playername[32];
+extern float	scr_playericonalpha;
+
 #define	CMD_BACKUP		64	// allow a lot of command backups for very fast systems
 
 //
@@ -137,15 +143,6 @@ typedef struct
 	char		layout[1024];		// general 2D overlay
 	int			inventory[MAX_ITEMS];
 
-	//
-	// non-gameserver infornamtion
-	// FIXME: move this cinematic stuff into the cin_t structure
-	FILE		*cinematic_file;
-	int			cinematictime;		// cls.realtime for first cinematic frame
-	int			cinematicframe;
-	char		cinematicpalette[768];
-	qboolean	cinematicpalette_active;
-
 	//
 	// server state information
 	//
@@ -234,6 +231,8 @@ typedef struct
 	char		downloadurl[MAX_OSPATH];  // for http downloads
 	qboolean	downloadhttp;
 
+	int			irc_connected;		//track irc connection
+
 // demo recording info must be here, so it isn't cleared on level change
 	qboolean	demorecording;
 	qboolean	demowaiting;	// don't record until a non-delta message is received
@@ -261,6 +260,12 @@ extern	cvar_t	*cl_noskins;
 extern	cvar_t	*cl_autoskins;
 extern  cvar_t	*cl_healthaura;
 extern  cvar_t	*cl_noblood;
+extern  cvar_t	*cl_disbeamclr;
+extern  cvar_t	*cl_brass;
+extern  cvar_t	*cl_playtaunts;
+
+extern	cvar_t	*cl_paindist;
+extern	cvar_t	*cl_explosiondist;
 
 extern	cvar_t	*cl_upspeed;
 extern	cvar_t	*cl_forwardspeed;
@@ -281,6 +286,7 @@ extern	cvar_t	*lookspring;
 extern	cvar_t	*lookstrafe;
 extern	cvar_t	*sensitivity;
 
+extern	cvar_t	*m_smoothing;
 extern	cvar_t	*m_pitch;
 extern	cvar_t	*m_yaw;
 extern	cvar_t	*m_forward;
@@ -288,8 +294,6 @@ extern	cvar_t	*m_side;
 
 extern	cvar_t	*freelook;
 
-extern	cvar_t	*cl_lightlevel;	// FIXME HACK
-
 extern	cvar_t	*cl_paused;
 extern	cvar_t	*cl_timedemo;
 
@@ -297,6 +301,9 @@ extern	cvar_t	*cl_vwep;
 
 extern  cvar_t  *background_music;
 
+extern  cvar_t	*cl_IRC_connect_at_startup;
+extern	cvar_t	*cl_IRC_server;
+
 typedef struct
 {
 	int		key;				// so entities can reuse same entry
@@ -324,7 +331,7 @@ extern	sizebuf_t	net_message;
 
 void DrawString (int x, int y, char *s);
 void DrawAltString (int x, int y, char *s);	// toggle high bit
-void Draw_ColorString ( int x, int y, char *str );
+void Draw_ColorString ( int x, int y, char *str, float scale);
 qboolean	CL_CheckOrDownloadFile (char *filename);
 
 void CL_AddNetgraph (void);
@@ -346,17 +353,6 @@ typedef struct cl_sustain
 } cl_sustain_t;
 
 #define MAX_SUSTAINS		32
-void CL_ParticleSteamEffect2(cl_sustain_t *self);
-void CL_ParticleFireEffect2(cl_sustain_t *self);
-void CL_ParticleSmokeEffect2(cl_sustain_t *self);
-void CL_ParticleEffect (vec3_t org, vec3_t dir, int color, int count);
-void CL_ParticleEffect2 (vec3_t org, vec3_t dir, int color, int count);
-void CL_BulletSparks ( vec3_t org, vec3_t dir);
-void CL_SplashEffect ( vec3_t org, vec3_t dir, int color, int count);
-void CL_LaserSparks ( vec3_t org, vec3_t dir, int color, int count);
-void CL_SmallHealthParticles(vec3_t org);
-void CL_MedHealthParticles(vec3_t org);
-void CL_LargeHealthParticles(vec3_t org);
 
 //=================================================
 
@@ -408,22 +404,75 @@ typedef struct particle_s
 // PGM
 // ========
 
+// Berserker client entities code
+#define     MAX_CLENTITIES     256
+#define CLM_BOUNCE		1
+#define CLM_FRICTION	2
+#define CLM_ROTATE		4
+#define CLM_NOSHADOW	8
+#define CLM_STOPPED		16
+#define CLM_BRASS		32
+
+typedef struct clentity_s {
+	struct clentity_s *next;
+	float time;
+	vec3_t org;
+	vec3_t lastOrg;
+	vec3_t vel;
+	vec3_t accel;
+	struct model_s *model;
+	float ang;
+	float avel;
+	float alpha;
+	float alphavel;
+	int flags;
+
+} clentity_t;
+
+void CL_AddClEntities(void);
+void CL_ClearClEntities(void);
 void CL_ClearEffects (void);
 void CL_ClearTEnts (void);
+
 void CL_BlasterBall (vec3_t start, vec3_t end);
 void CL_DisruptorBeam (vec3_t start, vec3_t end);
 void CL_LaserBeam (vec3_t start, vec3_t end);
 void CL_BlasterBeam (vec3_t start, vec3_t end);
-void CL_LaserBlast (vec3_t start, vec3_t end);
 void CL_RedBlasterBeam (vec3_t start, vec3_t end);
 void CL_VaporizerBeam (vec3_t start, vec3_t end);
 void CL_BubbleTrail (vec3_t start, vec3_t end);
 void CL_NewLightning (vec3_t start, vec3_t end);
 void CL_BlueTeamLight(vec3_t pos);
 void CL_RedTeamLight(vec3_t pos);
+void CL_FlagEffects(vec3_t pos, qboolean team);
+void CL_PoweredEffects (vec3_t pos);
 void CL_SmokeTrail (vec3_t start, vec3_t end, int colorStart, int colorRun, int spacing);
-void CL_ParticleSteamEffect (vec3_t org, vec3_t dir, int color, int count, int magnitude);
 void CL_SayIcon(vec3_t org);
+void CL_TeleportParticles (vec3_t org);
+void CL_BlasterParticles (vec3_t org, vec3_t dir);
+void CL_ExplosionParticles (vec3_t org);
+void CL_MuzzleParticles (vec3_t org);
+void CL_BlueMuzzleParticles (vec3_t org);
+void CL_SmartMuzzle (vec3_t org);
+void CL_Voltage(vec3_t org);
+void CL_Deathfield (vec3_t org, int type);
+void CL_BFGExplosionParticles (vec3_t org);
+void CL_DustParticles (vec3_t org);
+void CL_BlueBlasterParticles (vec3_t org, vec3_t dir);
+void CL_ParticleSteamEffect(cl_sustain_t *self);
+void CL_ParticleFireEffect2(cl_sustain_t *self);
+void CL_ParticleSmokeEffect2(cl_sustain_t *self);
+void CL_ParticleEffect (vec3_t org, vec3_t dir, int color, int count);
+void CL_ParticleEffect2 (vec3_t org, vec3_t dir, int color, int count);
+void CL_BulletSparks ( vec3_t org, vec3_t dir);
+void CL_SplashEffect ( vec3_t org, vec3_t dir, int color, int count);
+void CL_LaserSparks ( vec3_t org, vec3_t dir, int color, int count);
+void CL_SmallHealthParticles(vec3_t org);
+void CL_MedHealthParticles(vec3_t org);
+void CL_LargeHealthParticles(vec3_t org);
+void CL_BrassShells(vec3_t org, vec3_t dir, int count);
+void CL_MuzzleFlashParticle (vec3_t org, vec3_t angles, qboolean from_client);
+void CL_PlasmaFlashParticle (vec3_t org, vec3_t angles, qboolean from_client);
 
 int CL_ParseEntityBits (unsigned *bits);
 void CL_ParseDelta (entity_state_t *from, entity_state_t *to, int number, int bits);
@@ -450,6 +499,7 @@ void CL_AddLightStyles (void);
 
 void CL_PrepRefresh (void);
 void CL_RegisterSounds (void);
+void CL_RegisterModels (void);
 
 void CL_Quit_f (void);
 
@@ -504,6 +554,23 @@ void IN_CenterView (void);
 float CL_KeyState (kbutton_t *key);
 char *Key_KeynumToString (int keynum);
 
+//cl_irc.c
+typedef struct IRCresponse_s
+{
+	char word[200][50];
+	int words;
+} IRCresponse_t;
+
+typedef struct user_s
+{
+	char nick[32]; 
+	char email[100];
+} user_t;
+
+void CL_InitIRC(void);
+void CL_IRCShutdown(void);
+void CL_IRCSay(void);
+
 //
 // cl_demo.c
 //
@@ -522,11 +589,31 @@ void SHOWNET(char *s);
 void CL_ParseClientinfo (int player);
 void CL_Download_f (void);
 
+//
+// cl_scrn.c
+//
+void SCR_IRCPrintf (char *fmt, ...);
+
+//
+// cl_stats.c
+//
+
+typedef struct _PLAYERSTATS {
+	char playername[32];
+	int totalfrags;
+	double totaltime;
+	int ranking;
+} PLAYERSTATS;
+
+void getStatsDB( void );
+PLAYERSTATS getPlayerRanking ( PLAYERSTATS player );
+PLAYERSTATS getPlayerByRank ( int rank, PLAYERSTATS player );
+
 //
 // cl_view.c
 //
-extern	int			gun_frame;
-extern	struct model_s	*gun_model;
+
+extern qboolean need_free_vbo;
 
 qboolean loadingMessage;
 char loadingMessages[5][96];
@@ -560,6 +647,7 @@ int numitemicons;
 void V_Init (void);
 void V_RenderView( float stereo_separation );
 void V_AddEntity (entity_t *ent);
+void V_AddViewEntity (entity_t *ent);
 void V_AddParticle (vec3_t org, vec3_t angle, int color, int type, int texnum, int blenddst, int blendsrc, float alpha, float scale);
 void V_AddLight (vec3_t org, float intensity, float r, float g, float b);
 void V_AddTeamLight (vec3_t org, float intensity, float r, float g, float b, int team);
@@ -570,6 +658,7 @@ void V_AddStain (vec3_t org, float intensity, float r, float g, float b, float a
 // cl_tent.c
 //
 void CL_RegisterTEntSounds (void);
+void CL_RegisterTEntModels(void);
 
 //
 // cl_pred.c
@@ -585,9 +674,10 @@ trace_t CL_PMSurfaceTrace (vec3_t start, vec3_t mins, vec3_t maxs, vec3_t end, i
 cdlight_t *CL_AllocDlight (int key);
 void CL_BigTeleportParticles (vec3_t org);
 void CL_RocketTrail (vec3_t start, vec3_t end, centity_t *old);
+void CL_BlasterTrail (vec3_t start, vec3_t end, centity_t *old);
 void CL_ShipExhaust (vec3_t start, vec3_t end, centity_t *old);
 void CL_RocketExhaust (vec3_t start, vec3_t end, centity_t *old);
-void CL_BeamgunMark(vec3_t org, vec3_t dir, float dur);
+void CL_BeamgunMark(vec3_t org, vec3_t dir, float dur, qboolean isDis);
 void CL_BulletMarks(vec3_t org, vec3_t dir);
 void CL_VaporizerMarks(vec3_t org, vec3_t dir);
 void CL_BlasterMark(vec3_t org, vec3_t dir);
