@@ -27,9 +27,6 @@ cvar_t		*con_notifytime;
 
 
 #define		MAXCMDLINE	256
-extern	char	key_lines[32][MAXCMDLINE];
-extern	int		edit_line;
-extern	int		key_linepos;
 
 vec4_t	color_table[8] =
 {
@@ -61,13 +58,13 @@ void DrawString (int x, int y, char *s)
 Draw_ColorString
 =============
 */
-void Draw_ColorString ( int x, int y, char *str )
+void Draw_ColorString ( int x, int y, char *str, float scale)
 {
 	int		num;
 	vec4_t	scolor;
 	int		charscale;
 
-	charscale = (float)(viddef.height)*8/600;
+	charscale = (float)(viddef.height)*8*scale/600;
 	if(charscale < 8)
 		charscale = 8;
 
@@ -102,7 +99,7 @@ void Draw_ColorString ( int x, int y, char *str )
 
 void DrawAltString (int x, int y, char *s)
 {
-	int		charscale;
+	float	charscale;
 
 	charscale = (float)(viddef.height)*8/600;
 	if(charscale < 8)
@@ -120,7 +117,7 @@ void DrawAltString (int x, int y, char *s)
 void Key_ClearTyping (void)
 {
 	key_lines[edit_line][1] = 0;	// clear any typing
-	key_linepos = 1;
+	key_linelen = key_linepos = 1;
 }
 
 /*
@@ -273,7 +270,10 @@ Con_MessageMode_f
 void Con_MessageMode_f (void)
 {
 	chat_team = false;
+	chat_irc = false;
 	cls.key_dest = key_message;
+	Cbuf_AddText("chatbubble\n"); 
+	Cbuf_Execute ();
 }
 
 /*
@@ -284,7 +284,23 @@ Con_MessageMode2_f
 void Con_MessageMode2_f (void)
 {
 	chat_team = true;
+	chat_irc = false;
 	cls.key_dest = key_message;
+	Cbuf_AddText("chatbubble\n"); 
+	Cbuf_Execute ();
+}
+
+/*
+================
+Con_MessageMode3_f
+================
+*/
+void Con_MessageMode3_f (void)
+{
+	chat_team = false;
+	chat_irc = true;
+	cls.key_dest = key_message;
+	//Cbuf_AddText("chatbubble\n"); 
 }
 
 /*
@@ -296,10 +312,14 @@ If the line width has changed, reformat the buffer.
 */
 void Con_CheckResize (void)
 {
-	int		i, j, width, oldwidth, oldtotallines, numlines, numchars;
+	int		i, j, width, oldwidth, oldtotallines, numlines, numchars, charscale;
 	char	tbuf[CON_TEXTSIZE];
 
-	width = (viddef.width >> 3) - 2;
+	// find out the character's scale and use it to compute the width
+	charscale = (float)(viddef.height)*8/600;
+	if(charscale < 8)
+		charscale = 8;
+	width = (viddef.width / charscale) - 2;
 
 	if (width == con.linewidth)
 		return;
@@ -370,6 +390,7 @@ void Con_Init (void)
 	Cmd_AddCommand ("togglechat", Con_ToggleChat_f);
 	Cmd_AddCommand ("messagemode", Con_MessageMode_f);
 	Cmd_AddCommand ("messagemode2", Con_MessageMode2_f);
+	Cmd_AddCommand ("messagemode3", Con_MessageMode3_f);
 	Cmd_AddCommand ("clear", Con_Clear_f);
 	Cmd_AddCommand ("condump", Con_Dump_f);
 	con.initialized = true;
@@ -516,9 +537,9 @@ The input line scrolls horizontally if typing goes beyond the right edge
 */
 void Con_DrawInput (void)
 {
-	int		y;
 	int		i;
-	char	*text, output[2048];
+	char		text[2048];
+	char		*output;
 	int		charscale;
 
 	charscale = (float)(viddef.height)*8/600;
@@ -530,34 +551,25 @@ void Con_DrawInput (void)
 	if (cls.key_dest != key_console && cls.state == ca_active)
 		return;		// don't draw anything (always draw if not active)
 
-	text = key_lines[edit_line];
-
-// add the cursor frame
+// copy the text into the temporary buffer and add the cursor frame
+	memcpy (text, key_lines[edit_line], key_linepos);
 	text[key_linepos] = 10+((int)(cls.realtime>>8)&1);
+	if ( key_linelen > key_linepos )
+		memcpy (text + key_linepos + 1, key_lines[edit_line] + key_linepos, key_linelen - key_linepos);
 
 // fill out remainder with spaces
-	for (i=key_linepos+1 ; i< con.linewidth ; i++)
+	for (i = key_linelen + 1 ; i < con.linewidth ; i++)
 		text[i] = ' ';
+	text[i] = 0;
 
-//	prestep if horizontally scrolling
+// prestep if horizontally scrolling
+	output = text;
 	if (key_linepos >= con.linewidth)
-		text += 1 + key_linepos - con.linewidth;
-		
+		output += 1 + key_linepos - con.linewidth;
+
+
 // draw it
-	y = con.vislines-charscale*2;
-	
-	Com_sprintf (output, sizeof(output), "");
-	for (i=0 ; i<con.linewidth ; i++)
-	{
-		Com_sprintf (output, sizeof(output), "%s%c", output, text[i]);
-	}
-
-
-
-	Draw_ColorString ( charscale, (int)(con.vislines - 2.75*charscale), output);
-
-// remove cursor
-	key_lines[edit_line][key_linepos] = 0;
+	Draw_ColorString ( charscale, (int)(con.vislines - 2.75*charscale), output, 1);
 }
 
 
@@ -641,6 +653,11 @@ void Con_DrawNotify (void)
 			DrawString (charscale, v, "say_team:");
 			skip = 11;
 		}
+		else if (chat_irc)
+		{
+			DrawString (charscale, v, "say_IRC:");
+			skip = 10;
+		}
 		else
 		{
 			DrawString (charscale, v, "say:");
@@ -761,7 +778,7 @@ void Con_DrawConsole (float frac)
 		for (x=0 ; x<con.linewidth ; x++)
 			Com_sprintf (output, sizeof(output), "%s%c", output, text[x]);
 
-		Draw_ColorString ( 4, y, output);
+		Draw_ColorString ( 4, y, output, 1);
 	}
 
 //ZOID
