@@ -211,7 +211,7 @@ void ACEAI_PickLongRangeGoal(edict_t *self)
 	for(i=0;i<game.maxclients;i++)
 	{
 		ent = g_edicts + i + 1;
-		if(ent == self || !ent->inuse)
+		if(ent == self || !ent->inuse || (ent->client->invis_framenum > level.framenum))
 			continue;
 
 		node = ACEND_FindClosestReachableNode(ent,NODE_DENSITY,NODE_ALL);
@@ -266,6 +266,11 @@ void ACEAI_PickLongRangeGoal(edict_t *self)
 qboolean ACEIT_IsVisibleSolid(edict_t *self, edict_t *other)
 {
 	trace_t tr;
+
+	if(other->client) {
+		if(other->client->invis_framenum > level.framenum)
+			return false;
+	}
 	
 	tr = gi.trace (self->s.origin, vec3_origin, vec3_origin, other->s.origin, self, MASK_SOLID);
 		
@@ -304,7 +309,7 @@ void ACEAI_PickShortRangeGoal(edict_t *self)
 		}
 	    if (strcmp(target->classname, "player") == 0) //so players can't sneak RIGHT up on a bot
 		{
-			if(!target->deadflag && !self->in_deathball && !OnSameTeam(self, target))
+			if(!target->deadflag && !self->in_deathball && !OnSameTeam(self, target) && !(target->client->invis_framenum > level.framenum))
 			{
 				self->movetarget = target;
 			}
@@ -513,6 +518,82 @@ qboolean ACEAI_CheckShot(edict_t *self)
 	return true;
 }
 
+
+void ACEAI_Use_Invisibility (edict_t *ent)
+{
+	gitem_t *it;
+
+	it = FindItem("Invisibility");
+	ent->client->pers.inventory[ITEM_INDEX(it)]--;
+	ValidateSelectedItem (ent);
+
+	it = FindItem("Sproing");
+	ent->client->pers.inventory[ITEM_INDEX(it)] = 0;
+
+	it = FindItem("Haste");
+	ent->client->pers.inventory[ITEM_INDEX(it)] = 0;
+
+	ent->client->resp.reward_pts = 0;
+	ent->client->resp.powered = false;
+
+	if (ent->client->invis_framenum > level.framenum)
+		ent->client->invis_framenum += 300;
+	else
+		ent->client->invis_framenum = level.framenum + 300;
+
+	gi.sound(ent, CHAN_ITEM, gi.soundindex("items/powerup.wav"), 1, ATTN_NORM, 0);
+}
+
+void ACEAI_Use_Haste (edict_t *ent)
+{
+	gitem_t *it;
+
+	it = FindItem("Haste");
+	ent->client->pers.inventory[ITEM_INDEX(it)]--;
+	ValidateSelectedItem (ent);
+
+	it = FindItem("Sproing");
+	ent->client->pers.inventory[ITEM_INDEX(it)] = 0;
+
+	it = FindItem("Invisibility");
+	ent->client->pers.inventory[ITEM_INDEX(it)] = 0;
+
+	ent->client->resp.reward_pts = 0;
+	ent->client->resp.powered = false;
+
+	if (ent->client->haste_framenum > level.framenum)
+		ent->client->haste_framenum += 300;
+	else
+		ent->client->haste_framenum = level.framenum + 300;
+
+	gi.sound(ent, CHAN_ITEM, gi.soundindex("items/powerup.wav"), 1, ATTN_NORM, 0);
+}
+
+void ACEAI_Use_Sproing (edict_t *ent)
+{
+	gitem_t *it;
+
+	it = FindItem("Sproing");
+	ent->client->pers.inventory[ITEM_INDEX(it)]--;
+	ValidateSelectedItem (ent);
+
+	it = FindItem("Haste");
+	ent->client->pers.inventory[ITEM_INDEX(it)] = 0;
+
+	it = FindItem("Invisibility");
+	ent->client->pers.inventory[ITEM_INDEX(it)] = 0;
+
+	ent->client->resp.reward_pts = 0;
+	ent->client->resp.powered = false;
+
+	if (ent->client->sproing_framenum > level.framenum)
+		ent->client->sproing_framenum += 300;
+	else
+		ent->client->sproing_framenum = level.framenum + 300;
+
+	gi.sound(ent, CHAN_ITEM, gi.soundindex("items/powerup.wav"), 1, ATTN_NORM, 0);
+}
+
 ///////////////////////////////////////////////////////////////////////
 // Choose the best weapon for bot 
 ///////////////////////////////////////////////////////////////////////
@@ -520,6 +601,7 @@ void ACEAI_ChooseWeapon(edict_t *self)
 {	
 	float range;
 	vec3_t v;
+	float c;
 	
 	if (self->in_vehicle) {
 		return; 
@@ -529,6 +611,16 @@ void ACEAI_ChooseWeapon(edict_t *self)
 		return; //cannot switch or fire weapons when in a deathball.
 	}
 
+	if(self->client->resp.powered) { //got enough reward points, use something
+		c = random();
+		if( c < 0.5)
+			ACEAI_Use_Invisibility(self);
+		else if (c < 0.7)
+			ACEAI_Use_Haste(self);
+		else
+			ACEAI_Use_Sproing(self);
+	}
+
 	//mutators
 	if(instagib->value || rocket_arena->value)
 		return;
@@ -544,8 +636,7 @@ void ACEAI_ChooseWeapon(edict_t *self)
 	//what is the bot's favorite weapon? The bot will always check for it's favorite
 	//weapon first, which is set in the bot's config file. 
 	
-	if(!strcmp(self->faveweap, "Alien Vaporizer"))
-		
+	if(!strcmp(self->faveweap, "Alien Vaporizer") && self->skill > 1)
 	{
 		if(ACEIT_ChangeWeapon(self,FindItem(self->faveweap)))
 		{
@@ -553,6 +644,14 @@ void ACEAI_ChooseWeapon(edict_t *self)
 			return;
 		}
 	}
+	if(!strcmp(self->faveweap, "Alien Disruptor"))
+	{
+		if(ACEIT_ChangeWeapon(self,FindItem(self->faveweap)))
+		{
+			self->accuracy = self->weapacc[2];
+			return;
+		}
+	}
 	if(!strcmp(self->faveweap, "Disruptor")) 
 	{
 		if(ACEIT_ChangeWeapon(self,FindItem(self->faveweap)))
@@ -569,14 +668,6 @@ void ACEAI_ChooseWeapon(edict_t *self)
 			return;
 		}
 	}
-	if(!strcmp(self->faveweap, "Alien Disruptor"))
-	{
-		if(ACEIT_ChangeWeapon(self,FindItem(self->faveweap)))
-		{
-			self->accuracy = self->weapacc[2];
-			return;
-		}
-	}
 	if(!strcmp(self->faveweap, "Alien Smartgun"))
 	{
 		if(ACEAI_CheckShot(self) && ACEIT_ChangeWeapon(self, FindItem("Alien Smartgun")))
@@ -606,7 +697,7 @@ void ACEAI_ChooseWeapon(edict_t *self)
 				}
 		}
 	}
-	if(!strcmp(self->faveweap, "Violator"))
+	if(!strcmp(self->faveweap, "Violator") && self->skill > 0)
 	{
 		if(range < 300) { //because it's a fav weap, we want them to really try and use it
 				if(ACEIT_ChangeWeapon(self,FindItem("Violator")))
@@ -618,7 +709,7 @@ void ACEAI_ChooseWeapon(edict_t *self)
 	}
 	//now go through normal weapon favoring routine
 	// always favor the Vaporizor, unless close, then use the violator
-	if(range < 200) {
+	if(range < 200 && self->skill > 0) {
 		if(ACEIT_ChangeWeapon(self,FindItem("Violator")))
 		{
 			self->accuracy = 1.0;
@@ -626,10 +717,12 @@ void ACEAI_ChooseWeapon(edict_t *self)
 		}
 	}
 
-	if(ACEIT_ChangeWeapon(self,FindItem("Alien Vaporizer")))
-	{
-		self->accuracy = self->weapacc[9];
-		return;
+	if(self->skill > 1) {
+		if(ACEIT_ChangeWeapon(self,FindItem("Alien Vaporizer")))
+		{
+			self->accuracy = self->weapacc[9];
+			return;
+		}
 	}
 	
 	if(ACEAI_CheckShot(self) && ACEIT_ChangeWeapon(self, FindItem("Alien Smartgun")))
@@ -675,7 +768,7 @@ void ACEAI_ChooseWeapon(edict_t *self)
 		self->accuracy = self->weapacc[2];
 		return;
 	}	
-
+	
 	if(ACEIT_ChangeWeapon(self,FindItem("Blaster")))
    	{
 		self->accuracy = self->weapacc[1];
