@@ -119,12 +119,16 @@ void ACESP_LoadBots(edict_t *ent, int playerleft)
 	int i, j, count, spawnkicknum;
 	char *info;
 	char *skin;
-	char bot_filename[128];
+	char bot_filename[MAX_OSPATH];
 	int found;
 	int real_players, total_players;
 	edict_t *cl_ent;
+	
+	if (((int)(dmflags->value) & DF_BOTS)) {
+		return; // don't load any bots.
+	}
 
-//bots and configurations will be loaded level specific
+	//bots and configurations will be loaded level specific
 	if (((int)(dmflags->value) & DF_SKINTEAMS) || ctf->value || tca->value || cp->value)
 		strcpy(bot_filename, BOTDIR"/botinfo/team.tmp");
 	else if(sv_custombots->value)
@@ -137,11 +141,6 @@ void ACESP_LoadBots(edict_t *ent, int playerleft)
 
 	fread(&count,sizeof (int),1,pIn); 
 
-	if (((int)(dmflags->value) & DF_BOTS)) {
-		fclose(pIn);
-		return; // don't load any preconfigured bots.
-	}
-
 	if(g_duel->value) {
 		count = 1; //never more than 1 bot no matter what in duel mode
 		spawnkicknum = 2;
@@ -168,16 +167,15 @@ void ACESP_LoadBots(edict_t *ent, int playerleft)
 					real_players++;
 			}
 		}
+		if(count > spawnkicknum) //no need to load more bots than this number
+			count = spawnkicknum;
 	}
-	real_players -= playerleft; //done for when this is called as a player is disconnecting
-	
-	if(real_players >= game.maxclients)
-		return; //we have too many slots occupied, so lets not try loading any bots.
 
+	real_players -= playerleft; //done for when this is called as a player is disconnecting
 	total_players = 0;
+
 	for(i=0;i<count;i++)
 	{
-		
 		total_players = real_players + i + 1;
 
 		fread(userinfo,sizeof(char) * MAX_INFO_STRING,1,pIn); 
@@ -207,13 +205,13 @@ void ACESP_LoadBots(edict_t *ent, int playerleft)
 		//when clients enter the game
 		found = false;
 		found = ACESP_FindBot(info);
-
+		
 		if(!found && ((total_players <= spawnkicknum) || !spawnkicknum)) { 
 
 			if (((int)(dmflags->value) & DF_SKINTEAMS) || ctf->value || tca->value || cp->value) 
-				ACESP_SpawnBot(NULL, info, skin, NULL); //we may be changing the info further on
+				ACESP_SpawnBot(info, skin, NULL); //we may be changing the info further on
 			else 
-				ACESP_SpawnBot (NULL, NULL, NULL, userinfo);			
+				ACESP_SpawnBot (NULL, NULL, userinfo);			
 		}
 		else if(found && ((total_players > spawnkicknum) && spawnkicknum)) 
 			ACESP_KickBot(info);
@@ -224,6 +222,29 @@ void ACESP_LoadBots(edict_t *ent, int playerleft)
 
 }
 
+int ACESP_FindBotNum(void)
+{
+    FILE *pIn;
+	int count;
+	char bot_filename[MAX_OSPATH];
+
+	//bots and configurations are loaded level specific
+	if (((int)(dmflags->value) & DF_SKINTEAMS) || ctf->value || tca->value || cp->value)
+		strcpy(bot_filename, BOTDIR"/botinfo/team.tmp");
+	else if(sv_custombots->value)
+		sprintf(bot_filename, BOTDIR"/botinfo/custom%i.tmp", sv_custombots->integer);
+	else
+		sprintf(bot_filename, BOTDIR"/botinfo/%s.tmp", level.mapname);
+
+	if((pIn = fopen(bot_filename, "rb" )) == NULL)
+		return 0; // bail
+
+	fread(&count,sizeof (int),1,pIn); 
+
+	fclose(pIn);
+	
+	return count; 
+}
 ///////////////////////////////////////////////////////////////////////
 // Called by PutClient in Server to actually release the bot into the game
 // Keep from killin' each other when all spawned at once
@@ -282,16 +303,13 @@ void ACECO_ReadConfig(char config_file[128]) //use standard c routines for linux
 	}
 	else
 	{
-
-#ifdef _WIN32
-		length = filelength( fileno( fp  ) );
-#else
 		fseek(fp, 0, SEEK_END);
 		length = ftell(fp);
 		fseek(fp, 0, SEEK_SET);
-#endif
-		buffer = malloc( length );
+
+		buffer = malloc( length + 1);
 		fread( buffer, length, 1, fp );
+		buffer[length] = 0;
 	}
 	s = buffer;
 	
@@ -666,7 +684,7 @@ edict_t *ACESP_FindFreeClient (void)
 ///////////////////////////////////////////////////////////////////////
 // Set the name of the bot and update the userinfo
 ///////////////////////////////////////////////////////////////////////
-void ACESP_SetName(edict_t *bot, char *name, char *skin, char *team)
+void ACESP_SetName(edict_t *bot, char *name, char *skin)
 {
 	float rnd;
 	char userinfo[MAX_INFO_STRING];
@@ -675,13 +693,21 @@ void ACESP_SetName(edict_t *bot, char *name, char *skin, char *team)
 	char playerskin[MAX_INFO_STRING];
 	char playermodel[MAX_INFO_STRING];
 	int i, j, k, copychar;
+	char *skin2;
 
 	// Set the name for the bot.
 	// name
 	if(strlen(name) == 0)
+	{
 		sprintf(bot_name,"ACEBot_%d",bot->count);
+		sprintf(bot_skin,"martianenforcer/default");
+		skin2 = bot_skin;
+	}
 	else
+	{
 		strcpy(bot_name,name);
+		skin2 = skin;
+	}
 
 	bot->dmteam = NO_TEAM; //default
 
@@ -693,59 +719,42 @@ void ACESP_SetName(edict_t *bot, char *name, char *skin, char *team)
 		strcpy(playerskin, " ");
 		strcpy(playermodel, " ");
 		j = k = 0;
-		for(i = 0; i <= strlen(skin) && i < MAX_INFO_STRING; i++)
+		for(i = 0; i <= strlen(skin2) && i < MAX_INFO_STRING; i++)
 		{
 			if(copychar){ 
-				playerskin[k] = skin[i];
+				playerskin[k] = skin2[i];
 				k++;
 			}
 			else {
-				playermodel[j] = skin[i];
+				playermodel[j] = skin2[i];
 				j++;
 			}
-			if(skin[i] == '/')
+			if(skin2[i] == '/')
 				copychar = true;
 				
 			
 		}
 		playermodel[j] = 0;
 		
-		if((!strcmp(playerskin, "red"))	|| (!strcmp(playerskin, "blue"))) //was valid teamskin
+		if(blue_team_cnt < red_team_cnt)
 		{
-			if(!strcmp(playerskin, "red"))
-			{
-				bot->dmteam = RED_TEAM;
-	//			red_team_cnt++;
-			}
-			else
-			{
-				bot->dmteam = BLUE_TEAM;
-	//			blue_team_cnt++;
-			}
-
+			strcpy(playerskin, "blue");
+			//blue_team_cnt++;
+			bot->dmteam = BLUE_TEAM;
 		}
-		else //assign to team with fewest players
+		else
 		{
-			if(blue_team_cnt < red_team_cnt)
-			{
-				strcpy(playerskin, "blue");
-	//			blue_team_cnt++;
-				bot->dmteam = BLUE_TEAM;
-			}
-			else
-			{
-				strcpy(playerskin, "red");
-	//			red_team_cnt++;
-				bot->dmteam = RED_TEAM;
-			}
+			strcpy(playerskin, "red");
+			//red_team_cnt++;
+			bot->dmteam = RED_TEAM;
 		}
 	
-		strcpy(skin, playermodel);
-		strcat(skin, playerskin);
+		strcpy(skin2, playermodel);
+		strcat(skin2, playerskin);
 
 	}
 		
-	if(strlen(skin) == 0)
+	if(strlen(skin2) == 0)
 	{
 		// randomly choose skin 
 		rnd = random();
@@ -756,7 +765,7 @@ void ACESP_SetName(edict_t *bot, char *name, char *skin, char *team)
 			sprintf(bot_skin,"martianenforcer/blue");
 	}
 	else 
-		strcpy(bot_skin,skin);
+		strcpy(bot_skin,skin2);
 
 	// initialise userinfo
 	memset (userinfo, 0, sizeof(userinfo));
@@ -774,11 +783,9 @@ void ACESP_SetName(edict_t *bot, char *name, char *skin, char *team)
 ///////////////////////////////////////////////////////////////////////
 // Spawn the bot
 ///////////////////////////////////////////////////////////////////////
-void ACESP_SpawnBot (char *team, char *name, char *skin, char *userinfo)
+void ACESP_SpawnBot (char *name, char *skin, char *userinfo)
 {
 	edict_t *bot, *cl_ent;
-	char *info;
-	char sound[64];
 	int	i;
 
 	bot = ACESP_FindFreeClient ();
@@ -795,9 +802,10 @@ void ACESP_SpawnBot (char *team, char *name, char *skin, char *userinfo)
 
 	// To allow bots to respawn
 	if(userinfo == NULL)
-		ACESP_SetName(bot, name, skin, team);
+		ACESP_SetName(bot, name, skin);
 	else 
 	{
+		bot->dmteam = NO_TEAM; //default
 		if(!ClientConnect (bot, userinfo))
 		{	
 			/* Tony: Sometimes bots are refused entry to servers - give up gracefully */
@@ -820,11 +828,6 @@ void ACESP_SpawnBot (char *team, char *name, char *skin, char *userinfo)
 			game.num_bots++;
 	}
 
-	//play sound announcing arrival of bot
-	info = Info_ValueForKey (bot->client->pers.userinfo, "name");
-	sprintf(sound, "bots/%s.wav", info);
-	gi.sound (bot, CHAN_AUTO, gi.soundindex(sound), 1, ATTN_NONE, 0);
-
 	ACESP_PutClientInServer (bot,false,0);
 
 	if(g_duel->value) {
@@ -857,6 +860,17 @@ void ACESP_RemoveBot(char *name)
 			{
 				bot->health = 0;
 				player_die (bot, bot, bot, 100000, vec3_origin);
+				if(ctf->value)
+					CTFDeadDropFlag(bot);
+
+				DeadDropDeathball(bot);	
+				if (((int)(dmflags->value) & DF_SKINTEAMS) || ctf->value || tca->value || cp->value)  //adjust teams and scores
+				{
+					if(bot->dmteam == BLUE_TEAM)
+						blue_team_cnt-=1;
+					else
+						red_team_cnt-=1;
+				}
 				// don't even bother waiting for death frames
 				bot->deadflag = DEAD_DEAD;
 				bot->inuse = false;
