@@ -132,6 +132,19 @@ void ValidateSelectedItem (edict_t *ent)
 	SelectNextItem (ent, -1);
 }
 
+void DrawChatBubble (edict_t *ent)
+{
+
+	if(!ent->client)
+		return;
+	if(ent->client->resp.spectator)
+		return;
+
+	gi.WriteByte (svc_temp_entity);
+	gi.WriteByte (TE_SAYICON);
+	gi.WritePosition (ent->s.origin);
+	gi.multicast (ent->s.origin, MULTICAST_PVS);
+}
 
 //=================================================================================
 
@@ -902,6 +915,203 @@ void Cmd_PlayerList_f(edict_t *ent)
 	safe_cprintf(ent, PRINT_HIGH, "%s", text);
 }
 
+/*
+=================
+Cmd_CallVote_f
+=================
+*/
+void Cmd_CallVote_f (edict_t *ent)
+{
+
+	if(level.time <= warmuptime->value) {
+		safe_bprintf(PRINT_HIGH, "Cannot call a vote during warmup!\n");
+		return;
+	}
+
+	if(playervote.called) {
+		safe_bprintf(PRINT_HIGH, "Vote already in progress, please wait.\n");
+		return;
+	}
+
+	//start a vote
+	playervote.called = true; 
+	playervote.nay = playervote.nay = 0;
+	playervote.starttime = level.time;
+	if(strlen(gi.args()) < 128) {
+		strcpy(playervote.command, gi.args()); 
+		safe_bprintf(PRINT_HIGH, "%s called a vote: %s\n", ent->client->pers.netname, playervote.command);
+	}
+}
+
+/*
+=================
+Cmd_Vote_f
+=================
+*/
+void Cmd_Vote_f (edict_t *ent)
+{
+
+	int	i, j, mostvotes, winner;
+	int	n_candidates;
+	int	candidates[4];
+	char	buffer[512];
+	char	buffer2[60];
+	edict_t *cl_ent;
+
+	i = atoi (gi.argv(1));
+
+	if(g_callvote->value && playervote.called) {
+
+		switch(i) { //to do - move "voted" to persistant data
+			case 1:
+				if(!ent->client->resp.voted) {
+					ent->client->resp.voted = true;
+					playervote.yay++;
+					safe_bprintf(PRINT_HIGH, "%s voted ^2YES\n", ent->client->pers.netname);
+				}
+				break;
+			case 2:
+				if(!ent->client->resp.voted) {
+					ent->client->resp.voted = true;
+					playervote.nay++;
+					safe_bprintf(PRINT_HIGH, "%s voted ^1NO\n", ent->client->pers.netname);
+				}
+				break;
+		}
+	}
+
+	if (!level.intermissiontime || ! (g_mapvote && g_mapvote->value && !g_mapvote->modified) )
+		return;
+
+	ent->client->mapvote = i;
+	safe_bprintf(PRINT_HIGH, "%s voted for map %i\n", ent->client->pers.netname, i);
+
+	//update scoreboard
+	mostvotes = 0;
+	winner = 1; //next map in line
+	for(i = 0; i < 4; i++)
+		votedmap[i].tally = 0;
+	for (i=0 ; i<maxclients->value ; i++)
+	{
+		cl_ent = g_edicts + 1 + i;
+		if (!cl_ent->inuse || cl_ent->is_bot)
+			continue;
+		for(j = 0; j < 4; j++) {
+			if(cl_ent->client->mapvote-1 == j)
+				votedmap[j].tally++;
+			if(votedmap[j].tally > mostvotes){
+				mostvotes = votedmap[j].tally;
+			}
+		}
+	}
+
+	if ( g_voterand && g_voterand->value )
+	{
+		// random tie resolution
+		n_candidates = 0;
+		for (j = 0; j < 4; j ++) {
+			if ( votedmap[j].tally < mostvotes )
+				continue;
+			candidates[n_candidates ++] = j;
+		}
+
+		if ( n_candidates == 1 )
+		{
+			winner = candidates[0];
+			sprintf( buffer, "Map %s leads with %i vote%s!", votedmap[winner].mapname, votedmap[winner].tally, (mostvotes > 1) ? "s" : ""  );
+		}
+		else
+		{
+			strcpy( buffer, "It's a tie!\nMaps ");
+			for ( i = 0 ; i < n_candidates ; i ++ ) {
+				j = candidates[i];
+				if ( i > 0 )
+				{
+					if ( i == n_candidates - 1 )
+						strcat( buffer, " and " );
+					else
+						strcat( buffer, ", " );
+				}
+				strcat( buffer, votedmap[j].mapname );
+			}
+			sprintf( buffer2, "\nlead with %i vote%s!" , mostvotes , (mostvotes > 1) ? "s" : "" );
+			strcat( buffer, buffer2 );
+		}
+	}
+	else
+	{
+		// "old" voting system, leading map is the first one with enough votes
+		for (j = 0; j < 4; j ++) {
+			i = (j + 1) % 4;
+			if ( votedmap[i].tally < mostvotes )
+				continue;
+			winner = i;
+			break;
+		}
+		sprintf( buffer, "Map %s leads with %i vote%s!", votedmap[winner].mapname, votedmap[winner].tally, (mostvotes > 1) ? "s" : "" );
+	}
+
+	for (i=0 ; i<maxclients->value ; i++)
+	{
+		cl_ent = g_edicts + 1 + i;
+		if (!cl_ent->inuse || cl_ent->is_bot)
+			continue;
+		safe_centerprintf(cl_ent, "%s", buffer );
+	}
+}
+
+void Cmd_VoiceTaunt_f (edict_t *ent)
+{
+	int			index, i;
+	qboolean	done;
+	char		name[32];
+	char		string[256]; //a small string to send
+	char		playermodel[MAX_OSPATH], tauntsound[MAX_OSPATH];
+	char		*info;
+
+	index = atoi (gi.argv(1));
+
+	if(index > 5 || index < 1 || ent->is_bot) {
+		index = (int)(1 + random() * 5);
+		if(index > 5)
+			index = 5;
+	}
+
+	//get info about this client
+	if(ent->inuse && ent->client) {
+
+		if((level.time - ent->client->lasttaunttime) > 2) { //prevent flooding
+
+			ent->client->lasttaunttime = level.time;
+
+			strcpy(name, ent->client->pers.netname);
+			info = Info_ValueForKey (ent->client->pers.userinfo, "skin");
+			info[96] = 0; //truncate to prevent bad people from harming the server
+			
+			i = 0;
+			done = false;
+			while(!done)
+			{
+				if((info[i] == '/') || (info[i] == '\\'))
+					done = true;
+				playermodel[i] = info[i];
+				if(i > 62)
+					done = true;
+				i++;
+			}
+			playermodel[i-1] = 0;
+
+			sprintf(tauntsound, "taunts/%s/taunt%i.wav", playermodel, index);
+
+			Com_sprintf(string, sizeof(string), 
+				"%s %s %s ", info, tauntsound, name);
+
+			//send to all clients as a general config string
+			gi.configstring (CS_GENERAL, string);
+		}
+	}
+}
+
 /*
 =================
 ClientCommand
@@ -946,6 +1156,21 @@ void ClientCommand (edict_t *ent)
 		Cmd_Help_f (ent);
 		return;
 	}
+	if (Q_stricmp (cmd, "callvote") == 0)
+	{
+		Cmd_CallVote_f(ent);
+		return;
+	}
+	if (Q_stricmp (cmd, "vote") == 0)
+	{
+		Cmd_Vote_f(ent);
+		return;
+	}
+	if	(Q_stricmp (cmd, "vtaunt") == 0)
+	{
+		Cmd_VoiceTaunt_f(ent);
+		return;
+	}
 
 	if (level.intermissiontime)
 		return;
@@ -994,8 +1219,8 @@ void ClientCommand (edict_t *ent)
 		Cmd_Wave_f (ent);
 	else if (Q_stricmp(cmd, "playerlist") == 0)
 		Cmd_PlayerList_f(ent);
-	else if (Q_stricmp (cmd, "flashlight") == 0)
-		FL_make (ent);
+	else if (Q_stricmp (cmd, "chatbubble") == 0)
+		DrawChatBubble(ent);
 	else	// anything that doesn't match a command will be a chat
 		Cmd_Say_f (ent, false, true);
 
