@@ -228,6 +228,20 @@ qboolean CheckTeamDamage (edict_t *targ, edict_t *attacker)
 	return false;
 }
 
+void VerifyHeadShot( vec3_t point, vec3_t dir, float height, vec3_t newpoint)
+{
+        vec3_t normdir;
+        vec3_t normdir2;
+        
+        
+        VectorNormalize2(dir, normdir);
+        VectorScale( normdir, height, normdir2 );
+        VectorAdd( point, normdir2, newpoint );
+}
+
+#define HEAD_HEIGHT 12.0
+#define CROUCHING_MAXS2 16
+
 void T_Damage (edict_t *targ, edict_t *inflictor, edict_t *attacker, vec3_t dir, vec3_t point, vec3_t normal, int damage, int knockback, int dflags, int mod)
 {
 	gclient_t	*client;
@@ -235,6 +249,11 @@ void T_Damage (edict_t *targ, edict_t *inflictor, edict_t *attacker, vec3_t dir,
 	int			save;
 	int			asave;
 	int			te_sparks;
+	int         head_success = 0;
+    float       z_rel; 
+    int         height; 
+    float		from_top;
+    float       targ_maxs2;
 
 	if (!targ->takedamage)
 		return;
@@ -243,6 +262,42 @@ void T_Damage (edict_t *targ, edict_t *inflictor, edict_t *attacker, vec3_t dir,
 		if(targ->inuse && targ->client)
 			if(targ->client->spawnprotected)
 				return;
+
+	//headshots for disruptors
+	targ_maxs2 = targ->maxs[2];
+    if (targ_maxs2 == 4)
+		targ_maxs2 = CROUCHING_MAXS2; 
+        
+    height = abs(targ->mins[2]) + targ_maxs2; 
+        
+    if (targ->client && mod == MOD_DISRUPTOR) {
+		
+		z_rel = point[2] - targ->s.origin[2]; 
+        from_top = targ_maxs2 - z_rel;
+        
+		if (from_top < 0.0)     
+			from_top = 0.0;  
+
+        if ( from_top < 2*HEAD_HEIGHT ) {
+			vec3_t new_point;
+            VerifyHeadShot( point, dir, HEAD_HEIGHT, new_point );
+            VectorSubtract( new_point, targ->s.origin, new_point );
+                                        
+            if ( (targ_maxs2 - new_point[2]) < HEAD_HEIGHT 
+				&& (abs(new_point[1])) < HEAD_HEIGHT*.8 
+                && (abs(new_point[0])) < HEAD_HEIGHT*.8 ) {
+                                           
+				head_success = 1;
+            }
+        }
+                                
+        if ( head_success ) {
+                                        
+			damage = damage*1.8 + 1;
+            if (attacker->client)
+				mod = MOD_HEADSHOT;
+        }                           
+    }                
 			
 	// friendly fire avoidance
 	// if enabled you can't hurt teammates (but you can hurt yourself)
@@ -358,7 +413,12 @@ void T_Damage (edict_t *targ, edict_t *inflictor, edict_t *attacker, vec3_t dir,
 				SpawnDamage (TE_BLOOD, point, normal, take);
 		}
 		else {
-			SpawnDamage (te_sparks, point, normal, take);
+			if (targ->ctype == 0) //alien, robot, human
+				SpawnDamage (TE_GREENBLOOD, point, normal, take);
+			else if (targ->ctype == 2)
+				SpawnDamage (TE_GUNSHOT, point, normal, take);
+			else
+				SpawnDamage (TE_BLOOD, point, normal, take);
 			if(tca->value) {
 				if(!(strcmp(targ->classname, "misc_redspidernode")) || !(strcmp(targ->classname, "misc_bluespidernode")))
 					safe_centerprintf(attacker, "Spider health at %i percent", 100*targ->health/600);
@@ -431,6 +491,8 @@ void T_RadiusDamage (edict_t *inflictor, edict_t *attacker, float damage, edict_
 			{
 				VectorSubtract (ent->s.origin, inflictor->s.origin, dir);
 				T_Damage (ent, inflictor, attacker, dir, inflictor->s.origin, vec3_origin, (int)points, (int)points, DAMAGE_RADIUS, mod);
+				if (ent != attacker)
+					gi.sound (attacker, CHAN_VOICE, gi.soundindex("misc/hit.wav"), 1, ATTN_STATIC, 0);
 				if(weapon >=0)
 					attacker->client->resp.weapon_hits[weapon]++;
 			}
