@@ -27,20 +27,19 @@ int imageindex_sbfctf2;
 CTFScoreboardMessage
 ==================
 */
-void CTFScoreboardMessage (edict_t *ent, edict_t *killer)
+void CTFScoreboardMessage (edict_t *ent, edict_t *killer, int mapvote)
 {
 	char	entry[1024];
 	char	string[1400];
 	int		len;
-	int		i, j, k, n;
+	int		i, j, k, x, y;
 	int		sorted[2][MAX_CLIENTS];
 	int		sortedscores[2][MAX_CLIENTS];
 	int		score, total[2], totalscore[2];
-	int		last[2];
 	gclient_t	*cl;
 	edict_t		*cl_ent;
 	int team;
-	int maxsize = 1000;
+	int maxsize = 1024;
 	gitem_t *flag1_item, *flag2_item;
 
 	flag1_item = FindItemByClassname("item_flag_red");
@@ -48,7 +47,6 @@ void CTFScoreboardMessage (edict_t *ent, edict_t *killer)
 
 	// sort the clients by team and score
 	total[0] = total[1] = 0;
-	last[0] = last[1] = 0;
 	totalscore[0] = totalscore[1] = 0;
 	for (i=0 ; i<game.maxclients ; i++)
 	{
@@ -84,55 +82,49 @@ void CTFScoreboardMessage (edict_t *ent, edict_t *killer)
 	*string = 0;
 	len = 0;
 
-	// team one
-	sprintf(string, "xv -8 yv -8 picn i_team1 "
-		"xv 40 yv 28 string \"%4d/%-3d\" "
-		"xv 58 yv 12 num 3 21 "
-		"xv 184 yv -8 picn i_team2 "
-		"xv 230 yv 28 string \"%4d/%-3d\" "
-		"xv 248 yv 12 num 3 22 ",
-		totalscore[0], total[0],
-		totalscore[1], total[1]);
-	len = strlen(string);
-
+	if(ctf->value) {
+		//do ctf
+		sprintf(string, "newctfsb xv -16 yv -8 picn ctf1 "
+			"xv +12 yv 4 num 3 21 "
+			"xv 238 yv -8 picn ctf2 "
+			"xv 264 yv 4 num 3 22 ");
+		len = strlen(string);
+	}
+	else {
+		//regular team games
+		sprintf(string, "newctfsb xv -16 yv -8 picn team1 "
+			"xv +12 yv 4 num 3 21 "
+			"xv 238 yv -8 picn team2 "
+			"xv 264 yv 4 num 3 22 ");
+		len = strlen(string);
+	}
+	
 	for (i=0 ; i<16 ; i++)
 	{
 		if (i >= total[0] && i >= total[1])
 			break; // we're done
-
-#if 0 //ndef NEW_SCORE
-		// set up y
-		sprintf(entry, "yv %d ", 42 + i * 8);
-		if (maxsize - len > strlen(entry)) {
-			strcat(string, entry);
-			len = strlen(string);
-		}
-#else
+		
 		*entry = 0;
-#endif
 
 		// left side
 		if (i < total[0]) {
 			cl = &game.clients[sorted[0][i]];
 			cl_ent = g_edicts + 1 + sorted[0][i];
 
-
 			sprintf(entry+strlen(entry),
-				"ctf 0 %d %d %d %d ",
-				42 + i * 8,
+				"ctf -96 %d %d %d %d ",
+				42 + i * 16,
 				sorted[0][i],
 				cl->resp.score,
 				cl->ping > 999 ? 999 : cl->ping);
 
 			if (cl_ent->client->pers.inventory[ITEM_INDEX(flag2_item)])
-				sprintf(entry + strlen(entry), "xv 56 yv %d picn sbfctf2 ",
-					42 + i * 8);
-
-
+				sprintf(entry + strlen(entry), "xv -92 yv %d picn sbfctf2 ",
+					43 + i * 16);
+			
 			if (maxsize - len > strlen(entry)) {
 				strcat(string, entry);
 				len = strlen(string);
-				last[0] = i;
 			}
 		}
 
@@ -141,75 +133,66 @@ void CTFScoreboardMessage (edict_t *ent, edict_t *killer)
 			cl = &game.clients[sorted[1][i]];
 			cl_ent = g_edicts + 1 + sorted[1][i];
 
-
 			sprintf(entry+strlen(entry),
-				"ctf 190 %d %d %d %d ",
-				42 + i * 8,
+				"ctf 160 %d %d %d %d ",
+				42 + i * 16,
 				sorted[1][i],
 				cl->resp.score,
 				cl->ping > 999 ? 999 : cl->ping);
 
 			if (cl_ent->client->pers.inventory[ITEM_INDEX(flag1_item)])
-				sprintf(entry + strlen(entry), "xv 246 yv %d picn sbfctf1 ",
-					42 + i * 8);
+				sprintf(entry + strlen(entry), "xv 164 yv %d picn sbfctf1 ",
+					43 + i * 16);
 
 			if (maxsize - len > strlen(entry)) {
 				strcat(string, entry);
 				len = strlen(string);
-				last[1] = i;
 			}
 		}
 	}
 
-	// put in spectators if we have enough room
-	if (last[0] > last[1])
-		j = last[0];
-	else
-		j = last[1];
-	j = (j + 2) * 8 + 42;
-
-	k = n = 0;
-	if (maxsize - len > 50) {
-		for (i = 0; i < maxclients->value; i++) {
-			cl_ent = g_edicts + 1 + i;
-			cl = &game.clients[i];
-			if (!cl_ent->inuse ||
-				cl_ent->solid != SOLID_NOT ||
-				cl_ent->client->dmteam != NO_TEAM)
-				continue;
-
-			if (!k) {
-				k = 1;
-				sprintf(entry, "xv 0 yv %d string2 \"Spectators\" ", j);
-				strcat(string, entry);
-				len = strlen(string);
-				j += 8;
-			}
-
-			sprintf(entry+strlen(entry),
-				"ctf %d %d %d %d %d ",
-				(n & 1) ? 160 : 0, // x
-				j, // y
-				i, // playernum
-				cl->resp.score,
-				cl->ping > 999 ? 999 : cl->ping);
-			if (maxsize - len > strlen(entry)) {
-				strcat(string, entry);
-				len = strlen(string);
-			}
+	if(mapvote) {
+		y = 64;
+		x = 96;
+		Com_sprintf(entry, sizeof(entry), 
+			"xv %i yt %i string Vote ", x, y);
+		if(maxsize - len > strlen(entry)) {
+			strcat(string, entry);
+			len = strlen(string);
+		}
+		x = 136;
+		Com_sprintf(entry, sizeof(entry), 
+			"xv %i yt %i string for ", x, y);
+		if(maxsize - len > strlen(entry)) {
+			strcat(string, entry);
+			len = strlen(string);
+		}
+		x = 168;
+		Com_sprintf(entry, sizeof(entry), 
+			"xv %i yt %i string next ", x, y);
+		if(maxsize - len > strlen(entry)) {
+			strcat(string, entry);
+			len = strlen(string);
+		}
+		x = 208;
+		Com_sprintf(entry, sizeof(entry), 
+			"xv %i yt %i string map: ", x, y);
+		if(maxsize - len > strlen(entry)) {
+			strcat(string, entry);
+			len = strlen(string);
+		}
+		x = 96;
+		for(i=0; i<4; i++) {
 			
-			if (n & 1)
-				j += 8;
-			n++;
+			Com_sprintf(entry, sizeof(entry), 
+			"xv %i yt %i string %i.%s ", x, y+((i+1)*9)+9, i+1, votedmap[i].mapname);
+			if(maxsize - len > strlen(entry)) {
+				strcat(string, entry);
+				len = strlen(string);
+			}
 		}
-	}
 
-	if (total[0] - last[0] > 1) // couldn't fit everyone
-		sprintf(string + strlen(string), "xv 8 yv %d string \"..and %d more\" ",
-			42 + (last[0]+1)*8, total[0] - last[0] - 1);
-	if (total[1] - last[1] > 1) // couldn't fit everyone
-		sprintf(string + strlen(string), "xv 168 yv %d string \"..and %d more\" ",
-			42 + (last[1]+1)*8, total[1] - last[1] - 1);
+	}
 
 	gi.WriteByte (svc_layout);
 	gi.WriteString (string);
@@ -685,7 +668,7 @@ qboolean CTFPickup_Flag (edict_t *ent, edict_t *other)
 	int ctf_team;
 	char team_name[16] = " ";
 	char enemy_team_name[16] = " ";
-	gitem_t *flag_item, *enemy_flag_item;
+	gitem_t *flag_item, *enemy_flag_item, *it;
 
 	// figure out what team this flag is
 	if (strcmp(ent->classname, "item_flag_red") == 0)
@@ -734,6 +717,23 @@ qboolean CTFPickup_Flag (edict_t *ent, edict_t *other)
 
 				// other gets another 10 frag bonus
 				other->client->resp.score += 10;//CTF_CAPTURE_BONUS;
+				//reward points bonus
+				other->client->resp.reward_pts+=5;
+				
+				if(other->client->resp.reward_pts >= g_reward->integer && !other->client->resp.powered) { //give them speed and invis powerups
+					it = FindItem("Invisibility");
+					other->client->pers.inventory[ITEM_INDEX(it)] += 1;
+
+					it = FindItem("Sproing");
+					other->client->pers.inventory[ITEM_INDEX(it)] += 1;
+
+					it = FindItem("Haste");
+					other->client->pers.inventory[ITEM_INDEX(it)] += 1;
+
+					other->client->resp.powered = true;
+
+					gi.sound (other, CHAN_AUTO, gi.soundindex("misc/pc_up.wav"), 1, ATTN_STATIC, 0);
+				}			
 				
 				CTFResetFlags();
 				return false;
@@ -748,6 +748,24 @@ qboolean CTFPickup_Flag (edict_t *ent, edict_t *other)
 			gi.sound (ent, CHAN_AUTO, gi.soundindex("misc/red_returned.wav"), 1, ATTN_NONE, 0);
 		else
 			gi.sound (ent, CHAN_AUTO, gi.soundindex("misc/blue_returned.wav"), 1, ATTN_NONE, 0);
+
+		//reward points bonus
+		other->client->resp.reward_pts+=2;
+				
+		if(other->client->resp.reward_pts >= g_reward->integer && !other->client->resp.powered) { //give them speed and invis powerups
+			it = FindItem("Invisibility");
+			other->client->pers.inventory[ITEM_INDEX(it)] += 1;
+
+			it = FindItem("Sproing");
+			other->client->pers.inventory[ITEM_INDEX(it)] += 1;
+
+			it = FindItem("Haste");
+			other->client->pers.inventory[ITEM_INDEX(it)] += 1;
+
+			other->client->resp.powered = true;
+
+			gi.sound (other, CHAN_AUTO, gi.soundindex("misc/pc_up.wav"), 1, ATTN_STATIC, 0);
+		}	
 		
 		//CTFResetFlag will remove this entity!  We must return false
 		CTFResetFlag(ctf_team);
@@ -763,6 +781,23 @@ qboolean CTFPickup_Flag (edict_t *ent, edict_t *other)
 	else
 		gi.sound (ent, CHAN_AUTO, gi.soundindex("misc/blue_picked.wav"), 1, ATTN_NONE, 0);
 	
+	other->client->resp.reward_pts+=2;
+				
+	if(other->client->resp.reward_pts >= g_reward->integer && !other->client->resp.powered) { //give them speed and invis powerups
+		it = FindItem("Invisibility");
+		other->client->pers.inventory[ITEM_INDEX(it)] += 1;
+
+		it = FindItem("Sproing");
+		other->client->pers.inventory[ITEM_INDEX(it)] += 1;
+
+		it = FindItem("Haste");
+		other->client->pers.inventory[ITEM_INDEX(it)] += 1;
+
+		other->client->resp.powered = true;
+
+		gi.sound (other, CHAN_AUTO, gi.soundindex("misc/pc_up.wav"), 1, ATTN_STATIC, 0);
+	}	
+	
 	other->client->pers.inventory[ITEM_INDEX(flag_item)] = 1;
 
 	// pick up the flag
