@@ -255,9 +255,22 @@ void	Use_Invulnerability (edict_t *ent, gitem_t *item)
 
 void Use_Haste (edict_t *ent, gitem_t *item)
 {
+	gitem_t *it;
+
 	ent->client->pers.inventory[ITEM_INDEX(item)]--;
 	ValidateSelectedItem (ent);
 
+	if(ent->client->resp.powered) {
+		it = FindItem("Sproing");
+		ent->client->pers.inventory[ITEM_INDEX(it)] = 0;
+
+		it = FindItem("Invisibility");
+		ent->client->pers.inventory[ITEM_INDEX(it)] = 0;
+
+		ent->client->resp.reward_pts = 0;
+		ent->client->resp.powered = false;
+	}
+
 	if (ent->client->haste_framenum > level.framenum)
 		ent->client->haste_framenum += 300;
 	else
@@ -269,9 +282,23 @@ void Use_Haste (edict_t *ent, gitem_t *item)
 
 void Use_Sproing (edict_t *ent, gitem_t *item)
 {
+	gitem_t *it;
+
 	ent->client->pers.inventory[ITEM_INDEX(item)]--;
 	ValidateSelectedItem (ent);
 
+	if(ent->client->resp.powered) {
+
+		it = FindItem("Invisibility");
+		ent->client->pers.inventory[ITEM_INDEX(it)] = 0;
+
+		it = FindItem("Haste");
+		ent->client->pers.inventory[ITEM_INDEX(it)] = 0;
+
+		ent->client->resp.reward_pts = 0;
+		ent->client->resp.powered = false;
+	}
+
 	if (ent->client->sproing_framenum > level.framenum)
 		ent->client->sproing_framenum += 300;
 	else
@@ -279,6 +306,33 @@ void Use_Sproing (edict_t *ent, gitem_t *item)
 
 	gi.sound(ent, CHAN_ITEM, gi.soundindex("items/powerup.wav"), 1, ATTN_NORM, 0);
 }
+
+void Use_Invisibility (edict_t *ent, gitem_t *item)
+{
+	gitem_t *it;
+
+	ent->client->pers.inventory[ITEM_INDEX(item)]--;
+	ValidateSelectedItem (ent);
+
+	if(ent->client->resp.powered) {
+		it = FindItem("Sproing");
+		ent->client->pers.inventory[ITEM_INDEX(it)] = 0;
+
+		it = FindItem("Haste");
+		ent->client->pers.inventory[ITEM_INDEX(it)] = 0;
+
+		ent->client->resp.reward_pts = 0;
+		ent->client->resp.powered = false;
+	}
+
+	if (ent->client->invis_framenum > level.framenum)
+		ent->client->invis_framenum += 300;
+	else
+		ent->client->invis_framenum = level.framenum + 300;
+
+	gi.sound(ent, CHAN_ITEM, gi.soundindex("items/powerup.wav"), 1, ATTN_NORM, 0);
+}
+
 //======================================================================
 
 qboolean Pickup_Key (edict_t *ent, edict_t *other)
@@ -666,8 +720,6 @@ edict_t *Drop_Item (edict_t *ent, gitem_t *item)
 	gi.setmodel (dropped, dropped->item->world_model);
 	if(!strcmp(item->classname, "item_bomber"))
 		dropped->s.modelindex3 = gi.modelindex("vehicles/bomber/helmet.md2");
-	if(!strcmp(item->classname, "item_hover"))
-		dropped->s.modelindex3 = gi.modelindex("vehicles/hover/flames.md2");
 	dropped->solid = SOLID_TRIGGER;
 	dropped->movetype = MOVETYPE_TOSS;
 	dropped->touch = drop_temp_touch;
@@ -960,10 +1012,7 @@ void SpawnItem (edict_t *ent, gitem_t *item)
 	else if((strcmp(ent->classname, "item_quad") == 0)) {
 		ent->s.modelindex2 = gi.modelindex("models/items/quaddama/unit.md2");
 	}
-	if(strcmp(ent->classname, "item_armor_shard") == 0) {
-		//ent->s.renderfx = RF_CUSTOMSKIN; //shiny
-	}
-
+	
 	if (ent->model)
 		gi.modelindex (ent->model);
 
@@ -1097,7 +1146,7 @@ gitem_t	itemlist[] =
 		CTFDrop_Flag, //Should this be null if we don't want players to drop it manually?
 		NULL,
 		NULL,
-		"models/items/flags/flag1.md2", EF_FLAG1,
+		"models/items/flags/flag1.md2", EF_TEAM1,
 		NULL,
 /* icon */		"i_flag1",
 /* pickup */	"Red Flag",
@@ -1120,7 +1169,7 @@ gitem_t	itemlist[] =
 		CTFDrop_Flag, //Should this be null if we don't want players to drop it manually?
 		NULL,
 		NULL,
-		"models/items/flags/flag2.md2", EF_FLAG2,
+		"models/items/flags/flag2.md2", EF_TEAM2,
 		NULL,
 /* icon */		"i_flag2",
 /* pickup */	"Blue Flag",
@@ -1794,6 +1843,30 @@ gives +1 to maximum health
 		0,
 /* precache */ "items/sproingout.wav"
 	},
+
+	//these next two powerups are never placed in maps
+	{
+		"item_invisibility",
+		Pickup_Powerup,
+		Use_Invisibility,
+		NULL,
+		NULL,
+		"items/powerup.wav",
+		NULL, 
+		EF_ROTATE,
+		NULL,
+		NULL,
+		"Invisibility",
+		2,
+		60,
+		NULL,
+		IT_POWERUP,
+		0,
+		NULL,
+		0,
+/* precache */ "items/protect2.wav"
+	},
+	
 	// end of list marker
 	{NULL}
 };
