@@ -36,6 +36,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define	svc_temp_entity		3
 #define	svc_layout			4
 #define	svc_inventory		5
+#define svc_nop				6
 #define	svc_stufftext		11
 
 //==================================================================
@@ -71,6 +72,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #define	FRAMETIME		0.1
 
+//unlagged - true ping
+#define NUM_PING_SAMPLES 64
+//unlagged - true ping
+
 // memory tags to allow dynamic memory to be cleaned up
 #define	TAG_GAME	765		// clear when unloading the dll
 #define	TAG_LEVEL	766		// clear when loading a new level
@@ -203,6 +208,15 @@ MOVETYPE_BOUNCE
 } movetype_t;
 
 
+typedef struct
+{
+	char	command[128]; //limit to prevent malicious buffer overflows
+	int		yay;
+	int		nay;
+	float	starttime;
+	float	time;
+	qboolean called;
+} g_vote_t;
 
 typedef struct
 {
@@ -308,6 +322,7 @@ typedef struct
 {
 	int			framenum;
 	float		time;
+	int			previousTime; //in milleseconds
 
 	char		level_name[MAX_QPATH];	// the descriptive name (Outer Base, etc)
 	char		mapname[MAX_QPATH];		// the server name (base1, etc)
@@ -343,6 +358,11 @@ typedef struct
 	edict_t		*current_entity;	// entity running from G_RunFrame
 	int			body_que;			// dead bodies
 
+	//unlagged - backward reconciliation #4
+	// actual time this server frame started
+	float		frameStartTime;
+	//unlagged - backward reconciliation #4
+
 } level_locals_t;
 
 
@@ -457,9 +477,9 @@ extern	level_locals_t	level;
 extern	game_import_t	gi;
 extern	game_export_t	globals;
 extern	spawn_temp_t	st;
+extern	g_vote_t		playervote;
 
 extern	int	sm_meat_index;
-extern	int	snd_fry;
 
 extern	int red_team_cnt;
 extern  int blue_team_cnt;
@@ -499,14 +519,14 @@ extern  int print3;
 #define MOD_SPLASH			29
 #define MOD_TARGET_LASER	30
 #define MOD_TRIGGER_HURT	31
-#define MOD_HIT				32
+#define MOD_DEATHRAY		32
 #define MOD_TARGET_BLASTER	33
 #define MOD_GRAPPLE			34
-#define MOD_FRIENDLY_FIRE	0x8000000
+#define MOD_HEADSHOT		35
+#define MOD_FRIENDLY_FIRE   0x8000000
 
 extern	int	meansOfDeath;
 
-
 extern	edict_t			*g_edicts;
 
 #define FOFS(x)(ptrdiff_t)&(((edict_t *)0)->x)
@@ -531,6 +551,7 @@ extern	cvar_t	*spectator_password;
 extern	cvar_t	*needpass;
 extern	cvar_t	*g_select_empty;
 extern	cvar_t	*dedicated;
+extern	cvar_t	*motdfile;
 
 extern	cvar_t	*filterban;
 
@@ -609,6 +630,21 @@ extern cvar_t   *g_spawnprotect;
 //jousting
 extern cvar_t   *joustmode;
 
+//map voting
+extern cvar_t	*g_mapvote;
+extern cvar_t	*g_voterand;
+extern cvar_t	*g_votemode;
+extern cvar_t	*g_votesame;
+
+//call voting
+extern cvar_t	*g_callvote;
+
+//forced autobalanced teams
+extern cvar_t	*g_autobalance;
+
+//reward point threshold
+extern cvar_t	*g_reward;
+
 #define world	(&g_edicts[0])
 
 // item spawnflags
@@ -660,6 +696,7 @@ extern	gitem_t	itemlist[];
 //
 void Cmd_Help_f (edict_t *ent);
 void Cmd_Score_f (edict_t *ent);
+void Cmd_VoiceTaunt_f (edict_t *ent);
 
 //
 // g_items.c
@@ -683,8 +720,9 @@ void Touch_Item (edict_t *ent, edict_t *other, cplane_t *plane, csurface_t *surf
 
 //CTF
 void CTFDeadDropFlag(edict_t *self);
+void CTFResetFlag(int team);
 void CTFEffects(edict_t *player);
-void CTFScoreboardMessage (edict_t *ent, edict_t *killer);
+void CTFScoreboardMessage (edict_t *ent, edict_t *killer, int mapvote);
 void CTFPrecache(void);
 void CTFFlagSetup (edict_t *ent);
 qboolean CTFPickup_Flag (edict_t *ent, edict_t *other);
@@ -726,7 +764,8 @@ void	G_FreeEdict (edict_t *e);
 void	G_TouchTriggers (edict_t *ent);
 void	G_TouchSolids (edict_t *ent);
 
-char	*G_CopyString (char *in);
+char	*G_CopyString (const char *in);
+void	G_CleanPlayerName ( const char *source, char *dest );
 
 float	*tv (float x, float y, float z);
 char	*vtos (vec3_t v);
@@ -871,6 +910,17 @@ void ClientPlaceInQueue(edict_t *ent);
 void ClientCheckQueue(edict_t *ent);
 void MoveClientsDownQueue(edict_t *ent);
 
+//unlagged - g_unlagged.c
+void G_ResetHistory( edict_t *ent );
+void G_StoreHistory( edict_t *ent );
+void G_TimeShiftAllClients( int time, edict_t *skip );
+void G_UnTimeShiftAllClients( edict_t *skip );
+void G_DoTimeShiftFor( edict_t *ent );
+void G_UndoTimeShiftFor( edict_t *ent );
+void G_UnTimeShiftClient( edict_t *ent );
+void G_PredictPlayerMove( edict_t *ent, float frametime );
+//unlagged - g_unlagged.c
+
 //
 // g_player.c
 //
@@ -896,7 +946,7 @@ void G_SetStats (edict_t *ent);
 void G_SetSpectatorStats (edict_t *ent);
 void G_CheckChaseStats (edict_t *ent);
 void ValidateSelectedItem (edict_t *ent);
-void DeathmatchScoreboardMessage (edict_t *client, edict_t *killer);
+void DeathmatchScoreboardMessage (edict_t *client, edict_t *killer, int mapvote);
 
 //
 // p_weapon.c
@@ -959,7 +1009,7 @@ void GetChaseTarget(edict_t *ent);
 typedef struct
 {
 	char		userinfo[MAX_INFO_STRING];
-	char		netname[16];
+	char		netname[48];			// considering 1 colour change / character, that's 16
 	int			hand;
 
 	qboolean	connected;			// a loadgame will leave valid entities that
@@ -990,8 +1040,39 @@ typedef struct
 	int			spectator;			// client is a spectator
 	int	queue;				// client queue position for duel mode
 
+	//unlagged - client options
+	// these correspond with variables in the userinfo string
+	int			delag;
+	int			debugDelag;
+	int			cmdTimeNudge;
+//unlagged - client options
+//unlagged - lag simulation #2
+	int			latentSnaps;
+	int			latentCmds;
+	int			plOut;
+	usercmd_t	cmdqueue[MAX_LATENT_CMDS];
+	int			cmdhead;
+//unlagged - lag simulation #2
+//unlagged - true ping
+	int			realPing;
+	int			pingsamples[NUM_PING_SAMPLES];
+	int			samplehead;
+//unlagged - true ping
+
 } client_persistant_t;
 
+//unlagged - backward reconciliation #1
+// the size of history we'll keep
+#define NUM_CLIENT_HISTORY 17
+
+// everything we need to know to backward reconcile
+typedef struct {
+	vec3_t		mins, maxs;
+	vec3_t		currentOrigin;
+	int			leveltime;
+} clientHistory_t;
+//unlagged - backward reconciliation #1
+
 // client data that stays across deathmatch respawns
 typedef struct
 {
@@ -1008,6 +1089,13 @@ typedef struct
 	int weapon_shots[9];
 	int weapon_hits[9];
 
+	//reward points
+	int reward_pts;
+	qboolean powered;
+
+	//voting
+	qboolean voted;
+
 	int			spectator;			// client is a spectator
 } client_respawn_t;
 
@@ -1095,6 +1183,7 @@ struct gclient_s
 	float		haste_framenum;
 	float		sproing_framenum;
 	float		regen_framenum;
+	float		invis_framenum;
 
 	float		losehealth_framenum;
 
@@ -1151,6 +1240,36 @@ struct gclient_s
 	float spawnprotecttime;
 	qboolean spawnprotected;
 
+	//map voting
+	int mapvote;
+
+	//taunt message
+	float lasttaunttime;
+	
+	//deathray immunity
+	qboolean rayImmunity;
+	float rayTime;
+
+	//unlagged - backward reconciliation #1
+	// the serverTime the button was pressed
+	// (stored before pmove_fixed changes serverTime)
+	int			attackTime;
+	// the head of the history queue
+	int			historyHead;
+	// the history queue
+	clientHistory_t	history[NUM_CLIENT_HISTORY];
+	// the client's saved position
+	clientHistory_t	saved;			// used to restore after time shift
+	// an approximation of the actual server time we received this
+	// command (not in 50ms increments)
+	int			frameOffset;
+//unlagged - backward reconciliation #1
+
+//unlagged - smooth clients #1
+	// the last frame number we got an update from this client
+	int			lastUpdateFrame;
+//unlagged - smooth clients #1
+
 };
 
 
@@ -1377,13 +1496,23 @@ struct edict_s
 
 };
 
-void FL_think (edict_t *self);
-void FL_make (edict_t *self);
+struct gmapvote_s
+{
+	char	mapname[32]; //ugg, end the pointer madness
+	int		tally;
+} votedmap[4];
+
+extern char *winningmap;
+
 extern void CheckDeathcam_Viewent(edict_t *ent);
 extern void DeathcamRemove (edict_t *ent, char *opt);
 extern void DeathcamStart (edict_t *ent);
 extern void DeathcamTrack (edict_t *ent);
 
+//unlagged
+extern	cvar_t	*g_antilag;
+extern  cvar_t	*g_antilagdebug;
+
 // ACEBOT_ADD
 #include "acesrc/acebot.h"
 // ACEBOT_END
