@@ -25,13 +25,15 @@ level_locals_t	level;
 game_import_t	gi;
 game_export_t	globals;
 spawn_temp_t	st;
+g_vote_t		playervote;
 
 int	sm_meat_index;
-int	snd_fry;
 int meansOfDeath;
 int bot_won;
 qboolean stayed;
 
+char *winningmap;
+
 edict_t		*g_edicts;
 
 cvar_t	*deathmatch;
@@ -86,6 +88,21 @@ cvar_t  *g_spawnprotect;
 //joust mode
 cvar_t  *joustmode;
 
+//map voting
+cvar_t	*g_mapvote;
+cvar_t	*g_voterand;
+cvar_t	*g_votemode;
+cvar_t	*g_votesame;
+
+//call voting
+cvar_t	*g_callvote;
+
+//reward point threshold
+cvar_t	*g_reward;
+
+//force autobalanced teams
+cvar_t	*g_autobalance;
+
 cvar_t	*dmflags;
 cvar_t	*skill;
 cvar_t	*fraglimit;
@@ -98,6 +115,7 @@ cvar_t	*maxspectators;
 cvar_t	*maxentities;
 cvar_t	*g_select_empty;
 cvar_t	*dedicated;
+cvar_t	*motdfile;
 
 cvar_t	*filterban;
 
@@ -129,6 +147,10 @@ cvar_t  *background_music;
 cvar_t  *sv_botkickthreshold;
 cvar_t  *sv_custombots;
 
+//unlagged
+cvar_t	*g_antilag;
+cvar_t	*g_antilagdebug;
+
 void SpawnEntities (char *mapname, char *entities, char *spawnpoint);
 void ClientThink (edict_t *ent, usercmd_t *cmd);
 qboolean ClientConnect (edict_t *ent, char *userinfo);
@@ -139,7 +161,8 @@ void ClientCommand (edict_t *ent);
 void RunEntity (edict_t *ent);
 void InitGame (void);
 void G_RunFrame (void);
-
+int ACESP_FindBotNum(void);
+extern long filelength(int);
 
 //===================================================================
 
@@ -176,6 +199,7 @@ game_export_t *GetGameAPI (game_import_t *import)
 	globals.ClientDisconnect = ClientDisconnect;
 	globals.ClientBegin = ClientBegin;
 	globals.ClientCommand = ClientCommand;
+	globals.ACESP_FindBotNum = ACESP_FindBotNum;
 
 	globals.RunFrame = G_RunFrame;
 
@@ -194,7 +218,7 @@ void Sys_Error (char *error, ...)
 	char		text[1024];
 
 	va_start (argptr, error);
-	vsprintf (text, error, argptr);
+	vsnprintf(text, sizeof(text), error, argptr);
 	va_end (argptr);
 
 	gi.error (ERR_FATAL, "%s", text);
@@ -206,7 +230,7 @@ void Com_Printf (char *msg, ...)
 	char		text[1024];
 
 	va_start (argptr, msg);
-	vsprintf (text, msg, argptr);
+	vsnprintf(text, sizeof(text), msg, argptr);
 	va_end (argptr);
 
 	gi.dprintf ("%s", text);
@@ -335,6 +359,92 @@ void EndDMLevel (void)
 			DeathcamRemove (ent, "off");
 	}
 
+	//call voting
+	if(g_callvote->value) {
+		playervote.called = false;
+		playervote.yay = 0;
+		playervote.nay = 0;
+		playervote.command[0] = 0;
+	}
+
+	//map voting
+	if(g_mapvote->value) {
+		level.changemap = level.mapname;
+
+		// initialise map list using the current map's name
+		// this is done in all modes just in case
+		for ( i = 0 ; i < 4 ; i ++ )
+		{
+			strcpy(votedmap[i].mapname, level.mapname);
+			votedmap[i].tally = 0;
+		}
+
+		// if there is a map list, time to choose
+		if ( sv_maplist && sv_maplist->string && *(sv_maplist->string) )
+		{
+			char * names[200]; // 200 map names should be fine
+			int n_maps = 0;
+			int same_index = -1;
+			int mode = ( g_votemode ? g_votemode->value : 0 );
+			qboolean same = ( g_votesame ? (g_votesame->value == 1) : 1 );
+
+			memset( names, 0 , sizeof(names) );
+			s = strdup( sv_maplist->string );
+			t = strtok( s, seps );
+			do {
+				// if using the same map is disallowed, skip it
+				if ( !Q_stricmp(t, level.mapname) )
+				{
+					if ( same_index == -1 )
+						same_index = n_maps;
+					if ( !same )
+						continue;
+				}
+
+				// avoid duplicates
+				for ( i = 0 ; i < n_maps ; i ++ )
+				{
+					if ( ! Q_stricmp( t , names[i] ) )
+						break;
+				}
+				if ( i < n_maps )
+					continue;
+
+				names[n_maps ++] = t;
+			} while ( (t = strtok( NULL, seps)) != NULL );
+
+			if ( n_maps > 0 )
+			{
+				// if the map list has been changed and the
+				// current map is no longer in it, prevent
+				// screw-ups
+				if ( same_index == -1 )
+					same_index = 0;
+
+				if ( mode == 0 )
+				{
+					// standard vote mode, take the next
+					// 4 maps from the list
+					for ( i = 0 ; i < 4 ; i ++ )
+						strcpy( votedmap[i].mapname, names[(same_index + i) % n_maps] );
+				}
+				else
+				{
+					// random selection
+					for ( i = 0 ; i < 4 && n_maps > 0 ; i ++ )
+					{
+						int map = random() * (n_maps - 1);
+						strcpy( votedmap[i].mapname, names[map] );
+						names[map] = names[n_maps - 1];
+						n_maps --;
+					}
+				}
+			}
+
+			free(s);
+		}
+	}
+
 	// stay on same level flag
 	if ((int)dmflags->value & DF_SAME_LEVEL)
 	{
@@ -404,8 +514,9 @@ void EndDMLevel (void)
 		length = ftell(fp);
 		fseek(fp, 0, SEEK_SET);
 #endif
-		buffer = malloc( length );
+		buffer = malloc( length + 1 );
 		fread( buffer, length, 1, fp );
+		buffer[length] = 0;
 	}
 
 	s = buffer;
@@ -532,7 +643,6 @@ void ResetLevel (void) //for resetting players and items after warmup
 			PutClientInServer (ent);
 			ACESP_LoadBots(ent, 0);
 		}
-
 	}
 	blue_team_score = 0;
 	red_team_score = 0;
@@ -559,6 +669,17 @@ void ResetLevel (void) //for resetting players and items after warmup
 		}
 
 	}
+
+	if(g_callvote->value)
+		safe_bprintf(PRINT_HIGH, "Call voting is ^2ENABLED\n");
+	else
+		safe_bprintf(PRINT_HIGH, "Call voting is ^1DISABLED\n");
+
+	if(g_antilag->value) 
+		safe_bprintf(PRINT_HIGH, "Antilag is ^2ENABLED\n");
+	else 
+		safe_bprintf(PRINT_HIGH, "Antilag is ^1DISABLED\n");
+	
 	return;
 }
 
@@ -577,41 +698,25 @@ void CheckDMRules (void)
 	if(!tca->value && !ctf->value && !cp->value && !((int)(dmflags->value) & DF_SKINTEAMS)) {
 		if(level.time <= warmuptime->value) {
 				if((warmuptime->value - level.time ) == 3) {
-					for (i=0 ; i<maxclients->value ; i++)
-					{
-						cl_ent = g_edicts + 1 + i;
-						if (!cl_ent->inuse || cl_ent->is_bot)
-							continue;
-						gi.sound (cl_ent, CHAN_AUTO, gi.soundindex("misc/three.wav"), 1, ATTN_STATIC, 0);
-					}
+					cl_ent = g_edicts + 1; //need only one for broadcast sound
+					if (cl_ent->inuse && !cl_ent->is_bot)
+						gi.sound (cl_ent, CHAN_AUTO, gi.soundindex("misc/three.wav"), 1, ATTN_NONE, 0);
 				}
 				if((warmuptime->value - level.time ) == 2) {
-					for (i=0 ; i<maxclients->value ; i++)
-					{
-						cl_ent = g_edicts + 1 + i;
-						if (!cl_ent->inuse || cl_ent->is_bot)
-							continue;
-						gi.sound (cl_ent, CHAN_AUTO, gi.soundindex("misc/two.wav"), 1, ATTN_STATIC, 0);
-					}
+					cl_ent = g_edicts + 1;
+					if (cl_ent->inuse && !cl_ent->is_bot)
+						gi.sound (cl_ent, CHAN_AUTO, gi.soundindex("misc/two.wav"), 1, ATTN_NONE, 0);
 				}
 				if((warmuptime->value - level.time ) == 1) {
-					for (i=0 ; i<maxclients->value ; i++)
-					{
-						cl_ent = g_edicts + 1 + i;
-						if (!cl_ent->inuse || cl_ent->is_bot)
-							continue;
-						gi.sound (cl_ent, CHAN_AUTO, gi.soundindex("misc/one.wav"), 1, ATTN_STATIC, 0);
-					}
+					cl_ent = g_edicts + 1;
+					if (cl_ent->inuse && !cl_ent->is_bot)
+						gi.sound (cl_ent, CHAN_AUTO, gi.soundindex("misc/one.wav"), 1, ATTN_NONE, 0);
 				}
 				if(level.time == warmuptime->value) {
-					for (i=0 ; i<maxclients->value ; i++)
-					{
-					cl_ent = g_edicts + 1 + i;
-					if (!cl_ent->inuse || cl_ent->is_bot)
-						continue;
-						gi.sound (cl_ent, CHAN_AUTO, gi.soundindex("misc/fight.wav"), 1, ATTN_STATIC, 0);
+					cl_ent = g_edicts + 1;
+					if (cl_ent->inuse && !cl_ent->is_bot)
+						gi.sound (cl_ent, CHAN_AUTO, gi.soundindex("misc/fight.wav"), 1, ATTN_NONE, 0);
 						safe_centerprintf(cl_ent, "FIGHT!\n");
-					}
 					ResetLevel();
 				}
 				else if(level.time == ceil(level.time)){ //do only on the whole numer to avoid overflowing
@@ -715,7 +820,8 @@ void CheckDMRules (void)
 								cl_ent = g_edicts + 1 + i;
 								if (!cl_ent->inuse || cl_ent->is_bot)
 									continue;
-								gi.sound (cl_ent, CHAN_AUTO, gi.soundindex("misc/3frags.wav"), 1, ATTN_STATIC, 0);
+								if(i == 0)
+									gi.sound (cl_ent, CHAN_AUTO, gi.soundindex("misc/3frags.wav"), 1, ATTN_NONE, 0);
 								safe_centerprintf(cl_ent, "3 frags remain!\n");
 							}
 							print3 = true;
@@ -728,7 +834,8 @@ void CheckDMRules (void)
 								cl_ent = g_edicts + 1 + i;
 								if (!cl_ent->inuse || cl_ent->is_bot)
 									continue;
-								gi.sound (cl_ent, CHAN_AUTO, gi.soundindex("misc/2frags.wav"), 1, ATTN_STATIC, 0);
+								if(i == 0)
+									gi.sound (cl_ent, CHAN_AUTO, gi.soundindex("misc/2frags.wav"), 1, ATTN_NONE, 0);
 								safe_centerprintf(cl_ent, "2 frags remain!\n");
 							}
 							print2 = true;
@@ -741,7 +848,8 @@ void CheckDMRules (void)
 								cl_ent = g_edicts + 1 + i;
 								if (!cl_ent->inuse || cl_ent->is_bot)
 									continue;
-								gi.sound (cl_ent, CHAN_AUTO, gi.soundindex("misc/1frags.wav"), 1, ATTN_STATIC, 0);
+								if(i == 0)
+									gi.sound (cl_ent, CHAN_AUTO, gi.soundindex("misc/1frags.wav"), 1, ATTN_NONE, 0);
 								safe_centerprintf(cl_ent, "1 frag remains!\n");
 							}
 							print1 = true;
@@ -812,6 +920,7 @@ void ExitLevel (void)
 		if(stayed) {
 			ent->client->resp.score = 0;
 			ent->client->resp.deaths = 0;
+			ent->client->resp.reward_pts = 0;
 			if(!ent->is_bot) { //players
 				ent->takedamage = DAMAGE_AIM;
 				ent->solid = SOLID_BBOX;
@@ -860,6 +969,80 @@ void ExitLevel (void)
 		game.num_bots = 0; //do this *just* incase something got whacked or isn't getting set.
 }
 
+/*
+================
+G_ParseVoteCommand
+
+Parse and execute command
+================
+*/
+void G_ParseVoteCommand (void)
+{
+	int i, j;
+	char command[128];
+	char args[128];
+	edict_t *ent;
+	qboolean donearg = false;
+
+	//separate command from args
+	i = j = 0;
+	while(i < 128) {
+
+		if(playervote.command[i] == ' ')
+			donearg = true;
+
+		if(!donearg) 
+			command[i] = playervote.command[i];
+		else
+			command[i] = 0;
+
+		if(donearg && i < 127) { //skip the space between command and arg
+			args[j] = playervote.command[i+1];
+			j++;
+		}
+		i++;
+	}
+
+	if(!strcmp(command, "kick")) { //kick player
+
+		//get the correct client
+		for (i=0 ; i<maxclients->value ; i++)
+		{
+			ent = g_edicts + 1 + i;
+			if (!ent->inuse)
+				continue;
+
+			if(ent->client) {
+				if(!strcmp(ent->client->pers.netname, args)) {
+					if(ent->is_bot)
+						ACESP_KickBot(args);
+					else {
+						safe_bprintf(PRINT_HIGH, "%s was kicked\n", args);
+						ClientDisconnect (ent);
+					}
+				}
+				
+			}
+		}
+	}
+	else if(!strcmp(command, "fraglimit")) { //change fraglimit
+		gi.cvar_set("fraglimit", args);
+		safe_bprintf(PRINT_HIGH, "Fraglimit changed to %s\n", args);
+	}
+	else if(!strcmp(command, "timelimit")) { //change timelimit
+		gi.cvar_set("timelimit", args);
+		safe_bprintf(PRINT_HIGH, "Timelimit changed to %s\n", args);
+	}
+	else if(!strcmp(command, "map")) { //change map
+		Com_sprintf (command, sizeof(command), "map \"%s\"\n", args);
+		gi.AddCommandString (command);
+	}
+	else
+		safe_bprintf(PRINT_HIGH, "Invalid command!");
+
+}
+
+
 /*
 ================
 G_RunFrame
@@ -869,9 +1052,11 @@ Advances the world by 0.1 seconds
 */
 void G_RunFrame (void)
 {
-	int		i;
+	int		i, numActiveClients = 0;
 	edict_t	*ent;
 
+	level.previousTime = gi.Sys_Milliseconds() - 100; //100 milleseconds(1/10 of a second)
+
 	level.framenum++;
 	level.time = level.framenum*FRAMETIME;
 
@@ -889,7 +1074,8 @@ void G_RunFrame (void)
 	//
 	// treat each object in turn
 	// even the world gets a chance to think
-	//
+	//	
+	
 	ent = &g_edicts[0];
 	for (i=0 ; i<globals.num_edicts ; i++, ent++)
 	{
@@ -913,12 +1099,25 @@ void G_RunFrame (void)
 		if (i > 0 && i <= maxclients->value)
 		{
 			ClientBeginServerFrame (ent);
-// ACEBOT_ADD
-
-// ACEBOT_END
 		}
 
-		G_RunEntity (ent);
+		if(ent->inuse && ent->client && !ent->is_bot)
+			numActiveClients++;
+
+		//this next block of code may not be practical for a server running at 10fps
+/*		if(ent->movetype & MOVETYPE_FLYMISSILE) {
+			//unlagged
+			if ( g_antilag->integer)
+				G_TimeShiftAllClients( level.previousTime, NULL );	
+
+			G_RunEntity (ent);
+
+			//unlagged
+			if ( g_antilag->integer)
+				G_UnTimeShiftAllClients( NULL );
+		}
+		else*/
+			G_RunEntity (ent);
 	}
 
 	// see if it is time to end a deathmatch
@@ -929,5 +1128,48 @@ void G_RunFrame (void)
 
 	// build the playerstate_t structures for all players
 	ClientEndServerFrames ();
+
+	//unlagged
+	if ( g_antilag->integer)
+		level.frameStartTime = level.time;
+
+	//call voting
+	if(g_callvote->value && playervote.called) {
+
+		playervote.time = level.time;
+		if(playervote.time-playervote.starttime > 15 ){ //15 seconds
+			//execute command if votes are sufficient
+			if(numActiveClients <= 2 && playervote.yay > playervote.nay+1) { //minimum of 2 votes to pass
+				safe_bprintf(PRINT_HIGH, "Vote ^2Passed\n");
+				
+				//parse command(we will allow kick, map, fraglimit, timelimit).
+				G_ParseVoteCommand();
+				
+			}
+			else if(playervote.yay > 2 && playervote.yay > playervote.nay+1) { //3-1 minimum to pass
+				safe_bprintf(PRINT_HIGH, "Vote ^2Passed\n");
+				
+				//parse command(we will allow kick, map, fraglimit, timelimit).
+				G_ParseVoteCommand();
+				
+			}
+			else
+				safe_bprintf(PRINT_HIGH, "Vote ^1Failed\n");
+		
+			//clear
+			playervote.called = false;
+			playervote.yay = playervote.nay = 0;
+			playervote.command[0] = 0;
+
+			//do each ent
+			for (i=0 ; i<maxclients->value ; i++)
+			{
+				ent = g_edicts + 1 + i;
+				if (!ent->inuse || ent->is_bot)
+					continue;
+				ent->client->resp.voted = false;
+			}
+		}
+	}
 }
 
