@@ -237,6 +237,25 @@ void InitGame (void)
 	//joust mode
 	joustmode = gi.cvar("sv_joustmode", "0", CVAR_SERVERINFO);
 
+	//map voting
+	g_mapvote = gi.cvar("g_mapvote", "0", CVAR_SERVERINFO);
+	g_voterand = gi.cvar("g_voterand", "0", 0);
+	g_votemode = gi.cvar("g_votemode", "0", 0);
+	g_votesame = gi.cvar("g_votesame", "1", 0);
+
+	//call voting
+	g_callvote = gi.cvar("g_callvote", "1", CVAR_SERVERINFO); //change after testing
+
+	//forced autobalanced teams
+	g_autobalance = gi.cvar("g_autobalance", "0", 0);
+
+	//reward points threshold
+	g_reward = gi.cvar("g_reward", "20", CVAR_SERVERINFO);
+
+	//antilag
+	g_antilag = gi.cvar("g_antilag", "1", CVAR_SERVERINFO);
+	g_antilagdebug = gi.cvar("g_antilagdebug", "0", CVAR_SERVERINFO);
+
 	// change anytime vars
 	dmflags = gi.cvar ("dmflags", "0", CVAR_SERVERINFO);
 	fraglimit = gi.cvar ("fraglimit", "0", CVAR_SERVERINFO);
@@ -245,6 +264,7 @@ void InitGame (void)
 	spectator_password = gi.cvar ("spectator_password", "", CVAR_USERINFO);
 	needpass = gi.cvar ("needpass", "0", CVAR_SERVERINFO);
 	filterban = gi.cvar ("filterban", "1", 0);
+	motdfile = gi.cvar ("motdfile", "motd.txt", 0);
 
 	g_select_empty = gi.cvar ("g_select_empty", "0", CVAR_ARCHIVE);
 
