@@ -80,20 +80,15 @@ void SP_misc_teleporter (edict_t *self);
 void SP_misc_teleporter_dest (edict_t *self);
 
 void SP_npc_cow (edict_t *self);
+void SP_npc_deathray(edict_t *self);
 
-void SP_misc_electrode (edict_t *self);
-void SP_misc_scope (edict_t *self);
-void SP_misc_pod (edict_t *self);
-void SP_misc_receptor (edict_t *self);
-void SP_misc_etube (edict_t *self);
-void SP_misc_lamp (edict_t *self);
-void SP_misc_lamp2 (edict_t *self);
 void SP_misc_spiderpod (edict_t *self);
 void SP_misc_rednode (edict_t *self);
 void SP_misc_bluenode (edict_t *self);
 void SP_misc_bluespidernode (edict_t *self);
 void SP_misc_redspidernode (edict_t *self);
 void SP_misc_mapmodel (edict_t *self);
+void SP_misc_watersplash (edict_t *ent);
 
 spawn_t	spawns[] = {
 	{"item_health", SP_item_health},
@@ -169,20 +164,15 @@ spawn_t	spawns[] = {
 	{"misc_teleporter_dest", SP_misc_teleporter_dest},
 
 	{"npc_cow", SP_npc_cow},
+	{"npc_deathray", SP_npc_deathray},
 
-	{"misc_lamp", SP_misc_lamp},
-	{"misc_lamp2", SP_misc_lamp2},
-	{"misc_electrode", SP_misc_electrode},
-	{"misc_scope", SP_misc_scope},
-	{"misc_pod", SP_misc_pod},
-	{"misc_receptor", SP_misc_receptor},
-	{"misc_etube", SP_misc_etube},
 	{"misc_spiderpod", SP_misc_spiderpod},
 	{"misc_rednode", SP_misc_rednode},
 	{"misc_bluenode", SP_misc_bluenode},
 	{"misc_redspidernode", SP_misc_redspidernode},
 	{"misc_bluespidernode", SP_misc_bluespidernode},
 	{"misc_mapmodel", SP_misc_mapmodel},
+	{"misc_watersplash", SP_misc_watersplash},
 	{NULL, NULL}
 };
 
@@ -534,10 +524,12 @@ void SpawnEntities (char *mapname, char *entities, char *spawnpoint)
 
 	gi.dprintf ("%i entities inhibited\n", inhibit);
 
+	ACEND_InitNodes();
+	ACEND_LoadNodes();
+
 	G_FindTeams ();
 
 	PlayerTrail_Init ();
-
 }
 
 
@@ -587,18 +579,16 @@ char *dm_statusbar =
 "endif "
 
 // armor
-"if 4 "
 "	xl	142 "
 "	rnum "
-"endif "
 
 // timer
 "if 9 "
 "	xv	262 "
-"   yb  -36 "
+"   yb  -24 "
 "	num	2	10 "
 "	xv	296 "
-"   yb  -56 "
+"   yb  -32 "
 "	pic	9 "
 "endif "
 
@@ -622,10 +612,47 @@ char *dm_statusbar =
 //  high scorer
 "yt 80 "
 "num 3 20 "
+
+//  weapon stats
+"if 25 "
+"xr -72 "
+"yt 227 "
+"pic 25 "
+"endif"
+
+"if 26 "
+"yt 258 "
+"pic 26 "
+"endif "
+
+"if 27 "
+"yt 289 "
+"pic 27 "
+"endif "
+
+"if 28 "
+"yt 320 "
+"pic 28 "
+"endif "
+
+"if 29 "
+"yt 351 "
+"pic 29 "
+"endif "
+
+"if 30 "
+"yt 382 "
+"pic 30 "
+"endif "
+
+"if 31 "
+"yt 413 "
+"pic 31 "
+"endif "
 ;
 
 char *team_statusbar =
-//background
+// background
 "yb -256 "
 "xl	 0 "
 "pic 0 "
@@ -645,18 +672,16 @@ char *team_statusbar =
 "endif "
 
 // armor
-"if 4 "
 "	xl	142 "
 "	rnum "
-"endif "
 
 // timer
 "if 9 "
-"	xv	262 "
-"   yb  -36 "
+"	xv	324 "
+"   yb  -24 "
 "	num	2	10 "
-"	xv	296 "
-"   yb  -56 "
+"	xv	358 "
+"   yb  -32 "
 "	pic	9 "
 "endif "
 
@@ -688,10 +713,47 @@ char *team_statusbar =
 "yt 166 "
 "num 3 22 "
 
-//flag
+//  flag
 "   xv 128 "
 "   yb -64 "
 "   pic 23 "
+
+//  weapon stats
+"if 25 "
+"xr -72 "
+"yt 227 "
+"pic 25 "
+"endif"
+
+"if 26 "
+"yt 258 "
+"pic 26 "
+"endif "
+
+"if 27 "
+"yt 289 "
+"pic 27 "
+"endif "
+
+"if 28 "
+"yt 320 "
+"pic 28 "
+"endif "
+
+"if 29 "
+"yt 351 "
+"pic 29 "
+"endif "
+
+"if 30 "
+"yt 382 "
+"pic 30 "
+"endif "
+
+"if 31 "
+"yt 413 "
+"pic 31 "
+"endif "
 ;
 /*QUAKED worldspawn (0 0 0) ?
 
@@ -769,8 +831,6 @@ void SP_worldspawn (edict_t *ent)
 	else
 		gi.cvar_set("sv_gravity", st.gravity);
 
-	snd_fry = gi.soundindex ("player/fry.wav");	// standing in lava / slime
-
 	PrecacheItem (FindItem ("Blaster"));
 	PrecacheItem (FindItem ("Violator"));
 
@@ -835,18 +895,16 @@ void SP_worldspawn (edict_t *ent)
 	gi.modelindex ("models/objects/debris1/tris.md2");
 	gi.modelindex ("models/objects/debris3/tris.md2");
 
-	//precache all base player models.  this eliminates the "stutter" when a player joins the game
+//	gi.modelindex ("models/objects/laser/tris.md2");
+//	gi.modelindex ("models/objects/fireball/tris.md2");
+
+	//precache all base player models.  this eliminates "stutter" when a player joins the game
 	gi.modelindex ("players/martianenforcer/tris.md2");
-//	gi.modelindex ("players/martian/tris.md2");
 	gi.modelindex ("players/martiancyborg/tris.md2");
 	gi.modelindex ("players/enforcer/tris.md2");
-//	gi.modelindex ("players/infantry/tris.md2");
-//	gi.modelindex ("players/war/tris.md2");
-//	gi.modelindex ("players/brainlet/tris.md2");
-//	gi.modelindex ("players/robot/tris.md2");
-//	gi.modelindex ("players/construct/tris.md2");
 	gi.modelindex ("players/lauren/tris.md2");
-//	gi.modelindex ("players/rustbot/tris.md2");
+	gi.modelindex ("players/slashbot/tris.md2");
+	gi.modelindex ("players/commander/tris.md2");
 
 	//do the w_weps
 	// THIS ORDER MUST MATCH THE DEFINES IN g_local.h
@@ -908,5 +966,13 @@ void SP_worldspawn (edict_t *ent)
 
 	// 63 testing
 	gi.configstring(CS_LIGHTS+63, "a");
+
+	//call voting(we've started a new level, clear it)
+	if(g_callvote->value) {
+		playervote.called = false;
+		playervote.yay = 0;
+		playervote.nay = 0;
+		playervote.command[0] = 0;
+	}
 }
 
