@@ -10,7 +10,7 @@ fire_lead
 This is an internal support routine used for bullet/pellet based weapons.
 =================
 */
-static void fire_lead (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int kick, int te_impact, int hspread, int vspread, int mod)
+static void fire_lead (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int kick, int te_impact, int hspread, int vspread, int mod, int silentHit)
 {
 	trace_t		tr;
 	vec3_t		dir;
@@ -22,6 +22,9 @@ static void fire_lead (edict_t *self, vec3_t start, vec3_t aimdir, int damage, i
 	qboolean	water = false;
 	int			content_mask = MASK_SHOT | MASK_WATER;
 
+	if ( g_antilag->integer)
+		G_DoTimeShiftFor( self );
+
 	self->client->resp.weapon_shots[3]++;
 
 	tr = gi.trace (self->s.origin, NULL, NULL, start, self, MASK_SHOT);
@@ -105,7 +108,8 @@ static void fire_lead (edict_t *self, vec3_t start, vec3_t aimdir, int damage, i
 			{
 				T_Damage (tr.ent, self, self, aimdir, tr.endpos, tr.plane.normal, damage, kick, DAMAGE_BULLET, mod);
 				self->client->resp.weapon_hits[3]++;
-				gi.sound (self, CHAN_VOICE, gi.soundindex("misc/hit.wav"), 1, ATTN_STATIC, 0);
+				if(!silentHit)
+					gi.sound (self, CHAN_VOICE, gi.soundindex("misc/hit.wav"), 1, ATTN_STATIC, 0);
 			}
 			else
 			{
@@ -146,6 +150,9 @@ static void fire_lead (edict_t *self, vec3_t start, vec3_t aimdir, int damage, i
 		gi.WritePosition (tr.endpos);
 		gi.multicast (pos, MULTICAST_PVS);
 	}
+
+	if ( g_antilag->integer)
+		G_UndoTimeShiftFor( self );
 }
 
 
@@ -159,7 +166,7 @@ pistols, rifles, etc....
 */
 void fire_bullet (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int kick, int hspread, int vspread, int mod)
 {
-	fire_lead (self, start, aimdir, damage, kick, TE_GUNSHOT, hspread, vspread, mod);
+	fire_lead (self, start, aimdir, damage, kick, TE_GUNSHOT, hspread, vspread, mod, false);
 }
 
 
@@ -175,7 +182,7 @@ void fire_shotgun (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int k
 	int		i;
 
 	for (i = 0; i < count; i++)
-		fire_lead (self, start, aimdir, damage, kick, TE_GUNSHOT, hspread, vspread, mod);
+		fire_lead (self, start, aimdir, damage, kick, TE_GUNSHOT, hspread, vspread, mod, i);
 }
 
 
@@ -286,18 +293,25 @@ void fire_blasterball (edict_t *self, vec3_t start, vec3_t dir, int damage, int
 	bolt->movetype = MOVETYPE_FLYMISSILE;
 	bolt->clipmask = MASK_SHOT;
 	bolt->solid = SOLID_BBOX;
-	bolt->s.effects |= EF_PLASMA;
+	if(effect == EF_ROCKET) { //ack, kinda assbackwards, but the past mistakes haunt us
+		bolt->s.effects |= EF_BLASTER;
+		bolt->s.modelindex = gi.modelindex ("models/objects/laser/tris.md2");
+	}
+	else {
+		bolt->s.effects |= EF_PLASMA;
+		bolt->s.modelindex = gi.modelindex ("models/objects/fireball/tris.md2");
+	}
 	VectorClear (bolt->mins);
 	VectorClear (bolt->maxs);
-	bolt->s.modelindex = gi.modelindex ("models/objects/fireball/tris.md2");
 	bolt->s.sound = gi.soundindex ("misc/lasfly.wav");
 	bolt->owner = self;
 	bolt->touch = blasterball_touch;
 	bolt->nextthink = level.time + 2;
 	bolt->think = G_FreeEdict;
 	bolt->dmg = damage;
+	
 	bolt->classname = "bolt";
-
+	
 	gi.linkentity (bolt);
 
 	tr = gi.trace (self->s.origin, NULL, NULL, bolt->s.origin, bolt, MASK_SHOT);
@@ -317,6 +331,9 @@ int speed, int effect, qboolean hyper)
 	int			mask;
 	qboolean	water;
 
+	if ( g_antilag->integer)
+		G_DoTimeShiftFor( self );
+
 	self->client->resp.weapon_shots[6]++;
 
 	VectorMA (start, 8192, aimdir, end);
@@ -379,6 +396,9 @@ int speed, int effect, qboolean hyper)
 		gi.WriteDir (tr.plane.normal);
 		gi.multicast (self->s.origin, MULTICAST_PVS);
 	}
+
+	if ( g_antilag->integer)
+		G_UndoTimeShiftFor( self );
 }
 
 /*
@@ -526,6 +546,9 @@ void fire_blaster_beam (edict_t *self, vec3_t start, vec3_t aimdir, int damage,
 	vec3_t		water_start;
 	int			content_mask = MASK_SHOT | MASK_WATER;
 
+	if ( g_antilag->integer)
+		G_DoTimeShiftFor( self );
+
 	self->client->resp.weapon_shots[0]++;
 
 	VectorMA (start, 8192, aimdir, end);
@@ -664,6 +687,10 @@ void fire_blaster_beam (edict_t *self, vec3_t start, vec3_t aimdir, int damage,
 		gi.WritePosition (tr.endpos);
 		gi.multicast (pos, MULTICAST_PVS);
 	}
+
+	if ( g_antilag->integer)
+		G_UndoTimeShiftFor( self );
+
 }
 void fire_hover_beam (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int kick, qboolean detonate)
 {
@@ -677,6 +704,9 @@ void fire_hover_beam (edict_t *self, vec3_t start, vec3_t aimdir, int damage, in
 	vec3_t		water_start;
 	int			content_mask = MASK_SHOT | MASK_WATER;
 
+	if ( g_antilag->integer)
+		G_DoTimeShiftFor( self );
+
 	VectorMA (start, 8192, aimdir, end);
 	VectorCopy (start, from);
 	ignore = self;
@@ -817,6 +847,10 @@ void fire_hover_beam (edict_t *self, vec3_t start, vec3_t aimdir, int damage, in
 		gi.WritePosition (tr.endpos);
 		gi.multicast (pos, MULTICAST_PVS);
 	}
+
+	if ( g_antilag->integer)
+		G_UndoTimeShiftFor( self );
+
 }
 
 void fire_plasma (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int kick)
@@ -828,6 +862,9 @@ void fire_plasma (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int ki
 	int			mask;
 	qboolean	water;
 
+	if ( g_antilag->integer)
+		G_DoTimeShiftFor( self );
+
 	self->client->resp.weapon_shots[1]++;
 
 	VectorMA (start, 8192, aimdir, end);
@@ -891,6 +928,9 @@ void fire_plasma (edict_t *self, vec3_t start, vec3_t aimdir, int damage, int ki
 
 	if (self->client)
 		PlayerNoise(self, tr.endpos, PNOISE_IMPACT);
+
+	if ( g_antilag->integer)
+		G_UndoTimeShiftFor( self );
 }
 
 //vaporizer code
@@ -905,6 +945,9 @@ void fire_energy_field (edict_t *self, vec3_t start, vec3_t aimdir, int damage,
 	int			mask;
 	qboolean	water;
 
+	if ( g_antilag->integer)
+		G_DoTimeShiftFor( self );
+
 	self->client->resp.weapon_shots[7]++;
 
 	VectorMA (start, 8192, aimdir, end);
@@ -970,11 +1013,14 @@ void fire_energy_field (edict_t *self, vec3_t start, vec3_t aimdir, int damage,
 	bomb->think = G_FreeEdict;
 	bomb->classname = "bomb";
 	gi.linkentity (bomb);
-	T_RadiusDamage(bomb, self, 200, NULL, 150, MOD_VAPORIZER, 7); //ridiculously powerful!
+	T_RadiusDamage(bomb, self, 150, NULL, 150, MOD_VAPORIZER, 7); //ridiculously powerful!
 	G_FreeEdict (bomb);
 
 	if (self->client)
 		PlayerNoise(self, tr.endpos, PNOISE_IMPACT);
+
+	if ( g_antilag->integer)
+		G_UndoTimeShiftFor( self );
 }
 
 
@@ -1381,7 +1427,7 @@ void bomb_touch (edict_t *ent, edict_t *other, cplane_t *plane, csurface_t *surf
 		e->groundentity = NULL;
 		e->velocity[0] += crandom()* 50;
 		e->velocity[1] += crandom()* 50;
-		e->velocity[2] += crandom()* 100;
+		e->velocity[2] += 175 + crandom()* 50;
 	}
 	ent->think = bomb_blow;
 	ent->nextthink = level.time + .1;
@@ -1718,6 +1764,9 @@ void fire_violator(edict_t *self, vec3_t start, vec3_t aimdir, int damage, int k
 	trace_t		tr;
 	edict_t		*ignore;
 
+	if ( g_antilag->integer)
+		G_DoTimeShiftFor( self );
+
 	if(alt)
 		VectorMA (start, 6.4, aimdir, end);
 	else
@@ -1749,4 +1798,7 @@ void fire_violator(edict_t *self, vec3_t start, vec3_t aimdir, int damage, int k
 		}
 		VectorCopy (tr.endpos, from);
 	}
+
+	if ( g_antilag->integer)
+		G_UndoTimeShiftFor( self );
 }
