@@ -88,14 +88,17 @@ void ClientObituary (edict_t *self, edict_t *inflictor, edict_t *attacker)
 	char		*chatmsg;
 	char		*tauntmsg;
 	char		cleanname[16], cleanname2[16];
-	int			i, j, pos, total, place;
+	int			i, pos, total, place;
 	edict_t		*cl_ent;
+	gitem_t		*it;
 
 	if (deathmatch->value)
 	{
 		ff = meansOfDeath & MOD_FRIENDLY_FIRE;
 		mod = meansOfDeath & ~MOD_FRIENDLY_FIRE;
 		message = NULL;
+		chatmsg = NULL;
+		tauntmsg = NULL;
 		message2 = "";
 
 		switch (mod)
@@ -182,30 +185,10 @@ void ClientObituary (edict_t *self, edict_t *inflictor, edict_t *attacker)
 		self->enemy = attacker;
 		if (attacker && attacker->client)
 		{
-			//clean up name, get rid of escape chars
-			j = 0;
-			for (i = 0; i < 16; i++)
-				cleanname[i] = 0;
-			for (i = 0; i < strlen(self->client->pers.netname) && i < 16; i++) {
-				if ( self->client->pers.netname[i] == '^' ) {
-					i += 1;
-					continue;
-				}
-				cleanname[j] = self->client->pers.netname[i]+128;
-				j++;
-			}
-			//clean up name, get rid of escape chars
-			j = 0;
-			for (i = 0; i < 16; i++)
-				cleanname2[i] = 0;
-			for (i = 0; i < strlen(attacker->client->pers.netname) && i < 16; i++) {
-					if ( attacker->client->pers.netname[i] == '^' ) {
-						i += 1;
-						continue;
-					}
-				cleanname2[j] = attacker->client->pers.netname[i]+128;
-				j++;
-			}
+			//clean up names, get rid of escape chars
+			G_CleanPlayerName ( self->client->pers.netname , cleanname );
+			G_CleanPlayerName ( attacker->client->pers.netname , cleanname2 );
+
 			if(!attacker->is_bot) {
 				pos = 0;
 				total = 0;
@@ -300,6 +283,8 @@ void ClientObituary (edict_t *self, edict_t *inflictor, edict_t *attacker)
 				message = "was caught by";
 				message2 = "'s grapple";
 				break;
+			case MOD_HEADSHOT:
+				message = "had it's head blown off by";
 			}
 			//here is where the bot chat features will be added.
 			//default is on.  Setting to 1 turns it off.
@@ -307,7 +292,7 @@ void ClientObituary (edict_t *self, edict_t *inflictor, edict_t *attacker)
 #ifndef __unix__
 			if ((!((int)(dmflags->value) & DF_BOTCHAT)) && self->is_bot)
 			{
-				msg = random() * 12;
+				msg = random() * 9;
 				switch(msg){
 				case 1:
 					chatmsg = self->chatmsg1;
@@ -334,21 +319,23 @@ void ClientObituary (edict_t *self, edict_t *inflictor, edict_t *attacker)
 					chatmsg = self->chatmsg8;
 					break;
 				default:
-					chatmsg = "";
+					chatmsg = "%s: Stop it %s, you punk!";
 					break;
 				}
-				safe_bprintf (PRINT_CHAT, chatmsg, self->client->pers.netname, attacker->client->pers.netname);
-				safe_bprintf (PRINT_CHAT, "\n");
+				if(chatmsg) {
+					safe_bprintf (PRINT_CHAT, chatmsg, self->client->pers.netname, attacker->client->pers.netname);
+					safe_bprintf (PRINT_CHAT, "\n");
 
-				gi.WriteByte (svc_temp_entity);
-				gi.WriteByte (TE_SAYICON);
-				gi.WritePosition (self->s.origin);
-				gi.multicast (self->s.origin, MULTICAST_PVS);
+					gi.WriteByte (svc_temp_entity);
+					gi.WriteByte (TE_SAYICON);
+					gi.WritePosition (self->s.origin);
+					gi.multicast (self->s.origin, MULTICAST_PVS);
+				}
 			}
 #else
 			if ((!((int)(dmflags->value) & DF_BOTCHAT)) && self->is_bot)
 			{
-				msg = random() * 12;
+				msg = random() * 9;
 				switch(msg){
 				case 1:
 					chatmsg = "%s: You are a real jerk %s!";
@@ -375,62 +362,86 @@ void ClientObituary (edict_t *self, edict_t *inflictor, edict_t *attacker)
 					chatmsg = "%s: You're using a bot %s!";
 					break;
 				default:
-					chatmsg = "";
+					chatmsg = "%s: Stop it %s, you punk!";
 					break;
 				}
-				safe_bprintf (PRINT_CHAT, chatmsg, self->client->pers.netname, attacker->client->pers.netname);
-				safe_bprintf (PRINT_CHAT, "\n");
+				if(chatmsg) {
+					safe_bprintf (PRINT_CHAT, chatmsg, self->client->pers.netname, attacker->client->pers.netname);
+					safe_bprintf (PRINT_CHAT, "\n");
 
-				gi.WriteByte (svc_temp_entity);
-				gi.WriteByte (TE_SAYICON);
-				gi.WritePosition (self->s.origin);
-				gi.multicast (self->s.origin, MULTICAST_PVS);
+					gi.WriteByte (svc_temp_entity);
+					gi.WriteByte (TE_SAYICON);
+					gi.WritePosition (self->s.origin);
+					gi.multicast (self->s.origin, MULTICAST_PVS);
+				}
 			}
 #endif
 			//bot taunts
-			if(attacker->is_bot && (!attacker->client->ps.pmove.pm_flags & PMF_DUCKED) && attacker->skill == 3) {
+			if((!((int)(dmflags->value) & DF_BOTCHAT)) && attacker->is_bot) {
 
-				attacker->state = STATE_STAND;
-				attacker->s.frame = FRAME_taunt01-1;
-				attacker->client->anim_end = FRAME_taunt17;
+				if(!(attacker->client->ps.pmove.pm_flags & PMF_DUCKED)) {
+					attacker->state = STATE_STAND;
+					attacker->s.frame = FRAME_taunt01-1;
+					attacker->client->anim_end = FRAME_taunt17;			
 
-				//print a taunt
-				msg = random() * 9;
-				switch(msg){
-				case 1:
-					tauntmsg = "%s: You should have used a bigger gun %s.\n";
-					break;
-				case 2:
-					tauntmsg = "%s: You fight like your mom %s.\n";
-					break;
-				case 3:
-					tauntmsg = "%s: And stay down %s!\n";
-					break;
-				case 4:
-					tauntmsg = "%s: %s = pwned!\n";
-					break;
-				case 5:
-					tauntmsg = "%s: All too easy, %s, all too easy.\n";
-					break;
-				case 6:
-					tauntmsg = "%s: Ack! %s Ack! Ack!\n";
-					break;
-				case 7:
-					tauntmsg = "%s: What a loser you are %s!\n";
-					break;
-				case 8:
-					tauntmsg = "%s: %s, could you BE any more dead?\n";
-					break;
-				default:
-					tauntmsg = "%s: You are useless to me, %s\n";
-					break;
+					//print a taunt, or send taunt sound
+					msg = random() * 24;
+					switch(msg){
+					case 1:
+						tauntmsg = "%s: You should have used a bigger gun %s.\n";
+						break;
+					case 2:
+						tauntmsg = "%s: You fight like your mom %s.\n";
+						break;
+					case 3:
+						tauntmsg = "%s: And stay down %s!\n";
+						break;
+					case 4:
+						tauntmsg = "%s: %s = pwned!\n";
+						break;
+					case 5:
+						tauntmsg = "%s: All too easy, %s, all too easy.\n";
+						break;
+					case 6:
+						tauntmsg = "%s: Ack! %s Ack! Ack!\n";
+						break;
+					case 7:
+						tauntmsg = "%s: What a loser you are %s!\n";
+						break;
+					case 8:
+						tauntmsg = "%s: %s, could you BE any more dead?\n";
+						break;
+					case 9:
+					case 10:
+					case 11:
+					case 12:
+					case 13:
+					case 14:
+					case 15:
+					case 16:
+					case 17:
+					case 18:
+					case 19:
+					case 20:
+					case 21:
+					case 22:
+					case 23:
+					case 24:
+						Cmd_VoiceTaunt_f(attacker);
+						break;
+					default:
+						tauntmsg = "%s: You are useless to me, %s\n";
+						break;
+					}
+					if(tauntmsg) {
+						safe_bprintf (PRINT_CHAT, tauntmsg, attacker->client->pers.netname, self->client->pers.netname);
+						//send an effect to show that the bot is taunting
+						gi.WriteByte (svc_temp_entity);
+						gi.WriteByte (TE_SAYICON);
+						gi.WritePosition (attacker->s.origin);
+						gi.multicast (attacker->s.origin, MULTICAST_PVS);
+					}
 				}
-				safe_bprintf (PRINT_CHAT, tauntmsg, attacker->client->pers.netname, self->client->pers.netname);
-				//send an effect to show that the bot is taunting
-				gi.WriteByte (svc_temp_entity);
-				gi.WriteByte (TE_SAYICON);
-				gi.WritePosition (attacker->s.origin);
-				gi.multicast (attacker->s.origin, MULTICAST_PVS);
 			}
 
 			if (message)
@@ -450,7 +461,21 @@ void ClientObituary (edict_t *self, edict_t *inflictor, edict_t *attacker)
 						}
 					}
 					else {
+						
 						attacker->client->resp.score++;
+												
+						if(!self->groundentity) {
+							attacker->client->resp.reward_pts+=3;
+							safe_centerprintf(attacker, "Midair shot!\n");
+						}
+						else
+							attacker->client->resp.reward_pts++;
+
+						if(mod == MOD_HEADSHOT) { //3 more pts for a headshot
+							attacker->client->resp.reward_pts+=3;
+							safe_centerprintf(attacker, "HEADSHOT!\n");
+							gi.sound(attacker, CHAN_AUTO, gi.soundindex("misc/headshot.wav"), 1, ATTN_STATIC, 0);  
+						}
 
 						//mutators
 						if(vampire->value) {
@@ -459,6 +484,7 @@ void ClientObituary (edict_t *self, edict_t *inflictor, edict_t *attacker)
 								attacker->health = attacker->max_health;
 						}
 						self->client->resp.deaths++;
+
 						if (((int)(dmflags->value) & DF_SKINTEAMS)  && !ctf->value) {
 							if(attacker->dmteam == RED_TEAM){
 								red_team_score++;
@@ -493,6 +519,7 @@ void ClientObituary (edict_t *self, edict_t *inflictor, edict_t *attacker)
 									safe_centerprintf(cl_ent, "%s is on a rampage!\n", cleanname2);
 								}
 								gi.sound (self, CHAN_AUTO, gi.soundindex("misc/rampage.wav"), 1, ATTN_NONE, 0);
+								attacker->client->resp.reward_pts+=10;
 								break;
 							case 8:
 								for (i=0 ; i<maxclients->value ; i++)
@@ -512,6 +539,7 @@ void ClientObituary (edict_t *self, edict_t *inflictor, edict_t *attacker)
 									safe_centerprintf(cl_ent, "%s is a god!\n", cleanname2);
 								}
 								gi.sound (self, CHAN_AUTO, gi.soundindex("misc/godlike.wav"), 1, ATTN_NONE, 0);
+								attacker->client->resp.reward_pts+=20;
 								break;
 							default:
 								break;
@@ -528,6 +556,20 @@ void ClientObituary (edict_t *self, edict_t *inflictor, edict_t *attacker)
 					}
 
 				}
+				if(attacker->client->resp.reward_pts >= g_reward->integer && !attacker->client->resp.powered) { //give them speed and invis powerups
+					it = FindItem("Invisibility");
+					attacker->client->pers.inventory[ITEM_INDEX(it)] += 1;
+
+					it = FindItem("Sproing");
+					attacker->client->pers.inventory[ITEM_INDEX(it)] += 1;
+
+					it = FindItem("Haste");
+					attacker->client->pers.inventory[ITEM_INDEX(it)] += 1;
+
+					attacker->client->resp.powered = true;
+
+					gi.sound (attacker, CHAN_AUTO, gi.soundindex("misc/pc_up.wav"), 1, ATTN_STATIC, 0);
+				}
 				self->client->kill_streak = 0; //reset, you are dead
 				return;
 			}
@@ -535,6 +577,22 @@ void ClientObituary (edict_t *self, edict_t *inflictor, edict_t *attacker)
 
 	}
 
+	if(mod == MOD_DEATHRAY) {
+		safe_bprintf(PRINT_MEDIUM, "%s killed by Deathray!\n", self->client->pers.netname);
+
+		//immune player (activator) gets score increase
+		for (i=0 ; i<maxclients->value ; i++)
+		{
+			cl_ent = g_edicts + 1 + i;
+			if (!cl_ent->inuse || cl_ent->is_bot)
+				continue;
+			if(cl_ent->client)
+				if(cl_ent->client->rayImmunity)
+					cl_ent->client->resp.score++;
+		}
+		return;
+	}
+
 	safe_bprintf (PRINT_MEDIUM,"%s died.\n", self->client->pers.netname);
 	if (deathmatch->value) {
 		self->client->resp.score--;
@@ -598,7 +656,7 @@ void TossClientWeapon (edict_t *self)
 		drop->nextthink = level.time + (self->client->quad_framenum - level.framenum) * FRAMETIME;
 		drop->think = G_FreeEdict;
 	}
-	if (sproing)
+	if (sproing && !self->client->resp.powered)
 	{
 		self->client->v_angle[YAW] += spread;
 		drop = Drop_Item (self, FindItemByClassname ("item_sproing"));
@@ -609,7 +667,7 @@ void TossClientWeapon (edict_t *self)
 		drop->nextthink = level.time + (self->client->sproing_framenum - level.framenum) * FRAMETIME;
 		drop->think = G_FreeEdict;
 	}
-	if (haste)
+	if (haste && !self->client->resp.powered)
 	{
 		self->client->v_angle[YAW] += spread;
 		drop = Drop_Item (self, FindItemByClassname ("item_haste"));
@@ -661,6 +719,8 @@ void player_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damag
 	int	got_vehicle = 0;
 	int	number_of_gibs = 0;
 	int	gib_effect = EF_GREENGIB;
+	int hasFlag = false;
+	gitem_t *it, *flag1_item, *flag2_item;
 
 	if (self->in_vehicle) {
 		Reset_player(self);	//get the player out of the vehicle
@@ -673,7 +733,6 @@ void player_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damag
 	self->takedamage = DAMAGE_YES;
 	self->movetype = MOVETYPE_TOSS;
 
-
 	info = Info_ValueForKey (self->client->pers.userinfo, "skin");
 
 	self->s.modelindex2 = 0;	// remove linked weapon model
@@ -710,8 +769,27 @@ void player_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damag
 			if(!excessive->value)
 				TossClientWeapon (self);
 		}
-		if(ctf->value)
+
+		if(ctf->value) {			
+			//check to see if they had a flag
+			flag1_item = flag2_item = NULL;
+
+			flag1_item = FindItemByClassname("item_flag_red");
+			flag2_item = FindItemByClassname("item_flag_blue");
+
+			if (self->client->pers.inventory[ITEM_INDEX(flag1_item)] || self->client->pers.inventory[ITEM_INDEX(flag1_item)]) 
+				hasFlag = true;
+
 			CTFDeadDropFlag(self);
+			if(anticamp->value && meansOfDeath == MOD_SUICIDE && hasFlag) {
+				
+				//make campers really pay for hiding flags
+				if(self->dmteam == BLUE_TEAM)
+					CTFResetFlag(RED_TEAM);
+				else
+					CTFResetFlag(BLUE_TEAM);
+			}
+		}
 		if(self->in_deathball)
 			DeadDropDeathball(self);
 
@@ -720,6 +798,24 @@ void player_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damag
 		if (deathmatch->value)
 			Cmd_Help_f (self);		// show scores
 
+		if(self->health < -40 && attacker->client) {
+			attacker->client->resp.reward_pts++;
+			if(attacker->client->resp.reward_pts >= g_reward->integer && !attacker->client->resp.powered) { //give them speed and invis powerups
+				it = FindItem("Invisibility");
+				attacker->client->pers.inventory[ITEM_INDEX(it)] += 1;
+
+				it = FindItem("Sproing");
+				attacker->client->pers.inventory[ITEM_INDEX(it)] += 1;
+
+				it = FindItem("Haste");
+				attacker->client->pers.inventory[ITEM_INDEX(it)] += 1;
+
+				attacker->client->resp.powered = true;
+
+				gi.sound (attacker, CHAN_VOICE, gi.soundindex("misc/pc_up.wav"), 1, ATTN_STATIC, 0);
+			}
+		}
+
 	}
 
 	// remove powerups
@@ -727,20 +823,16 @@ void player_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damag
 	self->client->invincible_framenum = 0;
 	self->client->haste_framenum = 0;
 	self->client->sproing_framenum = 0;
+	self->client->invis_framenum = 0;
 
 	// clear inventory
 	memset(self->client->pers.inventory, 0, sizeof(self->client->pers.inventory));
-
+	
 	if (self->health < -40)
 	{	// gib
 		self->takedamage	= DAMAGE_NO;
 		self->s.modelindex3	= 0;    //remove helmet, if a martian
 	
-		gi.WriteByte (svc_temp_entity);
-		gi.WriteByte (TE_DEATHFIELD);
-		gi.WritePosition (self->s.origin);
-		gi.multicast (self->s.origin, MULTICAST_PVS);
-
 		if(self->client->chasetoggle == 1)
 		{
 			/* If deathcam is active, switch client model to nothing */
@@ -758,6 +850,11 @@ void player_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damag
 
 		if(self->ctype == 0) { //alien
 
+			gi.WriteByte (svc_temp_entity);
+			gi.WriteByte (TE_DEATHFIELD);
+			gi.WritePosition (self->s.origin);
+			gi.multicast (self->s.origin, MULTICAST_PVS);
+
 			for (n= 0; n < number_of_gibs; n++) {
 				ThrowGib (self, "models/objects/gibs/mart_gut/tris.md2", damage, GIB_METALLIC, EF_GREENGIB);
 				ThrowGib (self, "models/objects/debris2/tris.md2", damage, GIB_METALLIC, 0);
@@ -776,16 +873,18 @@ void player_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damag
 			gi.multicast (self->s.origin, MULTICAST_PHS);
 		}
 		else { //human
+
+			gi.WriteByte (svc_temp_entity);
+			gi.WriteByte (TE_DEATHFIELD2);
+			gi.WritePosition (self->s.origin);
+			gi.WriteDir (self->s.angles);
+			gi.multicast (self->s.origin, MULTICAST_PVS);
+
 			gib_effect = EF_GIB;
 			for (n= 0; n < number_of_gibs; n++)
 				ThrowGib (self, "models/objects/gibs/sm_meat/tris.md2", damage, GIB_METALLIC, EF_GIB);
 		}
 
-		//don't use until 6.11 is released, so we don't have lots of people with 
-		//triangles everywhere.  We could precache all this stuff tho and put the models
-		//on icculus perhaps, but we are having problems with the udp downloading of models.
-		//So, better safe than sorry and don't precache.
-		//So in other words, the flag files just won't be uploaded to SVN until release time.
 		if(self->usegibs) {
 			ThrowGib (self, self->head, damage, GIB_ORGANIC, gib_effect);
 			ThrowGib (self, self->leg, damage, GIB_ORGANIC, gib_effect);
@@ -826,6 +925,8 @@ void player_die (edict_t *self, edict_t *inflictor, edict_t *attacker, int damag
 		}
 	}
 
+	gi.sound (self, CHAN_VOICE, gi.soundindex("misc/death.wav"), 1, ATTN_STATIC, 0);
+
 	self->deadflag = DEAD_DEAD;
 
 	gi.linkentity (self);
@@ -925,6 +1026,18 @@ void InitClientPersistant (gclient_t *client)
 		client->pers.inventory[ITEM_INDEX(item)] = 1;
 	}
 
+	//if powered up, give back powerups
+	if(client->resp.powered) {
+		item = FindItem("Invisibility");
+		client->pers.inventory[ITEM_INDEX(item)] += 1;
+
+		item = FindItem("Sproing");
+		client->pers.inventory[ITEM_INDEX(item)] += 1;
+
+		item = FindItem("Haste");
+		client->pers.inventory[ITEM_INDEX(item)] += 1;
+	}
+
 	client->pers.connected = true;
 }
 
@@ -1092,7 +1205,7 @@ edict_t *SelectRandomCTFSpawnPoint (void)
 	int		count = 0;
 	int		selection;
 	float	range, range1, range2;
-	char	whichteam[16];
+	char	whichteam[32];
 
 	spot = NULL;
 	range1 = range2 = 99999;
@@ -1379,7 +1492,6 @@ void BodySink( edict_t *ent ) {
 	if ( level.time - ent->timestamp > 10.5 ) {
 		// the body ques are never actually freed, they are just unlinked
 		gi.unlinkentity( ent );
-		ent->solid = SOLID_NOT;
 		ent->s.modelindex = 0; //for good measure
 		ent->s.modelindex2 = 0;
 		ent->s.modelindex3 = 0;
@@ -1388,6 +1500,9 @@ void BodySink( edict_t *ent ) {
 	}
 	ent->nextthink = level.time + .1;
 	ent->s.origin[2] -= 1;
+	ent->s.effects |= EF_COLOR_SHELL;
+	ent->s.renderfx |= RF_SHELL_GREEN;
+	ent->solid = SOLID_NOT; //don't gib sinking bodies
 }
 
 void CopyToBodyQue (edict_t *ent)
@@ -1439,13 +1554,6 @@ void respawn (edict_t *self)
 	if (deathmatch->value)
 	{
 
-#if 0
-/* Don't think this is needed - deathcam is removed at very end of death,
-   so DeathcamRemove would get called twice */
-		if(!self->is_bot)
-			DeathcamRemove (self, "off");
-#endif
-
 // ACEBOT_ADD special respawning code
 		if (self->is_bot)
 		{
@@ -1597,6 +1705,12 @@ void PutClientInServer (edict_t *ent)
 	client->is_bot = 0;
 	client->kill_streak = 0;
 
+	client->mapvote = 0;
+
+	client->lasttaunttime = 0;
+
+	client->rayImmunity = false;
+
 	resp = client->resp;
 	memcpy (userinfo, client->pers.userinfo, sizeof(userinfo));
 	InitClientPersistant (client);
@@ -1800,24 +1914,41 @@ void PutClientInServer (edict_t *ent)
 	ChangeWeapon (ent);
 
 	client->spawnprotecttime = level.time;
+
+	//unlagged
+	if ( g_antilag->integer) {
+		G_ResetHistory( ent );
+		// and this is as good a time as any to clear the saved state
+		client->saved.leveltime = 0;
+	}
 }
 
 void ClientPlaceInQueue(edict_t *ent)
 {
 	int		i;
-	int		numplayers;
-	
-	numplayers = 0;
+	int		highestpos, induel, numplayers; 
 
+	highestpos = induel = numplayers = 0;
+	
 	for (i = 0; i < maxclients->value; i++) {
 		if(g_edicts[i+1].inuse && g_edicts[i+1].client) { 
+			if(g_edicts[i+1].client->pers.queue > highestpos) //only count players that are actually in
+				highestpos = g_edicts[i+1].client->pers.queue;
+			if(g_edicts[i+1].client->pers.queue && g_edicts[i+1].client->pers.queue < 3)
+				induel++;
 			if(g_edicts[i+1].client->pers.queue) //only count players that are actually in
 				numplayers++;
 		}
 	}
 
+	if(induel > 1) //make sure no more than two are in the duel at once
+		if(highestpos < 2)
+			highestpos = 2; //in case two people somehow managed to have pos 1
+	if(highestpos < numplayers)
+		highestpos = numplayers; 
+
 	if(!ent->client->pers.queue)
-		ent->client->pers.queue = numplayers+1;
+		ent->client->pers.queue = highestpos+1;
 }
 void ClientCheckQueue(edict_t *ent)
 {
@@ -1885,9 +2016,9 @@ void ClientBeginDeathmatch (edict_t *ent)
 {
 
 	FILE	*motd_file;
+	char	motd_file_name[MAX_QPATH];
 	char	line[80];
 	char	motd[500];
-	static char current_map[55];
 
 	G_InitEdict (ent);
 
@@ -1915,9 +2046,9 @@ void ClientBeginDeathmatch (edict_t *ent)
 		if(ent->dmteam == NO_TEAM) {
 			ent->client->showscores = true;
 			if(((int)(dmflags->value) & DF_SKINTEAMS) || ctf->value || tca->value || cp->value)
-				CTFScoreboardMessage (ent, NULL);
+				CTFScoreboardMessage (ent, NULL, false);
 			else
-				DeathmatchScoreboardMessage (ent, NULL);
+				DeathmatchScoreboardMessage (ent, NULL, false);
 			gi.unicast (ent, true);
 			ent->teamset = true;
 		}
@@ -1939,7 +2070,13 @@ void ClientBeginDeathmatch (edict_t *ent)
 
 	safe_bprintf (PRINT_HIGH, "%s entered the game\n", ent->client->pers.netname);
 
-	if ((motd_file = fopen("arena/motd.txt", "rb")) != NULL)
+	// get the name for the MOTD file
+	if ( motdfile && motdfile->string && motdfile->string[0] )
+		Com_sprintf (motd_file_name, sizeof(motd_file_name), "arena/%s", motdfile->string);
+	else
+		strcpy (motd_file_name, "arena/motd.txt");
+
+	if ((motd_file = fopen(motd_file_name, "rb")) != NULL)
 	{
 		// we successfully opened the file "motd.txt"
 		if ( fgets(motd, 500, motd_file) )
@@ -1960,15 +2097,9 @@ void ClientBeginDeathmatch (edict_t *ent)
 		fclose(motd_file);
 
 	}
-	else
-	
-	safe_centerprintf(ent,"\n======================================\nCodeRED ACE Bot's are running\non this server.\n\n'sv addbot' to add a new bot.\n'sv removebot <name>' to remove bot.\n======================================\n\n");
 
-	// If the map changes on us, init and reload the nodes.  
-	ACEND_InitNodes();
-	ACEND_LoadNodes();
+	//check bots with each player connect
 	ACESP_LoadBots(ent, 0);
-	strcpy(current_map,level.mapname);
 
 	// make sure all view stuff is valid
 	ClientEndServerFrame (ent);
@@ -2012,11 +2143,15 @@ The game can override any of the settings in place
 void ClientUserinfoChanged (edict_t *ent, char *userinfo, int whereFrom)
 {
 	char	*s;
+	edict_t *cl_ent;
 	int		playernum;
-	int		i, j, k, copychar, done;
+	int		i, j, k;
+	qboolean duplicate, done, copychar;
 	char playermodel[MAX_OSPATH] = " ";
 	char playerskin[MAX_INFO_STRING] = " ";
 	char modelpath[MAX_OSPATH] = " ";
+	char slot[2];
+	char colorName[32];
 	FILE *file;
 
 	// check for malformed or illegal info strings
@@ -2035,6 +2170,9 @@ void ClientUserinfoChanged (edict_t *ent, char *userinfo, int whereFrom)
 	if(whereFrom != SPAWN && whereFrom != CONNECT)
 		whereFrom = INGAME;
 
+	if(playervote.called && whereFrom == INGAME) 
+		return; //do not allow people to change info during votes
+
 	if((((int)(dmflags->value) & DF_SKINTEAMS) || ctf->value || tca->value || cp->value) && (ent->dmteam == RED_TEAM || ent->dmteam == BLUE_TEAM))
 		ent->client->pers.spectator = false; //cannot spectate if you've joined a team
 
@@ -2045,10 +2183,34 @@ void ClientUserinfoChanged (edict_t *ent, char *userinfo, int whereFrom)
 
 	// set name
 	s = Info_ValueForKey (userinfo, "name");
-	//fix player name if corrupted
-	if(s != NULL && strlen(s) > 16)
-		s[16] = 0;
-	strncpy (ent->client->pers.netname, s, sizeof(ent->client->pers.netname)-1);
+	strcpy(colorName, s);
+	i = j = k = 0;
+	while ( s && *s && i < 47 && j < 15 )
+	{
+		ent->client->pers.netname[i] = s[i];
+		if ( k == 0 )
+		{
+			if ( s[i] == Q_COLOR_ESCAPE )
+				k = 1;
+			else
+				j ++;
+		}
+		else if ( k == 1 )
+		{
+			if ( s[i] == Q_COLOR_ESCAPE )
+			{
+				j += 2;
+				if ( j == 16 )
+				{
+					i --;
+					break;
+				}
+			}
+			k = 0;
+		}
+		i ++;
+	}
+	ent->client->pers.netname[i] = 0;
 
 	//spectator mode
 	// set spectator
@@ -2132,6 +2294,42 @@ void ClientUserinfoChanged (edict_t *ent, char *userinfo, int whereFrom)
 	}
 	playernum = ent-g_edicts-1;
 
+	//check for duplicates(but not on respawns)
+	duplicate = false;
+	if(whereFrom != SPAWN) { 
+
+		for (j=0; j<maxclients->value ; j++) {
+
+			cl_ent = g_edicts + 1 + j;
+			if (!cl_ent->inuse)
+				continue;
+
+			if(!strcmp(ent->client->pers.netname, cl_ent->client->pers.netname)) {
+
+				if(playernum != j)
+					duplicate = true;
+			}
+		}
+		
+		if(duplicate && playernum < 100) { //just paranoia, should never be more than 64
+
+			sprintf(slot, "%i", playernum);
+
+			if(strlen(ent->client->pers.netname) < 31) { //small enough, just add to end
+
+				strcat(ent->client->pers.netname, slot);
+			}
+			else { //need to lop off end first
+
+				ent->client->pers.netname[30] = 0;
+				strcat(ent->client->pers.netname, slot);
+			}
+
+			Info_SetValueForKey (userinfo, "name", ent->client->pers.netname);
+			safe_bprintf(PRINT_HIGH, "Was a duplicate, changing name to %s\n", ent->client->pers.netname);
+		}
+	}
+
 	// combine name and skin into a configstring
 	gi.configstring (CS_PLAYERSKINS+playernum, va("%s\\%s", ent->client->pers.netname, s) );
 
@@ -2336,8 +2534,7 @@ qboolean ClientConnect (edict_t *ent, char *userinfo)
 	//spectator mode
 	// check for a spectator
 	value = Info_ValueForKey (userinfo, "spectator");
-	if (deathmatch->value && *value && strcmp(value, "0")) {
-		
+	if (deathmatch->value && *value && strcmp(value, "0")) {		
 
 		if (*spectator_password->string &&
 			strcmp(spectator_password->string, "none") &&
@@ -2504,24 +2701,77 @@ void ClientThink (edict_t *ent, usercmd_t *ucmd)
 {
 	gclient_t	*client;
 	edict_t	*other;
-	int		i, j;
+	int		i, j, mostvotes, n_candidates;
+	int		map_candidates[4];
 	pmove_t	pm;
 	qboolean sproing, haste;
-	vec3_t addspeed, forward, up;
+	vec3_t addspeed, forward, up, right;
 
 	level.current_entity = ent;
 	client = ent->client;
 
+	//unlagged
+	if ( g_antilag->integer)		
+		client->attackTime = gi.Sys_Milliseconds(); 
+
 	if (level.intermissiontime)
 	{
-
 		client->ps.pmove.pm_type = PM_FREEZE;
-		// can exit intermission after ten seconds
-		if (level.time > level.intermissiontime + 10.0
-			&& (ucmd->buttons & BUTTON_ANY) )
-			level.exitintermission = true;
+		
+		// can exit intermission after 10 seconds, or 20 if map voting enables
+		// (voting will only work if g_mapvote wasn't modified during intermission)
+		if (g_mapvote->value && ! g_mapvote->modified) {
+
+			//print out results, track winning map
+			mostvotes = 0;
+			
+			for (j = 0; j < 4; j++) {
+				if (votedmap[j].tally > mostvotes)
+					mostvotes = votedmap[j].tally;
+			}
+
+			if ( g_voterand && g_voterand->value )
+			{
+				// we're using a random value for the next map
+				// if a choice needs to be done
+				n_candidates = 0;
+				for ( j = 0 ; j < 4 ; j ++ ) {
+					if ( votedmap[j].tally < mostvotes )
+						continue;
+					map_candidates[n_candidates ++] = j;
+				}
+
+				j = random() * (n_candidates - 1);
+				strcpy(level.changemap, votedmap[map_candidates[j]].mapname);
+			}
+			else
+			{
+				// "old" voting system, take the first map that
+				// has enough votes
+				for ( j = 0 ; j < 4 ; j ++ ) {
+					i = (j + 1) % 4;
+					if ( votedmap[i].tally < mostvotes )
+						continue;
+					strcpy(level.changemap, votedmap[i].mapname);
+					break;
+				}
+			}
+			
+			if (level.time > level.intermissiontime + 20.0
+				&& (ucmd->buttons & BUTTON_ANY) )
+				level.exitintermission = true;
+		}
+		else {		
+			if (level.time > level.intermissiontime + 10.0
+				&& (ucmd->buttons & BUTTON_ANY) )
+				level.exitintermission = true;
+		}
 		return;
 	}
+	else if ( g_mapvote && g_mapvote->modified )
+	{
+		g_mapvote->modified = false;
+	}
 
 	pm_passent = ent;
 
@@ -2663,9 +2913,20 @@ void ClientThink (edict_t *ent, usercmd_t *ucmd)
 		{
 			sproing = client->sproing_framenum > level.framenum;
 			haste = client->haste_framenum > level.framenum;
-			if(sproing)
+			if(sproing) {
 				gi.sound(ent, CHAN_VOICE, gi.soundindex("items/sproing.wav"), 1, ATTN_NORM, 0);
+				ent->velocity[2] += 400;
+			}
 			if(haste) {
+				AngleVectors (ent->s.angles, addspeed, right, up);
+				addspeed[0] *= 400;
+				addspeed[1] *= 400;
+				for(i = 0; i < 2; i++) {
+				if(addspeed[i] > 200)
+					addspeed[i] = 200;
+				}
+				VectorAdd(ent->velocity, addspeed, ent->velocity);
+
 				gi.sound(ent, CHAN_VOICE, gi.soundindex("items/haste.wav"), 1, ATTN_NORM, 0);
 				gi.WriteByte (svc_temp_entity);
 				gi.WriteByte (TE_EXPLOSION2);
@@ -2737,7 +2998,6 @@ void ClientThink (edict_t *ent, usercmd_t *ucmd)
 			if(client->resp.spectator < 2) {
 				if((ctf->value || tca->value || cp->value) && (ent->dmteam == RED_TEAM || ent->dmteam == BLUE_TEAM)) {
 					client->pers.spectator = false; //we have a team, join
-				//	safe_bprintf(PRINT_HIGH, "red: %i blue: %i\n", red_team_cnt, blue_team_cnt);
 				}
 				else if((((int)(dmflags->value) & DF_SKINTEAMS) || (ctf->value || tca->value || cp->value) && ent->dmteam == NO_TEAM) && client->resp.spectator < 3) {
 					if(red_team_cnt < blue_team_cnt)
@@ -2746,7 +3006,6 @@ void ClientThink (edict_t *ent, usercmd_t *ucmd)
 						ent->dmteam = BLUE_TEAM;
 					client->pers.spectator = false;
 					ClientChangeSkin(ent);
-				//	safe_bprintf(PRINT_HIGH, "autojoin red: %i blue: %i\n", red_team_cnt, blue_team_cnt);
 				}
 			}
 
@@ -2764,14 +3023,20 @@ void ClientThink (edict_t *ent, usercmd_t *ucmd)
 
 	if (client->resp.spectator) {
 		if((((int)(dmflags->value) & DF_SKINTEAMS) || ctf->value || tca->value || cp->value) && client->resp.spectator < 2) {
-			if(ent->dmteam == NO_TEAM && (level.time/2 == ceil(level.time/2)))
-				safe_centerprintf(ent, "\n\n\nPress <fire> to autojoin\nor <jump> to join BLUE\nor <crouch> to join RED\n");
+			if(ent->dmteam == NO_TEAM && (level.time/2 == ceil(level.time/2))) {
+				if(g_autobalance->value)
+					safe_centerprintf(ent, "\n\n\nPress <fire> to join\nautobalanced team\n");
+				else
+					safe_centerprintf(ent, "\n\n\nPress <fire> to autojoin\nor <jump> to join BLUE\nor <crouch> to join RED\n");
+			}
 		}
 		if (ucmd->upmove >= 10) {
-			if(((((int)(dmflags->value) & DF_SKINTEAMS) || ctf->value || tca->value || cp->value) && ent->dmteam == NO_TEAM) && client->resp.spectator < 2) {
-				ent->dmteam = BLUE_TEAM; //join BLUE
-				client->pers.spectator = false;
-				ClientChangeSkin(ent);
+			if(!(g_autobalance->value)) {
+				if(((((int)(dmflags->value) & DF_SKINTEAMS) || ctf->value || tca->value || cp->value) && ent->dmteam == NO_TEAM) && client->resp.spectator < 2) {
+					ent->dmteam = BLUE_TEAM; //join BLUE
+					client->pers.spectator = false;
+					ClientChangeSkin(ent);
+				}
 			}
 			if (!(client->ps.pmove.pm_flags & PMF_JUMP_HELD)) {
 				client->ps.pmove.pm_flags |= PMF_JUMP_HELD;
@@ -2782,7 +3047,7 @@ void ClientThink (edict_t *ent, usercmd_t *ucmd)
 			}
 		}
 		else if(((((int)(dmflags->value) & DF_SKINTEAMS) || ctf->value || tca->value || cp->value) && ent->dmteam == NO_TEAM &&
-			(ucmd->upmove < 0))  && client->resp.spectator < 2){
+			(ucmd->upmove < 0))  && client->resp.spectator < 2 && !(g_autobalance->value)){
 
 			ent->dmteam = RED_TEAM; //join RED
 			client->pers.spectator = false;
@@ -2884,7 +3149,7 @@ void ClientBeginServerFrame (edict_t *ent)
 		{
 			// in deathmatch, only wait for attack button
 			if (deathmatch->value)
-				buttonMask = BUTTON_ATTACK;
+				buttonMask = BUTTON_ATTACK | BUTTON_ATTACK2;
 			else
 				buttonMask = -1;
 
