@@ -49,6 +49,7 @@ void MoveClientToIntermission (edict_t *ent)
 	ent->client->invincible_framenum = 0;
 	ent->client->haste_framenum = 0;
     ent->client->sproing_framenum = 0;
+	ent->client->invis_framenum = 0;
 	ent->client->grenade_blew_up = false;
 	ent->client->grenade_time = 0;
 
@@ -65,7 +66,10 @@ void MoveClientToIntermission (edict_t *ent)
 
 	if (deathmatch->value)
 	{
-		DeathmatchScoreboardMessage (ent, NULL);
+		if(g_mapvote->value)
+			DeathmatchScoreboardMessage (ent, NULL, true);
+		else
+			DeathmatchScoreboardMessage (ent, NULL, false);
 		gi.unicast (ent, true);
 	}
 
@@ -118,6 +122,7 @@ void PlaceWinnerOnVictoryPad(edict_t *winner, int offset)
 	winner->client->invincible_framenum = 0;
 	winner->client->haste_framenum = 0;
     winner->client->sproing_framenum = 0;
+	winner->client->invis_framenum = 0;
 	winner->client->grenade_blew_up = false;
 	winner->client->grenade_time = 0;
 
@@ -131,7 +136,10 @@ void PlaceWinnerOnVictoryPad(edict_t *winner, int offset)
 
 	if (deathmatch->value)
 	{
-		DeathmatchScoreboardMessage (winner, NULL);
+		if(g_mapvote->value)
+			DeathmatchScoreboardMessage (winner, NULL, true);
+		else
+			DeathmatchScoreboardMessage (winner, NULL, false);
 		gi.unicast (winner, true);
 	}
 
@@ -261,6 +269,8 @@ void BeginIntermission (edict_t *targ)
 			respawn(client);
 			client->deadflag = DEAD_DEAD; //so we can know if he's dead for placement offsetting
 		}
+		if(!client->is_bot && g_mapvote->value)
+			safe_centerprintf(client, "Type \"vote #\" to vote for next map!");
 	}
 
 	level.intermissiontime = level.time;
@@ -398,35 +408,67 @@ void BeginIntermission (edict_t *targ)
 
 }
 
+//duel code
+int highestpos, numplayers;
+void MoveEveryoneDownQueue(void) {
+	int i, induel = 0;
+
+	for (i = 0; i < maxclients->value; i++) {
+		if(g_edicts[i+1].inuse && g_edicts[i+1].client) {
+			//move everyone else down a notch(never less than 0)
+			if(induel > 1 && g_edicts[i+1].client->pers.queue <= 3) //houston, we have a problem
+				return; //abort, stop moving people
+			if(g_edicts[i+1].client->pers.queue > 1) 
+					g_edicts[i+1].client->pers.queue-=1; 
+			if(g_edicts[i+1].client->pers.queue < 3)
+				induel++;
+		}		
+	}
+}
 void CheckDuelWinner(void) {
 	int	i;
-	int highscore, numplayers;
+	int highscore, induel;
 
 	highscore = 0;
 	numplayers = 0;
+	highestpos = 0;
+	induel = 0;
 
 	for (i = 0; i < maxclients->value; i++) {
 		if(g_edicts[i+1].inuse && g_edicts[i+1].client) { 
 			if(g_edicts[i+1].client->resp.score > highscore)
 				highscore = g_edicts[i+1].client->resp.score;
-			if(g_edicts[i+1].client->pers.queue) //only people who are connected
-				numplayers++;
+			if(g_edicts[i+1].client->pers.queue > highestpos)
+				highestpos = g_edicts[i+1].client->pers.queue;
+			numplayers++;
 		}
 	}
+	if(highestpos < numplayers)
+		highestpos = numplayers;
 
 	for (i = 0; i < maxclients->value; i++) {
 		if(g_edicts[i+1].inuse && g_edicts[i+1].client) {
 			if((g_edicts[i+1].client->resp.score < highscore) && g_edicts[i+1].client->pers.queue < 3) {
-				g_edicts[i+1].client->pers.queue = numplayers+1; //loser, kicked to the back of the line
+				g_edicts[i+1].client->pers.queue = highestpos+1; //loser, kicked to the back of the line
+				highestpos++; 
 			}
 		}
 	}
-	for (i = 0; i < maxclients->value; i++) {
-		if(g_edicts[i+1].inuse && g_edicts[i+1].client) {
-			//move everyone else down a notch(never less than 0)
-			if(g_edicts[i+1].client->pers.queue > 1) 
-					g_edicts[i+1].client->pers.queue-=1; 
+
+	MoveEveryoneDownQueue();
+
+	//last resort checking after positions have changed
+	//check for any screwups and correct the queue
+	while(induel < 2 && numplayers > 1) {
+		induel = 0;
+		for (i = 0; i < maxclients->value; i++) {
+			if(g_edicts[i+1].inuse && g_edicts[i+1].client) {
+				if(g_edicts[i+1].client->pers.queue < 3 && g_edicts[i+1].client->pers.queue)
+					induel++;			
+			}
 		}
+		if(induel < 2) //something is wrong(i.e winner left), move everyone down.
+			MoveEveryoneDownQueue();
 	}
 }
 
@@ -470,7 +512,7 @@ DeathmatchScoreboardMessage
 
 ==================
 */
-void DeathmatchScoreboardMessage (edict_t *ent, edict_t *killer)
+void DeathmatchScoreboardMessage (edict_t *ent, edict_t *killer, int mapvote)
 {
 	char	entry[1024];
 	char	string[1400];
@@ -484,13 +526,12 @@ void DeathmatchScoreboardMessage (edict_t *ent, edict_t *killer)
 	edict_t		*cl_ent;
 	char    acc[16];
 	char	weapname[16];
-// ACEBOT_ADD
+
 	if (ent->is_bot)
 		return;
-// ACEBOT_END
 
 	if (((int)(dmflags->value) & DF_SKINTEAMS) || ctf->value || tca->value || cp->value) {
-		CTFScoreboardMessage (ent, killer);
+		CTFScoreboardMessage (ent, killer, mapvote);
 		return;
 	}
 	// sort the clients by score
@@ -521,7 +562,12 @@ void DeathmatchScoreboardMessage (edict_t *ent, edict_t *killer)
 	string[0] = 0;
 	
 	stringlength = strlen(string);
-	
+
+	Com_sprintf (entry, sizeof(entry), "newsb ");
+	j = strlen(entry);
+	strcpy (string + stringlength, entry);
+	stringlength += j;
+		
 	// add the clients in sorted order
 	if (total > 12)
 		total = 12;
@@ -531,12 +577,13 @@ void DeathmatchScoreboardMessage (edict_t *ent, edict_t *killer)
 		cl = &game.clients[sorted[i]];
 		cl_ent = g_edicts + 1 + sorted[i];
 
-		x = (i>=6) ? 160 : 0;
-		y = 32 + 32 * (i%6);
-
+		x = 0;
+		y = 32 + 32 * (i%12);
+		 
 		// add a background
 		Com_sprintf (entry, sizeof(entry),
-			"xv %i yv %i picn %s ",x+32, y, "tag2");
+			"xv %i yv %i picn %s ",x, y, "playerbox");
+
 		j = strlen(entry);
 		if (stringlength + j > 1024)
 			break;
@@ -551,9 +598,8 @@ void DeathmatchScoreboardMessage (edict_t *ent, edict_t *killer)
 		else //duel mode will have queued spectators
 			Com_sprintf (entry, sizeof(entry),
 				"queued %i %i %i %i %i %i ",
-				x, y, sorted[i], cl->resp.score, cl->ping, cl->pers.queue-2);
-	
-		
+				x, y, sorted[i], cl->resp.score, cl->ping, cl->pers.queue-2);	
+					
 		j = strlen(entry);
 		if (stringlength + j > 1024)
 			break;
@@ -561,86 +607,126 @@ void DeathmatchScoreboardMessage (edict_t *ent, edict_t *killer)
 		stringlength += j;
 	}
 
-	//weapon accuracy
-
-	//add a background
-	x = 0;
-	y = (total>=6) ? (32+(32*5)) : (32*total);
-	for (i=0 ; i<3 ; i++)
-	{
-	
+	//weapon accuracy(don't do if map voting)
+	if(!mapvote) {
+		//add a background
+		x = 96;
+		y = 16*(total+1);
+		
 		Com_sprintf (entry, sizeof(entry),
-			"xv %i yv %i picn %s ", x, y+((i+1)*32+24), "tag2");
-		j = strlen(entry);
-		if (stringlength + j > 1024)
-			break;
-		strcpy (string + stringlength, entry);
-		stringlength += j;
-		
-		// send the layout
-		if (stringlength + j > 1024)
-			break;
-	}
-
-	x = 0;
-	y = (total>=6) ? (32+(32*5)) : (32*total);
-	Com_sprintf(entry, sizeof(entry),
-		"xv %i yv %i string Accuracy ", x, y+56);
-	j = strlen(entry);
-	if(stringlength + j < 1024) {
-		strcpy(string + stringlength, entry);
-		stringlength +=j;
-	}
-	for(i = 0; i < 9; i++) {
-
-		//in case of scenarios where a weapon is scoring multiple hits per shot(ie smartgun)
-		if(ent->client->resp.weapon_hits[i] > ent->client->resp.weapon_shots[i])
-			ent->client->resp.weapon_hits[i] = ent->client->resp.weapon_shots[i];
-
-		if(ent->client->resp.weapon_shots[i] == 0)
-			strcpy(acc, "0%");
-		else {
-			sprintf(acc, "%i", (100*ent->client->resp.weapon_hits[i]/ent->client->resp.weapon_shots[i]));
-			strcat(acc, "%");
-		}
-		switch (i) {
-			case 0:
-				strcpy(weapname, "blaster");
-				break;
-			case 1:
-				strcpy(weapname, "disruptor");
-				break;
-			case 2:
-				strcpy(weapname, "smartgun");
-				break;
-			case 3:
-				strcpy(weapname, "chaingun");
-				break;
-			case 4:
-				strcpy(weapname, "flame");
-				break;
-			case 5:
-				strcpy(weapname, "rocket");
-				break;
-			case 6:
-				strcpy(weapname, "beamgun");
-				break;
-			case 7:
-				strcpy(weapname, "vaporizer");
-				break;
-			case 8:
-				strcpy(weapname, "violator");
-				break;
-		}
-		Com_sprintf(entry, sizeof(entry),
-			"xv %i yv %i string %s xv %i string %s ", x, y+((i+1)*9)+64, weapname, x+96, acc);
+			"xv %i yv %i picn %s ", x-4, y+48, "statbox");
 		j = strlen(entry);
 		if(stringlength + j < 1024) {
 			strcpy(string + stringlength, entry);
 			stringlength +=j;
 		}
+		
+		Com_sprintf(entry, sizeof(entry),
+			"xv %i yv %i string Accuracy ", x, y+56);
+		j = strlen(entry);
+		if(stringlength + j < 1024) {
+			strcpy(string + stringlength, entry);
+			stringlength +=j;
+		}
+		for(i = 0; i < 9; i++) {
+
+			//in case of scenarios where a weapon is scoring multiple hits per shot(ie smartgun)
+			if(ent->client->resp.weapon_hits[i] > ent->client->resp.weapon_shots[i])
+				ent->client->resp.weapon_hits[i] = ent->client->resp.weapon_shots[i];
+
+			if(ent->client->resp.weapon_shots[i] == 0)
+				strcpy(acc, "0%");
+			else {
+				sprintf(acc, "%i", (100*ent->client->resp.weapon_hits[i]/ent->client->resp.weapon_shots[i]));
+				strcat(acc, "%");
+			}
+			switch (i) {
+				case 0:
+					strcpy(weapname, "blaster");
+					break;
+				case 1:
+					strcpy(weapname, "disruptor");
+					break;
+				case 2:
+					strcpy(weapname, "smartgun");
+					break;
+				case 3:
+					strcpy(weapname, "chaingun");
+					break;
+				case 4:
+					strcpy(weapname, "flame");
+					break;
+				case 5:
+					strcpy(weapname, "rocket");
+					break;
+				case 6:
+					strcpy(weapname, "beamgun");
+					break;
+				case 7:
+					strcpy(weapname, "vaporizer");
+					break;
+				case 8:
+					strcpy(weapname, "violator");
+					break;
+			}
+			Com_sprintf(entry, sizeof(entry),
+				"xv %i yv %i string %s xv %i string %s ", x, y+((i+1)*9)+64, weapname, x+96, acc);
+			j = strlen(entry);
+			if(stringlength + j < 1024) {
+				strcpy(string + stringlength, entry);
+				stringlength +=j;
+			}
+		}
 	}
-	
+	//map voting
+	if(mapvote) {
+		y = 64;
+		x = 96;
+		Com_sprintf(entry, sizeof(entry), 
+			"xv %i yt %i string Vote ", x, y);
+		j = strlen(entry);
+		if(stringlength + j < 1024) {
+			strcpy(string + stringlength, entry);
+			stringlength +=j;
+		}
+		x = 136;
+		Com_sprintf(entry, sizeof(entry), 
+			"xv %i yt %i string for ", x, y);
+		j = strlen(entry);
+		if(stringlength + j < 1024) {
+			strcpy(string + stringlength, entry);
+			stringlength +=j;
+		}
+		x = 168;
+		Com_sprintf(entry, sizeof(entry), 
+			"xv %i yt %i string next ", x, y);
+		j = strlen(entry);
+		if(stringlength + j < 1024) {
+			strcpy(string + stringlength, entry);
+			stringlength +=j;
+		}
+		x = 208;
+		Com_sprintf(entry, sizeof(entry), 
+			"xv %i yt %i string map: ", x, y);
+		j = strlen(entry);
+		if(stringlength + j < 1024) {
+			strcpy(string + stringlength, entry);
+			stringlength +=j;
+		}
+		x = 96;
+		for(i=0; i<4; i++) {
+			
+			Com_sprintf(entry, sizeof(entry), 
+			"xv %i yt %i string F%i.%s ", x, y+((i+1)*9)+9, i+1, votedmap[i].mapname);
+			j = strlen(entry);
+			if(stringlength + j < 1024) {
+				strcpy(string + stringlength, entry);
+				stringlength +=j;
+			}
+		}
+
+	}
+
 	gi.WriteByte (svc_layout);
 	gi.WriteString (string);
 }
@@ -661,7 +747,7 @@ void DeathmatchScoreboard (edict_t *ent)
 		return;
 // ACEBOT_END
 
-	DeathmatchScoreboardMessage (ent, ent->enemy);
+	DeathmatchScoreboardMessage (ent, ent->enemy, false);
 	gi.unicast (ent, true);
 }
 
@@ -691,56 +777,6 @@ void Cmd_Score_f (edict_t *ent)
 	DeathmatchScoreboard (ent);
 }
 
-
-/*
-==================
-HelpComputer
-
-Draw help computer.
-==================
-*/
-void HelpComputer (edict_t *ent)
-{
-	char	string[1024];
-	char	*sk;
-
-// ACEBOT_ADD
-	if (ent->is_bot)
-		return;
-// ACEBOT_END
-
-	if (skill->value == 0)
-		sk = "easy";
-	else if (skill->value == 1)
-		sk = "medium";
-	else if (skill->value == 2)
-		sk = "hard";
-	else
-		sk = "hard+";
-	//We are going to simplify this layout considerably.
-	// send the layout
-	Com_sprintf (string, sizeof(string),
-		"xv -32 yv 0 picn help "			// background
-//		"xv 202 yv 12 string2 \"%s\" "		// skill
-//		"xv 0 yv 24 cstring2 \"%s\" "		// level name
-		"xv 0 yv 54 cstring2 \"%s\" ",		// help 1
-//		"xv 0 yv 110 cstring2 \"%s\" "		// help 2
-//		"xv 50 yv 164 string2 \" kills     goals    secrets\" "
-//		"xv 50 yv 172 string2 \"%3i/%3i     %i/%i       %i/%i\" ", 
-//		sk,
-//		level.level_name,
-		game.helpmessage1);
-//		game.helpmessage2,
-//		level.killed_monsters, level.total_monsters, 
-//		level.found_goals, level.total_goals,
-//		level.found_secrets, level.total_secrets);
-
-	gi.WriteByte (svc_layout);
-	gi.WriteString (string);
-	gi.unicast (ent, true);
-}
-
-
 /*
 ==================
 Cmd_Help_f
@@ -756,19 +792,6 @@ void Cmd_Help_f (edict_t *ent)
 		Cmd_Score_f (ent);
 		return;
 	}
-
-	ent->client->showinventory = false;
-	ent->client->showscores = false;
-
-	if (ent->client->showhelp && (ent->client->pers.game_helpchanged == game.helpchanged))
-	{
-		ent->client->showhelp = false;
-		return;
-	}
-
-	ent->client->showhelp = true;
-	ent->client->pers.helpchanged = 0;
-	HelpComputer (ent);
 }
 
 
@@ -818,17 +841,17 @@ void G_SetStats (edict_t *ent)
 	//
 
 	index = ArmorIndex (ent);
-	if (index)
-	{
-		item = GetItemByIndex (index);
-		ent->client->ps.stats[STAT_ARMOR_ICON] = gi.imageindex (item->icon);
-		ent->client->ps.stats[STAT_ARMOR] = ent->client->pers.inventory[index];
-	}
-	else
-	{
-		ent->client->ps.stats[STAT_ARMOR_ICON] = 0;
-		ent->client->ps.stats[STAT_ARMOR] = 0;
-	}
+    if (index)
+    {
+        item = GetItemByIndex (index);
+        ent->client->ps.stats[STAT_ARMOR_ICON] = 0;
+        ent->client->ps.stats[STAT_ARMOR] = ent->client->pers.inventory[index];
+    }
+    else
+    {
+        ent->client->ps.stats[STAT_ARMOR_ICON] = 0;
+        ent->client->ps.stats[STAT_ARMOR] = 0;
+    }
 
 	//
 	// pickup message
@@ -862,10 +885,20 @@ void G_SetStats (edict_t *ent)
 		ent->client->ps.stats[STAT_TIMER_ICON] = gi.imageindex ("p_sproing");
 		ent->client->ps.stats[STAT_TIMER] = (ent->client->sproing_framenum - level.framenum)/10;
 	}
+	else if (ent->client->invis_framenum > level.framenum)
+	{
+		ent->client->ps.stats[STAT_TIMER_ICON] = gi.imageindex ("p_invis");
+		ent->client->ps.stats[STAT_TIMER] = (ent->client->invis_framenum - level.framenum)/10;
+	}
+	else if (ent->client->resp.powered)
+	{
+		ent->client->ps.stats[STAT_TIMER_ICON] = gi.imageindex ("p_powered");
+		ent->client->ps.stats[STAT_TIMER] = ent->client->resp.reward_pts;
+	}
 	else
 	{
-		ent->client->ps.stats[STAT_TIMER_ICON] = 0;
-		ent->client->ps.stats[STAT_TIMER] = 0;
+		ent->client->ps.stats[STAT_TIMER_ICON] = gi.imageindex ("p_rewardpts");
+		ent->client->ps.stats[STAT_TIMER] = ent->client->resp.reward_pts;
 	}
 
 	//
@@ -953,8 +986,47 @@ void G_SetStats (edict_t *ent)
 	}
 	//end bot score info
 
+	//weapon/ammo inventories
+	for(i = 0; i < 7; i++) 
+		ent->client->ps.stats[STAT_WEAPN1+i] = 0;
+	i = 0;
+	if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Alien Disruptor"))] &&
+		ent->client->pers.weapon != (FindItem("Alien Disruptor"))) {
+			ent->client->ps.stats[STAT_WEAPN1] = gi.imageindex ("disruptor");
+			i++;
+	}
+	if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Alien Smartgun"))] &&
+		ent->client->pers.weapon != (FindItem("Alien Smartgun"))) {
+			ent->client->ps.stats[STAT_WEAPN1+i] = gi.imageindex ("smartgun");
+			i++;
+	}
+	if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Pulse Rifle"))] &&
+		ent->client->pers.weapon != (FindItem("Pulse Rifle"))) {
+			ent->client->ps.stats[STAT_WEAPN1+i] = gi.imageindex ("chaingun");
+			i++;
+	}
+	if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Flame Thrower"))] &&
+		ent->client->pers.weapon != (FindItem("Flame Thrower"))) {
+			ent->client->ps.stats[STAT_WEAPN1+i] = gi.imageindex ("flamethrower");
+			i++;
+	}
+	if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Rocket Launcher"))] &&
+		ent->client->pers.weapon != (FindItem("Rocket Launcher"))) {
+			ent->client->ps.stats[STAT_WEAPN1+i] = gi.imageindex ("rocketlauncher");
+			i++;
+	}
+	if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Disruptor"))] &&
+		ent->client->pers.weapon != (FindItem("Disruptor"))) {
+			ent->client->ps.stats[STAT_WEAPN1+i] = gi.imageindex ("beamgun");
+			i++;
+	}
+	if(ent->client->pers.inventory[ITEM_INDEX(FindItem("Alien Vaporizer"))] &&
+		ent->client->pers.weapon != (FindItem("Alien Vaporizer"))) {
+			ent->client->ps.stats[STAT_WEAPN1+i] = gi.imageindex ("vaporizor");
+	}
+
 	//
-	// help icon / current weapon if not shown
+	// current weapon
 	//
 	if (ent->client->pers.weapon)
 		ent->client->ps.stats[STAT_HELPICON] = gi.imageindex (ent->client->pers.weapon->icon);
