@@ -390,6 +390,106 @@ void SV_AddBlend (float r, float g, float b, float a, float *v_blend)
 	v_blend[3] = a2;
 }
 
+void ResetWeaponModel (edict_t *ent)
+{
+	char    *info;
+	char	weaponame[64] = " ";
+	char	weaponmodel[MAX_OSPATH] = " ";
+	int i;
+	int done;
+	char	weaponpath[MAX_OSPATH] = " ";
+	FILE *file;
+
+	if (ent->in_vehicle) {
+		return;
+	}
+
+	//set up code to set player world weapon model
+
+	info = Info_ValueForKey (ent->client->pers.userinfo, "skin");
+
+	i = 0;
+	done = 0;
+	strcpy(weaponame, " ");
+	weaponame[0] = 0;
+	while(!done)
+	{
+		if((info[i] == '/') || (info[i] == '\\'))
+			done = 1;
+		weaponame[i] = info[i];
+		if(i > 63)
+			done = 1;
+		i++;
+	}
+	strcpy(weaponmodel, " ");
+	weaponmodel[0] = 0;
+
+	sprintf(weaponmodel, "players/%s%s", weaponame, "weapon.md2"); //default
+
+#ifdef __unix__
+	if( !Q_strcasecmp( ent->client->pers.weapon->view_model, "models/weapons/v_violator/tris.md2" ))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_violator.md2");
+	else if( !Q_strcasecmp( ent->client->pers.weapon->view_model, "models/weapons/v_rocket/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_rlauncher.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"models/weapons/v_blast/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_blaster.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"models/weapons/v_bfg/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_bfg.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model, "models/weapons/v_rail/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_railgun.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model, "models/weapons/v_shotg2/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_sshotgun.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model, "models/weapons/v_shotg/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_shotgun.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model, "models/weapons/v_hyperb/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_hyperblaster.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model, "models/weapons/v_chain/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_chaingun.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model, "vehicles/deathball/v_wep.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_machinegun.md2");
+#else
+	if(ent->client->pers.weapon->view_model == "models/weapons/v_violator/tris.md2")
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_violator.md2");
+	if(ent->client->pers.weapon->view_model == "models/weapons/v_rocket/tris.md2")
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_rlauncher.md2");
+	if(ent->client->pers.weapon->view_model == "models/weapons/v_blast/tris.md2")
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_blaster.md2");
+	if(ent->client->pers.weapon->view_model == "models/weapons/v_bfg/tris.md2")
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_bfg.md2");
+	if(ent->client->pers.weapon->view_model == "models/weapons/v_rail/tris.md2")
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_railgun.md2");
+	if(ent->client->pers.weapon->view_model == "models/weapons/v_shotg2/tris.md2")
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_sshotgun.md2");
+	if(ent->client->pers.weapon->view_model == "models/weapons/v_shotg/tris.md2")
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_shotgun.md2");
+	if(ent->client->pers.weapon->view_model == "models/weapons/v_hyperb/tris.md2")
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_hyperblaster.md2");
+	if(ent->client->pers.weapon->view_model == "models/weapons/v_chain/tris.md2")
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_chaingun.md2");
+	if(ent->client->pers.weapon->view_model == "vehicles/deathball/v_wep.md2")
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_machinegun.md2");
+#endif
+
+	sprintf(weaponpath, "%s", weaponmodel);
+	Q2_FindFile (weaponpath, &file); //does it really exist?
+	if(!file) {
+#ifdef __unix__
+		sprintf(weaponpath, "%s%s", weaponame, "weapon.md2"); //no w_weaps, do we have this model?
+#else
+		sprintf(weaponpath, "%s", weaponame, "weapon.md2"); //no w_weaps, do we have this model?
+#endif
+		Q2_FindFile (weaponpath, &file);
+		if(!file) //server does not have this player model
+			sprintf(weaponmodel, "players/martianenforcer/weapon.md2");//default player(martian)
+		else { //have the model, but it has no w_weaps
+			sprintf(weaponmodel, "players/%s%s", weaponame, "weapon.md2"); //custom weapon
+			fclose(file);
+		}
+	}
+	else
+		fclose(file);
+	ent->s.modelindex2 = gi.modelindex(weaponmodel);
+}
 
 /*
 =============
@@ -490,6 +590,18 @@ void SV_CalcBlend (edict_t *ent)
 		if (remaining > 30 || (remaining & 4) )
 			SV_AddBlend (0.4, 1, 0.4, 0.04, ent->client->ps.blend);
 	}
+	else if (ent->client->invis_framenum > level.framenum)
+	{
+		remaining = ent->client->invis_framenum - level.framenum;
+		if (remaining == 30)	// beginning to fade
+			gi.sound(ent, CHAN_ITEM, gi.soundindex("items/protect2.wav"), 1, ATTN_NORM, 0);
+		if (remaining > 30 || (remaining & 4) )
+			SV_AddBlend (0.4, 1, 0.4, 0.04, ent->client->ps.blend);
+		if (remaining == 1) { //put weapon model back
+			ResetWeaponModel(ent);
+		}
+	}
+
 	// add for damage
 	if (ent->client->damage_alpha > 0)
 		SV_AddBlend (ent->client->damage_blend[0],ent->client->damage_blend[1]
@@ -743,9 +855,9 @@ TeamEffects
 void TeamEffects(edict_t *player)
 {
 	if(player->dmteam == RED_TEAM)
-		player->s.effects |= EF_FLAG1;
+		player->s.effects |= EF_TEAM1;
 	else if(player->dmteam == BLUE_TEAM)
-		player->s.effects |= EF_FLAG2;
+		player->s.effects |= EF_TEAM2;
 }
 
 /*
@@ -766,7 +878,7 @@ void G_SetClientEffects (edict_t *ent)
 	if(ctf->value)
 		CTFEffects(ent);
 
-	if (((int)(dmflags->value) & DF_SKINTEAMS) || ctf->value || tca->value || cp->value)
+	if (((int)(dmflags->value) & DF_SKINTEAMS) || ctf->value || tca->value || cp->value )
 		TeamEffects(ent);
 
 	if (ent->client->quad_framenum > level.framenum)
@@ -793,6 +905,15 @@ void G_SetClientEffects (edict_t *ent)
 	if(ent->client->spawnprotected)
 		ent->s.effects |= EF_PENT;
 
+	if(ent->client->kill_streak >= 8)
+		ent->s.effects |= EF_FLIES;
+
+	//invisibility
+	if(ent->client->invis_framenum > level.framenum) {
+		ent->s.renderfx |= RF_TRANSLUCENT; 
+		ent->s.modelindex2 = 0;
+	}
+
 }
 
 
@@ -810,6 +931,14 @@ void G_SetClientEvent (edict_t *ent)
 	{
 		if ( (int)(current_client->bobtime+bobmove) != bobcycle )
 			ent->s.event = EV_FOOTSTEP;
+		
+		if ( ((ent->waterlevel == 1) || (ent->waterlevel == 2)) && (xyspeed > 100) )
+		{
+			if ( (int)(current_client->bobtime+bobmove) != bobcycle )
+			{				
+				ent->s.event = EV_WADE;	 
+			}
+		}
 	}
 }
 
@@ -842,9 +971,7 @@ void G_SetClientSound (edict_t *ent)
 	else
 		weap = "";
 
-	if (ent->waterlevel && (ent->watertype&(CONTENTS_LAVA|CONTENTS_SLIME)) )
-		ent->s.sound = snd_fry;
-	else if (strcmp(weap, "weapon_bfg") == 0)
+	if (strcmp(weap, "weapon_bfg") == 0)
 		ent->s.sound = gi.soundindex("weapons/vaporizer_hum.wav");
 	else if (strcmp(weap, "weapon_shotgun") == 0)
 		ent->s.sound = gi.soundindex("weapons/smartgun_hum.wav");
@@ -984,12 +1111,9 @@ void ClientEndServerFrame (edict_t *ent)
 {
 	float	bobtime;
 	int		i;
-	vec3_t addspeed;
-	qboolean haste, sproing;
 	current_player = ent;
 	current_client = ent->client;
 
-
 	//
 	// If the origin or velocity have changed since ClientThink(),
 	// update the pmove values.  This will happen when the client
@@ -1039,19 +1163,6 @@ void ClientEndServerFrame (edict_t *ent)
 	// all cyclic walking effects
 	//
 
-	haste = current_client->haste_framenum > level.framenum;
-	if(haste) {
-		AngleVectors (ent->s.angles, addspeed, right, up);
-		addspeed[0] *= 100;
-		addspeed[1] *= 100;
- 		VectorAdd(ent->velocity, addspeed, ent->velocity);
-
-	}
-	
-	sproing = current_client->sproing_framenum > level.framenum;
-	if(sproing)
-		ent->velocity[2] += 50;
-
 	xyspeed = sqrt(ent->velocity[0]*ent->velocity[0] + ent->velocity[1]*ent->velocity[1]);
 
 	if (xyspeed < 5)
@@ -1121,16 +1232,16 @@ void ClientEndServerFrame (edict_t *ent)
 	// if the scoreboard is up, update it
 	if (ent->client->showscores && !(level.framenum & 31) )
 	{
-// ACEBOT_ADD
 		if (ent->is_bot)
 			return;
-// ACEBOT_END
 
-		DeathmatchScoreboardMessage (ent, ent->enemy);
+		DeathmatchScoreboardMessage (ent, ent->enemy, false);
 		gi.unicast (ent, false);
 	}
 	if (ent->client->chasetoggle == 1)
         CheckDeathcam_Viewent(ent);
 
+	if ( g_antilag->integer)
+		G_StoreHistory( ent );
 }
 
