@@ -137,8 +137,13 @@ qboolean Pickup_Weapon (edict_t *ent, edict_t *other)
 			{
 				if ((int)(dmflags->value) & DF_WEAPONS_STAY)
 					ent->flags |= FL_RESPAWN;
-				else
-					SetRespawn (ent, 5); //was 30
+				else {
+					//weapon = FindItem (ent->item->weapon);
+					if(ent->item->weapmodel == WEAP_VAPORIZER)
+						SetRespawn (ent, 10);
+					else
+						SetRespawn (ent, 5); 
+				}
 			}
 		}
 	}
@@ -270,6 +275,28 @@ void ChangeWeapon (edict_t *ent)
 
 	sprintf(weaponmodel, "players/%s%s", weaponame, "weapon.md2"); //default
 
+#ifdef __unix__
+	if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"models/weapons/v_violator/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_violator.md2");
+	else if( !Q_strcasecmp( ent->client->pers.weapon->view_model,"models/weapons/v_rocket/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_rlauncher.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"models/weapons/v_blast/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_blaster.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"models/weapons/v_bfg/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_bfg.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"models/weapons/v_rail/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_railgun.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"models/weapons/v_shotg2/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_sshotgun.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"models/weapons/v_shotg/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_shotgun.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"models/weapons/v_hyperb/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_hyperblaster.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"models/weapons/v_chain/tris.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_chaingun.md2");
+	else if( !Q_strcasecmp(ent->client->pers.weapon->view_model,"vehicles/deathball/v_wep.md2"))
+		sprintf(weaponmodel, "players/%s%s", weaponame, "w_machinegun.md2");
+#else
 	if(ent->client->pers.weapon->view_model == "models/weapons/v_violator/tris.md2")
 		sprintf(weaponmodel, "players/%s%s", weaponame, "w_violator.md2");
 	if(ent->client->pers.weapon->view_model == "models/weapons/v_rocket/tris.md2")
@@ -290,12 +317,16 @@ void ChangeWeapon (edict_t *ent)
 		sprintf(weaponmodel, "players/%s%s", weaponame, "w_chaingun.md2");
 	if(ent->client->pers.weapon->view_model == "vehicles/deathball/v_wep.md2")
 		sprintf(weaponmodel, "players/%s%s", weaponame, "w_machinegun.md2");
-
+#endif
 
 	sprintf(weaponpath, "%s", weaponmodel);
 	Q2_FindFile (weaponpath, &file); //does it really exist?
 	if(!file) {
+#ifdef __unix__
+		sprintf(weaponpath, "%s%s", weaponame, "weapon.md2"); //no w_weaps, do we have this model?
+#else
 		sprintf(weaponpath, "%s", weaponame, "weapon.md2"); //no w_weaps, do we have this model?
+#endif
 		Q2_FindFile (weaponpath, &file);
 		if(!file) //server does not have this player model
 			sprintf(weaponmodel, "players/martianenforcer/weapon.md2");//default player(martian)
@@ -308,14 +339,14 @@ void ChangeWeapon (edict_t *ent)
 		fclose(file);
 	ent->s.modelindex2 = gi.modelindex(weaponmodel);
 
-	//some models don't have visible weapons, ie war machine. hack.
-	if((info[0] == 'w') && (info[1]  == 'a') && (info[2] == 'r'))
-		ent->s.modelindex2 = 0;
-
 	//play a sound like in Q3, except for blaster, so it doesn't do it on spawn.
+#ifdef __unix__
+	if( Q_strcasecmp( ent->client->pers.weapon->view_model,"models/weapons/v_blast/tris.md2") )
+		gi.sound(ent, CHAN_VOICE, gi.soundindex("weapons/whoosh.wav"), 1, ATTN_NORM, 0);
+#else
 	if(!(ent->client->pers.weapon->view_model == "models/weapons/v_blast/tris.md2"))
 		gi.sound(ent, CHAN_VOICE, gi.soundindex("weapons/whoosh.wav"), 1, ATTN_NORM, 0);
-
+#endif
 	ent->client->anim_priority = ANIM_PAIN;
 	if(ent->client->ps.pmove.pm_flags & PMF_DUCKED)
 	{
@@ -428,10 +459,6 @@ void Use_Weapon (edict_t *ent, gitem_t *item)
 		return;
 	}
 
-	//mutators
-//	if(instagib->value || rocket_arena->value)
-//		return;
-
 	// see if we're already using it
 	if (item == ent->client->pers.weapon)
 		return;
@@ -1034,7 +1061,7 @@ void Blaster_Fire (edict_t *ent, vec3_t g_offset, int damage, qboolean hyper, in
 			}
 		}
 		else {
-			gi.sound(ent, CHAN_AUTO, gi.soundindex("weapons/hyprbd1a.wav"), 1, ATTN_NORM, 0);
+			// [no file]gi.sound(ent, CHAN_AUTO, gi.soundindex("weapons/hyprbd1a.wav"), 1, ATTN_NORM, 0);
 			fire_blaster (ent, start, forward, damage, 2800, effect, hyper);
 		}
 	}
@@ -1047,18 +1074,18 @@ void Blaster_Fire (edict_t *ent, vec3_t g_offset, int damage, qboolean hyper, in
 			fire_blasterball (ent, start, forward, damage, 1200, effect, hyper);
 	}
 	// send muzzle flash
-	if(!(ent->client->buttons & BUTTON_ATTACK2)){
-		gi.WriteByte (svc_muzzleflash);
-		gi.WriteShort (ent-g_edicts);
-		if (hyper)
-			gi.WriteByte (MZ_HYPERBLASTER | is_silenced);
-		else
-			gi.WriteByte (MZ_BLASTER | is_silenced);
-		gi.multicast (ent->s.origin, MULTICAST_PVS);
-		PlayerNoise(ent, start, PNOISE_WEAPON);
-	}
+	gi.WriteByte (svc_muzzleflash);
+	gi.WriteShort (ent-g_edicts);
+	if (hyper)
+		gi.WriteByte (MZ_HYPERBLASTER | is_silenced);
+	else if (ent->client->buttons & BUTTON_ATTACK2)
+		gi.WriteByte (MZ_RAILGUN | is_silenced);
+	else
+		gi.WriteByte (MZ_BLASTER | is_silenced);
+	gi.multicast (ent->s.origin, MULTICAST_PVS);
+	PlayerNoise(ent, start, PNOISE_WEAPON);
+	
 	//create visual muzzle flash sprite!
-
 	if(!hyper || (ent->client->buttons & BUTTON_ATTACK2))
 	{
 		VectorAdd(start, forward, start);
@@ -1117,9 +1144,19 @@ void Weapon_Bomber_Fire (edict_t *ent)
 
 	VectorSet(offset, 8, 8, ent->viewheight-4);
 	P_ProjectSource (ent->client, ent->s.origin, offset, forward, right, start);
-	forward[0] = forward[0] * 2.6;
-	forward[1] = forward[1] * 2.6;
-	fire_bomb (ent, start, forward, damage, 250, damage_radius, radius_damage, 8);
+	
+	if(ent->client->buttons & BUTTON_ATTACK2 && ent->client->ps.gunframe != 12) {
+		fire_rocket (ent, start, forward, damage/3, 1400, damage_radius/2, radius_damage/2);
+		gi.sound (ent, CHAN_WEAPON, gi.soundindex("weapons/rocklr1b.wav"), 1, ATTN_NORM, 0);
+		ent->client->ps.gunframe = 12;
+	}
+	else if(ent->client->ps.gunframe != 6){
+		forward[0] = forward[0] * 2.6;
+		forward[1] = forward[1] * 2.6;
+
+		fire_bomb (ent, start, forward, damage, 250, damage_radius, radius_damage, 8);
+		gi.sound (ent, CHAN_WEAPON, gi.soundindex("vehicles/shootbomb.wav"), 1, ATTN_NORM, 0);
+	}
 
 	// send muzzle flash
 	gi.WriteByte (svc_muzzleflash);
@@ -1129,8 +1166,6 @@ void Weapon_Bomber_Fire (edict_t *ent)
 
 	ent->client->ps.gunframe++;
 
-	gi.sound (ent, CHAN_WEAPON, gi.soundindex("vehicles/shootbomb.wav"), 1, ATTN_NORM, 0);
-
 	//might want to work with this some
 	if (! ( (int)dmflags->value & DF_INFINITE_AMMO ) )
 		ent->client->pers.inventory[ent->client->ammo_index]--;
@@ -1138,7 +1173,7 @@ void Weapon_Bomber_Fire (edict_t *ent)
 void Weapon_Bomber (edict_t *ent)
 {
 	static int	pause_frames[]	= {30, 0};
-	static int	fire_frames[]	= {12,0};
+	static int	fire_frames[]	= {6,12,0};
 	static int	excessive_fire_frames[]	= {6,8,10,12,0};
 
 	if(excessive->value)
@@ -1152,6 +1187,11 @@ void Weapon_Strafer_Fire (edict_t *ent)
 	vec3_t	start;
 	vec3_t	offset;
 	int damage;
+	float	damage_radius;
+	int		radius_damage;
+
+	radius_damage = 100;
+	damage_radius = 100;
 
 	if(excessive->value)
 		damage = 60;
@@ -1173,7 +1213,10 @@ void Weapon_Strafer_Fire (edict_t *ent)
 
 	P_ProjectSource (ent->client, ent->s.origin, offset, forward, right, start);
 
-	fire_blaster_beam (ent, start, forward, damage, 0, true);
+	if(ent->client->buttons & BUTTON_ATTACK2) 
+		fire_rocket (ent, start, forward, damage, 1200, damage_radius, radius_damage);
+	else
+		fire_blaster_beam (ent, start, forward, damage, 0, true);
 
 	// send muzzle flash
 	gi.WriteByte (svc_muzzleflash);
@@ -1201,7 +1244,14 @@ void Weapon_Strafer_Fire (edict_t *ent)
 
 	P_ProjectSource (ent->client, ent->s.origin, offset, forward, right, start);
 
-	fire_blaster_beam (ent, start, forward, damage, 0, true);
+	if(ent->client->buttons & BUTTON_ATTACK2) {
+		fire_rocket (ent, start, forward, damage, 1200, damage_radius, radius_damage);
+		gi.sound (ent, CHAN_WEAPON, gi.soundindex("weapons/rocklr1b.wav"), 1, ATTN_NORM, 0);
+	}
+	else {		
+		fire_blaster_beam (ent, start, forward, damage, 0, true);
+		gi.sound (ent, CHAN_WEAPON, gi.soundindex("vehicles/shootlaser.wav"), 1, ATTN_NORM, 0);
+	}
 
 	// send muzzle flash
 	gi.WriteByte (svc_muzzleflash);
@@ -1216,13 +1266,11 @@ void Weapon_Strafer_Fire (edict_t *ent)
 	gi.multicast (start, MULTICAST_PVS);
 
 	ent->client->ps.gunframe++;
-
-	gi.sound (ent, CHAN_WEAPON, gi.soundindex("vehicles/shootlaser.wav"), 1, ATTN_NORM, 0);
 }
 void Weapon_Strafer (edict_t *ent) //for now
 {
 	static int	pause_frames[]	= {30, 0};
-	static int	fire_frames[]	= {7,0};
+	static int	fire_frames[]	= {6,0};
 	static int	excessive_fire_frames[]	= {6,8,10,0};
 
 	if(excessive->value)
@@ -1254,7 +1302,20 @@ void Weapon_Hover_Fire (edict_t *ent)
 
 	P_ProjectSource (ent->client, ent->s.origin, offset, forward, right, start);
 
-	fire_hover_beam (ent, start, forward, damage, 0, true);
+	if(ent->client->buttons & BUTTON_ATTACK2) {
+		fire_blasterball (ent, start, forward, damage*3, 1500, EF_ROCKET, false);
+		gi.sound(ent, CHAN_AUTO, gi.soundindex("weapons/hypbrl1a.wav"), 1, ATTN_NORM, 0);
+	}
+	else if(ent->client->ps.gunframe == 6) {
+		fire_hover_beam (ent, start, forward, damage, 0, true);
+		gi.sound (ent, CHAN_WEAPON, gi.soundindex("weapons/biglaser.wav"), 1, ATTN_NORM, 0);
+		
+		VectorAdd(start, forward, start);
+		gi.WriteByte (svc_temp_entity);
+		gi.WriteByte (TE_CHAINGUNSMOKE);
+		gi.WritePosition (start);
+		gi.multicast (start, MULTICAST_PVS);
+	}
 
 	// send muzzle flash
 	gi.WriteByte (svc_muzzleflash);
@@ -1262,20 +1323,23 @@ void Weapon_Hover_Fire (edict_t *ent)
 	gi.WriteByte (MZ_BFG | is_silenced);
 	gi.multicast (ent->s.origin, MULTICAST_PVS);
 
-	VectorAdd(start, forward, start);
-	gi.WriteByte (svc_temp_entity);
-	gi.WriteByte (TE_CHAINGUNSMOKE);
-	gi.WritePosition (start);
-	gi.multicast (start, MULTICAST_PVS);
+	if(ent->client->buttons & BUTTON_ATTACK2) {
+		forward[0] = forward[0] * 10;
+		forward[1] = forward[1] * 10;
+
+		VectorAdd(start, forward, start);
+		gi.WriteByte (svc_temp_entity);
+		gi.WriteByte (TE_SMART_MUZZLEFLASH);
+		gi.WritePosition (start);
+		gi.multicast (start, MULTICAST_PVS);
+	}
 
 	ent->client->ps.gunframe++;
-
-	gi.sound (ent, CHAN_WEAPON, gi.soundindex("weapons/biglaser.wav"), 1, ATTN_NORM, 0);
 }
 void Weapon_Hover (edict_t *ent) //for now
 {
 	static int	pause_frames[]	= {30, 0};
-	static int	fire_frames[]	= {7,0};
+	static int	fire_frames[]	= {6,8,10,0};
 	static int	excessive_fire_frames[]	= {6,8,10,0};
 
 	if(excessive->value)
@@ -1540,11 +1604,12 @@ void weapon_floater_fire (edict_t *ent)
 	P_ProjectSource (ent->client, ent->s.origin, offset, forward, right, start);
 	forward[0] = forward[0] * 2.6;
 	forward[1] = forward[1] * 2.6;
+	forward[2] = forward[2] * 2.6;
 	if(ent->altfire) {
 		if(excessive->value)
-			fire_floater (ent, start, forward, damage, 500, damage_radius, radius_damage, 8);
+			fire_floater (ent, start, forward, damage, 400, damage_radius, radius_damage, 8);
 		else
-			fire_prox (ent, start, forward, damage-50, 500, damage_radius, radius_damage-50, 8);
+			fire_prox (ent, start, forward, damage-50, 200, damage_radius, radius_damage-50, 8);
 	}
 	else
 		fire_floater (ent, start, forward, damage, 500, damage_radius, radius_damage, 8);
