@@ -39,6 +39,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <string.h>
 #include <stdlib.h>
 #include <time.h>
+#include <ctype.h>
 
 #if (defined _M_IX86 || defined __i386__) && !defined C_ONLY && !defined __sun__
 #define id386	1
@@ -161,6 +162,7 @@ extern long Q_ftol( float f );
 #define VectorClear(a)			(a[0]=a[1]=a[2]=0)
 #define VectorNegate(a,b)		(b[0]=-a[0],b[1]=-a[1],b[2]=-a[2])
 #define VectorSet(v, x, y, z)	(v[0]=(x), v[1]=(y), v[2]=(z))
+#define	SnapVector(v) {v[0]=((int)(v[0]));v[1]=((int)(v[1]));v[2]=((int)(v[2]));}
 
 void VectorMA (vec3_t veca, float scale, vec3_t vecb, vec3_t vecc);
 
@@ -224,6 +226,11 @@ void RotatePointAroundVector( vec3_t dst, const vec3_t dir, const vec3_t point,
 
 //=============================================
 
+/* FIXME: Beware - _vsnprintf does not end with \0 - vsnprintf (*nix) does */
+#ifdef _WIN32
+#define vsnprintf	_vsnprintf
+#endif
+
 char *COM_SkipPath (char *pathname);
 void COM_StripExtension (char *in, char *out);
 void COM_FileBase (char *in, char *out);
@@ -251,6 +258,7 @@ int Q_strcasecmp (char *s1, char *s2);
 int Q_strncasecmp (char *s1, char *s2, int n);
 void Q_strcat (char *dst, const char *src, int dstSize);
 int Q_strnicmp (const char *string1, const char *string2, int n);
+char *Q_strlwr(char *s);
 //=============================================
 
 short	BigShort(short l);
@@ -349,13 +357,14 @@ CVARS (console variables)
 typedef struct cvar_s
 {
 	char		*name;
+	unsigned int	hash_key;
 	char		*string;
 	char		*latched_string;	// for CVAR_LATCH vars
-	int			flags;
-	qboolean	modified;	// set each time the cvar is changed
+	int		flags;
+	qboolean	modified;		// set each time the cvar is changed
 	float		value;
-	int			integer; //BLOOMS
-	struct cvar_s *next;
+	int		integer;		//BLOOMS
+	struct cvar_s	*next;
 } cvar_t;
 
 #endif		// CVAR
@@ -604,8 +613,8 @@ typedef struct
 #define	EF_QUAD				0x00008000
 #define	EF_PENT				0x00010000
 #define	EF_TELEPORTER		0x00020000		// particle fountain
-#define EF_FLAG1			0x00040000
-#define EF_FLAG2			0x00080000
+#define EF_TEAM1			0x00040000
+#define EF_TEAM2			0x00080000
 #define EF_GREENGIB			0x00200000
 #define EF_PLASMA			0x01000000
 #define EF_SHIPEXHAUST  	0x02000000
@@ -619,7 +628,7 @@ typedef struct
 #define	RF_DEPTHHACK		16		// for view weapon Z crunching
 #define	RF_TRANSLUCENT		32
 #define	RF_FRAMELERP		64
-#define RF_BEAM				128
+#define RF_BEAM				128		// unused
 #define	RF_CUSTOMSKIN		256		// skin is an index in image_precache
 #define	RF_GLOW				512		// pulse lighting for bonus items
 #define RF_SHELL_RED		1024
@@ -630,6 +639,7 @@ typedef struct
 #define	RF_SHELL_HALF_DAM	0x00020000
 #define RF_NOSHADOWS		0x00040000 //use this one for turning off shadows, etc.
 #define RF_MONSTER  		0x00080000
+#define RF_MENUMODEL		0x01280000 //for player menu
 
 // player_state_t->refdef flags
 #define	RDF_UNDERWATER		1		// warp the screen as apropriate
@@ -677,15 +687,15 @@ typedef struct
 #define	TE_SPLASH					10
 #define	TE_BUBBLETRAIL				11
 #define	TE_SCREEN_SPARKS			12
-#define	TE_SHIELD_SPARKS			13
+#define	TE_DEATHFIELD2				13
 #define	TE_BULLET_SPARKS			14
 #define	TE_LASER_SPARKS				15
-#define	TE_PARASITE_ATTACK			16
+#define	TE_DONTUSE					16
 #define TE_ROCKET_EXPLOSION_WATER	17
 #define	TE_REDLASER					19
 #define	TE_BFG_BIGEXPLOSION			21
 #define	TE_BOSSTPORT				22			// used as '22' in a map, so DON'T RENUMBER!!!
-#define	TE_BFG_LASER				23
+#define	TE_DONTUSE2					23
 #define	TE_GREENBLOOD				26
 #define	TE_LIGHTNING				33
 #define	TE_VAPORBEAM				38
@@ -758,6 +768,13 @@ typedef struct
 #define STAT_BLUESCORE			22
 #define STAT_FLAG_ICON			23
 #define STAT_ZOOMED				24
+#define STAT_WEAPN1				25
+#define STAT_WEAPN2				26
+#define STAT_WEAPN3				27
+#define STAT_WEAPN4				28
+#define STAT_WEAPN5				29
+#define STAT_WEAPN6				30
+#define STAT_WEAPN7				31
 
 #define	MAX_STATS				32
 
@@ -843,7 +860,8 @@ typedef enum
 	EV_FALL,
 	EV_FALLFAR,
 	EV_PLAYER_TELEPORT,
-	EV_OTHER_TELEPORT
+	EV_OTHER_TELEPORT, 
+	EV_WADE
 } entity_event_t;
 
 
@@ -949,3 +967,7 @@ typedef struct
 #define S_COLOR_MAGENTA	"^6"
 #define S_COLOR_WHITE	"^7"
 
+//unlagged - lag simulation #2
+#define MAX_LATENT_CMDS 64
+//unlagged - lag simulation #2
+
