@@ -23,7 +23,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "../game/q_shared.h"
 
 
-#define	VERSION		7.0
+#define	VERSION		7.33
 
 #define	BASEDIRNAME	"data1"
 
@@ -92,14 +92,28 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 //============================================================================
 
+// Hash key computation macro, because that piece of code gets around a lot
+
+#define COMPUTE_HASH_KEY(key, str, counter) \
+{ \
+	key = 0; \
+	for ( counter = 0; str[counter] ; counter ++ ) \
+		key = 31 * key + tolower(str[counter]); \
+}
+
+//============================================================================
+
 typedef struct sizebuf_s
 {
 	qboolean	allowoverflow;	// if false, do a Com_Error
 	qboolean	overflowed;		// set to true if the buffer size failed
-	byte	*data;
+	byte		*data;
 	int		maxsize;
 	int		cursize;
 	int		readcount;
+#ifdef	BUFFER_DEBUG
+	char		name[48];
+#endif	//BUFFER_DEBUG
 } sizebuf_t;
 
 void SZ_Init (sizebuf_t *buf, byte *data, int length);
@@ -108,6 +122,12 @@ void *SZ_GetSpace (sizebuf_t *buf, int length);
 void SZ_Write (sizebuf_t *buf, void *data, int length);
 void SZ_Print (sizebuf_t *buf, char *data);	// strcats onto the sizebuf
 
+#ifdef	BUFFER_DEBUG
+void SZ_SetName( sizebuf_t *buf, const char * name, qboolean print_it );
+#else	//BUFFER_DEBUG
+# define SZ_SetName(buf, name, print)
+#endif	// BUFFER_DEBUG
+
 //============================================================================
 
 struct usercmd_s;
@@ -808,7 +828,6 @@ extern	vec3_t	bytedirs[NUMVERTEXNORMALS];
 // this is in the client code, but can be used for debugging from server
 void SCR_DebugGraph (float value, int color);
 
-
 /*
 ==============================================================
 
