@@ -28,14 +28,11 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <GL/glu.h>
 
 image_t		gltextures[MAX_GLTEXTURES];
+image_t		*r_mirrortexture;
+image_t		*r_depthtexture;
 int			numgltextures;
 int			base_textureid;		// gltextures[i] = base_textureid+i
 
-static byte			 intensitytable[256];
-static unsigned char gammatable[256];
-
-cvar_t		*intensity;
-
 extern cvar_t	*cl_hudimage1; //custom huds
 extern cvar_t	*cl_hudimage2;
 
@@ -43,7 +40,6 @@ unsigned	d_8to24table[256];
 
 qboolean GL_Upload8 (byte *data, int width, int height,  qboolean mipmap, qboolean is_sky );
 qboolean GL_Upload32 (unsigned *data, int width, int height,  qboolean mipmap, qboolean is_normalmap);
-qboolean GL_Upload32_hires (unsigned *data, int width, int height,  qboolean mipmap, qboolean is_normalmap);
 
 int		gl_solid_format = 3;
 int		gl_alpha_format = 4;
@@ -112,12 +108,28 @@ void GL_EnableMultitexture( qboolean enable )
 		GL_SelectTexture( GL_TEXTURE1 );
 		qglEnable( GL_TEXTURE_2D );
 		GL_TexEnv( GL_REPLACE );
+
+		GL_SelectTexture( GL_TEXTURE2 );
+		qglEnable( GL_TEXTURE_2D );
+		GL_TexEnv( GL_REPLACE );
+
+		GL_SelectTexture( GL_TEXTURE3 );
+		qglEnable( GL_TEXTURE_2D );
+		GL_TexEnv( GL_REPLACE );
 	}
 	else
 	{
 		GL_SelectTexture( GL_TEXTURE1 );
 		qglDisable( GL_TEXTURE_2D );
 		GL_TexEnv( GL_REPLACE );
+
+		GL_SelectTexture( GL_TEXTURE2 );
+		qglDisable( GL_TEXTURE_2D );
+		GL_TexEnv( GL_REPLACE );
+
+		GL_SelectTexture( GL_TEXTURE3 );
+		qglDisable( GL_TEXTURE_2D );
+		GL_TexEnv( GL_REPLACE );
 	}
 	GL_SelectTexture( GL_TEXTURE0 );
 	GL_TexEnv( GL_REPLACE );
@@ -1055,86 +1067,75 @@ void GL_ResampleTexture (unsigned *in, int inwidth, int inheight, unsigned *out,
 }
 
 /*
-================
-GL_LightScaleTexture
+R_FilterTexture
 
-Scale up the pixel values in a texture to increase the
-lighting range
-================
+Applies brightness and contrast to the specified image while optionally computing 
+the image's average color.  Also handles image inversion and monochrome.  This is 
+all munged into one function to reduce loops on level load.
 */
-void GL_LightScaleTexture (unsigned *in, int inwidth, int inheight, qboolean only_gamma )
+void R_FilterTexture(unsigned *in, int width, int height)
 {
-	if ( only_gamma )
-	{
-		int		i, c;
-		byte	*p;
+	byte *pcb;
+	int count;
+	float fcb;
+	int ix;
+	static byte lut[256];
+	static int first_time = 1;
+	static float lut_gamma = 0.0f;
+	static float lut_contrast = 0.0f;
 
-		p = (byte *)in;
-
-		c = inwidth*inheight;
-		for (i=0 ; i<c ; i++, p+=4)
-		{
-			p[0] = gammatable[p[0]];
-			p[1] = gammatable[p[1]];
-			p[2] = gammatable[p[2]];
+	if ( lut_gamma != vid_gamma->value || lut_contrast != vid_contrast->value || first_time ) {
+		first_time = 0;
+		lut_gamma = vid_gamma->value;
+		lut_contrast = vid_contrast->value;
+		// build lookup table
+		for ( ix = 0; ix < sizeof(lut); ix++ ) {
+			fcb = (float)ix / 255.0f;
+			fcb *= lut_gamma;
+			if (fcb < 0.0f )
+				fcb = 0.0f;
+			fcb -= 0.5f;
+			fcb *= lut_contrast;
+			fcb += 0.5f;
+			fcb *= 255.0f;
+			if ( fcb >= 255.0f )
+				lut[ix] = 255 ;
+			else if (fcb <= 0.0f )
+				lut[ix] = 0;
+			else
+				lut[ix] = (byte)fcb;
 		}
 	}
-	else
-	{
-		int		i, c;
-		byte	*p;
 
-		p = (byte *)in;
-
-		c = inwidth*inheight;
-		for (i=0 ; i<c ; i++, p+=4)
-		{
-			p[0] = gammatable[intensitytable[p[0]]];
-			p[1] = gammatable[intensitytable[p[1]]];
-			p[2] = gammatable[intensitytable[p[2]]];
-		}
+	// apply to image
+	pcb = (byte*)in;
+	count = width * height;
+	while ( count-- ) {
+		*pcb = lut[*pcb];
+		++pcb;
+		*pcb = lut[*pcb];
+		++pcb;
+		*pcb = lut[*pcb];
+		++pcb;
+		++pcb;
 	}
+
 }
 
-/*
-================
-GL_MipMap
-
-Operates in place, quartering the size of the texture
-================
-*/
-void GL_MipMap (byte *in, int width, int height)
-{
-	int		i, j;
-	byte	*out;
-
-	width <<=2;
-	height >>= 1;
-	out = in;
-	for (i=0 ; i<height ; i++, in+=width)
-	{
-		for (j=0 ; j<width ; j+=8, out+=4, in+=8)
-		{
-			out[0] = (in[0] + in[4] + in[width+0] + in[width+4])>>2;
-			out[1] = (in[1] + in[5] + in[width+1] + in[width+5])>>2;
-			out[2] = (in[2] + in[6] + in[width+2] + in[width+6])>>2;
-			out[3] = (in[3] + in[7] + in[width+3] + in[width+7])>>2;
-		}
-	}
-}
-
-
 int		upload_width, upload_height;
 qboolean uploaded_paletted;
-// Fixed 256x256 texture size limitation - MrG
-qboolean GL_Upload32_hires (unsigned *data, int width, int height,  qboolean mipmap, qboolean is_normalmap)
+
+qboolean GL_Upload32 (unsigned *data, int width, int height,  qboolean mipmap, qboolean is_normalmap)
 {	int		samples;
 	unsigned 	*scaled;
 	int		scaled_width, scaled_height;
 	int		i, c;
 	byte		*scan;
 	int comp;
- 
+
+	if(mipmap && !is_normalmap)
+		R_FilterTexture(data, width, height);
+
     uploaded_paletted = false;    // scan the texture for any non-255 alpha 
     c = width*height; 
     scan = ((byte *)data) + 3; 
@@ -1178,7 +1179,12 @@ qboolean GL_Upload32_hires (unsigned *data, int width, int height,  qboolean mip
             }            if (scaled_width && scaled_height) 
                 break; 
         } 
-
+		// let people sample down the world textures for speed
+		if (mipmap)
+		{
+			scaled_width >>= (int)gl_picmip->value;
+			scaled_height >>= (int)gl_picmip->value;
+		}
         if (scaled_width > max_size) 
             scaled_width = max_size; 
         if (scaled_height > max_size) 
@@ -1194,9 +1200,6 @@ qboolean GL_Upload32_hires (unsigned *data, int width, int height,  qboolean mip
         scaled=data; 
     } 
 
-	if(!is_normalmap)
-		GL_LightScaleTexture (scaled, scaled_width, scaled_height, !mipmap);
-    
 	if (mipmap) 
         gluBuild2DMipmaps (GL_TEXTURE_2D, samples, scaled_width, scaled_height, GL_RGBA, GL_UNSIGNED_BYTE, scaled); 
     else   
@@ -1217,223 +1220,12 @@ qboolean GL_Upload32_hires (unsigned *data, int width, int height,  qboolean mip
         qglTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, gl_filter_max); 
         qglTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, gl_filter_max); 
     }     
+
+	qglTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, r_anisotropic->value);
+
     return (samples == gl_alpha_format); 
 } 
 
-/*
-===============
-GL_Upload32
-
-Returns has_alpha
-===============
-*/
-void GL_BuildPalettedTexture( unsigned char *paletted_texture, unsigned char *scaled, int scaled_width, int scaled_height )
-{
-	int i;
-
-	for ( i = 0; i < scaled_width * scaled_height; i++ )
-	{
-		unsigned int r, g, b, c;
-
-		r = ( scaled[0] >> 3 ) & 31;
-		g = ( scaled[1] >> 2 ) & 63;
-		b = ( scaled[2] >> 3 ) & 31;
-
-		c = r | ( g << 5 ) | ( b << 11 );
-
-		paletted_texture[i] = gl_state.d_16to8table[c];
-
-		scaled += 4;
-	}
-}
-
-qboolean GL_Upload32 (unsigned *data, int width, int height, qboolean mipmap, qboolean is_normalmap)
-{
-	int			samples;
-	unsigned	scaled[256*256];
-	unsigned char paletted_texture[256*256];
-	int			scaled_width, scaled_height;
-	int			i, c;
-	byte		*scan;
-	int comp;
-
-	uploaded_paletted = false;
-
-	for (scaled_width = 1 ; scaled_width < width ; scaled_width<<=1)
-		;
-	if (gl_round_down->value && scaled_width > width && mipmap)
-		scaled_width >>= 1;
-	for (scaled_height = 1 ; scaled_height < height ; scaled_height<<=1)
-		;
-	if (gl_round_down->value && scaled_height > height && mipmap)
-		scaled_height >>= 1;
-
-	// let people sample down the world textures for speed
-	if (mipmap)
-	{
-		scaled_width >>= (int)gl_picmip->value;
-		scaled_height >>= (int)gl_picmip->value;
-	}
-
-	// don't ever bother with >256 textures
-	if (scaled_width > 256)
-		scaled_width = 256;
-	if (scaled_height > 256)
-		scaled_height = 256;
-
-	if (scaled_width < 1)
-		scaled_width = 1;
-	if (scaled_height < 1)
-		scaled_height = 1;
-
-	upload_width = scaled_width;
-	upload_height = scaled_height;
-
-	if (scaled_width * scaled_height > sizeof(scaled)/4)
-		Sys_Error ("GL_Upload32: too big");
-
-	// scan the texture for any non-255 alpha
-	c = width*height;
-	scan = ((byte *)data) + 3;
-	samples = gl_solid_format;
-	for (i=0 ; i<c ; i++, scan += 4)
-	{
-		if ( *scan != 255 )
-		{
-			samples = gl_alpha_format;
-			break;
-		}
-	}
-
-	if (samples == gl_solid_format)
-	    comp = gl_tex_solid_format;
-	else if (samples == gl_alpha_format)
-	    comp = gl_tex_alpha_format;
-	else {
-	    comp = samples;
-	}
-
-#if 0
-	if (mipmap)
-		gluBuild2DMipmaps (GL_TEXTURE_2D, samples, width, height, GL_RGBA, GL_UNSIGNED_BYTE, trans);
-	else if (scaled_width == width && scaled_height == height)
-		qglTexImage2D (GL_TEXTURE_2D, 0, comp, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, trans);
-	else
-	{
-		gluScaleImage (GL_RGBA, width, height, GL_UNSIGNED_BYTE, trans,
-			scaled_width, scaled_height, GL_UNSIGNED_BYTE, scaled);
-		qglTexImage2D (GL_TEXTURE_2D, 0, comp, scaled_width, scaled_height, 0, GL_RGBA, GL_UNSIGNED_BYTE, scaled);
-	}
-#else
-
-	if (scaled_width == width && scaled_height == height)
-	{
-		if (!mipmap)
-		{
-			// paletted texture.
-			if ( qglColorTableEXT && gl_ext_palettedtexture->value && samples == gl_solid_format )
-			{
-				uploaded_paletted = true;
-				GL_BuildPalettedTexture( paletted_texture, ( unsigned char * ) data, scaled_width, scaled_height );
-				qglTexImage2D( GL_TEXTURE_2D,
-							  0,
-							  GL_COLOR_INDEX8_EXT,
-							  scaled_width,
-							  scaled_height,
-							  0,
-							  GL_COLOR_INDEX,
-							  GL_UNSIGNED_BYTE,
-							  paletted_texture );
-			}
-			else
-			{
-				qglTexImage2D (GL_TEXTURE_2D, 0, comp, scaled_width, scaled_height, 0, GL_RGBA, GL_UNSIGNED_BYTE, data);
-			}
-			goto done;
-		}
-		memcpy (scaled, data, width*height*4);
-	}
-	else
-		GL_ResampleTexture (data, width, height, scaled, scaled_width, scaled_height);
-
-	if(!is_normalmap)
-		GL_LightScaleTexture (scaled, scaled_width, scaled_height, !mipmap );
-
-	// textures.
-	if ( qglColorTableEXT && gl_ext_palettedtexture->value && ( samples == gl_solid_format ) )
-	{
-		uploaded_paletted = true;
-		GL_BuildPalettedTexture( paletted_texture, ( unsigned char * ) scaled, scaled_width, scaled_height );
-		qglTexImage2D( GL_TEXTURE_2D,
-					  0,
-					  GL_COLOR_INDEX8_EXT,
-					  scaled_width,
-					  scaled_height,
-					  0,
-					  GL_COLOR_INDEX,
-					  GL_UNSIGNED_BYTE,
-					  paletted_texture );
-	}
-	else
-	{
-		qglTexImage2D( GL_TEXTURE_2D, 0, comp, scaled_width, scaled_height, 0, GL_RGBA, GL_UNSIGNED_BYTE, scaled );
-	}
-
-	if (mipmap)
-	{
-		int		miplevel;
-
-		miplevel = 0;
-		while (scaled_width > 1 || scaled_height > 1)
-		{
-			GL_MipMap ((byte *)scaled, scaled_width, scaled_height);
-			scaled_width >>= 1;
-			scaled_height >>= 1;
-			if (scaled_width < 1)
-				scaled_width = 1;
-			if (scaled_height < 1)
-				scaled_height = 1;
-			miplevel++;
-
-			// paletted textures.
-			if ( qglColorTableEXT && gl_ext_palettedtexture->value && samples == gl_solid_format )
-			{
-				uploaded_paletted = true;
-				GL_BuildPalettedTexture( paletted_texture, ( unsigned char * ) scaled, scaled_width, scaled_height );
-				qglTexImage2D( GL_TEXTURE_2D,
-							  miplevel,
-							  GL_COLOR_INDEX8_EXT,
-							  scaled_width,
-							  scaled_height,
-							  0,
-							  GL_COLOR_INDEX,
-							  GL_UNSIGNED_BYTE,
-							  paletted_texture );
-			}
-			else
-			{
-				qglTexImage2D (GL_TEXTURE_2D, miplevel, comp, scaled_width, scaled_height, 0, GL_RGBA, GL_UNSIGNED_BYTE, scaled);
-			}
-		}
-	}
-done: ;
-#endif
-
-
-	if (mipmap)
-	{
-		qglTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, gl_filter_min);
-		qglTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, gl_filter_max);
-	}
-	else
-	{
-		qglTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, gl_filter_max);
-		qglTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, gl_filter_max);
-	}
-
-	return (samples == gl_alpha_format);
-}
-
 /*
 ===============
 GL_Upload8
@@ -1498,10 +1290,7 @@ qboolean GL_Upload8 (byte *data, int width, int height,  qboolean mipmap, qboole
 				((byte *)&trans[i])[2] = ((byte *)&d_8to24table[p])[2];
 			}
 		}
-		if(gl_texres->value)
-			return GL_Upload32_hires (trans, width, height, mipmap, false);
-		else
-			return GL_Upload32 (trans, width, height, mipmap, false);
+		return GL_Upload32 (trans, width, height, mipmap, false);
 	}
 }
 
@@ -1579,10 +1368,7 @@ nonscrap:
 		if (bits == 8)
 			image->has_alpha = GL_Upload8 (pic, width, height, (image->type != it_pic && image->type != it_sky), image->type == it_sky );
 		else {
-			if(gl_texres->value)
-				image->has_alpha = GL_Upload32_hires ((unsigned *)pic, width, height, (image->type != it_pic && image->type != it_sky), image->type == it_bump );
-			else
-				image->has_alpha = GL_Upload32 ((unsigned *)pic, width, height, (image->type != it_pic && image->type != it_sky), image->type == it_bump );
+			image->has_alpha = GL_Upload32 ((unsigned *)pic, width, height, (image->type != it_pic && image->type != it_sky), image->type == it_bump );
 		}
 		image->upload_width = upload_width;		// after power of 2 and scales
 		image->upload_height = upload_height;
@@ -1593,6 +1379,7 @@ nonscrap:
 		image->th = 1;
 	}
 
+	COMPUTE_HASH_KEY( image->hash_key, name, i );
 	return image;
 }
 
@@ -1633,19 +1420,23 @@ GL_FindImage
 Finds or loads the given image
 ===============
 */
+
+static double totalTimeFindImage = 0;
+static unsigned int totalCountFindImage = 0;
 image_t	*GL_FindImage (char *name, imagetype_t type)
 {
-	image_t	*image;
+	image_t		*image = NULL;
 	int		i, len;
-	byte	*pic, *palette;
+	byte		*pic, *palette;
 	int		width, height;
-	char	shortname[MAX_QPATH];
+	char		shortname[MAX_QPATH];
+	unsigned int	hash_key;
 
 	if (!name)
-		return NULL;	//	Com_Error (ERR_DROP, "GL_FindImage: NULL name");
+		goto ret_image;	//	Com_Error (ERR_DROP, "GL_FindImage: NULL name");
 	len = strlen(name);
-	if (len<5)
-		return NULL;	//	Com_Error (ERR_DROP, "GL_FindImage: bad name: %s", name);
+	if (len < 5)
+		goto ret_image;	//	Com_Error (ERR_DROP, "GL_FindImage: bad name: %s", name);
 
 	//if HUD, then we want to load the one according to what it is set to.
 	if(!strcmp(name, "pics/i_health.pcx")) 
@@ -1654,12 +1445,13 @@ image_t	*GL_FindImage (char *name, imagetype_t type)
 			strcpy(name, cl_hudimage2->string);
 
 	// look for it
+	COMPUTE_HASH_KEY( hash_key, name, i );
 	for (i=0, image=gltextures ; i<numgltextures ; i++,image++)
 	{
-		if (!strcmp(name, image->name))
+		if (hash_key == image->hash_key && !strcmp(name, image->name))
 		{
 			image->registration_sequence = registration_sequence;
-			return image;
+			goto ret_image;
 		}
 	}
 
@@ -1704,6 +1496,7 @@ done:
 	if (palette)
 		free(palette);
 
+ret_image:
 	return image; 
 }
 
@@ -1733,11 +1526,14 @@ void GL_FreeUnusedImages (void)
 	int		i;
 	image_t	*image;
 
-	// never free r_notexture or particle texture
+	// never free r_notexture or particle textures
 	r_notexture->registration_sequence = registration_sequence;
 	r_particletexture->registration_sequence = registration_sequence;
-    r_shelltexture->registration_sequence = registration_sequence;    // c14 added shell texture
+    r_shelltexture->registration_sequence = registration_sequence;   
+	r_shelltexture2->registration_sequence = registration_sequence;
+	r_shellnormal->registration_sequence = registration_sequence;
 	r_reflecttexture->registration_sequence = registration_sequence;
+	r_mirrorspec->registration_sequence = registration_sequence;
 	r_flare->registration_sequence = registration_sequence;
 	r_smoketexture->registration_sequence = registration_sequence;
   	r_explosiontexture->registration_sequence = registration_sequence;
@@ -1750,26 +1546,36 @@ void GL_FreeUnusedImages (void)
 	r_explosion7texture->registration_sequence = registration_sequence;
 	r_bloodtexture->registration_sequence = registration_sequence;
 	r_pufftexture->registration_sequence = registration_sequence;
-	r_blastertexture->registration_sequence = registration_sequence;
 	r_bflashtexture->registration_sequence = registration_sequence;
 	r_cflashtexture->registration_sequence = registration_sequence;
+	r_beamtexture->registration_sequence = registration_sequence;
+	r_beam2texture->registration_sequence = registration_sequence;
+	r_beam3texture->registration_sequence = registration_sequence;
 	r_leaderfieldtexture->registration_sequence = registration_sequence;
 	r_deathfieldtexture->registration_sequence = registration_sequence;
+	r_deathfieldtexture2->registration_sequence = registration_sequence;
 	r_hittexture->registration_sequence = registration_sequence;
 	r_bubbletexture->registration_sequence = registration_sequence;
 	r_shottexture->registration_sequence = registration_sequence;
 	r_bullettexture->registration_sequence = registration_sequence;
+	r_bulletnormal->registration_sequence = registration_sequence;
 	r_sayicontexture->registration_sequence = registration_sequence;
 	r_voltagetexture->registration_sequence = registration_sequence;
-	r_cubemap->registration_sequence = registration_sequence;
 	r_raintexture->registration_sequence = registration_sequence;
-
-	//minimap stuff - only if those textures are present, as this was a feature added later on
-	//and if crx has been updated via galaxy, clients may not have these images
-	if(r_radarmap)
-		r_radarmap->registration_sequence = registration_sequence;
-	if(r_around)
-		r_around->registration_sequence = registration_sequence;
+	r_leaftexture->registration_sequence = registration_sequence;
+	r_splashtexture->registration_sequence = registration_sequence;
+	r_splash2texture->registration_sequence = registration_sequence;
+	r_flagtexture->registration_sequence = registration_sequence;
+	r_logotexture->registration_sequence = registration_sequence;
+	r_distort->registration_sequence = registration_sequence;
+	r_radarmap->registration_sequence = registration_sequence;
+	r_around->registration_sequence = registration_sequence;
+	sun_object->registration_sequence = registration_sequence;
+	sun1_object->registration_sequence = registration_sequence;
+	sun2_object->registration_sequence = registration_sequence;
+	r_dis1texture->registration_sequence = registration_sequence;
+	r_dis2texture->registration_sequence = registration_sequence;
+	r_dis3texture->registration_sequence = registration_sequence;
 
 	for (i=0, image=gltextures ; i<numgltextures ; i++, image++)
 	{
@@ -1823,6 +1629,42 @@ int Draw_GetPalette (void)
 	return 0;
 }
 
+void R_InitMirrorTextures( void )
+{
+	byte	*data;
+	int		size;
+
+	//init the partial screen texture
+	size = 512 * 512 * 4;
+	data = malloc( size );
+	memset( data, 255, size );
+	r_mirrortexture = GL_LoadPic( "***r_mirrortexture***", (byte *)data, 512, 512, it_pic, 32 );
+	free ( data );
+}
+
+void R_InitDepthTextures( void )
+{
+	byte	*data;
+	int		size, texture_height, texture_width;
+
+	//find closer power of 2 to screen size 
+	for (texture_width = 1;texture_width < viddef.width;texture_width *= 2);
+	for (texture_height = 1;texture_height < viddef.height;texture_height *= 2);
+
+	//limit to 2048x2048 - anything larger is generally going to cause problems, and AA doesn't support res higher
+	if(texture_width > 2048)
+		texture_width = 2048;
+	if(texture_height > 2048)
+		texture_height = 2048;
+	
+	//init the framebuffer texture
+	size = texture_width * texture_height * 4;
+	data = malloc( size );
+	memset( data, 255, size );
+	r_depthtexture = GL_LoadPic( "***r_depthtexture***", (byte *)data, texture_width, texture_height, it_pic, 3 );
+	free ( data );	
+}
+
 
 /*
 ===============
@@ -1831,18 +1673,10 @@ GL_InitImages
 */
 void	GL_InitImages (void)
 {
-	int		i, j;
-	float	g = vid_gamma->value;
 
 	registration_sequence = 1;
 
-	// init intensity conversions
-	intensity = Cvar_Get ("intensity", "1", 0);
-
-	if ( intensity->value <= 1 )
-		Cvar_Set( "intensity", "1" );
-
-	gl_state.inverse_intensity = 1 / intensity->value;
+	gl_state.inverse_intensity = 1;
 
 	Draw_GetPalette ();
 
@@ -1853,38 +1687,10 @@ void	GL_InitImages (void)
 			Sys_Error( ERR_FATAL, "Couldn't load pics/16to8.pcx");
 	}
 
-	if ( gl_config.renderer & ( GL_RENDERER_VOODOO | GL_RENDERER_VOODOO2 ) )
-	{
-		g = 1.0F;
-	}
-
-	for ( i = 0; i < 256; i++ )
-	{
-		if ( g == 1 )
-		{
-			gammatable[i] = i;
-		}
-		else
-		{
-			float inf;
-
-			inf = 255 * pow ( (i+0.5)/255.5 , g ) + 0.5;
-			if (inf < 0)
-				inf = 0;
-			if (inf > 255)
-				inf = 255;
-			gammatable[i] = inf;
-		}
-	}
-
-	for (i=0 ; i<256 ; i++)
-	{
-		j = i*intensity->value;
-		if (j > 255)
-			j = 255;
-		intensitytable[i] = j;
-	}
 	R_InitBloomTextures();//BLOOMS
+	R_InitMirrorTextures();//MIRRORS
+	R_InitDepthTextures();//DEPTH(SHADOWMAPS)
+	R_FB_InitTextures();//FULLSCREEN EFFECTS
 }
 
 /*
