@@ -23,9 +23,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 int	r_dlightframecount;
 
-// #define	DLIGHT_CUTOFF	64
-#define DLIGHT_CUTOFF	0
-
 /*
 =============================================================================
 
@@ -41,42 +38,49 @@ R_MarkLights
 */
 void R_MarkLights (dlight_t *light, int bit, mnode_t *node)
 {
-	cplane_t	*splitplane;
-	float		dist;
-	msurface_t	*surf;
-	int			i;
-
-	if (node->contents != -1)
-		return;
-
-	splitplane = node->plane;
-	dist = DotProduct (light->origin, splitplane->normal) - splitplane->dist;
-
-	if (dist > light->intensity-DLIGHT_CUTOFF)
-	{
-		R_MarkLights (light, bit, node->children[0]);
-		return;
-	}
-	if (dist < -light->intensity+DLIGHT_CUTOFF)
-	{
-		R_MarkLights (light, bit, node->children[1]);
-		return;
-	}
-
+          cplane_t  *splitplane;
+          float              dist;
+          msurface_t         *surf;
+          int                i, sidebit;
+          
+          if (node->contents != -1)
+                   return;
+ 
+          splitplane = node->plane;
+          dist = DotProduct (light->origin, splitplane->normal) - splitplane->dist;
+ 
+          if (dist > light->intensity-DLIGHT_CUTOFF) {
+                   R_MarkLights (light, bit, node->children[0]);
+                   return;
+          }
+          if (dist < -light->intensity+DLIGHT_CUTOFF) {
+                   R_MarkLights (light, bit, node->children[1]);
+                   return;
+          }
+ 
 // mark the polygons
-	surf = r_worldmodel->surfaces + node->firstsurface;
-	for (i=0 ; i<node->numsurfaces ; i++, surf++)
-	{
-		if (surf->dlightframe != r_dlightframecount)
-		{
-			surf->dlightbits = 0;
-			surf->dlightframe = r_dlightframecount;
-		}
-		surf->dlightbits |= bit;
-	}
-
-	R_MarkLights (light, bit, node->children[0]);
-	R_MarkLights (light, bit, node->children[1]);
+          surf = r_worldmodel->surfaces + node->firstsurface;
+          for (i=0 ; i<node->numsurfaces ; i++, surf++)
+          {
+                   dist = DotProduct (light->origin, surf->plane->normal) - surf->plane->dist;         //Discoloda
+                   if (dist >= 0)                                                                              //Discoloda
+                             sidebit = 0;                                                                      //Discoloda
+                   else                                                                                        //Discoloda
+                             sidebit = SURF_PLANEBACK;                                                //Discoloda
+ 
+                   if ( (surf->flags & SURF_PLANEBACK) != sidebit )                                   //Discoloda
+                             continue;                                                                //Discoloda
+ 
+                   if (surf->dlightframe != r_dlightframecount)
+                   {
+                             surf->dlightbits = bit;
+                             surf->dlightframe = r_dlightframecount;
+                   } else
+                             surf->dlightbits |= bit;
+          }
+ 
+          R_MarkLights (light, bit, node->children[0]);
+          R_MarkLights (light, bit, node->children[1]);
 }
 
 
@@ -298,7 +302,7 @@ void R_LightPoint (vec3_t p, vec3_t color, qboolean addDynamic)
 	VectorClear ( dlightcolor );
 	for (lnum=0 ; lnum<r_newrefdef.num_dlights ; lnum++, dl++)
 	{
-		VectorSubtract (currententity->origin,
+		VectorSubtract (p,
 						dl->origin,
 						dist);
 		add = dl->intensity - VectorLength(dist);
@@ -429,256 +433,11 @@ void R_LightPointDynamics (vec3_t p, vec3_t color, m_dlight_t *list, int *amount
 
 }
 
-//===================================================================
-
-/*
-=============
-R_AddStainMap
-=============
-*/
-void R_AddStainMap (stain_t *st, msurface_t *surf)
-{
-	int			i;
-	mtexinfo_t	*tex;
-	int			sd, td;
-	float		fsacc, ftacc;
-	float		dist, fdist, frad, fminlight;
-	vec3_t		impact, local;
-	int			s, t, smax, tmax;
-	byte		*pfBL;
-
-	if (!surf->stainsamples)
-		return;
-	tex = surf->texinfo;
-	if ( (tex->flags & (SURF_SKY|SURF_TRANS33|SURF_TRANS66|SURF_WARP) ) )
-		return;
-
-	if ( surf->plane->type < 3 ) {
-		dist = st->origin[surf->plane->type] - surf->plane->dist;
-	} else {
-		dist = DotProduct (st->origin, surf->plane->normal) - surf->plane->dist;
-	}
-
-	if (surf->flags & SURF_PLANEBACK)
-		fdist = -dist;
-	else
-		fdist = dist;
-
-	frad = st->intensity - fabs (fdist);
-	if (frad < 0)
-		return;
-
-	smax = (surf->extents[0]>>4)+1;
-	tmax = (surf->extents[1]>>4)+1;
-
-	fminlight = frad;
-
-	for (i = 0 ; i<3 ; i++)
-		impact[i] = st->origin[i] - surf->plane->normal[i] * fdist;
-
-	local[0] = DotProduct (impact, tex->vecs[0]) + tex->vecs[0][3] - surf->texturemins[0];
-	local[1] = DotProduct (impact, tex->vecs[1]) + tex->vecs[1][3] - surf->texturemins[1];
-
-	pfBL = surf->stainsamples;
-	surf->cached_light[0] = 0;
-
-	for (t = 0, ftacc = 0 ; t<tmax ; t++, ftacc += 16)
-	{
-		td = local[1] - ftacc;
-		if ( td < 0 )
-			td = -td;
-		for ( s=0, fsacc = 0 ; s<smax ; s++, fsacc += 16, pfBL += 3)
-		{
-			sd = Q_ftol( local[0] - fsacc );
-
-			if ( sd < 0 )
-				sd = -sd;
-
-			if (sd > td)
-				fdist = sd + (td>>1);
-			else
-				fdist = td + (sd>>1);
-
-			if ( fdist < fminlight )
-			{
-				float alpha, mult;
-				int test;
-
-				mult = frad / fdist;
-				if (mult > 5.0)
-					mult = 5.0;
-
-				alpha = st->alpha * mult;
-				if (alpha > 255)
-					alpha = 255;
-				if (alpha > st->alpha)
-					alpha = st->alpha;
-				if (alpha < 0)
-					alpha = 0;
-				alpha *= (1.0 / 255.0);
-
-				for (i=0;i<3;i++)
-				{
-					test = Q_ftol( (1-alpha) * (float)pfBL[i] + alpha * st->color[i] );
-
-					if (test > 255)
-						pfBL[i] = 255;
-					else if (test < 0)
-						pfBL[i] = 0;
-					else
-						pfBL[i] = (byte)(test);
-				}
-			}
-		}
-	}
-}
-
-/*
-=============
-R_StainNode
-=============
-*/
-void R_StainNode (stain_t *st, mnode_t *node)
-{
-	int		c;
-	float	dist;
-	msurface_t *surf;
-
-	if (node->contents != -1)
-		return;
-
-	if ( node->plane->type < 3 ) {
-		dist = st->origin[node->plane->type] - node->plane->dist;
-	} else {
-		dist = DotProduct (st->origin, node->plane->normal) - node->plane->dist;
-	}
-
-	if (dist > st->intensity)
-	{
-		R_StainNode (st, node->children[0]);
-		return;
-	}
-
-	if (dist < -st->intensity)
-	{
-		R_StainNode (st, node->children[1]);
-		return;
-	}
-
-	for (c = node->numsurfaces, surf = r_worldmodel->surfaces + node->firstsurface; c ; c--, surf++)
-	{
-		R_AddStainMap ( st, surf );
-	}
-
-	if (node->contents == -1)
-		R_StainNode (st, node->children[0]);
-	if (node->contents == -1)
-		R_StainNode (st, node->children[1]);
-}
-
-/*
-=============
-R_PushStains
-=============
-*/
-void R_PushStains (void)
-{
-	int		i;
-	stain_t	*st;
-
-	st = r_newrefdef.stains;
-	for (i=0 ; i<r_newrefdef.num_stains ; i++, st++) {
-		R_StainNode ( st, r_worldmodel->nodes );
-	}
-}
-
-/*
-=============
-R_PushStainsForBModel
-=============
-*/
-void R_PushStainsForBModel (entity_t *e)
-{
-	int i, c;
-	stain_t	*st;
-	model_t *model;
-	msurface_t *surf;
-
-	st = r_newrefdef.stains;
-	model = e->model;
-
-	if (e->angles[0] || e->angles[1] || e->angles[2])
-	{
-		vec3_t temp;
-		vec3_t forward, right, up;
-
-		AngleVectors ( e->angles, forward, right, up );
-
-		for (i=0 ; i<r_newrefdef.num_stains ; i++, st++) {
-			VectorSubtract (st->origin, e->origin, temp);
-			st->origin[0] = DotProduct (temp, forward);
-			st->origin[1] = -DotProduct (temp, right);
-			st->origin[2] = DotProduct (temp, up);
-
-			surf = &model->surfaces[model->firstmodelsurface];
-			for (c=0 ; c<model->nummodelsurfaces ; c++, surf++)
-			{
-				R_AddStainMap ( st, surf );
-			}
-
-			VectorAdd ( temp, e->origin, st->origin );
-		}
-	}
-	else
-	{
-		for (i=0 ; i<r_newrefdef.num_stains ; i++, st++) {
-			VectorSubtract (st->origin, e->origin, st->origin);
-
-			surf = &model->surfaces[model->firstmodelsurface];
-			for (c=0 ; c<model->nummodelsurfaces ; c++, surf++)
-			{
-				R_AddStainMap ( st, surf );
-			}
-
-			VectorAdd ( st->origin, e->origin, st->origin );
-		}
-	}
-}
 
 //===================================================================
 
 static float s_blocklights[34*34*3];
 
-/*
-===============
-R_AddStainMaps
-===============
-*/
-void R_AddStainMaps (msurface_t *surf)
-{
-	int			s, t, i;
-	int			smax, tmax;
-	float		*pfBL;
-	byte		*pfBS;
-	float		scale;
-
-	scale = gl_modulate->value;
-
-	smax = (surf->extents[0]>>4)+1;
-	tmax = (surf->extents[1]>>4)+1;
-
-	pfBL = s_blocklights;
-	pfBS = surf->stainsamples;
-
-	for (t = 0; t<tmax; t++)
-		for ( s=0; s<smax; s++, pfBL += 3, pfBS +=3)
-			for (i=0;i<3;i++)
-			{
-				if (pfBL[i] > pfBS[i]*scale)
-					pfBL[i] = pfBS[i]*scale;
-			}
-}
-
 /*
 ===============
 R_AddDynamicLights
@@ -919,9 +678,6 @@ void R_BuildLightMap (msurface_t *surf, byte *dest, int stride)
 	if (surf->dlightframe == r_framecount)
 		R_AddDynamicLights (surf);
 
-	if (surf->stainsamples)
-		R_AddStainMaps (surf);
-
 // put into texture format
 store:
 	stride -= (smax<<2);
@@ -988,7 +744,10 @@ void R_RenderFlare (flare_t *light)
 	float	dist;
 	unsigned	flaretex;
 
-    flaretex =  r_flare->texnum;
+	if(light->style == 0)
+		flaretex =  r_flare->texnum;
+	else
+		flaretex = r_flare1->texnum;
 
 	VectorSubtract (light->origin, r_origin, v);
 	dist = VectorLength(v) * (light->size*0.01);
@@ -997,6 +756,8 @@ void R_RenderFlare (flare_t *light)
 	if(dist > 10*light->size)
 		dist = 10*light->size;
 
+	R_InitQuadVarrays();
+
 	qglDisable(GL_DEPTH_TEST);
 	qglEnable (GL_TEXTURE_2D);
 	GL_Bind(flaretex);
@@ -1031,6 +792,8 @@ void R_RenderFlare (flare_t *light)
 	qglDisable (GL_TEXTURE_2D);
 	qglDisableClientState(GL_COLOR_ARRAY);
 
+	R_KillVArrays();
+
 }
 
 int r_numflares;
@@ -1062,7 +825,7 @@ void R_RenderFlares (void)
 		// periodically test visibility to ramp alpha
 		if(rs_realtime - l->time > 0.02){
 			
-			r_trace = CM_BoxTrace(r_origin, l->origin, mins, maxs, r_worldmodel->firstnode, MASK_SHOT);
+			r_trace = CM_BoxTrace(r_origin, l->origin, mins, maxs, r_worldmodel->firstnode, MASK_VISIBILILITY);
 			visible = r_trace.fraction == 1.0;
 			
 			l->alpha += (visible ? 0.03 : -0.15);  // ramp
@@ -1088,6 +851,213 @@ void R_RenderFlares (void)
 	qglDepthMask (1);
 }
 
+
+/*
+================
+Berserker@quake2 Sun
+================
+*/
+qboolean draw_sun = false;
+float sun_size = 0.20f;
+vec3_t sun_origin;
+float sun_x, sun_y;
+qboolean spacebox = false;
+
+void transform_point(float out[4], const float m[16], const float in[4])
+{
+#define M(row,col)  m[col*4+row]
+	out[0] =
+		M(0, 0) * in[0] + M(0, 1) * in[1] + M(0, 2) * in[2] + M(0,
+																3) * in[3];
+	out[1] =
+		M(1, 0) * in[0] + M(1, 1) * in[1] + M(1, 2) * in[2] + M(1,
+																3) * in[3];
+	out[2] =
+		M(2, 0) * in[0] + M(2, 1) * in[1] + M(2, 2) * in[2] + M(2,
+																3) * in[3];
+	out[3] =
+		M(3, 0) * in[0] + M(3, 1) * in[1] + M(3, 2) * in[2] + M(3,
+																3) * in[3];
+#undef M
+}
+
+
+qboolean gluProject2(float objx, float objy, float objz, const float model[16], const float proj[16], const int viewport[4], float *winx, float *winy)	// /, 
+																																						// float 
+																																						// *winz)
+{
+	/* matrice de transformation */
+	float in[4], out[4], temp;
+
+	/* initilise la matrice et le vecteur a transformer */
+	in[0] = objx;
+	in[1] = objy;
+	in[2] = objz;
+	in[3] = 1.0;
+	transform_point(out, model, in);
+	transform_point(in, proj, out);
+
+	/* d'ou le resultat normalise entre -1 et 1 */
+	if (in[3] == 0.0)
+		return false;
+
+	temp = 1.0 / in[3];
+	in[0] *= temp;
+	in[1] *= temp;
+	in[2] *= temp;
+
+	/* en coordonnees ecran */
+	*winx = viewport[0] + (1 + in[0]) * (float) viewport[2] * 0.5;
+	*winy = viewport[1] + (1 + in[1]) * (float) viewport[3] * 0.5;
+///   /* entre 0 et 1 suivant z */
+///   *winz = (1 + in[2]) * 0.5;
+	return true;
+}
+
+void R_InitSun()
+{
+	draw_sun = false;
+
+	if (!sun_size)
+		return;
+
+	if (!r_drawsun->value)
+		return;
+
+	if (spacebox)
+		sun_size = 0.1f;
+	else
+		sun_size = 0.2f;
+
+	if (R_CullOrigin(sun_origin))
+		return;
+
+	draw_sun = true;
+
+	gluProject2(sun_origin[0], sun_origin[1], sun_origin[2], r_world_matrix, r_project_matrix, (int *) r_viewport, &sun_x, &sun_y);	// /, 
+																																	// &sun_z);
+	sun_y = r_newrefdef.height - sun_y;
+}
+
+
+void R_RenderSunFlare(image_t * tex, float offset, float size, float r,
+					  float g, float b, float alpha)
+{
+	float minx, miny, maxx, maxy;
+	float new_x, new_y, corr;
+
+	qglColor4f(r, g, b, alpha);
+	GL_Bind(tex->texnum);
+
+	if (offset) {
+		new_x = offset * (r_newrefdef.width / 2 - sun_x) + sun_x;
+		new_y = offset * (r_newrefdef.height / 2 - sun_y) + sun_y;
+	} else {
+		new_x = sun_x;
+		new_y = sun_y;
+	}
+
+	corr = 1;
+
+	minx = new_x - size * corr;
+	miny = new_y - size;
+	maxx = new_x + size * corr;
+	maxy = new_y + size;
+
+	qglBegin(GL_QUADS);
+	qglTexCoord2f(0, 0);
+	qglVertex2f(minx, miny);
+	qglTexCoord2f(1, 0);
+	qglVertex2f(maxx, miny);
+	qglTexCoord2f(1, 1);
+	qglVertex2f(maxx, maxy);
+	qglTexCoord2f(0, 1);
+	qglVertex2f(minx, maxy);
+	qglEnd();
+}
+
+float sun_time = 0;
+float sun_alpha = 0;
+void R_RenderSun()
+{
+	float l, hx, hy;
+	float vec[2];
+	float size;
+
+	if (!draw_sun)
+		return;
+
+	if (r_newrefdef.rdflags & RDF_NOWORLDMODEL)
+		return;
+
+	qglReadPixels(sun_x, r_newrefdef.height - sun_y, 1, 1,
+				  GL_DEPTH_COMPONENT, GL_FLOAT, &l);
+
+	// periodically test visibility to ramp alpha
+	if(rs_realtime - sun_time > 0.02) {
+
+		sun_alpha += (l == 1.0 ? 0.15 : -0.15);  // ramp
+			
+		if(sun_alpha > 1.0)  // clamp
+			sun_alpha = 1.0;
+		else if(sun_alpha < 0)
+			sun_alpha = 0.0;
+
+		sun_time = rs_realtime;
+	}
+
+	if (sun_alpha > 0)
+	{
+
+		hx = r_newrefdef.width / 2;
+		hy = r_newrefdef.height / 2;
+		vec[0] = 1 - fabs(sun_x - hx) / hx;
+		vec[1] = 1 - fabs(sun_y - hy) / hy;
+		l = 3 * vec[0] * vec[1] + 0.25;
+
+		// set 2d
+		qglMatrixMode(GL_PROJECTION);
+		qglPushMatrix();
+		qglLoadIdentity();
+		qglOrtho(0, r_newrefdef.width, r_newrefdef.height, 0, -99999,
+				 99999);
+		qglMatrixMode(GL_MODELVIEW);
+		qglPushMatrix();
+		qglLoadIdentity();
+		qglEnable(GL_BLEND);
+		qglBlendFunc(GL_SRC_ALPHA, GL_ONE);
+		GL_TexEnv(GL_MODULATE);
+		qglDepthRange(0, 0.3);
+
+		size = r_newrefdef.width * sun_size;
+		R_RenderSunFlare(sun_object, 0, size, .75, .75, .75, sun_alpha);
+		if (r_drawsun->value == 2) {
+	
+			R_RenderSunFlare(sun2_object, -0.9, size * 0.07, 0.1, 0.1, 0, sun_alpha);
+			R_RenderSunFlare(sun2_object, -0.7, size * 0.15, 0, 0, 0.1, sun_alpha);
+			R_RenderSunFlare(sun2_object, -0.5, size * 0.085, 0.1, 0, 0, sun_alpha);
+			R_RenderSunFlare(sun1_object, 0.3, size * 0.25, 0.1, 0.1, 0.1, sun_alpha);
+			R_RenderSunFlare(sun2_object, 0.5, size * 0.05, 0.1, 0, 0, sun_alpha);
+			R_RenderSunFlare(sun2_object, 0.64, size * 0.05, 0, 0.1, 0, sun_alpha);
+			R_RenderSunFlare(sun2_object, 0.7, size * 0.25, 0.1, 0.1, 0, sun_alpha);
+			R_RenderSunFlare(sun1_object, 0.85, size * 0.5, 0.1, 0.1, 0.1, sun_alpha);
+			R_RenderSunFlare(sun2_object, 1.1, size * 0.125, 0.1, 0, 0, sun_alpha);
+			R_RenderSunFlare(sun2_object, 1.25, size * 0.08, 0.1, 0.1, 0, sun_alpha);
+		}
+
+		qglDepthRange(0, 1);
+		qglColor4f(1, 1, 1, 1);
+		qglDisable(GL_BLEND);
+		qglBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+		// set 3d
+		qglPopMatrix();
+		qglMatrixMode(GL_PROJECTION);
+		qglPopMatrix();
+		qglMatrixMode(GL_MODELVIEW);
+	}
+}
+
+
 /*
 ===============
 R_ShadowLight
@@ -1171,13 +1141,14 @@ float R_ShadowLight (vec3_t pos, vec3_t lightAdd, int type)
 
 		for (i=0; i<r_numWorldLights; i++) {
 
-			wl = &r_worldLights[i];
+			wl = &r_worldLights[i];			
+			
 			VectorSubtract (wl->origin, pos, dist);
-			add = sqrt(wl->intensity - VectorLength(dist)/3);
+			add = sqrt(wl->intensity*10 - VectorLength(dist));
 			VectorNormalize(dist);
-			if (add > 0)
+			if (add > 0 && wl->origin[2] > pos[2]) 
 			{
-				VectorScale(dist, add, dist);
+				VectorScale(dist, sqrt(add), dist);
 				VectorAdd (lightAdd, dist, lightAdd);
 				lintens = wl->intensity;
 				if(lintens < 300)
@@ -1185,6 +1156,7 @@ float R_ShadowLight (vec3_t pos, vec3_t lightAdd, int type)
 				intens+=(lintens/1000 - VectorLength(dist)/30); //darken shadows where light is stronger
 			}
 		}
+
 		//cap some limits of lightness, darkness, subtley.
 		if (intens < 0.3)
 			intens = 0.3;
