@@ -39,7 +39,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include "qgl.h"
 #include "r_math.h"
-#define	REF_VERSION	"GL 0.01"
 
 // up / down
 #define	PITCH	0
@@ -50,6 +49,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 // fall over
 #define	ROLL	2
 
+#define	DLIGHT_CUTOFF	64
+
 extern	viddef_t	vid;
 
 #include "r_image.h"
@@ -68,6 +69,8 @@ typedef enum
 
 #include "r_model.h"
 
+extern float	r_frametime;
+
 void GL_BeginRendering (int *x, int *y, int *width, int *height);
 void GL_EndRendering (void);
 
@@ -87,6 +90,8 @@ extern	int			r_framecount;
 extern	cplane_t	frustum[4];
 extern	int			c_brush_polys, c_alias_polys;
 extern 	int c_flares;
+extern  int c_grasses;
+extern	int c_beams;
 
 extern	int			gl_filter_min, gl_filter_max;
 
@@ -117,8 +122,6 @@ extern	cvar_t	*r_lerpmodels;
 extern	cvar_t	*r_lightlevel;	// FIXME: This is a HACK to get the client's light level
 extern  cvar_t  *r_wave; //water waves
 
-extern cvar_t	*gl_vertex_arrays;
-
 extern cvar_t	*gl_ext_swapinterval;
 extern cvar_t	*gl_ext_palettedtexture;
 extern cvar_t	*gl_ext_multitexture;
@@ -132,7 +135,6 @@ extern cvar_t	*gl_particle_att_a;
 extern cvar_t	*gl_particle_att_b;
 extern cvar_t	*gl_particle_att_c;
 
-extern	cvar_t	*gl_nosubimage;
 extern	cvar_t	*gl_bitdepth;
 extern	cvar_t	*gl_mode;
 extern	cvar_t	*gl_log;
@@ -144,6 +146,7 @@ extern	cvar_t	*gl_round_down;
 extern	cvar_t	*gl_picmip;
 extern	cvar_t	*gl_skymip;
 extern	cvar_t	*gl_showtris;
+extern	cvar_t	*gl_showpolys;
 extern	cvar_t	*gl_finish;
 extern	cvar_t	*gl_clear;
 extern	cvar_t	*gl_cull;
@@ -163,12 +166,16 @@ extern	cvar_t	*gl_texturealphamode;
 extern	cvar_t	*gl_texturesolidmode;
 extern	cvar_t	*gl_lockpvs;
 extern	cvar_t	*gl_rtlights;
-extern	cvar_t	*gl_texres;
 
 extern	cvar_t	*vid_fullscreen;
 extern	cvar_t	*vid_gamma;
+extern  cvar_t	*vid_contrast;
 
 extern	cvar_t	*intensity;
+
+extern cvar_t *r_anisotropic;
+extern cvar_t *r_ext_max_anisotropy;
+
 // Vic - begin
 
 extern cvar_t	*r_overbrightbits;
@@ -176,26 +183,31 @@ extern cvar_t	*gl_ext_mtexcombine;
 
 // Vic - end
 extern cvar_t	*gl_normalmaps;
-extern cvar_t	*gl_specularmaps;
-extern cvar_t	*gl_cubemaps;
+extern cvar_t	*gl_shadowmaps;
+extern cvar_t	*gl_parallaxmaps;
+extern cvar_t	*gl_specular;
+extern cvar_t	*gl_glsl_postprocess;
 
 extern	cvar_t	*r_shaders;
 extern	cvar_t	*r_bloom;
 extern	cvar_t	*r_lensflare;
 extern	cvar_t	*r_lensflare_intens;
+extern	cvar_t	*r_drawsun;
 
 extern	qboolean	map_fog;
 extern	char		map_music[128];
+extern  unsigned	r_weather;
 
 extern  cvar_t		*r_minimap;
 extern  cvar_t		*r_minimap_size;
 extern  cvar_t		*r_minimap_zoom;
 extern  cvar_t		*r_minimap_style;
 
-extern	cvar_t	*gl_reflection;			// MPO
-extern	cvar_t	*gl_reflection_debug;	// MPO	for debugging the reflection
-extern	cvar_t	*gl_reflection_max;		// MPO  max number of water reflections
-extern	cvar_t	*gl_arb_fragment_program; // jit
+extern	cvar_t	*gl_mirror;
+
+extern	cvar_t	*gl_arb_fragment_program; 
+extern	cvar_t	*gl_glsl_shaders;
+
 
 extern	cvar_t	*sys_affinity;
 extern	cvar_t	*sys_priority;
@@ -203,6 +215,8 @@ extern	cvar_t	*sys_priority;
 extern	cvar_t	*gl_screenshot_type;
 extern	cvar_t	*gl_screenshot_jpeg_quality;
 
+extern	cvar_t	*r_legacy;
+
 extern	int		gl_lightmap_format;
 extern	int		gl_solid_format;
 extern	int		gl_alpha_format;
@@ -213,6 +227,8 @@ extern	int		c_visible_lightmaps;
 extern	int		c_visible_textures;
 
 extern	float	r_world_matrix[16];
+extern float r_project_matrix[16];
+extern int	r_viewport[4];
 extern	float		r_farclip, r_farclip_min, r_farclip_bias;
 void GL_Bind (int texnum);
 void GL_MBind( GLenum target, int texnum );
@@ -223,10 +239,8 @@ void vectoangles (vec3_t value1, vec3_t angles);
 void R_LightPoint (vec3_t p, vec3_t color, qboolean addDynamic);
 void R_LightPointDynamics (vec3_t p, vec3_t color, m_dlight_t *list, int *amount, int max);
 void R_PushDlights (void);
-void R_PushStains (void);
-void R_PushStainsForBModel (entity_t *e);
 void R_PushDlightsForBModel (entity_t *e);
-
+void SetVertexOverbrights (qboolean toggle);
 void RefreshFont (void);
 
 //====================================================================
@@ -253,8 +267,8 @@ extern	int		registration_sequence;
 
 void V_AddBlend (float r, float g, float b, float a, float *v_blend);
 
-int 	R_Init( void *hinstance, void *hWnd );
-void	R_Shutdown( void );
+int	R_Init( void *hinstance, void *hWnd );
+void R_Shutdown( void );
 
 void R_RenderView (refdef_t *fd);
 void GL_ScreenShot_f (void);
@@ -265,25 +279,37 @@ void R_DrawWorld (void);
 void R_RenderDlights (void);
 void R_DrawAlphaSurfaces (void);
 void R_RenderBrushPoly (msurface_t *fa);
+void R_DrawSpecialSurfaces(void);
 void R_InitParticleTexture (void);
+void R_DrawParticles (void);
 void GL_DrawRadar(void);
 void Draw_InitLocal (void);
 void GL_SubdivideSurface (msurface_t *fa);
 qboolean R_CullBox (vec3_t mins, vec3_t maxs);
+qboolean R_CullOrigin(vec3_t origin);
 void R_RotateForEntity (entity_t *e);
 void R_MarkLeaves (void);
 void R_AddSkySurface (msurface_t *fa);
-void EmitWaterPolys (msurface_t *fa);
-void EmitWaterPolys_original (msurface_t *fa);
+void GL_RenderWaterPolys (msurface_t *fa, int texnum, float scaleX, float scaleY);
 void R_ClearSkyBox (void);
 void R_DrawSkyBox (void);
-void R_DrawSkyBoxVooDoo (void);
 void R_MarkLights (dlight_t *light, int bit, mnode_t *node);
 float R_ShadowLight (vec3_t pos, vec3_t lightAdd, int type);
+void R_DrawShadowVolume(entity_t * e);
+void R_ShadowBlend(float alpha);
 #ifdef __unix__
 void R_ReadFogScript(char config_file[128]);
 void R_ReadMusicScript(char config_file[128]);
 #endif
+void R_GLSLPostProcess(void);
+void R_FB_InitTextures(void);
+
+//VBO
+void R_VCInit(void);
+void GL_BindVBO(vertCache_t *cache);
+vertCache_t *R_VCFindCache(vertStoreMode_t store, entity_t *ent);
+vertCache_t *R_VCLoadData(vertCacheMode_t mode, int size, void *buffer, vertStoreMode_t store, entity_t *ent);
+void R_VCShutdown(void);
 
 //BLOOMS[start]
 //
@@ -293,11 +319,17 @@ void R_BloomBlend( refdef_t *fd );
 void R_InitBloomTextures( void );
 //BLOOMS[end]
 
+//FLARES and SUN
 void R_RenderFlares (void);
-//lava haze [start]
-//void R_InitSmokes (void);
-//void R_SmokeFrame (void);
-//lava haze [end]
+void R_DrawVegetationSurface (void);
+void R_DrawBeamSurface ( void );
+void R_InitSun();
+void R_RenderSun();
+vec3_t sun_origin;
+qboolean spacebox;
+
+//Player icons
+extern float	scr_playericonalpha;
 
 void	Draw_GetPicSize (int *w, int *h, char *name);
 void	Draw_Pic (int x, int y, char *name);
@@ -305,8 +337,8 @@ void	Draw_ScaledPic (int x, int y, float scale, char *pic);
 void	Draw_StretchPic (int x, int y, int w, int h, char *name);
 void	Draw_Char (int x, int y, int c);
 void	Draw_ColorChar (int x, int y, int num, vec4_t color);
-void	Draw_ScaledChar(int x, int y, int num, int scale);
-void	Draw_ScaledColorChar (int x, int y, int num, vec4_t color, int scale);
+void	Draw_ScaledChar(float x, float y, int num, float scale);
+void	Draw_ScaledColorChar (float x, float y, int num, vec4_t color, float scale);
 void	Draw_TileClear (int x, int y, int w, int h, char *name);
 void	Draw_Fill (int x, int y, int w, int h, int c);
 void	Draw_FadeScreen (void);
@@ -322,6 +354,7 @@ void	GL_ResampleTexture (unsigned *in, int inwidth, int inheight, unsigned *out,
 
 image_t *R_RegisterSkin (char *name);
 image_t *R_RegisterParticlePic(char *name);
+image_t *R_RegisterParticleNormal(char *name);
 image_t *R_RegisterGfxPic(char *name);
 
 void	LoadPCX (char *filename, byte **pic, byte **palette, int *width, int *height);
@@ -402,14 +435,8 @@ typedef struct
 	const char *vendor_string;
 	const char *version_string;
 	const char *extensions_string;
-
 	qboolean	allow_cds;
-
-	// Vic - begin
-
 	qboolean 	mtexcombine;
-
-	// Vic - end
 } glconfig_t;
 
 typedef struct
@@ -423,7 +450,7 @@ typedef struct
 
 	int			lightmap_textures;
 
-	int			currenttextures[2];
+	int			currenttextures[3];
 	int			currenttmu;
 
 	float		camera_separation;
@@ -432,14 +459,17 @@ typedef struct
 	qboolean	alpha_test;
 	qboolean	blend;
 	qboolean	texgen;
-	qboolean	texshaders;
-	qboolean	fragment_program; // jitwater
+	qboolean	fragment_program; 
+	qboolean	glsl_shaders;
+	qboolean	separateStencil;
+	qboolean	stencil_wrap;
+	qboolean	vbo;
+
 } glstate_t;
 
 extern glconfig_t  gl_config;
 extern glstate_t   gl_state;
 
-
 // vertex arrays
 
 #define MAX_ARRAY MAX_PARTICLES*4
@@ -451,6 +481,133 @@ extern glstate_t   gl_state;
 extern float	tex_array[MAX_ARRAY][2];
 extern float	vert_array[MAX_ARRAY][3];
 extern float	col_array[MAX_ARRAY][4];
+extern float	norm_array[MAX_ARRAY][3];
+
+#define MAX_VARRAY_VERTS MAX_VERTS + 2
+#define MAX_VARRAY_VERTEX_SIZE 11
+
+#define MAX_VERTICES		16384
+#define MAX_INDICES		MAX_VERTICES * 4
+#define MAX_SHADOW_VERTS	16384
+
+extern float VArrayVerts[MAX_VARRAY_VERTS * MAX_VARRAY_VERTEX_SIZE];
+extern int VertexSizes[];
+extern float *VArray;
+static vec3_t NormalsArray[MAX_TRIANGLES*3];
+extern vec3_t ShadowArray[MAX_SHADOW_VERTS];
+
+// define our vertex types
+#define VERT_SINGLE_TEXTURED			0		// verts and st for 1 tmu
+#define VERT_BUMPMAPPED					1		// verts and st for 1 tmu, but with 2 texcoord pointers
+#define VERT_MULTI_TEXTURED				2		// verts and st for 2 tmus
+#define VERT_COLOURED_UNTEXTURED		3		// verts and colour
+#define VERT_COLOURED_TEXTURED			4		// verts, st for 1 tmu and colour
+#define VERT_COLOURED_MULTI_TEXTURED	5		// verts, st for 2 tmus and colour
+#define VERT_DUAL_TEXTURED				6		// verts, st for 2 tmus both with same st
+#define VERT_NO_TEXTURE					7		// verts only, no textures
+#define VERT_BUMPMAPPED_COLOURED		8		// verts and st for 1 tmu, 2 texoord pointers and colour
+#define VERT_NORMAL_COLOURED_TEXTURED	9		// verts and st for 1tmu and color, with normals
+
+// vertex array kill flags
+#define KILL_TMU0_POINTER	1
+#define KILL_TMU1_POINTER	2
+#define KILL_TMU2_POINTER	3
+#define KILL_TMU3_POINTER	4
+#define KILL_RGBA_POINTER	5
+#define KILL_NORMAL_POINTER 6
+
+// vertex array subsystem
+void R_InitVArrays (int varraytype);
+void R_KillVArrays (void);
+void R_InitQuadVarrays(void);
+void R_AddTexturedSurfToVArray (msurface_t *surf, float scroll);
+void R_AddLightMappedSurfToVArray (msurface_t *surf, float scroll);
+void R_AddGLSLShadedSurfToVArray (msurface_t *surf, float scroll, qboolean lightmap);
+void R_AddGLSLShadedWarpSurfToVArray (msurface_t *surf, float scroll);
+void R_KillNormalTMUs(void);
+
+// stencil volumes
+extern glStencilFuncSeparatePROC	qglStencilFuncSeparate;
+extern glStencilOpSeparatePROC		qglStencilOpSeparate;
+extern glStencilMaskSeparatePROC	qglStencilMaskSeparate;
+
+//shadow maps
+GLuint fboId;
+void setupMatrices(float position_x,float position_y,float position_z,float lookAt_x,float lookAt_y,float lookAt_z);
+void setTextureMatrix();
+void R_DrawShadowMapWorld (void);
+extern image_t *r_depthtexture;
+
+//arb fragment
+extern unsigned int g_water_program_id;
+
+//glsl
+extern GLhandleARB	g_programObj;
+extern GLhandleARB	g_waterprogramObj;
+extern GLhandleARB	g_meshprogramObj;
+extern GLhandleARB	g_fbprogramObj;
+
+extern GLhandleARB	g_vertexShader;
+extern GLhandleARB	g_fragmentShader;
+
+//standard bsp surfaces
+extern GLuint		g_location_testTexture;
+extern GLuint		g_location_eyePos;
+extern GLuint		g_tangentSpaceTransform;
+extern GLuint		g_location_heightTexture;
+extern GLuint		g_location_lmTexture;
+extern GLuint		g_location_normalTexture;
+extern GLuint		g_location_bspShadowmapTexture;
+extern GLuint		g_heightMapID;
+extern GLuint		g_location_fog;
+extern GLuint		g_location_parallax;
+extern GLuint		g_location_dynamic;
+extern GLuint		g_location_specular;
+extern GLuint		g_location_shadowmap;
+extern GLuint		g_location_lightPosition;
+extern GLuint		g_location_lightColour;
+extern GLuint		g_location_lightCutoffSquared;
+
+//water
+extern GLuint		g_location_baseTexture;
+extern GLuint		g_location_normTexture;
+extern GLuint		g_location_refTexture;
+extern GLuint		g_location_tangent;
+extern GLuint		g_location_time;
+extern GLuint		g_location_lightPos;
+extern GLuint		g_location_reflect;
+extern GLuint		g_location_trans;
+extern GLuint		g_location_fogamount;
+
+//mesh
+extern GLuint		g_location_meshlightPosition;
+extern GLuint		g_location_meshnormal;
+extern GLuint		g_location_baseTex;
+extern GLuint		g_location_normTex;
+extern GLuint		g_location_fxTex;
+extern GLuint		g_location_color;
+extern GLuint		g_location_meshNormal;
+extern GLuint		g_location_meshTangent;
+extern GLuint		g_location_meshTime;
+extern GLuint		g_location_meshFog;
+extern GLuint		g_location_useFX;
+extern GLuint		g_location_useGlow;
+
+//fullscreen
+extern GLuint		g_location_framebuffTex;
+extern GLuint		g_location_distortTex;
+extern GLuint		g_location_frametime;
+extern GLuint		g_location_fxType;
+extern GLuint		g_location_fxPos;
+extern GLuint		g_location_fxColor;
+extern GLuint		g_location_fbSampleSize;
+
+#define TURBSCALE2 (256.0 / (2 * M_PI)) 
+
+// reduce runtime calcs
+#define TURBSCALE 40.743665431525205956834243423364
+
+extern float r_turbsin[];
 /*
 ====================================================================
 
