@@ -166,34 +166,6 @@ static void R_ParseLightEntities (void)
 	}
 }
 
-//only add one light per light surface
-void GL_AddSurfaceWorldLight (msurface_t *surf)
-{
-	int i, intensity;
-	glpoly_t *poly;
-	vec3_t origin = {0,0,0}, color = {0,0,0};
-
-	if (!(surf->texinfo->flags & SURF_LIGHT))
-		return;
-	if (r_numWorldLights == MAX_LIGHTS)
-		return;
-
-	for (poly=surf->polys, i=0 ; poly ; poly=poly->next, i++)
-		VectorAdd(origin, poly->center, origin);
-
-	VectorScale(origin, 1.0/(float)i, origin);
-	VectorCopy(origin, r_worldLights[r_numWorldLights].origin);
-
-	intensity = surf->texinfo->value/2;
-	if (intensity>200) intensity = 200;
-	r_worldLights[r_numWorldLights].intensity = intensity;
-	
-	r_worldLights[r_numWorldLights].surf = surf;
-
-	r_numWorldLights++;
-}
-
-
 /*
 ===============
 Mod_PointInLeaf
@@ -368,8 +340,11 @@ model_t *Mod_ForName (char *name, qboolean crash)
 					strcpy(rs,mod->skins[i]->name);
 					rs[strlen(rs)-4]=0;
 
-					//(struct rscript_s *)mod->script[i] = RS_FindScript(rs);
+#ifdef _WINDOWS
+					(struct rscript_s *)mod->script[i] = RS_FindScript(rs);
+#else
 					mod->script[i] = RS_FindScript(rs); //make it gcc 4.1.1 compatible
+#endif
 					if (mod->script[i])
 						RS_ReadyScript((rscript_t *)mod->script[i]);
 					i++;
@@ -647,7 +622,7 @@ void Mod_LoadTexinfo (lump_t *l)
 		if (out->image->script)
 			RS_ReadyScript(out->image->script);
 
-		//check for normal and specular maps
+		//check for normal, height and specular maps
 		strcpy(sv_name, name);
 		if( ( strlen( name ) + 8 ) <= MAX_QPATH )
 		{
@@ -660,16 +635,17 @@ void Mod_LoadTexinfo (lump_t *l)
 		else
 			out->normalMap = out->image; 
 
-		if( ( strlen( sv_name ) + 8 ) <= MAX_QPATH )
+		strcpy(name, sv_name);
+		if( ( strlen( name ) + 8 ) <= MAX_QPATH )
 		{
-			strcat( sv_name, "_sm.tga" );
-			out->specularMap = GL_FindImage( sv_name, it_bump );
-			if( out->specularMap == NULL ) 
-				out->specularMap = out->image; 
+			strcat( name, "_hm.tga" );
+			out->heightMap = GL_FindImage( name, it_bump );
+			if( out->heightMap == NULL ) 
+				out->heightMap = out->image; 
 		
 		}
 		else
-			out->specularMap = out->image; 
+			out->heightMap = out->image; 
 	}
 
 	// count animation frames
@@ -743,8 +719,7 @@ static void R_ClearFlares(void)
 	r_numflares = 0;
 }
 
-//this will get added in the future from a shader or a script 
-void GL_AddFlareSurface (msurface_t *surf )
+void GL_AddFlareSurface (msurface_t *surf, int type )
 {
      int i, width, height, intens;
      glpoly_t *poly;
@@ -858,14 +833,232 @@ void GL_AddFlareSurface (msurface_t *surf )
      VectorMA(origin, 2, tmp, origin);
      VectorCopy(origin, light->origin);
     
-     light->style = 1;
+     light->style = type;
 
      free (buffer); 
 }
 
+//vegetation
+int r_numgrasses;
+grass_t r_grasses[MAX_GRASSES];
+
+static void R_ClearGrasses(void)
+{
+	memset(r_grasses, 0, sizeof(r_grasses));
+	r_numgrasses = 0;
+}
+
+void GL_AddVegetationSurface (msurface_t *surf, int texnum, vec3_t color, float size, char name[MAX_QPATH], int type)
+{
+    glpoly_t *poly;
+    grass_t  *grass; 
+	image_t *gl;
+	vec3_t origin = {0,0,0}, binormal, tangent, tmp;
+
+	if (r_numgrasses >= MAX_GRASSES)
+			return;
+
+	if(size == 0.0)
+		size = 1.0f;
+ 	  
+	poly = surf->polys;
+         
+	VectorCopy(poly->verts[0], origin);
+   
+	AngleVectors(surf->plane->normal, NULL, tangent, binormal);
+	VectorNormalize(tangent);
+	VectorNormalize(binormal);
+	
+	VectorMA(origin, -32*frand(), tangent, origin);
+
+	if (surf->flags & SURF_PLANEBACK)
+		VectorNegate(surf->plane->normal, tmp);
+	else
+		VectorCopy(surf->plane->normal, tmp);
+     
+	VectorMA(origin, 2, tmp, origin);
+		
+	grass = &r_grasses[r_numgrasses++];
+	VectorCopy(origin, grass->origin);
+	
+	gl = R_RegisterGfxPic (name);
+	if (gl)
+		grass->texsize = gl->height;
+	else
+		grass->texsize = 64; //sane default
+	
+	grass->texnum = texnum;
+	VectorCopy(color, grass->color);
+	grass->size = size;
+	strcpy(grass->name, name);
+	grass->type = type;
+}
+
+int r_numbeams;
+beam_t r_beams[MAX_BEAMS];
+
+static void R_ClearBeams(void)
+{
+	memset(r_beams, 0, sizeof(r_beams));
+	r_numbeams = 0;
+}
+
+void GL_AddBeamSurface (msurface_t *surf, int texnum, vec3_t color, float size, char name[MAX_QPATH], int type)
+{
+    glpoly_t *poly;
+    beam_t  *beam; 
+	image_t *gl;
+	vec3_t poly_center, mins, maxs;
+	float *v;
+	int i;
+	vec3_t origin = {0,0,0}, binormal, tangent, tmp;
+
+	if (r_numbeams >= MAX_BEAMS)
+			return;
+
+	if(size == 0.0)
+		size = 1.0f;
+ 	  
+	poly = surf->polys;
+         
+	 /*
+	===================
+	find poligon centre
+	===================
+	*/
+	VectorSet(mins, 999999, 999999, 999999);
+	VectorSet(maxs, -999999, -999999, -999999);
+
+    for ( poly = surf->polys; poly; poly = poly->chain ) { 
+         for (i=0, v=poly->verts[0] ; i< poly->numverts; i++, v+= VERTEXSIZE) { 
+                
+               if(v[0] > maxs[0])   maxs[0] = v[0]; 
+               if(v[1] > maxs[1])   maxs[1] = v[1]; 
+               if(v[2] > maxs[2])   maxs[2] = v[2]; 
+
+               if(v[0] < mins[0])   mins[0] = v[0]; 
+               if(v[1] < mins[1])   mins[1] = v[1]; 
+               if(v[2] < mins[2])   mins[2] = v[2]; 
+            } 
+      } 
+
+     poly_center[0] = (mins[0] + maxs[0]) /2; 
+     poly_center[1] = (mins[1] + maxs[1]) /2; 
+     poly_center[2] = (mins[2] + maxs[2]) /2;
+	 VectorCopy(poly_center, origin);
+   
+	AngleVectors(surf->plane->normal, NULL, tangent, binormal);
+	VectorNormalize(tangent);
+	VectorNormalize(binormal);
+
+	if (surf->flags & SURF_PLANEBACK)
+		VectorNegate(surf->plane->normal, tmp);
+	else
+		VectorCopy(surf->plane->normal, tmp);
+     
+	VectorMA(origin, 2, tmp, origin);
+		
+	beam = &r_beams[r_numbeams++];
+	VectorCopy(origin, beam->origin);
+	
+	gl = R_RegisterGfxPic (name);
+	if (gl)
+		beam->texsize = gl->height;
+	else
+		beam->texsize = 64; //sane default
+	
+	beam->texnum = texnum;
+	VectorCopy(color, beam->color);
+	beam->size = size;
+	strcpy(beam->name, name);
+	beam->type = type;
+
+}
+
+GL_CalcSurfaceNormals(msurface_t *surf) {
+
+	glpoly_t *p = surf->polys;
+	float	*v;	
+	int		i;
+
+	for (; p; p = p->chain)
+	{
+		vec3_t	v01, v02, temp1, temp2, temp3;
+		vec3_t	normal, binormal, tangent;
+		float	s = 0;
+		float	*vec;
+
+		_VectorSubtract( p->verts[ 1 ], p->verts[0], v01 );
+		vec = p->verts[0];
+		_VectorCopy(surf->plane->normal, normal); 
+
+		for (v = p->verts[0], i = 0 ; i < p->numverts; i++, v += VERTEXSIZE)
+		{
+
+			float currentLength;
+			vec3_t currentNormal;
+
+			//do calculations for normal, tangent and binormal
+			if( i > 1) {
+				_VectorSubtract( p->verts[ i ], p->verts[0], temp1 );
+
+				CrossProduct( temp1, v01, currentNormal );
+				currentLength = VectorLength( currentNormal );
+
+				if( currentLength > s )
+				{
+					s = currentLength;
+					_VectorCopy( currentNormal, normal );
+
+					vec = p->verts[i];
+					_VectorCopy( temp1, v02 );
+
+				}
+			}
+		}
+
+		VectorNormalize( normal ); //we have the largest normal
+
+		surf->tangentSpaceTransform[ 0 ][ 2 ] = normal[ 0 ];
+		surf->tangentSpaceTransform[ 1 ][ 2 ] = normal[ 1 ];
+		surf->tangentSpaceTransform[ 2 ][ 2 ] = normal[ 2 ];
+
+		//now get the tangent
+		s = ( p->verts[ 1 ][ 3 ] - p->verts[ 0 ][ 3 ] )
+			* ( vec[ 4 ] - p->verts[ 0 ][ 4 ] );
+		s -= ( vec[ 3 ] - p->verts[ 0 ][ 3 ] )
+			 * ( p->verts[ 1 ][ 4 ] - p->verts[ 0 ][ 4 ] );
+		s = 1.0f / s;
+
+		VectorScale( v01, vec[ 4 ] - p->verts[ 0 ][ 4 ], temp1 );
+		VectorScale( v02, p->verts[ 1 ][ 4 ] - p->verts[ 0 ][ 4 ], temp2 );
+		_VectorSubtract( temp1, temp2, temp3 );
+		VectorScale( temp3, s, tangent );
+		VectorNormalize( tangent ); 
+
+		surf->tangentSpaceTransform[ 0 ][ 0 ] = tangent[ 0 ];
+		surf->tangentSpaceTransform[ 1 ][ 0 ] = tangent[ 1 ];
+		surf->tangentSpaceTransform[ 2 ][ 0 ] = tangent[ 2 ];
+
+		//now get the binormal
+		VectorScale( v02, p->verts[ 1 ][ 3 ] - p->verts[ 0 ][ 3 ], temp1 );
+		VectorScale( v01, vec[ 3 ] - p->verts[ 0 ][ 3 ], temp2 );
+		_VectorSubtract( temp1, temp2, temp3 );
+		VectorScale( temp3, s, binormal );
+		VectorNormalize( binormal ); 
+
+		surf->tangentSpaceTransform[ 0 ][ 1 ] = binormal[ 0 ];
+		surf->tangentSpaceTransform[ 1 ][ 1 ] = binormal[ 1 ];
+		surf->tangentSpaceTransform[ 2 ][ 1 ] = binormal[ 2 ];
+	}
+}
+
+
+
+
+
 void GL_BuildPolygonFromSurface(msurface_t *fa);
 void GL_CreateSurfaceLightmap (msurface_t *surf);
-void GL_CreateSurfaceStainmap (msurface_t *surf);
 void GL_EndBuildingLightmaps (void);
 void GL_BeginBuildingLightmaps (model_t *m);
 
@@ -881,6 +1074,9 @@ void Mod_LoadFaces (lump_t *l)
 	int			i, count, surfnum;
 	int			planenum, side;
 	int			ti;
+	rscript_t	*rs;
+	vec3_t		color;
+	qboolean	fod;
 
 	in = (void *)(mod_base + l->fileofs);
 	if (l->filelen % sizeof(*in))
@@ -926,9 +1122,14 @@ void Mod_LoadFaces (lump_t *l)
 			out->samples = NULL;
 		else
 			out->samples = loadmodel->lightdata + i;
-		out->stainsamples = NULL;
 
-	// set the drawing flags
+		//special case - note one day we should find a way to check for contents such as mist to do this.
+		if(!Q_stricmp(out->texinfo->image->name, "textures/arena6/fodblue.wal") || !Q_stricmp(out->texinfo->image->name, "textures/arena5/fod.wal"))
+			fod = true;
+		else
+			fod = false;
+
+		// set the drawing flags
 		if (out->texinfo->flags & SURF_WARP)
 		{
 			out->flags |= SURF_DRAWTURB;
@@ -937,36 +1138,52 @@ void Mod_LoadFaces (lump_t *l)
 				out->extents[i] = 16384;
 				out->texturemins[i] = -8192;
 			}
-			GL_SubdivideSurface (out);	// cut up polygon for warps
+			if(!(gl_state.glsl_shaders && gl_glsl_shaders->value) || fod)
+				GL_SubdivideSurface (out);	// cut up polygon for warps
 		}
 
 		// create lightmaps and polygons
 		if ( !SurfaceHasNoLightmap(out) ) 
 		{
 			GL_CreateSurfaceLightmap (out);
-			GL_CreateSurfaceStainmap (out);
 		}
 
-		if (! (out->texinfo->flags & SURF_WARP) ) 
+		if ( (! (out->texinfo->flags & SURF_WARP)) || (gl_state.glsl_shaders && gl_glsl_shaders->value && !fod)) 
 			GL_BuildPolygonFromSurface(out);
 		
-		if(r_lensflare->value) {
-			rscript_t *rs;
-			rs = (rscript_t *)out->texinfo->image->script;
-			if(rs)	{
-#ifdef __unix__
-				rs_stage_t	*stage;
-				stage = rs->stage;
-#else
-				rs_stage_t	*stage = rs->stage;
-#endif
-				do {
-					if (stage->lensflare) 
-						GL_AddFlareSurface(out);
-				} while (stage = stage->next);
-			}
-		}
+		rs = (rscript_t *)out->texinfo->image->script;
 	
+		if(rs)	{
+#ifdef __unix__
+			rs_stage_t	*stage;
+			stage = rs->stage;
+#else
+			rs_stage_t	*stage = rs->stage;
+#endif
+			do {
+				if (stage->lensflare) {
+					if(r_lensflare->value) 	
+						GL_AddFlareSurface(out, stage->flaretype);
+				}
+				if (stage->grass && stage->texture) {
+					if(stage->colormap.enabled) {
+						color[0] = stage->colormap.red;
+						color[1] = stage->colormap.green;
+						color[2] = stage->colormap.blue;
+					}
+					GL_AddVegetationSurface(out, stage->texture->texnum, color, stage->scale.scaleX, stage->texture->bare_name, stage->grasstype);
+				}
+				if (stage->beam && stage->texture) {
+					if(stage->colormap.enabled) {
+						color[0] = stage->colormap.red;
+						color[1] = stage->colormap.green;
+						color[2] = stage->colormap.blue;
+					}
+					GL_AddBeamSurface(out, stage->texture->texnum, color, stage->scale.scaleX, stage->texture->bare_name, stage->beamtype);
+				}
+			} while (stage = stage->next);
+		}
+		GL_CalcSurfaceNormals(out);	
 	}
 	GL_EndBuildingLightmaps ();
 }
@@ -1193,10 +1410,12 @@ void Mod_LoadBrushModel (model_t *mod, void *buffer)
 	int			i;
 	dheader_t	*header;
 	mmodel_t 	*bm;
-	char		rs_name[MAX_OSPATH], tmp[16];		// rscript - MrG
+	char		rs_name[MAX_OSPATH], tmp[MAX_QPATH];		// rscript - MrG
 
 	if(r_lensflare->value)
 		R_ClearFlares();
+	R_ClearGrasses();
+	R_ClearBeams();
 	// rscript - MrG
 	RS_FreeUnmarked();
 	strcpy(tmp,loadmodel->name+5);
@@ -1282,50 +1501,83 @@ ALIAS MODELS
 */
 
 /*
-=================
-Mod_AliasCalculateVertexNormals
-=================
+========================
+Mod_FindTriangleWithEdge
+Shadow volumes stuff
+========================
 */
-void Mod_AliasCalculateVertexNormals ( int numtris, dtriangle_t *tris, int numxyz, mtrivertx_t *v )
-{
+static int Mod_FindTriangleWithEdge(neighbors_t * neighbors, dtriangle_t * tris, int numtris, int triIndex, int edgeIndex){
 
-	int i, j;
-	vec3_t dir1, dir2;
-	vec3_t normals[MAX_VERTS], trnormals[MAX_TRIANGLES];
 
-	if(gl_rtlights->value) {
-		for ( i = 0; i < numtris; i++ ) {
-			// calculate two mostly perpendicular edge directions
-			VectorSubtract ( v[tris[i].index_xyz[0]].v, v[tris[i].index_xyz[1]].v, dir1 );
-			VectorSubtract ( v[tris[i].index_xyz[2]].v, v[tris[i].index_xyz[1]].v, dir2 );
+	int i, j, found = -1, foundj = 0;
+	dtriangle_t *current = &tris[triIndex];
+	qboolean dup = false;
 
-			// we have two edge directions, we can calculate a third vector from
-			// them, which is the direction of the surface normal
-			CrossProduct ( dir1, dir2, trnormals[i] );
-			VectorNormalize ( trnormals[i] );
-		}
+	for (i = 0; i < numtris; i++) {
+		if (i == triIndex)
+			continue;
 
-		// sum all triangle normals
-		for ( i = 0; i < numxyz; i++ ) {
-			VectorClear ( normals[i] );
-
-			for ( j = 0; j < numtris; j++ ) {
-				if ( tris[j].index_xyz[0] == i 
-					|| tris[j].index_xyz[1] == i
-					|| tris[j].index_xyz[2] == i ) {
-					VectorAdd ( normals[i], trnormals[j], normals[i] );
-				}
+		for (j = 0; j < 3; j++) {
+			if (((current->index_xyz[edgeIndex] == tris[i].index_xyz[j]) &&
+				 (current->index_xyz[(edgeIndex + 1) % 3] ==
+				  tris[i].index_xyz[(j + 1) % 3]))
+				||
+				((current->index_xyz[edgeIndex] ==
+				  tris[i].index_xyz[(j + 1) % 3])
+				 && (current->index_xyz[(edgeIndex + 1) % 3] ==
+					 tris[i].index_xyz[j]))) {
+				// no edge for this model found yet?
+				if (found == -1) {
+					found = i;
+					foundj = j;
+				} else
+					dup = true;	// the three edges story
 			}
-
-			VectorNormalize ( normals[i] );
 		}
+	}
 
-		// copy normals back
-		for ( i = 0; i < numxyz; i++ ) {
-			NormalToLatLong ( normals[i], v[i].latlong );
+	// normal edge, setup neighbour pointers
+	if (!dup && found != -1) {	// / FIXED by BERSERKER:  Tenebrae 
+								//  ,  found == -1
+								// ->   !
+		neighbors[found].n[foundj] = triIndex;
+		return found;
+	}
+	// naughty egde let no-one have the neighbour
+	return -1;
+}
+
+/*
+===============
+Mod_BuildTriangleNeighbors
+
+===============
+*/
+static void Mod_BuildTriangleNeighbors(neighbors_t * neighbors,
+									   dtriangle_t * tris, int numtris)
+{
+	int i, j;
+
+	// set neighbours to -1
+	for (i = 0; i < numtris; i++) {
+		for (j = 0; j < 3; j++)
+			neighbors[i].n[j] = -1;
+	}
+
+	// generate edges information (for shadow volumes)
+	// NOTE: We do this with the original vertices not the reordered onces 
+	// since reordering them
+	// duplicates vertices and we only compare indices
+	for (i = 0; i < numtris; i++) {
+		for (j = 0; j < 3; j++) {
+			if (neighbors[i].n[j] == -1)
+				neighbors[i].n[j] =
+					Mod_FindTriangleWithEdge(neighbors, tris, numtris, i,
+											 j);
 		}
 	}
 }
+
 /*
 R_LoadMd2VertexArrays
 */
@@ -1363,14 +1615,17 @@ Mod_LoadAliasModel
 */
 void Mod_LoadAliasModel (model_t *mod, void *buffer)
 {
-	int					i, j;
-	dmdl_t				*pinmodel, *pheader;
+	int					i, j, l;
+	dmdl_t				*pinmodel, *pheader, *paliashdr;
 	dstvert_t			*pinst, *poutst;
-	dtriangle_t			*pintri, *pouttri;
-	daliasframe_t		*pinframe, *poutframe;
+	dtriangle_t			*pintri, *pouttri, *tris;
+	daliasframe_t		*pinframe, *poutframe, *pframe;
 	int					*pincmd, *poutcmd;
 	int					version;
-
+	int					cx;
+	float				s, t;
+	float				iw, ih;
+	fstvert_t			*st;
 	pinmodel = (dmdl_t *)buffer;
 
 	version = LittleLong (pinmodel->version);
@@ -1430,6 +1685,12 @@ void Mod_LoadAliasModel (model_t *mod, void *buffer)
 		}
 	}
 
+//
+// find neighbours
+//
+	mod->neighbors = malloc(pheader->num_tris * sizeof(neighbors_t));
+	Mod_BuildTriangleNeighbors(mod->neighbors, pouttri, pheader->num_tris);
+
 //
 // load the frames
 //
@@ -1477,22 +1738,103 @@ void Mod_LoadAliasModel (model_t *mod, void *buffer)
 		strcpy(rs,(char *)pinmodel + LittleLong(pinmodel->ofs_skins) + i*MAX_SKINNAME);
 
 		rs[strlen(rs)-4]=0;
-	
-		//(struct rscript_s *)mod->script[i] = RS_FindScript(rs);
+
+#ifdef _WINDOWS
+		(struct rscript_s *)mod->script[i] = RS_FindScript(rs);
+#else
 		mod->script[i] = RS_FindScript(rs); //make it gcc 4.1.1 compatible
+#endif
 		
 		if (mod->script[i])
 			RS_ReadyScript((rscript_t *)mod->script[i]);
 	}
 
-	mod->mins[0] = -32;
-	mod->mins[1] = -32;
-	mod->mins[2] = -32;
-	mod->maxs[0] = 32;
-	mod->maxs[1] = 32;
-	mod->maxs[2] = 32;
+	cx = pheader->num_st * sizeof(fstvert_t);
+     mod->st = st = (fstvert_t*)Hunk_Alloc (cx);
+		
+	// Calculate texcoords for triangles
+    iw = 1.0 / pheader->skinwidth;
+    ih = 1.0 / pheader->skinheight;
+    for (i=0; i<pheader->num_st ; i++)
+    {
+        s = poutst[i].s;
+        t = poutst[i].t;
+        st[i].s = (s + 1.0) * iw; //tweak by one pixel
+        st[i].t = (t + 1.0) * ih;
+    }
 
 	R_LoadMd2VertexArrays(mod);
+
+	paliashdr = (dmdl_t *)mod->extradata;
+
+	//just use frame zero - we never scale frames in Alien Arena
+	pframe = ( daliasframe_t * ) ( ( byte * ) paliashdr +
+		                              paliashdr->ofs_frames);
+
+	/*
+	** compute axially aligned mins and maxs
+	*/
+	for ( i = 0; i < 3; i++ )
+	{
+		mod->mins[i] = pframe->translate[i];
+		mod->maxs[i] = mod->mins[i] + pframe->scale[i]*255;
+	}
+
+	/*
+	** compute a full bounding box
+	*/
+	for ( i = 0; i < 8; i++ )
+	{
+		vec3_t   tmp;
+
+		if ( i & 1 )
+			tmp[0] = mod->mins[0];
+		else
+			tmp[0] = mod->maxs[0];
+
+		if ( i & 2 )
+			tmp[1] = mod->mins[1];
+		else
+			tmp[1] = mod->maxs[1];
+
+		if ( i & 4 )
+			tmp[2] = mod->mins[2];
+		else
+			tmp[2] = mod->maxs[2];
+
+		VectorCopy( tmp, mod->bbox[i] );
+	}
+	
+	if (gl_state.vbo)
+	{
+		int		index_st;
+		float	*map;
+
+		mod->vbo_st = NULL;
+		mod->vbo_xyz = NULL;
+
+		tris = (dtriangle_t *) ((byte *)pheader + pheader->ofs_tris);
+		map = (float*) vbo_shadow;	
+
+		for (l=0, i=0; i<pheader->num_tris; i++)
+		{
+			for (j=0; j<3; j++)
+			{
+				index_st = tris[i].index_st[j];
+				map[l++] = mod->st[index_st].s;
+				map[l++] = mod->st[index_st].t;
+			}
+		}
+
+		if (l>3*MAX_VBO_XYZs)
+			Com_Error(ERR_FATAL, "Temporary buffer overflow\n");
+
+
+		mod->vbo_st = R_VCLoadData(VBO_STATIC, l*sizeof(float), &vbo_shadow, VBO_STORE_ANY, NULL);
+		//mod->vbo_xyz = R_VCLoadData(VBO_STATIC, paliashdr->num_xyz*sizeof(vec3_t), &mod->r_mesh_verts, VBO_STORE_ANY, NULL);
+		GL_BindVBO(NULL);
+	}
+
 }
 
 //=============================================================================
@@ -1519,7 +1861,8 @@ int R_FindFile (char *filename, FILE **file)
 
 void R_BeginRegistration (char *model)
 {
-	char	fullname[MAX_QPATH];
+	char	fullname[MAX_OSPATH];
+ 	char    *path;
 	cvar_t	*flushmap;
 	FILE	*file;
 	int		i;
@@ -1529,77 +1872,52 @@ void R_BeginRegistration (char *model)
 
 	r_weather = 0; //default is 0
 
-	//check for fog file. (gamedir first) 
-	Com_sprintf(fullname, sizeof(fullname), "%s/maps/scripts/%s.fog", FS_Gamedir(), model);
-	i = 0;
-	do
-		fullname[i] = tolower(fullname[i]);
-	while (fullname[i++]);
-
-	R_FindFile (fullname, &file); //does a fogfile exist?
-	if(file) {
-		//read the file, get fog information
-		fclose(file);
-		R_ReadFogScript(fullname);
-		map_fog = true;
-	}
-	else
-		map_fog = false;
-
-	if(!map_fog) {
-		//check for fog file.  
-		Com_sprintf(fullname, sizeof(fullname), "%s/maps/scripts/%s.fog", BASEDIRNAME, model);
+	// check for fog file, using file system search path
+	path = NULL;
+	map_fog = false;
+	for(;;)
+	{
+		path = FS_NextPath( path );
+		if( !path )
+		{
+			break;
+		}
+		Com_sprintf(fullname, sizeof(fullname), "%s/maps/scripts/%s.fog", path, model);
 		i = 0;
 		do
 			fullname[i] = tolower(fullname[i]);
 		while (fullname[i++]);
-
-		R_FindFile (fullname, &file); //does a fogfile exist?
+		R_FindFile( fullname, &file ); //does a fog file exist?
 		if(file) {
 			//read the file, get fog information
 			fclose(file);
 			R_ReadFogScript(fullname);
-			map_fog = true;
+			break;
 		}
-		else
-			map_fog = false;
-	}
-	
-
-	//check for music file.
-	Com_sprintf(fullname, sizeof(fullname), "%s/maps/scripts/%s.mus", FS_Gamedir(), model);
-	i = 0;
-	do
-		fullname[i] = tolower(fullname[i]);
-	while (fullname[i++]);
-
-	R_FindFile (fullname, &file); //does a music file exist?
-	if(file) {
-		//read the file, get music information
-		fclose(file);
-		R_ReadMusicScript(fullname);
-
-	}
-	else {
-		strcpy(map_music, "music/none.wav");
 	}
 
-	if(!strcmp(map_music, "music/none.wav")) {
-		//check for music file.
-		Com_sprintf(fullname, sizeof(fullname), "%s/maps/scripts/%s.mus", BASEDIRNAME, model);
+	// check for background music file, , using file system search path
+	strcpy(map_music, "music/none.wav");
+	path = NULL;
+	for(;;)
+	{
+		path = FS_NextPath( path );
+		if( !path )
+		{
+			break;
+		}
+		Com_sprintf(fullname, sizeof(fullname), "%s/maps/scripts/%s.mus", path, model);
 		i = 0;
 		do
 			fullname[i] = tolower(fullname[i]);
 		while (fullname[i++]);
-
-		R_FindFile (fullname, &file); //does a fogfile exist?
+		R_FindFile( fullname, &file ); //does a music file exist?
 		if(file) {
 			//read the file, get music information
-			fclose(file);
-			R_ReadMusicScript(fullname);
+			fclose( file );
+			R_ReadMusicScript( fullname );
+			break;
 		}
-		else
-			strcpy(map_music, "music/none.wav");
 	}
 
 	Com_sprintf (fullname, sizeof(fullname), "maps/%s.bsp", model);
