@@ -31,13 +31,14 @@ void CIN_FreeCin (int texnum);
 extern float	r_turbsin[];
 extern  void GL_BlendFunction (GLenum sfactor, GLenum dfactor);
 
-#define		TURBSCALE (256.0 / (2 * M_PI))
-
 #define		TOK_DELIMINATORS "\r\n\t "
 
 float		rs_realtime = 0;
 rscript_t	*rs_rootscript = NULL;
 
+int r_numgrasses;
+int r_numbeams;
+
 int	RS_Random(rs_stage_t *stage, msurface_t *surf)
 {
 	random_stage_t	*randStage = stage->rand_stage;
@@ -223,12 +224,20 @@ void RS_ClearStage (rs_stage_t *stage)
 	VectorClear(stage->angle);
 
 	stage->texture = NULL;
+	stage->texture2 = NULL;
 
 	stage->depthhack = 0;
 	stage->envmap = false;
 	stage->dynamic = false;
 	stage->lensflare = false;
+	stage->flaretype = 0;
 	stage->normalmap = false;
+	stage->grass = false;
+	stage->grasstype = 0;
+	stage->beam = false;
+	stage->beamtype = 0;
+	stage->fx = false;
+	stage->glow = false;
 
 	stage->lightmap = true;
 
@@ -238,6 +247,7 @@ void RS_ClearStage (rs_stage_t *stage)
 rscript_t *RS_NewScript (char *name)
 {
 	rscript_t	*rs;
+	unsigned int	i;
 
 	if (!rs_rootscript) 
 	{
@@ -255,6 +265,7 @@ rscript_t *RS_NewScript (char *name)
 		rs = rs->next;
 	}
 
+	COMPUTE_HASH_KEY(rs->hash_key, name, i);
 	strncpy (rs->name, name, sizeof(rs->name));
 
 	rs->stage = NULL;
@@ -293,6 +304,7 @@ rs_stage_t *RS_NewStage (rscript_t *rs)
 	}
 
 	strncpy (stage->name, "***r_notexture***", sizeof(stage->name));
+	strncpy (stage->name2, "***r_notexture***", sizeof(stage->name2));
 
 	stage->rand_stage = NULL;
 	stage->anim_stage = NULL;
@@ -370,21 +382,23 @@ void RS_FreeUnmarked (void)
 rscript_t *RS_FindScript(char *name)
 {
 	rscript_t	*rs = rs_rootscript;
+	unsigned int	i, hash_key;
+
+	COMPUTE_HASH_KEY(hash_key, name, i);
 
 	while (rs != NULL) 
 	{
-		if (!_stricmp(rs->name, name))
+		if (rs->hash_key == hash_key && !_stricmp(rs->name, name))
 		{
-			if (rs->stage)
-			   return rs;
-			else
-			   return NULL;
+			if (! rs->stage)
+				rs = NULL;
+			break;
 		}
 
 		rs = rs->next;
 	}
 
-	return NULL;
+	return rs;
 }
 
 void RS_ReadyScript (rscript_t *rs)
@@ -402,6 +416,10 @@ void RS_ReadyScript (rscript_t *rs)
 
 	while (stage != NULL)
 	{
+		//if normalmap, set mode
+		if(stage->normalmap)
+			mode = it_bump;
+
 		//set anim
 		anim = stage->anim_stage;
 		while (anim != NULL) 
@@ -429,6 +447,10 @@ void RS_ReadyScript (rscript_t *rs)
 			stage->texture = GL_FindImage (stage->name, mode);
 		if (!stage->texture)
 			stage->texture = r_notexture;
+		if (stage->name2[0]) 
+			stage->texture2 = GL_FindImage (stage->name2, mode);
+		if (!stage->texture2)
+			stage->texture2 = r_notexture;
 
 		//check alpha
 		if (stage->blendfunc.blend)
@@ -442,54 +464,6 @@ void RS_ReadyScript (rscript_t *rs)
 	rs->ready = true;
 }
 
-void RS_UpdateRegistration (void)
-{
-	rscript_t		*rs = rs_rootscript;
-	rs_stage_t		*stage;
-	anim_stage_t	*anim;
-	random_stage_t	*randStage;
-	int				mode;
-
-	while (rs != NULL) 
-	{
-		mode = (rs->dontflush) ? it_pic : it_wall;
-		stage = rs->stage;
-
-		while (stage != NULL) 
-		{
-			anim = stage->anim_stage;
-			while (anim != NULL) 
-			{
-				anim->texture = GL_FindImage(anim->name,mode);
-				if (!anim->texture)
-					anim->texture = r_notexture;
-				anim = anim->next;
-			}
-
-			randStage = (random_stage_t*)stage->rand_stage;
-			while (randStage != NULL) 
-			{
-				randStage->texture = GL_FindImage(randStage->name,mode);
-				if (!randStage->texture)
-					randStage->texture = r_notexture;
-				randStage = randStage->next;
-			}
-
-			if (stage->name[0]) 
-			{
-				stage->texture = GL_FindImage(stage->name, mode);
-			}
-
-			if (!stage->texture)
-				stage->texture = r_notexture;
-
-			stage = stage->next;
-		}
-
-		rs = rs->next;
-	}
-}
-
 int RS_BlendID (char *blend)
 {
 	if (!blend[0])
@@ -540,6 +514,7 @@ scriptname
 	safe
 	{
 		map <texturename>
+		map2 <texturename>
 		scroll <xtype> <xspeed> <ytype> <yspeed>
 		blendfunc <source> <dest>
 		alphashift <speed> <min> <max>
@@ -548,7 +523,14 @@ scriptname
 		nolightmap
 		alphamask
 		lensflare
+		flaretype
 		normalmap
+		grass
+		grasstype
+		beam
+		beamtype
+		fx
+		glow
 	}
 }
 */
@@ -560,6 +542,13 @@ void rs_stage_map (rs_stage_t *stage, char **token)
 	strncpy (stage->name, *token, sizeof(stage->name));
 }
 
+void rs_stage_map2 (rs_stage_t *stage, char **token)
+{
+	*token = strtok (NULL, TOK_DELIMINATORS);
+
+	strncpy (stage->name2, *token, sizeof(stage->name2));
+}
+
 void rs_stage_model (rs_stage_t *stage, char **token)
 {
 	*token = strtok (NULL, TOK_DELIMINATORS);
@@ -803,14 +792,46 @@ void rs_stage_lensflare (rs_stage_t *stage, char **token)
 {
 	stage->lensflare = true;
 }
+void rs_stage_flaretype (rs_stage_t *stage, char **token)
+{
+	*token = strtok (NULL, TOK_DELIMINATORS);
+	stage->flaretype = atoi(*token);
+}
 void rs_stage_normalmap (rs_stage_t *stage, char **token)
 {
 	stage->normalmap = true;
 }
+void rs_stage_grass (rs_stage_t *stage, char **token)
+{
+	stage->grass = true;
+}
+void rs_stage_grasstype (rs_stage_t *stage, char **token)
+{
+	*token = strtok (NULL, TOK_DELIMINATORS);
+	stage->grasstype = atoi(*token);
+}
+void rs_stage_beam (rs_stage_t *stage, char **token)
+{
+	stage->beam = true;
+}
+void rs_stage_beamtype (rs_stage_t *stage, char **token)
+{
+	*token = strtok (NULL, TOK_DELIMINATORS);
+	stage->beamtype = atoi(*token);
+}
+void rs_stage_fx (rs_stage_t *stage, char **token)
+{
+	stage->fx = true;
+}
+void rs_stage_glow (rs_stage_t *stage, char **token)
+{
+	stage->glow = true;
+}
 static rs_stagekey_t rs_stagekeys[] = 
 {
 	{	"colormap",		&rs_stage_colormap		},
 	{	"map",			&rs_stage_map			},
+	{	"map2",			&rs_stage_map2			},
 	{	"model",		&rs_stage_model			},
 	{	"scroll",		&rs_stage_scroll		},
 	{	"frames",		&rs_stage_frames		},
@@ -829,7 +850,14 @@ static rs_stagekey_t rs_stagekeys[] =
 	{	"dynamic",		&rs_stage_dynamic		},
 	{	"alphafunc",	&rs_stage_alphafunc		},
 	{	"lensflare",	&rs_stage_lensflare		},
+	{	"flaretype",	&rs_stage_flaretype		},
 	{   "normalmap",	&rs_stage_normalmap		},
+	{	"grass",		&rs_stage_grass			},
+	{	"grasstype",	&rs_stage_grasstype		},
+	{	"beam",			&rs_stage_beam			},
+	{	"beamtype",		&rs_stage_beamtype		},
+	{	"fx",			&rs_stage_fx			},
+	{	"glow",			&rs_stage_glow			},
 
 	{	NULL,			NULL					}
 };
@@ -1041,7 +1069,7 @@ void RS_ScanPathForScripts (void)
 	}
 
 	script_count = 0;
-	if(gl_normalmaps->value) { //search for normal map scripts ONLY if we are using normal mapping
+	if(gl_normalmaps->value) { //search for normal map scripts ONLY if we are using normal mapping, do last to overide anything
 		
 		script_list = FS_ListFilesInFS("scripts/normals/*.rscript", &script_count, 0, SFF_SUBDIR | SFF_HIDDEN | SFF_SYSTEM);
 
@@ -1295,13 +1323,14 @@ rscript_t	*surfaceScript(msurface_t *surf)
 }
 void SetVertexOverbrights (qboolean toggle)
 {
-	
+	if (!r_overbrightbits->value || !gl_ext_mtexcombine->value) 
+		return;
 
 	if (toggle)//turn on
 	{
 		qglTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);
 		qglTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_MODULATE);
-		qglTexEnvi(GL_TEXTURE_ENV, GL_RGB_SCALE_ARB, 3);
+		qglTexEnvi(GL_TEXTURE_ENV, GL_RGB_SCALE_ARB, 1);
 		qglTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA_ARB, GL_MODULATE);
 		
 		GL_TexEnv( GL_COMBINE_EXT );
@@ -1332,20 +1361,290 @@ void ToggleLightmap (qboolean toggle)
 	}
 }
 
-//This is now the primary shader drawing routine for surfaces - it will draw on top of the 
-//existing texture.  This will help in preventing cheats, in which people create shaders that
-//allow you to see through walls.
+extern int c_grasses;
+grass_t r_grasses[MAX_GRASSES];
+void R_DrawVegetationSurface ( void )
+{
+    int		i, k;
+	grass_t *grass;
+    float   scale;
+	vec3_t	origin, mins, maxs, angle, right, up, corner[4];
+	float	*corner0 = corner[0];
+	qboolean visible;
+	float	lightLevel[3];
+	trace_t r_trace;
+	float	sway;
+
+	if(r_newrefdef.rdflags & RDF_NOWORLDMODEL)
+		return;
+
+	grass = r_grasses;
+
+	VectorSet(mins, 0, 0, 0);
+	VectorSet(maxs,	0, 0, 0);	
+
+	R_InitVArrays (VERT_SINGLE_TEXTURED);
+
+    for (i=0; i<r_numgrasses; i++, grass++) {
+
+		scale = 10.0*grass->size; 
+		 
+		VectorCopy(r_newrefdef.viewangles, angle);
+
+		if(!grass->type) 
+			angle[0] = 0;  // keep vertical by removing pitch(grass and plants grow upwards)
+
+		AngleVectors(angle, NULL, right, up);	
+		VectorScale(right, scale, right);
+		VectorScale(up, scale, up);
+		VectorCopy(grass->origin, origin);
+
+		origin[2] += (grass->texsize/16)*(scale/10); //dynamically get image size and adjust? 4 pixels correlated to a 64x64 image
+								   //so image size/16 would work.
+
+		if(!grass->type) {
+			r_trace = CM_BoxTrace(r_origin, origin, maxs, mins, r_worldmodel->firstnode, MASK_VISIBILILITY);
+			visible = r_trace.fraction == 1.0;
+		}
+		else
+			visible = true; //leaves tend to use much larger images, culling results in undesired effects
+
+		if(visible) {
+	
+			//render grass polygon
+			qglDepthMask( GL_FALSE );	 	
+			qglEnable( GL_BLEND);
+			qglBlendFunc ( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
+					
+			GL_Bind(grass->texnum);
+
+			if(gl_dynamic->value)
+				R_LightPoint (origin, lightLevel, true);
+			else
+				R_LightPoint (origin, lightLevel, false);
+			VectorScale(lightLevel, 2.0, lightLevel);
+			qglColor4f( grass->color[0]*(lightLevel[0]+0.1),grass->color[1]*(lightLevel[1]+0.1),grass->color[2]*(lightLevel[2]+0.1), 1 );
+			GL_TexEnv( GL_MODULATE );
+			
+			VectorSet (corner[0],
+				origin[0] + (up[0] + right[0])*(-0.5),
+				origin[1] + (up[1] + right[1])*(-0.5),
+				origin[2] + (up[2] + right[2])*(-0.5));
+
+			//the next two statements create a slight swaying in the wind
+			//perhaps we should add a parameter to control ammount in shader?
+
+			if(grass->type) {
+				sway = 3;
+			}
+			else
+				sway = 2;
+
+			VectorSet ( corner[1],
+				corner0[0] + up[0] + sway*sin (rs_realtime*sway), 
+				corner0[1] + up[1] + sway*sin (rs_realtime*sway), 
+				corner0[2] + up[2]);
+
+			VectorSet ( corner[2], 
+				corner0[0] + (up[0]+right[0] + sway*sin (rs_realtime*sway)),
+				corner0[1] + (up[1]+right[1] + sway*sin (rs_realtime*sway)), 
+				corner0[2] + (up[2]+right[2]));
+
+			VectorSet ( corner[3],
+				corner0[0] + right[0], 
+				corner0[1] + right[1], 
+				corner0[2] + right[2]);
+
+			VArray = &VArrayVerts[0];
+
+			for(k = 0; k < 4; k++) {
+
+				VArray[0] = corner[k][0];
+				VArray[1] = corner[k][1];
+				VArray[2] = corner[k][2];
+
+				switch(k) {
+					case 0:
+						VArray[3] = 1;
+						VArray[4] = 1;
+						break;
+					case 1:
+						VArray[3] = 0;
+						VArray[4] = 1;
+						break;
+					case 2:
+						VArray[3] = 0;
+						VArray[4] = 0;
+						break;
+					case 3:
+						VArray[3] = 1;
+						VArray[4] = 0;
+						break;
+				}
+
+				VArray += VertexSizes[VERT_SINGLE_TEXTURED];
+			}
+
+			if(qglLockArraysEXT)						
+				qglLockArraysEXT(0, 4);
+
+			qglDrawArrays(GL_QUADS,0,4);
+					
+			if(qglUnlockArraysEXT)						
+				qglUnlockArraysEXT();
+
+			c_grasses++;
+		}
+	}
+
+	R_KillVArrays ();
+
+	qglTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
+	qglBlendFunc ( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
+	qglColor4f( 1,1,1,1 );
+	qglDisable(GL_BLEND);
+	qglDepthMask( GL_TRUE );	
+	GL_TexEnv( GL_REPLACE );
+}
+
+extern int c_beams;
+beam_t r_beams[MAX_BEAMS];
+void R_DrawBeamSurface ( void )
+{
+    int		i, k;
+	beam_t *beam;
+    float   scale;
+	vec3_t	origin, mins, maxs, angle, right, up, corner[4];
+	float	*corner0 = corner[0];
+	qboolean visible;
+	trace_t r_trace;
+
+	if(r_newrefdef.rdflags & RDF_NOWORLDMODEL)
+		return;
+
+	beam = r_beams;
+
+	VectorSet(mins, 0, 0, 64);
+	VectorSet(maxs, 0, 0, -64);	
+
+	R_InitVArrays (VERT_SINGLE_TEXTURED);
+
+    for (i=0; i<r_numbeams; i++, beam++) {
+		 
+		scale = 10.0*beam->size; 
+
+		VectorCopy(r_newrefdef.viewangles, angle);
+
+		angle[0] = 0;  // keep vertical by removing pitch
+
+		AngleVectors(angle, NULL, right, up);	
+		VectorScale(right, scale, right);
+		VectorScale(up, scale, up); 
+		VectorCopy(beam->origin, origin);
+
+		if(!beam->type)
+			origin[2] -= up[2]/2; //contingent on direction of beam
+		else
+			origin[2] += up[2]/2;
+
+		r_trace = CM_BoxTrace(r_origin, origin, mins, maxs, r_worldmodel->firstnode, MASK_VISIBILILITY);
+		visible = r_trace.fraction == 1.0;
+		
+		if(visible) {
+
+			//render polygon
+			qglDepthMask( GL_FALSE );	 	
+			qglEnable( GL_BLEND);
+			qglBlendFunc   (GL_SRC_ALPHA, GL_ONE);	
+
+			qglColor4f( beam->color[0],beam->color[1],beam->color[2], 1 );
+
+			GL_Bind(beam->texnum);
+
+			VectorSet (corner[0],
+				origin[0] + (up[0] + right[0])*(-0.5),
+				origin[1] + (up[1] + right[1])*(-0.5),
+				origin[2] + (up[2] + right[2])*(-0.5));
+
+			VectorSet ( corner[1],
+				corner0[0] + up[0], 
+				corner0[1] + up[1], 
+				corner0[2] + up[2]);
+
+			VectorSet ( corner[2], 
+				corner0[0] + (up[0]+right[0]),
+				corner0[1] + (up[1]+right[1]), 
+				corner0[2] + (up[2]+right[2]));
+
+			VectorSet ( corner[3],
+				corner0[0] + right[0], 
+				corner0[1] + right[1], 
+				corner0[2] + right[2]);
+
+				VArray = &VArrayVerts[0];
+
+			for(k = 0; k < 4; k++) {
+
+				VArray[0] = corner[k][0];
+				VArray[1] = corner[k][1];
+				VArray[2] = corner[k][2];
+
+				switch(k) {
+					case 0:
+						VArray[3] = 1;
+						VArray[4] = 1;
+						break;
+					case 1:
+						VArray[3] = 0;
+						VArray[4] = 1;
+						break;
+					case 2:
+						VArray[3] = 0;
+						VArray[4] = 0;
+						break;
+					case 3:
+						VArray[3] = 1;
+						VArray[4] = 0;
+						break;
+				}
+
+				VArray += VertexSizes[VERT_SINGLE_TEXTURED];
+			}
+
+			if(qglLockArraysEXT)						
+				qglLockArraysEXT(0, 4);
+
+			qglDrawArrays(GL_QUADS,0,4);
+					
+			if(qglUnlockArraysEXT)						
+				qglUnlockArraysEXT();
+
+			c_beams++;
+		}
+	}
+
+	R_KillVArrays ();
+
+	qglTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
+	qglBlendFunc ( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
+	qglColor4f( 1,1,1,1 );
+	qglDisable(GL_BLEND);
+	qglDepthMask( GL_TRUE );	
+	GL_TexEnv( GL_REPLACE );
+}
+
+
+//to do - rewrite using vertex arrays
+
+//This is the shader drawing routine for bsp surfaces - it will draw on top of the 
+//existing texture.  
 void RS_DrawSurfaceTexture (msurface_t *surf, rscript_t *rs)
 {
 	glpoly_t	*p;
 	float		*v;
 	int			i, nv;
 	vec3_t		wv, vectors[3];
-#ifdef __unix__
 	rs_stage_t	*stage;
-#else
-	rs_stage_t	*stage = rs->stage;
-#endif
 	float		os, ot, alpha;
 	float		scale, time, txm, tym;
 	
@@ -1365,12 +1664,9 @@ void RS_DrawSurfaceTexture (msurface_t *surf, rscript_t *rs)
 	do
 	{
 
-		if (stage->lensflare)
+		if (stage->lensflare || stage->grass || stage->beam)
 			break;
 
-		if (stage->normalmap)
-			continue;
-
 		if (stage->colormap.enabled)
 			qglDisable (GL_TEXTURE_2D);
 		else if (stage->rand_count) {
@@ -1506,13 +1802,6 @@ void RS_DrawSurfaceTexture (msurface_t *surf, rscript_t *rs)
 					{
 						float red=255, green=255, blue=255;
 
-						if (stage->lightmap && p->vertexlight)
-						{
-							red = p->vertexlight[i*3+0]/255.0f;
-							green = p->vertexlight[i*3+1]/255.0f;
-							blue = p->vertexlight[i*3+2]/255.0f;
-						}
-						
 						if (stage->colormap.enabled)
 						{
 							red *= stage->colormap.red/255.0f;
@@ -1628,10 +1917,16 @@ void RS_DrawSurfaceTexture (msurface_t *surf, rscript_t *rs)
 	if(!r_bloom->value)
 		SetVertexOverbrights(false);
 
+	// restore the original blend mode
+	qglBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+	qglEnable (GL_BLEND);
+
+	
 }
 
 rscript_t *rs_caustics;
 rscript_t *rs_glass;
+extern cvar_t *cl_gun;
 
 void RS_LoadSpecialScripts (void) //the special cases of glass and water caustics
 {
@@ -1646,7 +1941,8 @@ void RS_SpecialSurface (msurface_t *surf)
 	rscript_t *rs_shader;
 
 	//Underwater Caustics
-	if (surf->flags & SURF_UNDERWATER)
+	//hack - can't seem to find fix for when there is no gun model
+	if (surf->flags & SURF_UNDERWATER && cl_gun->value && r_lefthand->value != 2.0F) 
 			RS_DrawSurfaceTexture(surf, rs_caustics);
 
 	//this was moved here to handle all textures shaders as well.
