@@ -71,6 +71,8 @@ typedef struct rs_stage_s
 {
 	image_t					*texture;				// texture
 	char					name[MAX_OSPATH];		// tex name
+	image_t					*texture2;				// texture for combining(GLSL)
+	char					name2[MAX_OSPATH];		// texture name
 
 	char					model[MAX_OSPATH];		// name of model
 	
@@ -105,9 +107,20 @@ typedef struct rs_stage_s
 	int						alphafunc;		// software alpha effects
 
 	qboolean				has_alpha;		// for sorting
-	qboolean				lensflare;		// for adding lensflares
 	qboolean				normalmap;		// for normalmaps
 
+	qboolean				lensflare;		// for adding lensflares
+	int						flaretype;		// type of flare
+
+	qboolean				grass;			// grass and vegetation
+	int						grasstype;		// the type of vegetation
+	
+	qboolean				beam;			// for adding light beams
+	int						beamtype;		// the type of beam(up vs down)
+	
+	qboolean				fx;				// for glsl effect layer
+	qboolean				glow;			// for glsl effect layer
+
 	struct rs_stage_s		*next;			// next stage
 } rs_stage_t;
 
@@ -127,7 +140,8 @@ typedef struct
 // Base script
 typedef struct rscript_s 
 {
-	char					name[MAX_OSPATH];	// name of script
+	char				name[MAX_OSPATH];	// name of script
+	unsigned int			hash_key;		// hash key for the script's name
 	
 	unsigned char			subdivide;	// Heffo - chop the surface up this much for vertex warping
 	float					warpdist;	// Heffo - vertex warping distance;
@@ -160,9 +174,7 @@ void RS_FreeScript(rscript_t *rs);
 void RS_FreeUnmarked(void);
 rscript_t *RS_FindScript(char *name);
 void RS_ReadyScript(rscript_t *rs);
-#ifdef _WINDOWS
 void RS_ScanPathForScripts(void);
-#endif
 int RS_Animate(rs_stage_t *stage);
 void RS_UpdateRegistration(void);
 void RS_DrawSurface (msurface_t *surf, qboolean lightmap);
