@@ -21,14 +21,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <assert.h>
 
 #include "r_local.h"
-#include "r_refl.h"	// MPO
 
 static vec3_t	modelorg;		// relative to viewpoint
 
 msurface_t	*r_alpha_surfaces;
 msurface_t	*r_special_surfaces;
 msurface_t *r_normalsurfaces;
-msurface_t *r_specularsurfaces;
 
 #define DYNAMIC_LIGHT_WIDTH  128
 #define DYNAMIC_LIGHT_HEIGHT 128
@@ -105,10 +103,8 @@ image_t *R_TextureAnimation (mtexinfo_t *tex)
 DrawGLPoly
 ================
 */
-void DrawGLPoly (glpoly_t *p, int flags)
+void DrawGLPoly (msurface_t *fa, int flags)
 {
-	int		i;
-	float	*v = p->verts[0];
 	float	scroll;
 
 	scroll = 0;
@@ -119,13 +115,10 @@ void DrawGLPoly (glpoly_t *p, int flags)
 			scroll = -64.0;
 	}
 
-	qglBegin (GL_POLYGON);
-	for (i=0 ; i<p->numverts ; i++, v+= VERTEXSIZE)
-	{
-		qglTexCoord2f (v[3] + scroll, v[4]);
-		qglVertex3fv (v);
-	}
-	qglEnd ();
+	R_InitVArrays(VERT_SINGLE_TEXTURED);
+	R_AddTexturedSurfToVArray (fa, scroll);
+	R_KillVArrays();
+
 }
 
 /*
@@ -167,7 +160,7 @@ void R_DrawTriangleOutlines (void)
 }
 
 /*
-** DrawGLPolyLightmap
+** DrawGLPolyLightmap (does this ever really get used?)
 */
 void DrawGLPolyLightmap( glpoly_t *p, float soffset, float toffset )
 {
@@ -331,6 +324,7 @@ void R_RenderBrushPoly (msurface_t *fa)
 	int			maps;
 	image_t		*image;
 	qboolean is_dynamic = false;
+	float		scroll;
 
 	c_brush_polys++;
 
@@ -346,10 +340,8 @@ void R_RenderBrushPoly (msurface_t *fa)
 			        gl_state.inverse_intensity,
 					gl_state.inverse_intensity,
 					1.0F );
-		if(!gl_reflection->value || (!Q_stricmp(fa->texinfo->image->name, "textures/arena2/lava.wal"))) //lava HACK!
-			EmitWaterPolys_original(fa);
-		else
-			EmitWaterPolys (fa);
+		GL_RenderWaterPolys(fa, 0, 1, 1);
+		
 		GL_TexEnv( GL_REPLACE );
 
 		return;
@@ -363,19 +355,26 @@ void R_RenderBrushPoly (msurface_t *fa)
 
 	fa->normalchain = r_normalsurfaces;
 	r_normalsurfaces = fa;
-	fa->specularchain = r_specularsurfaces;
-	r_specularsurfaces = fa;
 
 	if (SurfaceIsAlphaBlended(fa))
 		qglEnable( GL_ALPHA_TEST );
 
-	DrawGLPoly (fa->polys, fa->texinfo->flags);
+	scroll = 0;
+	if (fa->texinfo->flags & SURF_FLOWING)
+	{
+		scroll = -64 * ( (r_newrefdef.time / 40.0) - (int)(r_newrefdef.time / 40.0) );
+		if (scroll == 0.0)
+			scroll = -64.0;
+	}
+	R_InitVArrays(VERT_SINGLE_TEXTURED);
+	R_AddTexturedSurfToVArray (fa, scroll);
+	R_KillVArrays();
 
 	if (SurfaceIsAlphaBlended(fa))
 	{
 		qglDisable( GL_ALPHA_TEST );
 		return;
-	}//for now until Vic fixes problems with colored lightmaps on these surfaces
+	}
 
 	/*
 	** check for lightmap modification
@@ -450,8 +449,11 @@ void R_DrawAlphaSurfaces (void)
 {
 	msurface_t	*s;
 	float		intens;
-	rscript_t *rs_shader;
-
+	rscript_t	*rs_shader;
+	rs_stage_t	*stage = NULL;
+	int			texnum = 0;
+	float		scaleX = 1, scaleY = 1;
+ 
 	// the textures are prescaled up for a better lighting range,
 	// so scale it back down
 	intens = gl_state.inverse_intensity;
@@ -485,13 +487,24 @@ void R_DrawAlphaSurfaces (void)
 		}
 
 		if (s->flags & SURF_DRAWTURB) {
-			if(!gl_reflection->value || (!Q_stricmp(s->texinfo->image->name, "textures/arena2/lava.wal"))) //lava HACK!
-				EmitWaterPolys_original (s);
-			else
-				EmitWaterPolys (s);
+			//water shaders		
+			if(r_shaders->value) {
+				rs_shader = (rscript_t *)s->texinfo->image->script;
+				if(rs_shader) {
+					stage = rs_shader->stage;
+					if(stage) { //for now, just map a reflection texture
+						texnum = stage->texture->texnum; //pass this to renderwaterpolys
+					}
+					if(stage->scale.scaleX != 0 && stage->scale.scaleY !=0) {
+						scaleX = stage->scale.scaleX;
+						scaleY = stage->scale.scaleY;
+					}
+				}
+			}
+			GL_RenderWaterPolys (s, texnum, scaleX, scaleY);
 		}
 		else {
-			if(r_shaders->value) {
+			if(r_shaders->value && !(s->texinfo->flags & SURF_FLOWING)) {
 				rs_shader = (rscript_t *)s->texinfo->image->script;
 				if(rs_shader) {
 
@@ -509,10 +522,10 @@ void R_DrawAlphaSurfaces (void)
 					qglDepthMask(true);
 				}
 				else
-					DrawGLPoly (s->polys, s->texinfo->flags);
+					DrawGLPoly (s, s->texinfo->flags);
 			}
 			else
-				DrawGLPoly (s->polys, s->texinfo->flags);
+				DrawGLPoly (s, s->texinfo->flags);
 		}
 
 	}
@@ -622,22 +635,19 @@ void DrawTextureChains (void)
 	GL_TexEnv( GL_REPLACE );
 }
 
-
+extern int KillFlags;
 static void GL_RenderLightmappedPoly( msurface_t *surf )
 {
-	int		i, nv = surf->polys->numverts;
+	int		nv = surf->polys->numverts;
 	int		map;
 	float	scroll;
-	float	*v;
 	image_t *image = R_TextureAnimation( surf->texinfo );
 	qboolean is_dynamic = false;
 	unsigned lmtex = surf->lightmaptexturenum;
 	
 	surf->normalchain = r_normalsurfaces;
 	r_normalsurfaces = surf;
-	surf->specularchain = r_specularsurfaces;
-	r_specularsurfaces = surf;
-	
+
 	for ( map = 0; map < MAXLIGHTMAPS && surf->styles[map] != 255; map++ )
 	{
 		if ( r_newrefdef.lightstyles[surf->styles[map]].white != surf->cached_light[map] )
@@ -655,9 +665,9 @@ dynamic:
 		}
 	}
 
-	if ( is_dynamic )
+	if ( is_dynamic && !(gl_glsl_shaders->value && gl_state.glsl_shaders) ) 
 	{
-		unsigned	temp[128*128];
+		unsigned	temp[DYNAMIC_LIGHT_WIDTH*DYNAMIC_LIGHT_HEIGHT];
 		int			smax, tmax;
 
 		smax = (surf->extents[0]>>4)+1;
@@ -684,9 +694,6 @@ dynamic:
 	if (SurfaceIsAlphaBlended(surf))
 		qglEnable( GL_ALPHA_TEST );
 
-	GL_MBind( GL_TEXTURE0, image->texnum );
-	GL_MBind( GL_TEXTURE1, gl_state.lightmap_textures + lmtex );
-
 	scroll = 0;
 	if (surf->texinfo->flags & SURF_FLOWING)
 	{
@@ -695,15 +702,166 @@ dynamic:
 			scroll = -64.0;
 	}
 
-	v = surf->polys->verts[0];
-	qglBegin (GL_POLYGON);
-	for (i=0 ; i< nv; i++, v+= VERTEXSIZE)
-	{
-		qglMTexCoord2fSGIS( GL_TEXTURE0, v[3]+scroll, v[4]);
-		qglMTexCoord2fSGIS( GL_TEXTURE1, v[5], v[6]);
-		qglVertex3fv (v);
+	if(gl_glsl_shaders->value && gl_state.glsl_shaders) {
+		dlight_t	*dl;
+		int			lnum, sv_lnum = 0;	
+		float		add, brightest = 0;
+		vec3_t		lightVec;
+		float		lightCutoffSquared;
+
+		if(is_dynamic) {
+
+			dl = r_newrefdef.dlights;
+			for (lnum=0; lnum<r_newrefdef.num_dlights; lnum++, dl++)
+			{
+				VectorSubtract (r_origin, dl->origin, lightVec);
+				add = dl->intensity - VectorLength(lightVec)/10;
+				if (add > brightest) //only bother with lights close by
+				{
+					brightest = add;
+					sv_lnum = lnum; //remember the position of most influencial light
+				}
+			}
+			if(brightest > 0) { //we have a light
+				dl = r_newrefdef.dlights;
+				dl += sv_lnum; //our most influential light
+				VectorSubtract(dl->origin, r_origin, lightVec);
+				VectorNormalize(lightVec);
+				VectorScale(lightVec, brightest/20, lightVec);
+			
+				lightCutoffSquared = ( dl->intensity - DLIGHT_CUTOFF );
+
+				if( lightCutoffSquared <= 0.0f ) 
+					lightCutoffSquared = 0.0f;
+
+				lightCutoffSquared *= 2.0f;
+				lightCutoffSquared *= lightCutoffSquared;
+			}
+		}
+
+		//parallax maps
+		if(gl_parallaxmaps->value && strcmp(surf->texinfo->heightMap->name, surf->texinfo->image->name) 
+			&& strcmp(surf->texinfo->normalMap->name, surf->texinfo->image->name)) {
+		
+			R_InitVArrays (VERT_MULTI_TEXTURED);
+
+			glUseProgramObjectARB( g_programObj );
+    		
+			GL_MBind( GL_TEXTURE0,  surf->texinfo->image->texnum);
+			glUniform1iARB( g_location_testTexture, 0); 
+		
+			GL_MBind( GL_TEXTURE1, surf->texinfo->heightMap->texnum);
+			glUniform1iARB( g_location_heightTexture, 1); 
+			
+			glUniform1iARB( g_location_lmTexture, 2);
+			qglActiveTextureARB(GL_TEXTURE2);
+			qglBindTexture(GL_TEXTURE_2D, gl_state.lightmap_textures + lmtex);
+			KillFlags |= KILL_TMU2_POINTER;
+
+			glUniform1iARB( g_location_normalTexture, 3);
+			qglActiveTextureARB(GL_TEXTURE3);
+			qglBindTexture(GL_TEXTURE_2D, surf->texinfo->normalMap->texnum);
+			KillFlags |= KILL_TMU3_POINTER;
+			
+			if(is_dynamic) {
+						
+				if(brightest > 0) {
+					
+					glUniform3fARB( g_location_lightPosition, dl->origin[0], dl->origin[1], dl->origin[2]);
+					glUniform3fARB( g_location_lightColour, dl->color[0], dl->color[1], dl->color[2]);
+
+					glUniform1fARB( g_location_lightCutoffSquared, lightCutoffSquared);
+					
+					glUniform1iARB( g_location_dynamic, 1);
+					if(gl_shadowmaps->value) { //dynamic world shadows
+						glUniform1iARB( g_location_bspShadowmapTexture, 7);
+						qglActiveTextureARB(GL_TEXTURE7);
+						qglBindTexture(GL_TEXTURE_2D, r_depthtexture->texnum);
+						glUniform1iARB( g_location_shadowmap, 1);
+					}
+				}
+				else {
+					glUniform1iARB( g_location_dynamic, 0);
+					glUniform1iARB( g_location_shadowmap, 0);
+				}
+			}
+			else 
+				glUniform1iARB( g_location_dynamic, 0);
+				
+			glUniform1iARB( g_location_parallax, 1);
+			if(gl_specular->value)
+				glUniform1iARB( g_location_specular, 1);
+			else
+				glUniform1iARB( g_location_specular, 0);
+			
+			R_AddGLSLShadedSurfToVArray (surf, scroll, true);
+
+			glUseProgramObjectARB( 0 );
+
+		}
+		//normal mapped surface for dynamic lights
+		else if(is_dynamic && brightest > 0 && strcmp(surf->texinfo->normalMap->name, surf->texinfo->image->name)) {
+					
+			R_InitVArrays (VERT_MULTI_TEXTURED);
+
+			glUseProgramObjectARB( g_programObj );
+    		
+			GL_MBind( GL_TEXTURE0,  surf->texinfo->image->texnum);
+			glUniform1iARB( g_location_testTexture, 0); 
+
+			GL_MBind( GL_TEXTURE1,  surf->texinfo->heightMap->texnum);
+			glUniform1iARB( g_location_heightTexture, 1);
+
+			glUniform1iARB( g_location_lmTexture, 2);
+			qglActiveTextureARB(GL_TEXTURE2);
+			qglBindTexture(GL_TEXTURE_2D, gl_state.lightmap_textures + lmtex);
+			KillFlags |= KILL_TMU2_POINTER;
+
+			glUniform1iARB( g_location_normalTexture, 3);
+			qglActiveTextureARB(GL_TEXTURE3);
+			qglBindTexture(GL_TEXTURE_2D, surf->texinfo->normalMap->texnum);
+			KillFlags |= KILL_TMU3_POINTER;
+
+			if(gl_shadowmaps->value) { //dynamic world shadows
+				glUniform1iARB( g_location_bspShadowmapTexture, 7);
+				qglActiveTextureARB(GL_TEXTURE7);
+				qglBindTexture(GL_TEXTURE_2D, r_depthtexture->texnum);
+				glUniform1iARB( g_location_shadowmap, 1);
+			}
+	
+			glUniform3fARB( g_location_lightPosition, dl->origin[0], dl->origin[1], dl->origin[2]);
+			glUniform3fARB( g_location_lightColour, dl->color[0], dl->color[1], dl->color[2]);
+			glUniform1fARB( g_location_lightCutoffSquared, lightCutoffSquared);
+			glUniform1iARB( g_location_dynamic, 1);
+			glUniform1iARB( g_location_parallax, 0);
+			
+			R_AddGLSLShadedSurfToVArray (surf, scroll, true);
+
+			glUseProgramObjectARB( 0 );
+
+		}
+		//surface has no normalmap
+		else {
+		
+			GL_MBind( GL_TEXTURE0, image->texnum );
+			GL_MBind( GL_TEXTURE1, gl_state.lightmap_textures + lmtex );
+		
+			R_InitVArrays (VERT_MULTI_TEXTURED);	
+			
+			R_AddLightMappedSurfToVArray (surf, scroll);
+		}
 	}
-	qglEnd ();
+	else {	//no glsl, standard render
+		
+		GL_MBind( GL_TEXTURE0, image->texnum );
+		GL_MBind( GL_TEXTURE1, gl_state.lightmap_textures + lmtex );
+		
+		R_InitVArrays (VERT_MULTI_TEXTURED);	
+			
+		R_AddLightMappedSurfToVArray (surf, scroll);
+	}
+		
+	R_KillVArrays ();
 
 	if (SurfaceIsAlphaBlended(surf))
 		qglDisable( GL_ALPHA_TEST);
@@ -711,52 +869,30 @@ dynamic:
 
 //This next section deals with bumpmapped surfaces.  Much of this was gathered from Mike Hiney
 //and "Paul's Projects" tutorials.
-static void R_InitNormalSurfaces (void)
-{
-	dlight_t	*dl;
-	int			lnum;
-	vec3_t		lightAdd, angle;
-	float		add;
+extern GLuint normalisationCubeMap;
+static void R_InitNormalSurfaces ()
+{	
 
 	qglActiveTextureARB (GL_TEXTURE0);
 	qglDisable (GL_TEXTURE_2D);
 	qglEnable (GL_TEXTURE_CUBE_MAP_ARB);
 
-	qglBindTexture (GL_TEXTURE_CUBE_MAP_ARB, r_cubemap->texnum);
+	qglBindTexture (GL_TEXTURE_CUBE_MAP_ARB, normalisationCubeMap);
 	qglMatrixMode (GL_TEXTURE);
 	qglLoadIdentity ();
 
 	// rotate around Y to bring blue/pink to the front
 	qglRotatef (145, 0, 1, 0);
 
-	// now reposition so that the bright spot is center screen
+	// now reposition so that the bright spot is center screen, and up a little 
 	qglRotatef (-45, 1, 0, 0);
 	qglRotatef (-45, 0, 0, 1);
 
-	// rotate by viewangles  
+	//rotate by view angles(area in front of view brightens up, lower depth)
 	qglRotatef (-r_newrefdef.viewangles[2], 1, 0, 0);
 	qglRotatef (-r_newrefdef.viewangles[0], 0, 1, 0);
 	qglRotatef (-r_newrefdef.viewangles[1], 0, 0, 1);
 
-	//now rotate according to dynamic lights
-	VectorClear(lightAdd);
-	dl = r_newrefdef.dlights;
-	for (lnum=0; lnum<r_newrefdef.num_dlights; lnum++, dl++)
-	{
-
-		VectorSubtract (r_newrefdef.vieworg, dl->origin, angle);
-		add = sqrt(dl->intensity - VectorLength(angle));
-		VectorNormalize(angle);
-		if (add > 0)
-		{
-			VectorScale(angle, add, angle);
-			VectorAdd (lightAdd, angle, lightAdd);
-		}
-	}
-	qglRotatef (-lightAdd[2], 1, 0, 0);
-	qglRotatef (-lightAdd[0], 0, 1, 0);
-	qglRotatef (-lightAdd[1], 0, 0, 1);
-
 	// the next 2 statements will move the cmstr calculations into hardware so that we don;t
 	// have to evaluate them once per vert...
 
@@ -773,12 +909,25 @@ static void R_InitNormalSurfaces (void)
 	qglTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 }
 
+void R_KillNormalTMUs(void) {
+	
+	//kill TMU1
+	qglActiveTextureARB (GL_TEXTURE1);
+	qglTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
+	qglDisable (GL_TEXTURE_2D);
+
+	//kill TMU0
+	qglActiveTextureARB (GL_TEXTURE0);
+	qglMatrixMode (GL_TEXTURE);
+	qglLoadIdentity ();
+	qglMatrixMode (GL_MODELVIEW);
+	qglDisable (GL_TEXTURE_CUBE_MAP_ARB);
+	qglEnable (GL_TEXTURE_2D);
+}
+
 static void R_DrawNormalSurfaces (void)
 {
 	msurface_t *surf = r_normalsurfaces;
-	int		i;
-	float	*v;
-	glpoly_t *p;
 
 	// nothing to draw!
 	if (!surf)
@@ -787,6 +936,7 @@ static void R_DrawNormalSurfaces (void)
 	if (!gl_normalmaps->value)
 		return;
 
+	R_InitVArrays (VERT_BUMPMAPPED);
 	R_InitNormalSurfaces();
 
 	qglActiveTextureARB (GL_TEXTURE1);
@@ -813,106 +963,18 @@ static void R_DrawNormalSurfaces (void)
 		else
 			continue;
 
-		for (p = surf->polys; p; p = p->chain)
-		{
-			qglBegin (GL_POLYGON);
-
-			for (v = p->verts[0], i = 0 ; i < p->numverts; i++, v += VERTEXSIZE)
-			{		
-				qglMTexCoord3fSGIS(GL_TEXTURE0, v[0], v[1], v[2]);
-				qglMTexCoord2fSGIS(GL_TEXTURE1, v[3], v[4]);
-				qglVertex3fv (v);
-			}
-
-			qglEnd ();
-		}
+		R_AddTexturedSurfToVArray (surf, 0);
 	}
 
-	//kill TMU1
-	qglActiveTextureARB (GL_TEXTURE1);
-	qglTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
-	qglDisable (GL_TEXTURE_2D);
-
-	//kill TMU0
-	qglActiveTextureARB (GL_TEXTURE0);
-	qglMatrixMode (GL_TEXTURE);
-	qglLoadIdentity ();
-	qglMatrixMode (GL_MODELVIEW);
-	qglDisable (GL_TEXTURE_CUBE_MAP_ARB);
-	qglEnable (GL_TEXTURE_2D);
+	R_KillNormalTMUs();
+	R_KillVArrays ();
 
 	// restore original blend
 	qglTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
 	qglBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 	qglDisable (GL_BLEND);
 }
-static void R_DrawSpecularSurfaces (void)
-{
-	msurface_t *surf = r_specularsurfaces;
-	int		i;
-	float	*v;
-	glpoly_t *p;
 
-	// nothing to draw!
-	if (!surf)
-		return;
-
-	if (!gl_normalmaps->value || !gl_specularmaps->value)
-		return;
-
-	R_InitNormalSurfaces();
-
-	qglActiveTextureARB (GL_TEXTURE1);
-	qglEnable (GL_TEXTURE_2D);
-	qglTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
-
-	qglDepthMask (GL_FALSE); 
-	qglEnable (GL_BLEND); 
-	qglBlendFunc (GL_ONE, GL_ONE);
-	
-	for (; surf; surf = surf->specularchain)
-	{
-		if (SurfaceIsAlphaBlended(surf))
-			continue;
-		if(strcmp(surf->texinfo->specularMap->name, surf->texinfo->image->name))
-			qglBindTexture (GL_TEXTURE_2D, surf->texinfo->specularMap->texnum);
-		else
-			continue;
-
-		for (p = surf->polys; p; p = p->chain)
-		{
-			qglBegin (GL_POLYGON);
-
-			for (v = p->verts[0], i = 0 ; i < p->numverts; i++, v += VERTEXSIZE)
-			{			
-				qglMTexCoord3fSGIS(GL_TEXTURE0, v[0], v[1], v[2]);
-				qglMTexCoord2fSGIS(GL_TEXTURE1, v[3], v[4]);
-				qglVertex3fv (v);
-			}
-
-			qglEnd ();
-		}
-	}
-
-		//kill TMU1
-	qglActiveTextureARB (GL_TEXTURE1);
-	qglTexEnvi (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
-	qglDisable (GL_TEXTURE_2D);
-
-	//kill TMU0
-	qglActiveTextureARB (GL_TEXTURE0);
-	qglMatrixMode (GL_TEXTURE);
-	qglLoadIdentity ();
-	qglMatrixMode (GL_MODELVIEW);
-	qglDisable (GL_TEXTURE_CUBE_MAP_ARB);
-	qglEnable (GL_TEXTURE_2D);
-
-	// restore original blend
-	qglDepthMask (GL_TRUE);
-	qglTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
-	qglBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-	qglDisable (GL_BLEND);
-}
 /*
 =================
 R_DrawInlineBModel
@@ -935,7 +997,6 @@ void R_DrawInlineBModel (entity_t *e)
 	}
 
 	r_normalsurfaces = NULL;
-	r_specularsurfaces = NULL;
 
 	//
 	// draw texture
@@ -976,13 +1037,13 @@ void R_DrawInlineBModel (entity_t *e)
 	{
 		if ( !qglMTexCoord2fSGIS ) {
 			R_BlendLightmaps ();
-
-			GL_EnableMultitexture (true);
-			R_DrawNormalSurfaces ();
-			R_DrawSpecularSurfaces ();
-			GL_EnableMultitexture (false);
 		}
-
+		else
+		{
+			GL_EnableMultitexture( false );
+			R_DrawNormalSurfaces();
+			GL_EnableMultitexture( true );
+		}
 	}
 	else
 	{
@@ -1025,8 +1086,6 @@ void R_DrawBrushModel (entity_t *e)
 		VectorAdd (e->origin, currentmodel->maxs, maxs);
 	}
 
-	R_PushStainsForBModel ( e );
-
 	if (R_CullBox (mins, maxs)) {
 		return;
 	}
@@ -1034,30 +1093,7 @@ void R_DrawBrushModel (entity_t *e)
 	qglColor3f (1,1,1);
 	memset (gl_lms.lightmap_surfaces, 0, sizeof(gl_lms.lightmap_surfaces));
 
-	//VectorSubtract (r_newrefdef.vieworg, e->origin, modelorg);
-
-	// start MPO
-	// if this is a reflection we're drawing we need to flip the vertical position across the water
-	if (g_drawing_refl)
-	{
-		/*
-		vec3_t tmp;
-		VectorCopy(r_newrefdef.vieworg, tmp);
-		tmp[2] = (2*g_refl_Z[g_active_refl]) - tmp[2];	// flip
-		VectorSubtract(tmp, e->origin, modelorg);
-		*/
-
-		// equivalent, faster code
-        modelorg[0] = r_newrefdef.vieworg[0] - e->origin[0];
-        modelorg[1] = r_newrefdef.vieworg[1] - e->origin[1];
-        modelorg[2] = ((2*g_refl_Z[g_active_refl]) - r_newrefdef.vieworg[2])
-        	- e->origin[2];
-	}
-	else
-	{
-		VectorSubtract (r_newrefdef.vieworg, e->origin, modelorg);
-	}
-	// stop MPO
+	VectorSubtract (r_newrefdef.vieworg, e->origin, modelorg);
 
 	if (rotated)
 	{
@@ -1081,10 +1117,10 @@ void R_DrawBrushModel (entity_t *e)
 	GL_EnableMultitexture( true );
 	GL_SelectTexture( GL_TEXTURE0);
 
-	// Vic - begin
 	if ( !gl_config.mtexcombine ) {
 
 		GL_TexEnv( GL_REPLACE );
+
 		GL_SelectTexture( GL_TEXTURE1);
 
 		if ( gl_lightmap->value )
@@ -1121,8 +1157,6 @@ void R_DrawBrushModel (entity_t *e)
 
 	}
 
-	// Vic - end
-
 	R_DrawInlineBModel (e);
 	GL_EnableMultitexture( false );
 
@@ -1150,6 +1184,7 @@ void R_RecursiveWorldNode (mnode_t *node, int clipflags)
 	mleaf_t		*pleaf;
 	float		dot;
 	image_t		*image;
+	rscript_t *rs_shader;
 
 	if (node->contents == CONTENTS_SOLID)
 		return;		// solid
@@ -1238,6 +1273,7 @@ void R_RecursiveWorldNode (mnode_t *node, int clipflags)
 
 		if ( (surf->flags & SURF_PLANEBACK) != sidebit )
 			continue;		// wrong side
+
 		if (surf->texinfo->flags & SURF_SKY)
 		{	// just adds to visible sky bounds
 			R_AddSkySurface (surf);
@@ -1254,8 +1290,13 @@ void R_RecursiveWorldNode (mnode_t *node, int clipflags)
 			{
 				GL_RenderLightmappedPoly( surf );
 
-				surf->specialchain = r_special_surfaces;
-				r_special_surfaces = surf;
+				if(r_shaders->value) { //only add to the chain if there is actually a shader
+					rs_shader = (rscript_t *)surf->texinfo->image->script;
+					if(rs_shader || (surf->flags & SURF_UNDERWATER)) {
+						surf->specialchain = r_special_surfaces;
+						r_special_surfaces = surf;
+					}
+				}
 			}
 			else
 			{
@@ -1266,8 +1307,13 @@ void R_RecursiveWorldNode (mnode_t *node, int clipflags)
 				surf->texturechain = image->texturechain;
 				image->texturechain = surf;
 
-				surf->specialchain = r_special_surfaces;
-				r_special_surfaces = surf;
+				if(r_shaders->value) { //only add to the chain if there is actually a shader
+					rs_shader = (rscript_t *)surf->texinfo->image->script;
+					if(rs_shader) {
+						surf->specialchain = r_special_surfaces;
+						r_special_surfaces = surf;
+					}
+				}
 			}
 		}
 	}
@@ -1296,9 +1342,6 @@ void R_DrawWorld (void)
 
 	VectorCopy (r_newrefdef.vieworg, modelorg);
 
-	if (g_drawing_refl) // jitwater / MPO
-		modelorg[2] = (2.0f * g_refl_Z[g_active_refl]) - modelorg[2]; // flip
-
 	// auto cycle the world frame for texture animation
 	memset (&ent, 0, sizeof(ent));
 	ent.frame = (int)(r_newrefdef.time*2);
@@ -1310,7 +1353,6 @@ void R_DrawWorld (void)
 	memset (gl_lms.lightmap_surfaces, 0, sizeof(gl_lms.lightmap_surfaces));
 
 	r_normalsurfaces = NULL;
-	r_specularsurfaces = NULL;
 
 	R_ClearSkyBox ();
 
@@ -1320,15 +1362,15 @@ void R_DrawWorld (void)
 
 		GL_SelectTexture( GL_TEXTURE0);
 
-		// Vic - begin
 		if ( !gl_config.mtexcombine ) {
 			GL_TexEnv( GL_REPLACE );
-			GL_SelectTexture( GL_TEXTURE1);
+				GL_SelectTexture( GL_TEXTURE1);
 
 			if ( gl_lightmap->value )
 				GL_TexEnv( GL_REPLACE );
 			else
 				GL_TexEnv( GL_MODULATE );
+		
 		} else {
 			GL_TexEnv ( GL_COMBINE_EXT );
 			qglTexEnvi ( GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_REPLACE );
@@ -1356,8 +1398,7 @@ void R_DrawWorld (void)
 				qglTexEnvi ( GL_TEXTURE_ENV, GL_RGB_SCALE_EXT, r_overbrightbits->value );
 			}
 		}
-		// Vic - end
-
+		
 		R_RecursiveWorldNode (r_worldmodel->nodes, 15);
 
 		GL_EnableMultitexture( false );
@@ -1376,10 +1417,13 @@ void R_DrawWorld (void)
 
 	GL_EnableMultitexture( true );
 	R_DrawNormalSurfaces ();
-	R_DrawSpecularSurfaces ();
 	GL_EnableMultitexture( false );	
 
-	R_DrawSkyBox ();
+	R_InitSun();
+
+	qglDepthMask(0);
+	R_DrawSkyBox();
+	qglDepthMask(1);
 
 	R_DrawTriangleOutlines ();
 }
@@ -1672,25 +1716,6 @@ void GL_CreateSurfaceLightmap (msurface_t *surf)
 	R_BuildLightMap (surf, base, BLOCK_WIDTH*LIGHTMAP_BYTES);
 }
 
-/*
-========================
-GL_CreateSurfaceStainmap
-========================
-*/
-void GL_CreateSurfaceStainmap (msurface_t *surf)
-{
-	int		smax, tmax;
-
-	if (!surf->samples)
-		return;
-
-	smax = (surf->extents[0]>>4)+1;
-	tmax = (surf->extents[1]>>4)+1;
-
-	surf->stainsamples = Hunk_Alloc (smax*tmax*3);
-	memset (surf->stainsamples, 255, smax*tmax*3);
-}
-
 /*
 ==================
 GL_BeginBuildingLightmaps
