@@ -27,7 +27,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ** GLimp_EndFrame
 ** GLimp_Init
 ** GLimp_Shutdown
-** GLimp_SwitchFullscreen
 **
 */
 
@@ -68,13 +67,15 @@ static int scrnum;
 static Window win;
 static GLXContext ctx = NULL;
 static Atom wmDeleteWindow;
+static Atom cor_clipboard;
 
 qboolean have_stencil = false; // Stencil shadows - MrG
 
 #define KEY_MASK (KeyPressMask | KeyReleaseMask)
 #define MOUSE_MASK (ButtonPressMask | ButtonReleaseMask | \
 		    PointerMotionMask | ButtonMotionMask )
-#define X_MASK (KEY_MASK | MOUSE_MASK | VisibilityChangeMask | StructureNotifyMask )
+#define X_MASK (KEY_MASK | MOUSE_MASK | VisibilityChangeMask | \
+	       	StructureNotifyMask | PropertyChangeMask )
 
 /*****************************************************************************/
 /* MOUSE                                                                     */
@@ -118,8 +119,6 @@ static Cursor CreateNullCursor(Display *display, Window root)
 
 void install_grabs(void)
 {
-
-	XDefineCursor(dpy, win, CreateNullCursor(dpy, win));
 	XGrabPointer(dpy, win, True, 0, GrabModeAsync, GrabModeAsync, win, None, CurrentTime);
 
 	if (in_dgamouse->integer)
@@ -127,8 +126,8 @@ void install_grabs(void)
 		int MajorVersion, MinorVersion;
 
 		if (XF86DGAQueryVersion(dpy, &MajorVersion, &MinorVersion)) {
-			XF86DGADirectVideo(dpy, DefaultScreen(dpy), XF86DGADirectMouse);
 			XWarpPointer(dpy, None, win, 0, 0, 0, 0, 0, 0);
+			XF86DGADirectVideo(dpy, DefaultScreen(dpy), XF86DGADirectMouse);
 			dgamouse = true;
 		} else {
 			// unable to query, probalby not supported
@@ -159,8 +158,6 @@ void uninstall_grabs(void)
 	XUngrabPointer(dpy, CurrentTime);
 	XUngrabKeyboard(dpy, CurrentTime);
 
-	XUndefineCursor(dpy, win);
-
 	mouse_active = false;
 }
 
@@ -341,8 +338,8 @@ void HandleEvents(void)
 		case MotionNotify:
 			if (mouse_active) {
 				if (dgamouse) {
-					mx += (event.xmotion.x + win_x) * 2;
-					my += (event.xmotion.y + win_y) * 2;
+					mx += (event.xmotion.x + (vidmode_active ? 0 : win_x)) * 2;
+					my += (event.xmotion.y + (vidmode_active ? 0 : win_y)) * 2;
 				} 
 				else 
 				{
@@ -353,6 +350,12 @@ void HandleEvents(void)
 						dowarp = true;
 				}
 			}
+			else
+			{
+				// allow mouse movement on menus in windowed mode
+				mx = event.xmotion.x;
+				my = event.xmotion.y;
+			}
 			break;
 
 
@@ -533,7 +536,8 @@ int GLimp_SetMode( int *pwidth, int *pheight, int mode, qboolean fullscreen )
 	}
 	else
 		have_stencil = true;
-	
+
+	vidmode_active = false;
 	if (vidmode_ext) {
 		int best_fit, best_dist, dist, x, y;
 		
@@ -614,6 +618,8 @@ int GLimp_SetMode( int *pwidth, int *pheight, int mode, qboolean fullscreen )
 	wmDeleteWindow = XInternAtom(dpy, "WM_DELETE_WINDOW", False);
 	XSetWMProtocols(dpy, win, &wmDeleteWindow, 1);
 
+	cor_clipboard = XInternAtom(dpy, "COR_CLIPBOARD", False);
+
 	XMapWindow(dpy, win);
 
 	if (vidmode_active) {
@@ -637,18 +643,12 @@ int GLimp_SetMode( int *pwidth, int *pheight, int mode, qboolean fullscreen )
 	// let the sound and input subsystems know about the new window
 	VID_NewWindow (width, height);
 
+	XDefineCursor(dpy, win, CreateNullCursor(dpy, win));
+
 	qglXMakeCurrent(dpy, win, ctx);
 
 	RS_ScanPathForScripts();		// load all found scripts
 
-	// Vertex arrays
-	qglEnableClientState (GL_VERTEX_ARRAY);
-	qglEnableClientState (GL_TEXTURE_COORD_ARRAY);
-
-	qglTexCoordPointer (2, GL_FLOAT, sizeof(tex_array[0]), tex_array[0]);
-	qglVertexPointer (3, GL_FLOAT, sizeof(vert_array[0]), vert_array[0]);
-	qglColorPointer (4, GL_FLOAT, sizeof(col_array[0]), col_array[0]);
-
 	return rserr_ok;
 }
 
@@ -680,7 +680,6 @@ void GLimp_Shutdown( void )
 	ctx = NULL;
 	dpy = NULL;
 	win = 0;
-	ctx = NULL;
 }
 
 /*
@@ -712,8 +711,7 @@ void GLimp_BeginFrame( float camera_seperation )
 */
 void GLimp_EndFrame (void)
 {
-	qglFlush();
-	qglXSwapBuffers(dpy, win);
+	qglXSwapBuffers(dpy, win); // does glFlush()
 
 	// rscript - MrG
 	rs_realtime=Sys_Milliseconds() * 0.0005f;
@@ -745,6 +743,64 @@ void Fake_glColorTableEXT( GLenum target, GLenum internalformat,
 
 
 /*------------------------------------------------*/
-/* X11 Input Stuff
+/* X11 Input Stuff				  */
 /*------------------------------------------------*/
 
+/*
+** Sys_GetClipboardData()
+**
+** Returns the contents of the X clipboard; needs to
+** be here instead of inside sys_unix.c because it
+** needs to access the X display.
+*/
+char *Sys_GetClipboardData(void)
+{
+	XEvent evt;
+	int sent_at;
+	Bool received;
+	Atom prop_type;
+	int prop_fmt;
+	unsigned long prop_items, prop_size;
+	unsigned char *buffer, *output = NULL;
+
+	// request the contents of the clipboard
+	XConvertSelection( dpy, XA_PRIMARY, XA_STRING,
+		cor_clipboard, win, CurrentTime );
+	sent_at = Sys_Milliseconds();
+
+	// now we need to wait until either:
+	//  1) we get a response
+	//  2) 250ms have gone by and we got nothing
+	do
+       	{
+		received = XCheckTypedEvent( dpy, SelectionNotify, &evt );
+	} while ( !( received || Sys_Milliseconds() - sent_at > 250 ) );
+
+	// no reply received, return NULL
+	if ( !received )
+		return NULL;
+
+	// get information about property
+	XGetWindowProperty( dpy , win, cor_clipboard, 0, 0, False,
+			AnyPropertyType, &prop_type, &prop_fmt,
+			&prop_items, &prop_size, &buffer );
+	XFree(buffer);
+
+	// only get the text if it's actually ASCII
+	if ( prop_fmt == 8 )
+	{
+		XGetWindowProperty( dpy , win, cor_clipboard, 0,
+				prop_size, False, AnyPropertyType,
+				&prop_type, &prop_fmt, &prop_items,
+				&prop_size, &buffer );
+
+		output = malloc( prop_items + 1 );
+		memcpy( output, buffer, prop_items );
+		output[prop_items] = 0;
+	}
+
+	XDeleteProperty(dpy, win, cor_clipboard);
+
+	return output;
+}
+
